/**********************************************************************************/
/*  vproweather 1.0.9                                                            */
/*    A program for acquiring data from a Davis Vantage Pro Weather Station.      */
/*                                                                                */
/* (c)2004 Joe Jaworski    email: jj@joejaworski.com                              */
/* (c)2005-2013 Curt Blank email: curt@curtronics.com                             */
/* VPROWEATHER is covered under the GNU general license. See the attached         */
/* LICENSE file for details.                                                      */
/*                                                                                */
/* This software is provided as-is, without any expressed or implied              */
/* warranties, including, without limitation, the implied warranties of           */
/* merchantibility and fitness for any particular purpose.                        */
/*                                                                                */
/*     v0.6-1  cjb - fixed _HUM_LO_BY_DAYS copying data for                       */
/*                   HUM_OUT_DAY_LOW_TIMES instead of HUM_OUT_DAY_LOW             */
/*     v0.6-2  cjb - added and used bMtime in TimeConvert                         */
/*     v0.6-3  cjb - Don't bother using NEXT_10MIN_PTR or NEXT_15MIN_PTR          */
/*                   they never get updated per Davis                             */
/*     v0.6-4  cjb - print _BARO_CURR a.k.a rtBaroCurr out to 3 decimal places    */
/*                 - do the same for all _BARO_ attributes                        */
/*     v0.6-5  cjb - changed strftime, got rid of the commas for one thing        */
/*     v0.6-6  cjb - added PrintSolarSet to output Solar graph data               */
/*                 - do not divide Solar data by 10 (.i.e. not / 10.0)            */
/*                 - added PrintETByteSet & PrintETSet to output ET graph data	  */
/*                 - also output THSW data                                        */
/*     v0.6-7  cjb - fix _WIND_DIR_ROSE (compass rose version) when testing for	  */
/*                   >= 347 && < 12                                               */
/*     v0.6-8  cjb - convert hld.wTHSWHi(Day|Month|Year) using (signed short)     */
/*                 - change THSW availibity check to                              */
/*                   hld.wTHSWHi(Day|Month|Year) == 65535                         */
/*     v0.7-1  cjb - we seemed to always have ignored WIND_SPEED_HOUR_HIGHS       */
/*                   (from the day of Joe) so let's add it now, no reason not to  */
/*     v0.7-3  cjb - Vantage Pro2 reports some null values as 32768 so zero them  */
/*     v0.7-4  cjb - added WIND_HI_DIR_BY_DAYS                                    */
/*                 - added WIND_HI_DIR_BY_MONTHS                                  */
/*                 - added WIND_HI_DIR_BY_YEARS                                   */
/*                 - added UV_MEDS_BY_HOURS                                       */
/*                 - added UV_MEDS_BY_DAYS                                        */
/*     v0.7-5  cjb - looks like VP2's offset RAIN_RATE_1_MIN by 32767, gonna      */
/*                  have to keep an eye on this                                   */
/*     v0.8-0  cjb - add access to mySQL db                                       */
/*                 - add (unsigned char *) cast to all passing of pData to        */
/*                   finally get rid of the annoying 'differ in signedness'       */
/*                   compiler warning                                             */
/*                 - create functions BaroTrend, BaroTrendImage, WindRoseDir and  */
/*                   move respective code to them                                 */
/*     v0.8-7  cjb - add and use bVerbose                                         */
/*                 - fix buffer overflow in TimeRef10Min, TimeRef15Min, &         */
/*                   TimeRefHours when using 12 hour time                         */
/*     v0.8-8  cjb - add Sunrise and Sunset to output of -Z option                */
/*     v0.8-9  cjb - add handling for Little Endian machines via compiler flag    */
/*                   for RT & HL data (graph data after testing)                  */
/*     v0.8-10 cjb - modify BaroTrend & BaroTrendImage to take into account       */
/*                   unsigned values along with singed values (multi-architecture */
/*                   support)                                                     */
/*                 - modify THSW check (i.e. no solar radiation detector)         */
/*     v0.8-14 cjb - check for 65536 on 64 bit and zero out bad values		  */
/*     v0.8-43 cjb - math format screw up in new HeatIndex calculation            */
/*     v0.9.0  cjb - added routines for handling DMP & DMPAFT                     */
/*     v0.9.1  cjb - added dmpdConvertEndian routine to swap Endian             */
/*     v0.9.6  cjb - in OutputData print [OK] if bRTDSuccess                      */
/*     v0.9.7  cjb - in OutputData fixed sunrise being printed for both sunrise & */
/*                   sunset                                                       */
/*     v0.9.9  cjb - added a "-" (dash) for the Barometric Trend in the -Z output */
/*                   for easier column parsing by applications                    */
/*                   i.e. Falling-Slowly.                                         */
/*                 - -Z option is now documented and supported                    */
/*                 - added ListOutputDataCols function in support of the now      */
/*                   documented -Z option                                         */
/*                 - calculate CurRain to include in output of -Z option if it    */
/*                   can be determined from the previous entry in the -Z file -   */
/*                   if not it's value will match that of RainDay                 */
/*     v1.0.0  cjb - switch to using wStormRain to calculate CurRain              */
/*                 - changed CurRain to IncRain - there is no way to insure it is */
/*                   the current rain, only that it is the incremental amount     */
/*                   since the last reading                                       */
/*                 - runDST fromat in OutputData does not need to be %3i should   */
/*                   be %i                                                        */
/*                 - added calcSunRiseSetNoon and all its associated functions    */
/*                   using NOAA's algorithm                                       */
/*                 - in OutputData (-Z) don't output erroneous data for Barometer */
/*                   Trend and TrendImg if getting Real Time data was not         */
/*                   successful                                                   */
/*                 - added -C option to use "," (commas) as deliminators in -Z    */
/*                   output instead of tab deliminated                            */
/*     v1.0.0  cjb - In TimeConvert check if value to convert is 0x7fff or 0xfff  */
/*                   or O and if so return "n/a"                                  */
/*     v1.0.1 cjb  - added rtDayEToz water ounces of Evapotranspiration           */
/*                 - modified TimeRefMonths issue that may occur on last day of   */
/*                   month generating list of months                              */
/*                 - In TimeRefYears only print 24 years not 25. The Davis EEPROM */
/*                   keeps 24+1 records in some data sets, which the extra one is */
/*                   the current value (month or year). We discard this to keep   */
/*                   all graphs consistent at 24.                                 */
/*                 - added grETOZByHours, grETOZDayTotalByDays,                   */
/*                   grETQTMonthTotalByMonths, grETGALYearTotalByYears sets       */
/*     v1.0.4  cjb - added -2 option for use with Vantage Pro 2 and removed       */
/*                   compile flag, now a runtime option                           */
/*     v1.0.5  cjb - when making graph set lists check for overflow and return    */
/*                   'overflow error' if found                                    */
/*                 - graph data values are now reported as "n/a" instead of 0     */
/*                   when sensors do not exist or when historical data is not     */
/*                   available                                                    */
/*     v1.0.6  cjb - finally finish up the metric conversion option               */
/*                 - add grUVHiByYears to the output                              */
/*                 - added new "-F, --format" option                              */
/*                 - archive Prevailing and High Wind Direction now displayed as  */
/*                   direction and not number                                     */
/*     v1.0.7 cjb  - added check for bTipMod                                      */
/*     v1.0.9 cjb  - metric conversion in PrintRainRateByteSet needed value[7]    */
/*                                                                                */
/* dhandler.c - data handler functions                                            */
/*                                                                                */
/**********************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <memory.h>
#include <math.h>
#include <sys/stat.h>
#include "include/names.h"
#include "include/dhandler.h"
#ifndef noMySQL
#include "include/mySQLnames.h"
#endif

/* CCITT table of CRC values */
static unsigned short crc_table [] = {
0x0,     0x1021,  0x2042,  0x3063,  0x4084,  0x50a5,  0x60c6,  0x70e7,  
0x8108,  0x9129,  0xa14a,  0xb16b,  0xc18c,  0xd1ad,  0xe1ce,  0xf1ef,  
0x1231,  0x210,   0x3273,  0x2252,  0x52b5,  0x4294,  0x72f7,  0x62d6,  
0x9339,  0x8318,  0xb37b,  0xa35a,  0xd3bd,  0xc39c,  0xf3ff,  0xe3de,  
0x2462,  0x3443,  0x420,   0x1401,  0x64e6,  0x74c7,  0x44a4,  0x5485,  
0xa56a,  0xb54b,  0x8528,  0x9509,  0xe5ee,  0xf5cf,  0xc5ac,  0xd58d,  
0x3653,  0x2672,  0x1611,  0x630,   0x76d7,  0x66f6,  0x5695,  0x46b4,  
0xb75b,  0xa77a,  0x9719,  0x8738,  0xf7df,  0xe7fe,  0xd79d,  0xc7bc,  
0x48c4,  0x58e5,  0x6886,  0x78a7,  0x840,   0x1861,  0x2802,  0x3823,  
0xc9cc,  0xd9ed,  0xe98e,  0xf9af,  0x8948,  0x9969,  0xa90a,  0xb92b,  
0x5af5,  0x4ad4,  0x7ab7,  0x6a96,  0x1a71,  0xa50,   0x3a33,  0x2a12,  
0xdbfd,  0xcbdc,  0xfbbf,  0xeb9e,  0x9b79,  0x8b58,  0xbb3b,  0xab1a,  
0x6ca6,  0x7c87,  0x4ce4,  0x5cc5,  0x2c22,  0x3c03,  0xc60,   0x1c41,  
0xedae,  0xfd8f,  0xcdec,  0xddcd,  0xad2a,  0xbd0b,  0x8d68,  0x9d49,  
0x7e97,  0x6eb6,  0x5ed5,  0x4ef4,  0x3e13,  0x2e32,  0x1e51,  0xe70,  
0xff9f,  0xefbe,  0xdfdd,  0xcffc,  0xbf1b,  0xaf3a,  0x9f59,  0x8f78,  
0x9188,  0x81a9,  0xb1ca,  0xa1eb,  0xd10c,  0xc12d,  0xf14e,  0xe16f,  
0x1080,  0xa1,    0x30c2,  0x20e3,  0x5004,  0x4025,  0x7046,  0x6067,  
0x83b9,  0x9398,  0xa3fb,  0xb3da,  0xc33d,  0xd31c,  0xe37f,  0xf35e,  
0x2b1,   0x1290,  0x22f3,  0x32d2,  0x4235,  0x5214,  0x6277,  0x7256,  
0xb5ea,  0xa5cb,  0x95a8,  0x8589,  0xf56e,  0xe54f,  0xd52c,  0xc50d,  
0x34e2,  0x24c3,  0x14a0,  0x481,   0x7466,  0x6447,  0x5424,  0x4405,  
0xa7db,  0xb7fa,  0x8799,  0x97b8,  0xe75f,  0xf77e,  0xc71d,  0xd73c,  
0x26d3,  0x36f2,  0x691,   0x16b0,  0x6657,  0x7676,  0x4615,  0x5634,  
0xd94c,  0xc96d,  0xf90e,  0xe92f,  0x99c8,  0x89e9,  0xb98a,  0xa9ab,  
0x5844,  0x4865,  0x7806,  0x6827,  0x18c0,  0x8e1,   0x3882,  0x28a3,  
0xcb7d,  0xdb5c,  0xeb3f,  0xfb1e,  0x8bf9,  0x9bd8,  0xabbb,  0xbb9a,  
0x4a75,  0x5a54,  0x6a37,  0x7a16,  0xaf1,   0x1ad0,  0x2ab3,  0x3a92,  
0xfd2e,  0xed0f,  0xdd6c,  0xcd4d,  0xbdaa,  0xad8b,  0x9de8,  0x8dc9,  
0x7c26,  0x6c07,  0x5c64,  0x4c45,  0x3ca2,  0x2c83,  0x1ce0,  0xcc1,  
0xef1f,  0xff3e,  0xcf5d,  0xdf7c,  0xaf9b,  0xbfba,  0x8fd9,  0x9ff8,  
0x6e17,  0x7e36,  0x4e55,  0x5e74,  0x2e93,  0x3eb2,  0xed1,   0x1ef0,  
};


static char szForeStrings[] = 

"Mostly clear and cooler.\0"
"Mostly clear with little temperature change.\0"
"Mostly clear for 12 hrs. with little temperature change.\0"
"Mostly clear for 12 to 24 hrs. and cooler.\0"
"Mostly clear with little temperature change.\0"
"Partly cloudy and cooler.\0"
"Partly cloudy with little temperature change.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear and warmer.\0"
"Partly cloudy with little temperature change.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and warmer. Precipitation possible within 24 to 48 hrs.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds with little temperature change. Precipitation possible within 24 hrs.\0"
"Mostly clear with little temperature change.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds with little temperature change. Precipitation possible within 12 hrs.\0"
"Mostly clear with little temperature change.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and warmer. Precipitation possible within 24 hrs.\0"
"Mostly clear and warmer. Increasing winds.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and warmer. Precipitation possible within 12 hrs. Increasing winds.\0"
"Mostly clear and warmer. Increasing winds.\0"
"Increasing clouds and warmer.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and warmer. Precipitation possible within 12 hrs. Increasing winds.\0"
"Mostly clear and warmer. Increasing winds.\0"
"Increasing clouds and warmer.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and warmer. Precipitation possible within 12 hrs. Increasing winds.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Mostly clear and warmer. Precipitation possible within 48 hrs.\0"
"Mostly clear and warmer.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds with little temperature change. Precipitation possible within 24 to 48 hrs.\0"
"Increasing clouds with little temperature change.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and warmer. Precipitation possible within 12 to 24 hrs.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and warmer. Precipitation possible within 12 to 24 hrs. Windy.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and warmer. Precipitation possible within 12 to 24 hrs. Windy.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and warmer. Precipitation possible within 6 to 12 hrs.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and warmer. Precipitation possible within 6 to 12 hrs. Windy.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and warmer. Precipitation possible within 12 to 24 hrs. Windy.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and warmer. Precipitation possible within 12 hrs.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and warmer. Precipitation likely.\0"
"clearing and cooler. Precipitation ending within 6 hrs.\0"
"Partly cloudy with little temperature change.\0"
"clearing and cooler. Precipitation ending within 6 hrs.\0"
"Mostly clear with little temperature change.\0"
"Clearing and cooler. Precipitation ending within 6 hrs.\0"
"Partly cloudy and cooler.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear and cooler.\0"
"clearing and cooler. Precipitation ending within 6 hrs.\0"
"Mostly clear with little temperature change.\0"
"Clearing and cooler. Precipitation ending within 6 hrs.\0"
"Mostly clear and cooler.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds with little temperature change. Precipitation possible within 24 hrs.\0"
"Mostly cloudy and cooler. Precipitation continuing.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Mostly cloudy and cooler. Precipitation likely.\0"
"Mostly cloudy with little temperature change. Precipitation continuing.\0"
"Mostly cloudy with little temperature change. Precipitation likely.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and cooler. Precipitation possible and windy within 6 hrs.\0"
"Increasing clouds with little temperature change. Precipitation possible and windy within 6 hrs.\0"
"Mostly cloudy and cooler. Precipitation continuing. Increasing winds.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Mostly cloudy and cooler. Precipitation likely. Increasing winds.\0"
"Mostly cloudy with little temperature change. Precipitation continuing. Increasing winds.\0"
"Mostly cloudy with little temperature change. Precipitation likely. Increasing winds.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and cooler. Precipitation possible within 12 to 24 hrs. Possible wind shift to the W, NW, or N.\0"
"Increasing clouds with little temperature change. Precipitation possible within 12 to 24 hrs. Possible wind shift to the W, NW, or N.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and cooler. Precipitation possible within 6 hrs. Possible wind shift to the W, NW, or N.\0"
"Increasing clouds with little temperature change. Precipitation possible within 6 hrs. Possible wind shift to the W, NW, or N.\0"
"Mostly cloudy and cooler. Precipitation ending within 12 hrs. Possible wind shift to the W, NW, or N.\0"
"Mostly cloudy and cooler. Possible wind shift to the W, NW, or N.\0"
"Mostly cloudy with little temperature change. Precipitation ending within 12 hrs. Possible wind shift to the W, NW, or N.\0"
"Mostly cloudy with little temperature change. Possible wind shift to the W, NW, or N.\0"
"Mostly cloudy and cooler. Precipitation ending within 12 hrs. Possible wind shift to the W, NW, or N.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Mostly cloudy and cooler. Precipitation possible within 24 hrs. Possible wind shift to the W, NW, or N.\0"
"Mostly cloudy with little temperature change. Precipitation ending within 12 hrs. Possible wind shift to the W, NW, or N.\0"
"Mostly cloudy with little temperature change. Precipitation possible within 24 hrs. Possible wind shift to the W, NW, or N.\0"
"clearing, cooler and windy. Precipitation ending within 6 hrs.\0"
"clearing, cooler and windy.\0"
"Mostly cloudy and cooler. Precipitation ending within 6 hrs. Windy with possible wind shift to the W, NW, or N.\0"
"Mostly cloudy and cooler. Windy with possible wind shift to the W, NW, or N.\0"
"clearing, cooler and windy.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Mostly cloudy with little temperature change. Precipitation possible within 12 hrs. Windy.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and cooler. Precipitation possible within 12 hrs, possibly heavy at times. Windy.\0"
"Mostly cloudy and cooler. Precipitation ending within 6 hrs. Windy.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Mostly cloudy and cooler. Precipitation possible within 12 hrs. Windy.\0"
"Mostly cloudy and cooler. Precipitation ending in 12 to 24 hrs.\0"
"Mostly cloudy and cooler.\0"
"Mostly cloudy and cooler. Precipitation continuing, possible heavy at times. Windy.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Mostly cloudy and cooler. Precipitation possible within 6 to 12 hrs. Windy.\0"
"Mostly cloudy with little temperature change. Precipitation continuing, possibly heavy at times. Windy.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Mostly cloudy with little temperature change. Precipitation possible within 6 to 12 hrs. Windy.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds with little temperature change. Precipitation possible within 12 hrs, possibly heavy at times. Windy.\0"
"Mostly cloudy and cooler. Windy.\0"
"Mostly cloudy and cooler. Precipitation continuing, possibly heavy at times. Windy.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Mostly cloudy and cooler. Precipitation likely, possibly heavy at times. Windy.\0"
"Mostly cloudy with little temperature change. Precipitation continuing, possibly heavy at times. Windy.\0"
"Mostly cloudy with little temperature change. Precipitation likely, possibly heavy at times. Windy.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and cooler. Precipitation possible within 6 hrs. Windy.\0"
"Increasing clouds with little temperature change. Precipitation possible within 6 hrs. windy\0"
"Increasing clouds and cooler. Precipitation continuing. Windy with possible wind shift to the W, NW, or N.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Mostly cloudy and cooler. Precipitation likely. Windy with possible wind shift to the W, NW, or N.\0"
"Mostly cloudy with little temperature change. Precipitation continuing. Windy with possible wind shift to the W, NW, or N.\0"
"Mostly cloudy with little temperature change. Precipitation likely. Windy with possible wind shift to the W, NW, or N.\0"
"Increasing clouds and cooler. Precipitation possible within 6 hrs. Windy with possible wind shift to the W, NW, or N.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and cooler. Precipitation possible within 6 hrs. Possible wind shift to the W, NW, or N.\0"
"Increasing clouds with little temperature change. Precipitation possible within 6 hrs. Windy with possible wind shift to the W, NW, or N.\0"
"Increasing clouds with little temperature change. Precipitation possible within 6 hrs. Possible wind shift to the W, NW, or N.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and cooler. Precipitation possible within 6 hrs. Windy with possible wind shift to the W, NW, or N.\0"
"Increasing clouds with little temperature change. Precipitation possible within 6 hrs. Windy with possible wind shift to the W, NW, or N.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Increasing clouds and cooler. Precipitation possible within 12 to 24 hrs. Windy with possible wind shift to the W, NW, or N.\0"
"Increasing clouds with little temperature change. Precipitation possible within 12 to 24 hrs. Windy with possible wind shift to the W, NW, or N.\0"
"Mostly cloudy and cooler. Precipitation possibly heavy at times and ending within 12 hrs. Windy with possible wind shift to the W, NW, or N.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Mostly cloudy and cooler. Precipitation possible within 6 to 12 hrs, possibly heavy at times. Windy with possible wind shift to the W, NW, or N.\0"
"Mostly cloudy with little temperature change. Precipitation ending within 12 hrs. Windy with possible wind shift to the W, NW, or N.\0"
"Mostly cloudy with little temperature change. Precipitation possible within 6 to 12 hrs, possibly heavy at times. Windy with possible wind shift to the W, NW, or N.\0"
"Mostly cloudy and cooler. Precipitation continuing.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Mostly cloudy and cooler. Precipitation likely, windy with possible wind shift to the W, NW, or N.\0"
"Mostly cloudy with little temperature change. Precipitation continuing.\0"
"Mostly cloudy with little temperature change. Precipitation likely.\0"
"Partly cloudy with little temperature change.\0"
"Mostly clear with little temperature change.\0"
"Mostly cloudy and cooler. Precipitation possible within 12 hours, possibly heavy at times. Windy.\0"
"FORECAST REQUIRES 3 HRS. OF RECENT DATA\0"
"Mostly clear and cooler.\0\0" ;

static char MonthNameShort[] = "Jan\0" "Feb\0" "Mar\0" "Apr\0" "May\0" "Jun\0" "Jul\0" "Aug\0" "Sep\0" "Oct\0" "Nov\0" "Dec\0\0";
static char MonthDays[] = "31\0" "28\0" "31\0" "30\0" "31\0" "30\0" "31\0" "31\0" "30\0" "31\0" "30\0" "31\0\0";

//#if defined(mySQL) && defined(_RAIN_TABLE_NAME) && (defined(_RAIN_STORM_COL) || defined(_DAY_RAIN_COL))
extern unsigned long long int getLastIndex(int cid);
extern int ConsoleID;
extern float getIncRain(unsigned long long int IndexID, int vpID, float *PerMinute);
//#else
/* local functions */
static int readLastLine(char outFN[], char *Line, char flag[]);
static void parseLine(char *Line, int col, char delim[]);
//#endif
void calcSunRiseSetNoon(float lat, float lng, int year, int month, int day, int hour, int minute, int second, int dst, float tz);
static double calcTimeJulianCent(double jd);
static int isLeapYear(int yr) ;
static int calcDoyFromJD(double jd);
static double radToDeg(double angleRad) ;
static double degToRad(double angleDeg) ;
static double calcGeomMeanLongSun(double t);
static double calcGeomMeanAnomalySun(double t);
static double calcEccentricityEarthOrbit(double t);
static double calcSunEqOfCenter(double t);
static double calcSunTrueLong(double t);
static double calcSunTrueAnomaly(double t);
static double calcSunRadVector(double t);
static double calcSunApparentLong(double t);
static double calcMeanObliquityOfEcliptic(double t);
static double calcObliquityCorrection(double t);
static double calcSunDeclination(double t);
static double calcEquationOfTime(double t);
static double calcHourAngleSunrise(double lat, double solarDec);
static double getJD(int year, int month, int day);
static double getTimeLocal(int hour, int minute, int second, BOOL DST);
static double calcAzEl(double T, double localtime, double latitude, double longitude, double zone);
static void calcSolNoon(double jd, double longitude, double timezone, int dst);
static char* dayString(double jd, int next, int flag);
static char* timeDateString(double JD, double minutes);
static char* timeString(double minutes, int flag);
static double calcSunriseSetUTC(BOOL rise, double JD, double latitude, double longitude);
static void calcSunriseSet(BOOL rise, double JD, double latitude, double longitude, double timezone, double dst);
static double calcJDofNextPrevRiseSet(int next, int rise, double JD, double latitude, double longitude, double tz, double dst);

RTDATA rcd;			/* the one and only real time weather packet */
HLDATA hld;			/* the one and only highs/lows packet */
DMPDATA_A dmpda;
DMPDATA_B dmpdb;

char* TimeRef10Min(time_t tt);							/*   */
char* TimeRef15Min(time_t tt);							/*   */
char* TimeRefHours(time_t tt);							/*   */
char* TimeRefDays(time_t tt);							/*   */
char* TimeRefMonths(time_t tt);							/*   */
char* TimeRefYears(time_t tt);							/*   */

char* PrintTempSet24(BYTE *pData, int ptrOffset, BYTE rrOffset, int valOffset);		/*   */
char* PrintByteSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize, int cvtType);		/*   */
char* PrintSolarSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize);	/*   */
char* PrintBarSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize);		/*   */
char* PrintETByteSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize);	/*   */
char* PrintETOZByteSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize);       /*   */
char* PrintRainRateSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize);	/*   */
char* PrintRainRateByteSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize);	/*   */
char* PrintDateSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize);		/*   */
BYTE cvtMetricBYTE(BYTE value, int cvtType);
int cvtMetricINT(int value, int cvtType);
float cvtMetricFLOAT(float value, int cvtType);
void *getMemPtr(size_t size);

/* local functions */
static void PrintTimeSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize);	/*   */
static void PrintTempSet25(BYTE *pData, int ptrOffset, BYTE rrOffset, int valOffset);	/*   */
static char* PrintETSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize);	/*   */
static char* PrintETQUARTSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize);    /*   */
static char* PrintETGALSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize);    /*   */
static void PrintTimeRef(void);							/*   */
static char* ForecastString(WORD wRule);					/*   */
static void HeaderList( char *List);
static char* arWindRoseDir(int dir);

static void rcdConvertEndian(void);
static void hldConvertEndian(void);
static void dmpdConvertEndian(void);

unsigned short SwapEndian (unsigned short nValue)
{
   return (((nValue>> 8)) | (nValue << 8));
}


/*--------------------------------------------------------------------------
        getMemPtr
----------------------------------------------------------------------------*/
void *getMemPtr(size_t mSize)
{
    void *ptr;

   ptr = malloc(mSize);
   if (!ptr) {
       fprintf(stderr, "\nvproweather: malloc failed\n");
       exit(2);
   }
   return ptr;
}


/*--------------------------------------------------------------------------
        calcWindChill
----------------------------------------------------------------------------*/
float calcWindChill(float Temperature, float WindSpeed)
{
    float WindChill;

    if (WindSpeed == 0) {
        return Temperature;
    } else {
        WindChill = 35.74 + (0.6215*Temperature) - (35.75*pow(WindSpeed,0.16)) + (0.4275*Temperature*pow(WindSpeed,0.16));
    }
    if (WindChill > Temperature) { WindChill = Temperature; }

    return WindChill;
}

/*--------------------------------------------------------------------------
        calcHeatIndex
----------------------------------------------------------------------------*/
float calcHeatIndex(float Temperature, float RelHumidity)
{
    float HeatIndex;

    if (Temperature < 80.0 || RelHumidity < 40.0) {
        return Temperature;
    } else {
        HeatIndex = -42.379 + (2.04901523*Temperature) + (10.14333127*RelHumidity) - (0.22475541*Temperature*RelHumidity) - (0.00683783*pow(Temperature,2)) - (0.05481717*pow(RelHumidity,2)) + (0.00122874*pow(Temperature,2)*RelHumidity) + (0.00085282*Temperature*pow(RelHumidity,2)) - (0.00000199*pow(Temperature,2)*pow(RelHumidity,2));

    }
    if (HeatIndex < Temperature) HeatIndex = Temperature;

    return HeatIndex;
}


/*--------------------------------------------------------------------------
	cvtMetricBYTE
----------------------------------------------------------------------------*/
BYTE cvtMetricBYTE(BYTE value, int cvtType)
{

    if (cvtType == _CVT_NONE || !bMetric) return value;

    switch (cvtType) {
        case _CVT_TEMP:
            value = (value - 32.0) * (5.0/9.0);	/* Celsius */
            break;
        case _CVT_WIND:
            value = value * 1.609344;		/* Kilometers */
            break;
        case _CVT_RAIN:
            if (bTipMod) 
                value = value * 20.0;		/* Millimeters */
            else
                value = value * 25.4;		/* Millimeters */
            break;
        case _CVT_BARO:
            value = value * 33.8637526;		/* Millibars */
            break;
        case _CVT_ET_OZ:
            value = value * 29.5735295625;	/* Milliliters */
            break;
        case _CVT_ET_QT:
            value = value * 0.946352946;	/* Liters */
            break;
        case _CVT_ET_GL:
            value = value * 3.785411784;	/* Liters */
            break;
        case '?': 
        default: 
            break;
    }
    return value;
}


/*--------------------------------------------------------------------------
        cvtMetricINT
----------------------------------------------------------------------------*/
int cvtMetricINT(int value, int cvtType)
{

    if (cvtType == _CVT_NONE || !bMetric) return value;

    switch (cvtType) {
        case _CVT_TEMP:
            value = (value - 32.0) * (5.0/9.0);	/* Celsius */
            break;
        case _CVT_WIND:
            value = value * 1.609344;		/* Kilometers */
            break;
        case _CVT_RAIN:
            if (bTipMod)
                value = value * 20.0;		/* Millimeters */
            else
                value = value * 25.4;		/* Millimeters */
            break;
        case _CVT_BARO:
            value = value * 33.8637526;		/* Millibars */
            break;
        case _CVT_ET_OZ:
            value = value * 29.5735295625;	/* Milliliters */
            break;
        case _CVT_ET_QT:
            value = value * 0.946352946;	/* Liters */
            break;
        case _CVT_ET_GL:
            value = value * 3.785411784;	/* Liters */
            break;
        case '?': 
        default: 
            break;
    }
    return value;
}


/*--------------------------------------------------------------------------
        cvtMetricFLOAT
----------------------------------------------------------------------------*/
float cvtMetricFLOAT(float value, int cvtType)
{

    if (cvtType == _CVT_NONE || !bMetric) return value;

    switch (cvtType) {
        case _CVT_TEMP:
            value = (value - 32.0) * (5.0/9.0);	/* Celsius */
            value = (roundf(value * 10.0)) / 10.0;
            break;
        case _CVT_WIND:
            value = value * 1.609344;		/* Kilometers */
            value = roundf(value);
            break;
        case _CVT_RAIN:
            if (bTipMod)
                value = value * 20.0;		/* Millimeters */
            else
                value = value * 25.4;		/* Millimeters */
            value = (roundf(value * 100.0)) / 100.0;
            break;
        case _CVT_BARO:
            value = value * 33.8637526;		/* Millibars */
            value = (roundf(value * 1000.0)) / 1000.0;
            break;
        case _CVT_ET_OZ:
            value = value * 29.5735295625;	/* Milliliters */
            value = (roundf(value * 10.0)) / 10.0;
            break;
        case _CVT_ET_QT:
            value = value * 0.946352946;	/* Liters */
            value = (roundf(value * 10.0)) / 10.0;
            break;
        case _CVT_ET_GL:
            value = value * 3.785411784;	/* Liters */
            value = (roundf(value * 10.0)) / 10.0;
            break;
        case '?': 
        default: 
            break;
    }
    return value;
}


/*--------------------------------------------------------------------------
	CheckCRC
	Verifies the crc against a passed buffer. Returns the non-zero CRC code
	if error, zero if the crc passed.
----------------------------------------------------------------------------*/
int CheckCRC(int nCnt, char *pData)
{
	int i;
	WORD wCRC = 0;					/* zero checksum to start	*/
	BYTE y;
	
	for( i = 0; i < nCnt; i++) {
		y = *(pData)++;
		wCRC = crc_table[(wCRC >> 8) ^ y] ^ (wCRC << 8); /* CCITT std */
	}
	
	return wCRC;					/* if zero, it passed */
}	


/*--------------------------------------------------------------------------
        SubString
----------------------------------------------------------------------------*/
char* SubString(const char* text, char delim, int *ppos)
{
    int pos = 0;
    int spos = *ppos;
    static char value[255];

    while (spos < strlen(text) && text[spos] != delim && text[spos] != '\0')
        value[pos++] = text[spos++];

    value[pos] = '\0';

    if (spos < strlen(text) && text[spos] != '\0') spos++;

    *ppos = spos;

    return value;
}


/*--------------------------------------------------------------------------
        SetEnvVar
----------------------------------------------------------------------------*/
int SetEnvVar(char * pData, char envVars[])
{
   char ParamName[128];
   int SSpos;

// int setenv(const char *name, const char *value, int overwrite);

    while((sprintf(ParamName, "%s", SubString(envVars, ',', &SSpos))) && SSpos <= strlen(envVars)) {

    }
    return TRUE;
}


/*--------------------------------------------------------------------------
        DateTimeCT
----------------------------------------------------------------------------*/
time_t DateTimeCT(char * s1)
{
    struct tm stm;
    time_t tt;

    time(&tt);
    stm = *localtime(&tt);                          /* get system time */
    stm.tm_mon  += 1;
    stm.tm_year += 1900;

    sprintf( s1, "%4u-%02u-%02u %02u:%02u:%02u", stm.tm_year, stm.tm_mon, stm.tm_mday, stm.tm_hour, stm.tm_min, stm.tm_sec);
    s1[strlen(s1)] = '\0';
    return tt;
}

/*--------------------------------------------------------------------------
        PrintCurTime
        Prints the current time
----------------------------------------------------------------------------*/
char* getCurTime(void)
{
    struct tm stm;
    time_t tt;
    static char CurTime[20];

    tt = time(NULL);
    stm = *(localtime(&tt));
    strftime(CurTime,sizeof(CurTime),"%Y-%m-%d %H:%M:%S%z ",&stm);

    return CurTime;
}

/*--------------------------------------------------------------------------
	PrintTime
	Prints the time data retrieved from the weather station.
----------------------------------------------------------------------------*/
void PrintTime(char *szData)
{
	struct tm stm;
	time_t tt;
	char szBuf[255];

	time(&tt);
	stm = *localtime(&tt);			/* fill time struct */
	stm.tm_sec  = (int)szData[1];		/* copy in Davis values */
	stm.tm_min  = (int)szData[2];
	stm.tm_hour = (int)szData[3];
	stm.tm_mday = (int)szData[4];
	stm.tm_mon  = (int)(szData[5] - 1);
	stm.tm_year = (int)szData[6];
	
	tt = mktime(&stm);
	stm = *localtime(&tt);			/* correct bogus values */
	
	/* change the following line if you don't like the time format */
/*	strftime(szBuf, sizeof(szBuf), "%A, %B %d, %Y  %I:%M %p\n"  , &stm); */
	strftime(szBuf, sizeof(szBuf), "%A %B %d %Y %H:%M\n"  , &stm);

        if (toSTDOUT(TRUE, FALSE)) printf("DavisTime = %s", szBuf);
}	


/*--------------------------------------------------------------------------
	GetRTData
	Gets the real time weather data packet to the static RTDATA struct.
----------------------------------------------------------------------------*/
void GetRTData(char *szData)
{
	memcpy((char*)&rcd, szData, sizeof(RTDATA));
	if (bSwapEndian) rcdConvertEndian();
}


/*--------------------------------------------------------------------------
        GetDMPData
        Gets the archive weather data packet to the static DMPDATA struct.
----------------------------------------------------------------------------*/
void GetDMPData(char *szData)
{
        memcpy((char*)&dmpda, szData, sizeof(DMPDATA_A));
	memcpy((char*)&dmpdb, szData, sizeof(DMPDATA_B));
        if (bSwapEndian) dmpdConvertEndian();
}


/*--------------------------------------------------------------------------
	DumpData
	Dumps the weather data packet to a file
----------------------------------------------------------------------------*/
void DumpData(char *szData, int cCnt, char loopFN[])
{
	FILE *fp;
	int i = 0;

	fp = fopen(loopFN, "wb");
	if(fp == NULL) {
		fprintf(stderr, "can't open %s\n", loopFN);
		return;
	}
	while(i <= cCnt) {
		fputc(szData[i], fp);
		i++;
        }
	
	fclose(fp);
	
}	

/*--------------------------------------------------------------------------
        GetData
        Reads the weather data packet from a file
----------------------------------------------------------------------------*/
int GetData(char *szData, int cCnt, char loopFN[])
{
        FILE *fp;
	int pos = 0;

        fp = fopen(loopFN, "r");
        if(fp == NULL) {
                fprintf(stderr, "can't open %s\n", loopFN);
                return 0;
        }
        
	szData[pos] = '\0';
        while (pos <= cCnt) {
		szData[pos] = fgetc(fp);
		if (feof(fp)) break;
                pos++;
	}
        fclose(fp);
	pos--;
	if (bVerbose) fprintf(stderr, "Got (%d) characters from file (%s)\n", pos, loopFN);

	return pos;

}


/*--------------------------------------------------------------------------
        ListOutputDataCols
----------------------------------------------------------------------------*/
void ListOutputDataCols(char outFN[])
{
    FILE *fp;
    int pos = 0;
    char Data[HeaderStrLen];
    char List[HeaderStrLen];

    HeaderList(List);
    if (strlen(outFN) > 0) {
        fp = fopen(outFN, "r");
        if(fp != NULL) {
            while (pos < HeaderStrLen)
                Data[pos++] = fgetc(fp);
            fclose(fp);
            Data[pos-1] = '\0';
            if (strcmp(List, Data) == 0) return;
        }
        fp = fopen(outFN, "a");
        if(fp == NULL) {
                fprintf(stderr, "can't open %s\n", outFN);
                return;
        }
    } else
        fp = stdout;

    fprintf( fp, "%s\n", List);

    if (strlen(outFN) > 0) fclose(fp);
}


/*--------------------------------------------------------------------------
        HeaderList
----------------------------------------------------------------------------*/
void HeaderList( char *List)
{
    char *pos;

    sprintf( List, "DateTime\tInsideTemp\tInsideHum\tOutsideTemp\tOutsideHum\tWindChill\tHeatIndex\tSolarRad\tETDay\tForecastIcon\tForecastRule\tRainDay\tStormRain\tIncRain\tRainRate\tIsRaining\tWindDir\tWindSpeed\tWindRoseDir\tBarometer\tBaroTrendImage\tBaroTrend\tSunrise\tSunset\tRunTimeTT\tDST\tSolarNoon\tStatus");
    if (bZcommas) {
        pos = strchr(List, '\t');
        while(pos != NULL) {
            *pos = ',';
            pos = strchr(List, '\t');
        }
    }
    return;
}


/*--------------------------------------------------------------------------
        readLastLine
----------------------------------------------------------------------------*/
int readLastLine(char outFN[], char *Line, char flag[])
{
    FILE *fp;
    struct stat fStats;
    int Status = 0;
    int step = 0;
    int pos = 1;
    int i = 0;
    int fLen = 0;
    char OK[5];
    BOOL foundOK = FALSE;
    BOOL foundNL = FALSE;

    Line[0] = '\0';
    if ((Status = stat( outFN, &fStats)) == 0) {
        fLen = (int)fStats.st_size;
        fp = fopen(outFN, "r");
        if(fp == NULL) {
                fprintf(stderr, "can't open %s\n", outFN);
                return(-1);
        }
        step = -strlen(flag);
        strcpy(OK, "    ");
        OK[strlen(flag)] = '\0';
        while(Status == 0 && pos > 0 && (! foundNL || ! foundOK)) {
            Status = fseek(fp, step, SEEK_END);
            pos = ftell(fp);
            for(i = 0; i < strlen(flag); i++)
                OK[i] = fgetc(fp);
            if (foundOK && OK[strlen(flag)-1] == '\n') foundNL = TRUE;
            if (strcmp(OK, flag) == 0) foundOK = TRUE;
            step--;
        }
        Status = fseek(fp, pos+strlen(OK), SEEK_SET);
        i = 0;
        while(pos+i < fLen) {
            Line[i++] = fgetc(fp);
            if (Line[i-1] == '\n') break;
        }
        Line[i-1] = '\0';
        fclose(fp);
    }
    if (bVerbose) fprintf(stdout, "readLastLine Status %i\n", Status);

    return Status;
}

/*--------------------------------------------------------------------------
        parseLine
----------------------------------------------------------------------------*/
void parseLine(char *Line, int col, char delim[])
{
    char *cpos;
    int cur = 1;
    int pos = 0;

    cpos = strtok(Line, delim);

    while(cur++ < col && cpos != NULL)
        cpos = strtok(NULL, delim);
    if (cpos == NULL)
        Line[0] = '\0';
    else
        while(pos < strlen(Line) && *cpos != '\t')
            Line[pos++] = *cpos++;
}

/*--------------------------------------------------------------------------
	OutputData
	Function to output key data to a file in tab delimited
	columns. Columns are ordered as follows:

runTime rtInsideTemp rtInsideHum rtOutsideTemp rtOutsideHum rtWindChill rtHeatIndex rtSolarRad rtDayET rtForeIcon rtForeRule rtDayRain rtRainStorm curRain rtRainRate rtIsRaining rtWindDir rtWindSpeed rtWindDirRose rtBaroCurr rtBaroTrendImg rtBaroTrend sunrise sunset run-time_t dst

----------------------------------------------------------------------------*/
void OutputData(char * pData, char outFN[], BOOL bRTDSuccess, BOOL bGDSuccess)
{
    FILE *fp;
    time_t tt;
    struct tm stm;
    char datetime[15];
    char sunrise[26];
    char sunset[26];
    float IncRain = 0.0;
    float PerMinute = 0.0;
    char Trend[16];
    char TrendImage[16];
    char delim = '\t';
    char Line[1024];
    float lastIncRain = 0.0;
#ifndef noMySQL
    unsigned long long int LastPtr;
    extern BOOL bUseSQL;
    extern BOOL bEnableMYSQL;
#endif

    time(&tt);
    stm = *localtime(&tt);
    strftime(datetime, sizeof(datetime), "%Y%m%d-%H:%M", &stm);

//#if defined(mySQL) && defined(_RAIN_TABLE_NAME) && (defined(_RAIN_STORM_COL) || defined(_DAY_RAIN_COL))
#ifndef noMySQL
    if (bUseSQL && bEnableMYSQL) {
        LastPtr = getLastIndex(ConsoleID);
        IncRain = getIncRain(LastPtr+1, ConsoleID, &PerMinute);
        if (IncRain < 0.0) IncRain = 0.0;
        if (PerMinute < 0.0) PerMinute = 0.0;
    } else {
#endif
        readLastLine( outFN, Line, "[OK]" );
        parseLine(Line, 13, "\t");
        if (Line[0] != '\0') sscanf(Line, "%f", &lastIncRain );
        IncRain = ((int)(cvtMetricINT(rcd.wStormRain, _CVT_RAIN) - (lastIncRain*100)))/100.0;	/* try this to prevent a -0.00 value */
        /* for some reason the above can produce -0.00 when RainDay = lastIncRain */
        if (IncRain < 0.0) {
            fprintf(stderr, "WARN2: rcd.wRainDay %i rcd.wRainDay %f lastIncRain \"%s\" lastIncRain %f IncRain %f\n", rcd.wRainDay, rcd.wRainDay / 100.0, Line, lastIncRain, (rcd.wRainDay / 100.0) - lastIncRain);
        }
        if (bVerbose)
            fprintf(stderr, "rcd.wRainDay %.2f lastIncRain %f IncRain %.2f PerMinute %.2f\n", rcd.wRainDay / 100.0, lastIncRain, (rcd.wRainDay / 100.0) - lastIncRain, PerMinute);
#ifndef noMySQL
    }
#endif
        
    fp = fopen(outFN, "a");
    if(fp == NULL) {
        fprintf(stderr, "can't open %s\n", outFN);
        return;
    }

    sunrise[0] = '\0';
    sunset[0] = '\0';
    strcpy(sunrise,TimeConvert(rcd.wSunrise));
    strcpy(sunset,TimeConvert(rcd.wSunset));
    if (bCalcSun) {
        strcpy(sunrise,SunRise);
        strcpy(sunset,SunSet);
    }
    Trend[0] = '\0';
    TrendImage[0] = '\0';
    if (bRTDSuccess) {
        strcpy(Trend, BaroTrend(1));
        strcpy(TrendImage, BaroTrendImage());
    } else {
        strcpy(Trend, "unknown");
        strcpy(TrendImage, "unknown");
    }

    if (bZcommas) delim = ',';
    fprintf( fp, "%s%c%.1f%c%d%c%.1f%c%d%c%.1f%c%.1f%c%d%c%.3f%c%d%c%d%c%.2f%c%.2f%c%.2f%c%.2f%c%s%c%d%c%d%c%s%c%2.3f%c%s%c%s%c%s%c%s%c%llu%c%i", datetime, delim, cvtMetricFLOAT(((signed short)rcd.wInsideTemp) / 10.0, _CVT_TEMP), delim, rcd.yInsideHum, delim, cvtMetricFLOAT(((signed short)rcd.wOutsideTemp) / 10.0, _CVT_TEMP), delim, rcd.yOutsideHum, delim, cvtMetricFLOAT(calcWindChill( ((signed short)rcd.wOutsideTemp) / 10.0, rcd.yWindSpeed ), _CVT_TEMP), delim, cvtMetricFLOAT(calcHeatIndex( ((signed short)rcd.wOutsideTemp) / 10.0, rcd.yOutsideHum ), _CVT_TEMP), delim, rcd.wSolarRad, delim, cvtMetricFLOAT(rcd.wETDay / 1000.0, _CVT_RAIN), delim, rcd.yForeIcon, delim, rcd.yRule, delim, cvtMetricFLOAT(rcd.wRainDay / 100.0, _CVT_RAIN), delim, cvtMetricFLOAT(rcd.wStormRain / 100.0, _CVT_RAIN), delim, IncRain, delim, cvtMetricFLOAT(rcd.wRainRate / 100.0, _CVT_RAIN), delim, rcd.wRainRate ? "yes" : "no", delim, rcd.wWindDir, delim, cvtMetricBYTE(rcd.yWindSpeed, _CVT_WIND), delim, WindRoseDir(), delim, cvtMetricFLOAT(rcd.wBarometer / 1000.0, _CVT_BARO), delim, TrendImage, delim, Trend, delim, sunrise, delim, sunset, delim, (unsigned long long)ttRun, delim, dstRun);
    if (bCalcSun)
        fprintf( fp, "%c%s", delim, SolarNoon);
    else
        fprintf( fp, "%c--:--:--", delim);

    if (bRTDSuccess && bGDSuccess) 
        fprintf( fp, "%c[OK]\n", delim);
    else
        fprintf( fp, "\n");

    fclose(fp);
    
}	


/*--------------------------------------------------------------------------
	GetHLData
	Gets the high/low weather data packet to the static HLDATA struct.
----------------------------------------------------------------------------*/
void GetHLData(char *szData)
{
	memcpy((char*)&hld, szData, sizeof(HLDATA));
	if (bSwapEndian) hldConvertEndian();
}	

/*--------------------------------------------------------------------------
        BaroTrend
        3-hour rolling baro trend
----------------------------------------------------------------------------*/
char* BaroTrend(int ns)
{
	int i;
	static char Trend[16];
        char *pos;

	Trend[0] = '\0';
        i = rcd.cP;
        switch(i) {
                case -60: case 196: sprintf(Trend, "%s", "Falling Rapidly"); break;
                case -20: case 236: sprintf(Trend, "%s", "Falling Slowly"); break;
                case 0:   sprintf(Trend, "%s", "Steady"); break;
                case 20:  sprintf(Trend, "%s", "Rising Slowly"); break;
                case 60:  sprintf(Trend, "%s", "Rising Rapidly"); break;
                case 80:  sprintf(Trend, "n/a-%d-Rev.A", rcd.cP); break;
                default:  sprintf(Trend, "n/a-%d", rcd.cP);
        }
        if (ns == 1) {
            pos = strstr( Trend, " ");
            if (pos != NULL) *pos = '-';
        }
	return Trend;
}

/*--------------------------------------------------------------------------
        BaroTrendImage
        3-hour rolling baro trend image
----------------------------------------------------------------------------*/
char* BaroTrendImage()
{
        int i;
        static char TrendImg[16];

	TrendImg[0] = '\0';
        i = rcd.cP;
        switch(i) {
                case -60: case 196: sprintf(TrendImg, "%s", "baro_fr"); break;
                case -20: case 236: sprintf(TrendImg, "%s", "baro_fs"); break;
                case 0:   sprintf(TrendImg, "%s", "baro_s"); break;
                case 20:  sprintf(TrendImg, "%s", "baro_rs"); break;
                case 60:  sprintf(TrendImg, "%s", "baro_rr"); break;
                case 80:  sprintf(TrendImg, "%s", "baro_none-Rev.A"); break;
                default:  sprintf(TrendImg, "%s", "baro_none");
        }
	return TrendImg;
}

/*--------------------------------------------------------------------------
        WindRoseDir
----------------------------------------------------------------------------*/
char* WindRoseDir()
{
	static char RoseDir[4];

	/* This don't work because 1 to 11 is less than 347 and 347 to 359 is greater than 12 - cjb */
	/* original test --> if(rcd.wWindDir >= 347 && rcd.wWindDir < 12)       */      /* compass rose version */
	/* I was wondering for awhile why this seemed flakey, just now looked into it - cjb */
	/* These tests will work -  gcc compiler sems to dictate separate test for 0 (zero) - cjb */
	if((rcd.wWindDir == 0) || (rcd.wWindDir >= 1 && rcd.wWindDir < 12) || (rcd.wWindDir >= 347 && rcd.wWindDir <= 360))
		sprintf(RoseDir, "%s", "N");
	else if(rcd.wWindDir >= 12 && rcd.wWindDir < 34)
		sprintf(RoseDir, "%s", "NNE");
	else if(rcd.wWindDir >= 34 && rcd.wWindDir < 57)
		sprintf(RoseDir, "%s", "NE");
	else if(rcd.wWindDir >= 57 && rcd.wWindDir < 79)
		sprintf(RoseDir, "%s", "ENE");
	else if(rcd.wWindDir >= 79 && rcd.wWindDir < 102)
		sprintf(RoseDir, "%s", "E");
	else if(rcd.wWindDir >= 102 && rcd.wWindDir < 124)
		sprintf(RoseDir, "%s", "ESE");
	else if(rcd.wWindDir >= 124 && rcd.wWindDir < 147)
		sprintf(RoseDir, "%s", "SE");
	else if(rcd.wWindDir >= 147 && rcd.wWindDir < 170)
		sprintf(RoseDir, "%s", "SSE");
	else if(rcd.wWindDir >= 170 && rcd.wWindDir < 192)
		sprintf(RoseDir, "%s", "S");
	else if(rcd.wWindDir >= 192 && rcd.wWindDir < 214)
		sprintf(RoseDir, "%s", "SSW");
	else if(rcd.wWindDir >= 214 && rcd.wWindDir < 237)
		sprintf(RoseDir, "%s", "SW");
	else if(rcd.wWindDir >= 237 && rcd.wWindDir < 259)
		sprintf(RoseDir, "%s", "WSW");
	else if(rcd.wWindDir >= 259 && rcd.wWindDir < 280)
		sprintf(RoseDir, "%s", "W");
	else if(rcd.wWindDir >= 280 && rcd.wWindDir < 303)
		sprintf(RoseDir, "%s", "WNW");
	else if(rcd.wWindDir >= 303 && rcd.wWindDir < 347)
		sprintf(RoseDir, "%s", "NW");
	else	/*  >326 <347 */
		sprintf(RoseDir, "%s", "NNW");
	return RoseDir;
}


/*--------------------------------------------------------------------------
        arWindRoseDir
----------------------------------------------------------------------------*/
char* arWindRoseDir(int dir)
{
    static char RoseDir[4];

    switch (dir) {
        case 0:   sprintf(RoseDir, "%s", "N"); break;
        case 1:   sprintf(RoseDir, "%s", "NNE"); break;
        case 2:   sprintf(RoseDir, "%s", "NE"); break;
        case 3:   sprintf(RoseDir, "%s", "ENE"); break;
        case 4:   sprintf(RoseDir, "%s", "E"); break;
        case 5:   sprintf(RoseDir, "%s", "ESE"); break;
        case 6:   sprintf(RoseDir, "%s", "SE"); break;
        case 7:   sprintf(RoseDir, "%s", "SSE"); break;
        case 8:   sprintf(RoseDir, "%s", "S"); break;
        case 9:   sprintf(RoseDir, "%s", "SSW"); break;
        case 10:  sprintf(RoseDir, "%s", "SW"); break;
        case 11:  sprintf(RoseDir, "%s", "WSW"); break;
        case 12:  sprintf(RoseDir, "%s", "W"); break;
        case 13:  sprintf(RoseDir, "%s", "WNW"); break;
        case 14:  sprintf(RoseDir, "%s", "NW"); break;
        case 15:  sprintf(RoseDir, "%s", "NNW"); break;
        case 255: sprintf(RoseDir, "%s", "---"); break;
        case '?':
        default:
                  sprintf(RoseDir, "%s", "???"); break;
    }
    return RoseDir;
}


/*--------------------------------------------------------------------------
        PrintDMPData
        Dumps the archive weather data to stdout.
----------------------------------------------------------------------------*/
void PrintDMPData(int seq, int page, int rec, char mark[])
{
    printf("%s = %d.%03d.%1d%s%s\n", _SEQUENCE,seq,page,rec,mark,(dmpda.wTime != 65535) ? "" : " (empty)");
        if (dmpda.wTime == 65535) return;
    printf("%s = %s\n", _DATE_STAMP, PrintDateDMP(dmpda.wDate));
    printf("%s = %s\n", _TIME_STAMP, TimeConvert(dmpda.wTime));
    if (dmpda.wTempOut != 32767) printf("%s = %.1f\n", _TEMP_OUTSIDE, cvtMetricFLOAT((double)dmpda.wTempOut/10.0, _CVT_TEMP));
    if (dmpda.wTempOutHi != -32767) printf("%s = %.1f\n", _TEMP_OUT_HI, cvtMetricFLOAT((double)dmpda.wTempOutHi/10.0, _CVT_TEMP));
    if (dmpda.wTempOutLo != 32767) printf("%s = %.1f\n",_TEMP_OUT_LO, cvtMetricFLOAT((double)dmpda.wTempOutLo/10.0, _CVT_TEMP));
    printf("%s = %d\n", _RAINFALL, cvtMetricINT(dmpda.wRainFall, _CVT_RAIN));
    printf("%s = %d\n", _RAIN_RATE_HI, cvtMetricINT(dmpda.wRainRateHi, _CVT_RAIN));
    if (dmpda.wBarometer != 0) printf("%s = %.3f\n", _BAROMETER, cvtMetricFLOAT((double)dmpda.wBarometer/1000.0, _CVT_BARO));
    if (dmpda.wSolarRad != 32767) printf("%s = %d\n", _SOLAR_RADIATION, dmpda.wSolarRad);
    printf("%s = %d\n", _WIND_SAMPLES, dmpda.wWindSamples);
    if (dmpda.wTempIn != 32767) printf("%s = %.1f\n", _TEMP_INSIDE, cvtMetricFLOAT((double)dmpda.wTempIn/10.0, _CVT_TEMP));
    if (dmpda.yHumIn != 255) printf("%s = %d\n", _HUMIDITY_INSIDE, dmpda.yHumIn);
    if (dmpda.yHumOut != 255) printf("%s = %d\n", _HUMIDITY_OUTSIDE, dmpda.yHumOut);
    if (dmpda.yWindSpeedAvg != 255) printf("%s = %d\n", _WIND_SPEED_AVG, cvtMetricBYTE(dmpda.yWindSpeedAvg, _CVT_WIND));
    printf("%s = %d\n", _WIND_SPEED_HI, cvtMetricBYTE(dmpda.yWindSpeedHi, _CVT_WIND));
//    if (dmpda.yWindSpeedDirHi != 255) 
    printf("%s = %s\n", _WIND_SPEED_HI_DIR, arWindRoseDir(dmpda.yWindSpeedDirHi));
//    if (dmpda.yWindDirPrevail != 255) 
    printf("%s = %s\n", _WIND_DIR_PREVAIL, arWindRoseDir(dmpda.yWindDirPrevail));
    if (dmpda.yUVAvg != 255) printf("%s = %.1f\n", _UV_INDEX_AVG, (double)dmpda.yUVAvg/10.0);
    if (dmpda.yET != 0) printf("%s = %.3f\n", _ET, cvtMetricFLOAT(dmpda.yET/1000.0, _CVT_RAIN));
    if (dmpdb.yRecType != 0) {	/* A record */
        if (bVerbose) fprintf(stderr, "A record\n");
        if (dmpda.ySoilMoist1 != 255) printf("%s = %d\n", _SOIL_MOIST_1, dmpda.ySoilMoist1);
        if (dmpda.ySoilMoist2 != 255) printf("%s = %d\n", _SOIL_MOIST_2, dmpda.ySoilMoist2);
        if (dmpda.ySoilMoist3 != 255) printf("%s = %d\n", _SOIL_MOIST_3, dmpda.ySoilMoist3);
        if (dmpda.ySoilMoist4 != 255) printf("%s = %d\n", _SOIL_MOIST_4, dmpda.ySoilMoist4);
        if (dmpda.ySoilTemp1 != 255) printf("%s = %d\n", _SOIL_TEMP_1, cvtMetricBYTE(dmpda.ySoilTemp1-90, _CVT_TEMP));
        if (dmpda.ySoilTemp2 != 255) printf("%s = %d\n", _SOIL_TEMP_2, cvtMetricBYTE(dmpda.ySoilTemp2-90, _CVT_TEMP));
        if (dmpda.ySoilTemp3 != 255) printf("%s = %d\n", _SOIL_TEMP_3, cvtMetricBYTE(dmpda.ySoilTemp3-90, _CVT_TEMP));
        if (dmpda.ySoilTemp4 != 255) printf("%s = %d\n", _SOIL_TEMP_4, cvtMetricBYTE(dmpda.ySoilTemp4-90, _CVT_TEMP));
        if (dmpda.yLeafWet1 != 255) printf("%s = %d\n", _LEAF_WET_1, dmpda.yLeafWet1);
        if (dmpda.yLeafWet2 != 255) printf("%s = %d\n", _LEAF_WET_2, dmpda.yLeafWet2);
        if (dmpda.yLeafWet3 != 255) printf("%s = %d\n", _LEAF_WET_3, dmpda.yLeafWet3);
        if (dmpda.yLeafWet4 != 255) printf("%s = %d\n", _LEAF_WET_4, dmpda.yLeafWet4);
        if (dmpda.yExtraTemp1 != 255) printf("%s = %d\n", _TEMP_EXTRA_1, cvtMetricBYTE(dmpda.yExtraTemp1-90, _CVT_TEMP));
        if (dmpda.yExtraTemp2 != 255) printf("%s = %d\n", _TEMP_EXTRA_2, cvtMetricBYTE(dmpda.yExtraTemp2-90, _CVT_TEMP));
        if (dmpda.yExtraHum1 != 255) printf("%s = %d\n", _HUM_EXTRA_1, dmpda.yExtraHum1);
        if (dmpda.yExtraHum2 != 255) printf("%s = %d\n", _HUM_EXTRA_2, dmpda.yExtraHum2);
        if (dmpda.wReedClosed != 0) printf("%s = %d\n", _REED_CLOSED, dmpda.wReedClosed);
        if (dmpda.wReedOpened != 0) printf("%s = %d\n", _REED_OPENED, dmpda.wReedOpened);
    } else {			/* B record */
        if (bVerbose) fprintf(stderr, "B record\n");
        if (dmpdb.wSolarRadHi != 32767) printf("%s = %d\n", _SOLAR_RADIATION_HI, dmpdb.wSolarRadHi);
        if (dmpdb.yUVIndexHi != 255) printf("%s = %d\n", _UV_INDEX_HI, dmpdb.yUVIndexHi);
        printf("%s = %d\n", _FORECAST_RULE, dmpdb.yForecastRule);
        if (dmpdb.yLeafTemp1 != 255) printf("%s = %d\n", _LEAF_TEMP_1, cvtMetricBYTE(dmpdb.yLeafTemp1-90, _CVT_TEMP));
        if (dmpdb.yLeafTemp2 != 255) printf("%s = %d\n", _LEAF_TEMP_2, cvtMetricBYTE(dmpdb.yLeafTemp2-90, _CVT_TEMP));
        if (dmpdb.yLeafWet1 != 255) printf("%s = %d\n", _LEAF_WET_1, dmpdb.yLeafWet1);
        if (dmpdb.yLeafWet2 != 255) printf("%s = %d\n", _LEAF_WET_2, dmpdb.yLeafWet2);
        if (dmpdb.ySoilTemp1 != 255) printf("%s = %d\n", _SOIL_TEMP_1, cvtMetricBYTE(dmpdb.ySoilTemp1-90, _CVT_TEMP));
        if (dmpdb.ySoilTemp2 != 255) printf("%s = %d\n", _SOIL_TEMP_2, cvtMetricBYTE(dmpdb.ySoilTemp2-90, _CVT_TEMP));
        if (dmpdb.ySoilTemp3 != 255) printf("%s = %d\n", _SOIL_TEMP_3, cvtMetricBYTE(dmpdb.ySoilTemp3-90, _CVT_TEMP));
        if (dmpdb.ySoilTemp4 != 255) printf("%s = %d\n", _SOIL_TEMP_4, cvtMetricBYTE(dmpdb.ySoilTemp4-90, _CVT_TEMP));
        if (dmpdb.yExtraHum1 != 255) printf("%s = %d\n", _HUM_EXTRA_1, dmpdb.yExtraHum1);
        if (dmpdb.yExtraHum2 != 255) printf("%s = %d\n", _HUM_EXTRA_2, dmpdb.yExtraHum2);
        if (dmpdb.yExtraTemp1 != 255) printf("%s = %d\n", _TEMP_EXTRA_1, cvtMetricBYTE(dmpdb.yExtraTemp1-90, _CVT_TEMP));
        if (dmpdb.yExtraTemp2 != 255) printf("%s = %d\n", _TEMP_EXTRA_2, cvtMetricBYTE(dmpdb.yExtraTemp2-90, _CVT_TEMP));
        if (dmpdb.yExtraTemp3 != 255) printf("%s = %d\n", _TEMP_EXTRA_3, cvtMetricBYTE(dmpdb.yExtraTemp3-90, _CVT_TEMP));
        if (dmpdb.ySoilMoist1 != 255) printf("%s = %d\n", _SOIL_MOIST_1, dmpdb.ySoilMoist1);
        if (dmpdb.ySoilMoist2 != 255) printf("%s = %d\n", _SOIL_MOIST_2, dmpdb.ySoilMoist2);
        if (dmpdb.ySoilMoist3 != 255) printf("%s = %d\n", _SOIL_MOIST_3, dmpdb.ySoilMoist3);
        if (dmpdb.ySoilMoist4 != 255) printf("%s = %d\n", _SOIL_MOIST_4, dmpdb.ySoilMoist4);
    }
}


/*--------------------------------------------------------------------------
	PrintRTData
	Dumps the real time weather data to stdout.
----------------------------------------------------------------------------*/
void PrintRTData(void)
{
    char sunrise[26] = "";
    char sunset[26] = "";
    char solarnoon[11] = "";
    char *fpos, *lpos, *apos;
    int i;

    /* 3-hour rolling baro trend */
    printf("%s = %s\n", _BARO_TREND, BaroTrend(0));
    printf("%s = %s\n", _BARO_TREND_IMG, BaroTrendImage());
    if (!bMetric)
        printf("%s = %.3f\n", _BARO_CURR, rcd.wBarometer / 1000.0 );
    else
        printf("%s = %.1f\n", _BARO_CURR, cvtMetricFLOAT(rcd.wBarometer / 1000.0, _CVT_BARO) );
    printf("%s = %.1f\n", _INSIDE_TEMP, cvtMetricFLOAT(((signed short)rcd.wInsideTemp) / 10.0, _CVT_TEMP) );
    printf("%s = %d\n", _INSIDE_HUM, rcd.yInsideHum );
    printf("%s = %.1f\n", _OUTSIDE_TEMP, cvtMetricFLOAT(((signed short)rcd.wOutsideTemp) / 10.0, _CVT_TEMP) );
    for (i = 0; i < 7; i++) {
        if (rcd.yXtraTemps[i] != 255) printf("%s%i = %i\n", _EXTRA_TEMP, i+1, cvtMetricBYTE(rcd.yXtraTemps[i]-90, _CVT_TEMP) );
    }
    for (i = 0; i < 4; i++) {
        if (rcd.ySoilTemps[i] != 255) printf("%s%i = %i\n", _SOIL_TEMP, i+1, cvtMetricBYTE(rcd.ySoilTemps[i]-90, _CVT_TEMP) );
    }
    for (i = 0; i < 4; i++) {
        if (rcd.yLeafTemps[i] != 255) printf("%s%i = %i\n", _LEAF_TEMP, i+1, cvtMetricBYTE(rcd.yLeafTemps[i]-90, _CVT_TEMP) );
    }
    printf("%s = %d\n", _OUTSIDE_HUM, rcd.yOutsideHum );
    for (i = 0; i < 7; i++) {
        if (rcd.yXtraHums[i] != 255) printf("%s%i = %d\n", _EXTRA_HUM, i+1, rcd.yXtraHums[i] );
    }

    printf("%s = %d\n", _WIND_SPEED, cvtMetricBYTE(rcd.yWindSpeed, _CVT_WIND) );
    printf("%s = %d\n", _WIND_AVG_SPEED, cvtMetricBYTE(rcd.yAvgWindSpeed, _CVT_WIND) );
    printf("%s = %d\n", _WIND_DIR, rcd.wWindDir );
    printf("%s = %s\n", _WIND_DIR_ROSE, WindRoseDir());

    printf("%s = %.1f\n", _WIND_CHILL, cvtMetricFLOAT(calcWindChill( ((signed short)rcd.wOutsideTemp) / 10.0, rcd.yWindSpeed ), _CVT_TEMP));
    printf("%s = %.1f\n", _HEAT_INDEX, cvtMetricFLOAT(calcHeatIndex( ((signed short)rcd.wOutsideTemp) / 10.0, rcd.yOutsideHum ), _CVT_TEMP));

    printf("%s = %.2f\n", _RAIN_RATE, cvtMetricFLOAT(rcd.wRainRate / 100.0, _CVT_RAIN) );
    printf("%s = %s\n", _IS_RAINING, rcd.wRainRate ? "yes" : "no");
    printf("%s = ", _UV_INDEX);
    if((unsigned)rcd.yUVLevel == 0xff)
    	printf("n/a\n");
    else
    	printf("%.1f\n", rcd.yUVLevel / 10.0 );
    printf("%s = ", _SOLAR_RAD);
    if((unsigned)rcd.wSolarRad == 0x7fff)	// 32767
    	printf("n/a\n");
    else
    	printf("%d\n", rcd.wSolarRad );
    printf("%s = %.2f\n", _RAIN_STORM, cvtMetricFLOAT(rcd.wStormRain / 100.0, _CVT_RAIN) );
    printf("%s = %s\n", _STORM_START_DATE, PrintDate(rcd.wStormStart));
    
    printf("%s = %.2f\n", _DAY_RAIN, cvtMetricFLOAT(rcd.wRainDay / 100.0, _CVT_RAIN) );
    printf("%s = %.2f\n", _MONTH_RAIN, cvtMetricFLOAT(rcd.wRainMonth / 100.0, _CVT_RAIN) );
    printf("%s = %.2f\n", _YEAR_RAIN, cvtMetricFLOAT(rcd.wRainYear / 100.0, _CVT_RAIN) );
    printf("%s = %.3f\n", _DAY_ET, cvtMetricFLOAT(rcd.wETDay / 1000.0, _CVT_RAIN) );
    if (!bMetric)
        printf("%s = %.2f\n", _DAY_ET_U, (rcd.wETDay / 1000.0) * RAIN_OZINCH );
    else
        printf("%s = %.2f\n", _DAY_ET_M, cvtMetricFLOAT((rcd.wETDay / 1000.0) * RAIN_OZINCH, _CVT_ET_OZ) );
    printf("%s = %.3f\n", _MONTH_ET, cvtMetricFLOAT(rcd.wETMonth / 100.0, _CVT_RAIN) );
    printf("%s = %.3f\n", _YEAR_ET, cvtMetricFLOAT(rcd.wETYear / 100.0, _CVT_RAIN) );
    printf("%st = %d\n", _XMIT_BATT, rcd.yXmitBatt);
    printf("%s = %.1f\n", _BATT_VOLTAGE, ((rcd.wBattLevel * 300)/512)/100.0);
    printf("%s = %d\n", _FORE_ICON, rcd.yForeIcon);
    printf("%s = %d\n", _FORE_RULE, rcd.yRule);
    printf("%s = %s\n", _FORE_STRING, ForecastString(rcd.yRule));
    sunrise[0] = '\0';
    sunset[0] = '\0';
    solarnoon[0] = '\0';
    strcpy(sunrise,TimeConvert(rcd.wSunrise));
    strcpy(sunset,TimeConvert(rcd.wSunset));
    if (bCalcSun) {
        if (SunRise[0] != '\0') strcpy(sunrise,SunRise);
        if (SunSet[0] != '\0') strcpy(sunset,SunSet);
        if (SolarNoon[0] != '\0') {
            strcpy(solarnoon,SolarNoon);
            if (! bSolarNoonSec) {
                fpos = strchr(solarnoon,':');
                lpos = strrchr(solarnoon,':');
                apos = strstr(solarnoon,"M");
                if (lpos != NULL && lpos != fpos) {
                    if (apos != NULL) {
                        *lpos++ = *(apos-1);
                        *lpos++ = *apos;
                    }
                    *lpos = '\0';
                }
            }
            printf("%s = %s\n", _SOLAR_NOON, solarnoon);
        }
    }
    printf("%s = %s\n", _SUNRISE, sunrise);
    printf("%s = %s\n", _SUNSET, sunset);
    printf("%s = %s (%llu)\n", _RT_CAPTURE_TIMESTAMP, rtDateTimeCT, (unsigned long long)ttrt);

}


/*--------------------------------------------------------------------------
        ListRTDataFmt
        Dumps the real time weather formatted data to stdout.
----------------------------------------------------------------------------*/
void ListRTDataFmt()
{
    printf("\n       IDENTIFIER     DESCRIPTION\n");
    printf("           IN_TEMP : Indoor Temperature (%%f)\n");
    printf("          OUT_TEMP : Outdoor Temperature (%%f)\n");
    printf("       IN_HUMIDITY : Indoor Humidity (%%f)\n");
    printf("      OUT_HUMIDITY : Outdoor Humidity (%%f)\n");
    printf("        WIND_CHILL : Wind Chill (%%f)\n");
    printf("        HEAT_INDEX : Heat Index (%%f)\n");
    printf("         BAROMETER : Barometer (%%f)\n");
    printf("        BARO_TREND : Barometer Trend (%%s)\n");
    printf("    WIND_DIRECTION : Wind Direction (%%s)\n");
    printf("       WIND_DEGREE : Wind Degree (%%f)\n");
    printf("        WIND_SPEED : Wind Speed (%%f)\n");
    printf("        RAIN_TODAY : Rain Today (%%f)\n");
    printf("        RAIN_MONTH : Rain Month (%%f)\n");
    printf("         RAIN_YEAR : Rain Year (%%f)\n");
    printf("         SOLARNOON : Solar Noon (%%s)\n");
    printf("           SUNRISE : Sunrise Time (%%s)\n");
    printf("            SUNSET : Sunset Time (%%s)\n\n");
    printf(" Examples:\n");
printf("      \"Indoor Temp:       %%5.1f{IN_TEMP}     Outdoor Temp:      %%5.1f{OUT_TEMP}\n");
printf("      \"Indoor Humidity:     %%3.0f{IN_HUMIDITY}\\nOutdoor Humidity:    %%3.0f{OUT_HUMIDITY}\\n\"\n");
printf("      \"Barometer:      %%8.3f{BAROMETER}\\nBarometer Trend:   %%s{BARO_TREND}\\n\"\n");
printf("      See more examples in usable files in \"conf/\" directory\n\n");
}


/*--------------------------------------------------------------------------
        PrintRTDataFmt
        Dumps the real time weather formatted data to stdout.
----------------------------------------------------------------------------*/
void PrintRTDataFmt(char *fString)
{
    char sunrise[26] = "";
    char sunset[26] = "";
    char solarnoon[11] = "";
    char *format = NULL;
    int SSpos = 0, LLpos = 0;
    int aLen = 0, fLen = 0; 
    char *attr = NULL, *fmt = NULL, *posB = NULL;
    char *fpos, *lpos, *apos;

    sunrise[0] = '\0';
    sunset[0] = '\0';
    solarnoon[0] = '\0';
    strcpy(sunrise,TimeConvert(rcd.wSunrise));
    strcpy(sunset,TimeConvert(rcd.wSunset));
    if (bCalcSun) {
        if (SunRise[0] != '\0') strcpy(sunrise,SunRise);
        if (SunSet[0] != '\0') strcpy(sunset,SunSet);
        if (SolarNoon[0] != '\0') {
            strcpy(solarnoon,SolarNoon);
            if (! bSolarNoonSec) {
                fpos = strchr(solarnoon,':');
                lpos = strrchr(solarnoon,':');
                apos = strstr(solarnoon,"M");
                if (lpos != NULL && lpos != fpos) {
                    if (apos != NULL) {
                        *lpos++ = *(apos-1);
                        *lpos++ = *apos;
                    }
                    *lpos = '\0';
                }
            }
        }
    }
 
    while ((format = SubString(fString, '}', &SSpos)) && SSpos < strlen(fString)) {
        if (strstr(format, "\\n") == format) {
            printf("\n");
            format += 2;
        }
        if (strchr(format,'\n') == format || strchr(format,'\r') == format) {
            printf("\n");
            format++;
        }
        posB = strchr(format,'{');

        aLen = strlen(format)-(posB-format);
        if (posB > 0 && aLen > 0) {
            attr = getMemPtr(aLen);
            strncpy(attr, posB+1, aLen-1);
            *(attr+aLen-1)  = '\0';
        }
        if (posB > format) {
            fLen = (posB - format) + 1;
            fmt = getMemPtr(fLen+4);
            strncpy(fmt, format, fLen-1);
            *(fmt+fLen-1) = '\0';
        }

        if (strcmp(attr, "IN_TEMP") == 0) {
            strcat(fmt, bMetric ? " C" : " F");
            printf(fmt, cvtMetricFLOAT(rcd.wInsideTemp/10.0, _CVT_TEMP));
        }
        if (strcmp(attr, "IN_HUMIDITY") == 0) {
            strcat(fmt, " %%");
            printf(fmt, (float)rcd.yInsideHum);
        }
        if (strcmp(attr, "OUT_TEMP") == 0) {
            strcat(fmt, bMetric ? " C" : " F");
            printf(fmt, cvtMetricFLOAT(rcd.wOutsideTemp/10.0, _CVT_TEMP));
        }
        if (strcmp(attr, "OUT_HUMIDITY") == 0) {
            strcat(fmt, " %%");
            printf(fmt, (float)rcd.yOutsideHum);
        }
        if (strcmp(attr, "WIND_CHILL") == 0) {
            strcat(fmt, bMetric ? " C" : " F");
            printf(fmt, cvtMetricFLOAT(calcWindChill(rcd.wOutsideTemp/10.0, rcd.yWindSpeed), _CVT_TEMP));
        }
        if (strcmp(attr, "HEAT_INDEX") == 0) {
            strcat(fmt, bMetric ? " C" : " F");
            printf(fmt, cvtMetricFLOAT(calcHeatIndex(rcd.wOutsideTemp/10.0, rcd.yOutsideHum), _CVT_TEMP));
        }
        if (strcmp(attr, "BAROMETER") == 0) {
            strcat(fmt, bMetric ? " mb" : " in");
            printf(fmt, cvtMetricFLOAT(rcd.wBarometer/1000.0, _CVT_BARO));
        }
        if (strcmp(attr, "BARO_TREND") == 0) printf(fmt, BaroTrend(0));
        if (strcmp(attr, "WIND_DIRECTION") == 0) printf(fmt, WindRoseDir());
        if (strcmp(attr, "WIND_DEGREE") == 0) printf(fmt, rcd.wWindDir);
        if (strcmp(attr, "WIND_SPEED") == 0) {
            strcat(fmt, bMetric ? " kh " : " mph");
            printf(fmt, cvtMetricFLOAT(rcd.yWindSpeed, _CVT_WIND));
        }
        if (strcmp(attr, "RAIN_TODAY") == 0) {
            strcat(fmt, bMetric ? " mm" : " in");
            printf(fmt, cvtMetricFLOAT(rcd.wRainDay/100.0, _CVT_RAIN));
        }
        if (strcmp(attr, "RAIN_MONTH") == 0) {
            strcat(fmt, bMetric ? " mm" : " in");
            printf(fmt, cvtMetricFLOAT(rcd.wRainMonth/100.0, _CVT_RAIN));
        }
        if (strcmp(attr, "RAIN_YEAR") == 0) {
            strcat(fmt, bMetric ? " mm" : " in");
            printf(fmt, cvtMetricFLOAT(rcd.wRainYear/100.0, _CVT_RAIN));
        }
        if (strcmp(attr, "SUNRISE") == 0) printf(fmt, sunrise);
        if (strcmp(attr, "SUNSET") == 0) printf(fmt, sunset);
        if (strcmp(attr, "SOLARNOON") == 0 && solarnoon[0] != '\0') printf(fmt, solarnoon);

        free(attr);
        free(fmt);
        attr = fmt = posB = NULL;
        aLen = fLen = 0;
        LLpos = SSpos;
    }
    if (LLpos != strlen(fString)) {
        format = fString + LLpos;
        if (strstr(format, "\\n") == format || strcmp(format, "\n") == 0 || *format == '\n' || *format == '\r') printf("\n");
    }
}


/*--------------------------------------------------------------------------
	TimeConvert
	Converts a BCD encoded time value to ascii human readable form. Returns
	a pointer to a static buffer containing the converted ascii string. The
	form is '12:33PM'. If you don't like this format, here is where you need
	to change it.
----------------------------------------------------------------------------*/
char* TimeConvert(WORD wTime)
{

	static char szBuf[32];				/* static return buffer */
	BOOL bAM = TRUE;
	int nHours = wTime/100;
	int nMinutes = wTime - ((wTime/100) * 100);

//	if (bVerbose) fprintf(stderr, "TimeConvert: wTime %d 0x%x\n", wTime, (unsigned)wTime);

        if ((unsigned)wTime == 0x7fff || (unsigned)wTime == 0xffff) {      /* ignore missing data */
                sprintf(szBuf, "n/a");
                return szBuf;
        }

	if(!bMtime) {	
		if(nHours > 12) {		/* PM time ? */
			nHours -= 12;
			bAM = FALSE;
		}
		else if(nHours == 12)
			bAM = FALSE;

		if(nHours == 0 && bAM)
			nHours = 12;
	
		sprintf(szBuf, "%d:%02d%s", nHours, nMinutes, (bAM?"AM":"PM"));
	}
	else {
		sprintf(szBuf, "%02d:%02d", nHours, nMinutes);
	}

	return szBuf;
}


/*--------------------------------------------------------------------------
	PrintHLData
	Dumps the highs/lows weather data to stdout.
----------------------------------------------------------------------------*/
void PrintHLData(void)
{
    if (!bMetric) {
        printf("%s = %.3f\n", _BARO_LO_DAY, hld.wBaroLoDay / 1000.0 );
        printf("%s = %.3f\n", _BARO_HI_DAY, hld.wBaroHiDay / 1000.0 );
        printf("%s = %.3f\n", _BARO_LO_MONTH, hld.wBaroLoMonth / 1000.0 );
        printf("%s = %.3f\n", _BARO_HI_MONTH, hld.wBaroHiMonth / 1000.0 );
        printf("%s = %.3f\n", _BARO_LO_YEAR, hld.wBaroLoYear / 1000.0 );
        printf("%s = %.3f\n", _BARO_HI_YEAR, hld.wBaroHiYear / 1000.0 );
    } else {
        printf("%s = %.1f\n", _BARO_LO_DAY, cvtMetricFLOAT(hld.wBaroLoDay / 1000.0, _CVT_BARO) );
        printf("%s = %.1f\n", _BARO_HI_DAY, cvtMetricFLOAT(hld.wBaroHiDay / 1000.0, _CVT_BARO) );
        printf("%s = %.1f\n", _BARO_LO_MONTH, cvtMetricFLOAT(hld.wBaroLoMonth / 1000.0, _CVT_BARO) );
        printf("%s = %.1f\n", _BARO_HI_MONTH, cvtMetricFLOAT(hld.wBaroHiMonth / 1000.0, _CVT_BARO) );
        printf("%s = %.1f\n", _BARO_LO_YEAR, cvtMetricFLOAT(hld.wBaroLoYear / 1000.0, _CVT_BARO) );
        printf("%s = %.1f\n", _BARO_HI_YEAR, cvtMetricFLOAT(hld.wBaroHiYear / 1000.0, _CVT_BARO) );
    }
    printf("%s = %s\n", _BARO_LO_TIME, TimeConvert(hld.wBaroLoTime));
    printf("%s = %s\n", _BARO_HI_TIME, TimeConvert(hld.wBaroHiTime));

    printf("%s = %d\n", _WIND_HI_DAY, cvtMetricBYTE(hld.yWindHiDay, _CVT_WIND) );
    printf("%s = %s\n", _WIND_HI_TIME ,TimeConvert(hld.wWindHiTime));
    printf("%s = %d\n", _WIND_HI_MONTH, cvtMetricBYTE(hld.yWindHiMonth, _CVT_WIND) );
    printf("%s = %d\n", _WIND_HI_YEAR, cvtMetricBYTE(hld.yWindHiYear, _CVT_WIND) );

    printf("%s = %.1f\n", _IN_TEMP_HI_DAY, cvtMetricFLOAT(((signed short)hld.wInTempHiDay) / 10.0, _CVT_TEMP) );
    printf("%s = %.1f\n", _IN_TEMP_LO_DAY, cvtMetricFLOAT(((signed short)hld.wInTempLoDay) / 10.0, _CVT_TEMP) );
    printf("%s = %s\n", _IN_TEMP_HI_TIME, TimeConvert(hld.wInTempHiTime));
    printf("%s = %s\n", _IN_TEMP_LO_TIME, TimeConvert(hld.wInTempLoTime));
    printf("%s = %.1f\n", _IN_TEMP_LO_MONTH, cvtMetricFLOAT(((signed short)hld.wInTempLoMonth) / 10.0, _CVT_TEMP) );
    printf("%s = %.1f\n", _IN_TEMP_HI_MONTH, cvtMetricFLOAT(((signed short)hld.wInTempHiMonth) / 10.0, _CVT_TEMP) );
    printf("%s = %.1f\n", _IN_TEMP_LO_YEAR, cvtMetricFLOAT(((signed short)hld.wInTempLoYear) / 10.0, _CVT_TEMP) );
    printf("%s = %.1f\n", _IN_TEMP_HI_YEAR, cvtMetricFLOAT(((signed short)hld.wInTempHiYear) / 10.0, _CVT_TEMP) );
    
    printf("%s = %d\n", _IN_HUM_HI_DAY, hld.yInHumHiDay );
    printf("%s = %d\n", _IN_HUM_LO_DAY, hld.yInHumLoDay );
    printf("%s = %s\n", _IN_HUM_HI_TIME, TimeConvert(hld.wInHumHiTime));
    printf("%s = %s\n", _IN_HUM_LO_TIME, TimeConvert(hld.wInHumLoTime));
    printf("%s = %d\n", _IN_HUM_HI_MONTH, hld.yInHumHiMonth );
    printf("%s = %d\n", _IN_HUM_LO_MONTH, hld.yInHumLoMonth );
    printf("%s = %d\n", _IN_HUM_HI_YEAR, hld.yInHumHiYear );
    printf("%s = %d\n", _IN_HUM_LO_YEAR, hld.yInHumLoYear );

    printf("%s = %.1f\n", _TEMP_HI_DAY, cvtMetricFLOAT(((signed short)hld.wTempHiDay) / 10.0, _CVT_TEMP) );
    printf("%s = %.1f\n", _TEMP_LO_DAY, cvtMetricFLOAT(((signed short)hld.wTempLoDay) / 10.0, _CVT_TEMP) );
    printf("%s = %s\n", _TEMP_HI_TIME, TimeConvert(hld.wTempHiTime));
    printf("%s = %s\n", _TEMP_LO_TIME, TimeConvert(hld.wTempLoTime));
    printf("%s = %.1f\n", _TEMP_LO_MONTH, cvtMetricFLOAT(((signed short)hld.wTempLoMonth) / 10.0, _CVT_TEMP) );
    printf("%s = %.1f\n", _TEMP_HI_MONTH, cvtMetricFLOAT(((signed short)hld.wTempHiMonth) / 10.0, _CVT_TEMP) );
    printf("%s = %.1f\n", _TEMP_HI_YEAR, cvtMetricFLOAT(((signed short)hld.wTempHiYear) / 10.0, _CVT_TEMP) );
    printf("%s = %.1f\n", _TEMP_LO_YEAR, cvtMetricFLOAT(((signed short)hld.wTempLoYear) / 10.0, _CVT_TEMP) );
    
    printf("%s = %d\n", _DEW_LO_DAY, cvtMetricINT((signed short)hld.wDewLoDay, _CVT_TEMP) );
    printf("%s = %d\n", _DEW_HI_DAY, cvtMetricINT((signed short)hld.wDewHiDay, _CVT_TEMP) );
    printf("%s = %s\n", _DEW_LO_TIME, TimeConvert(hld.wDewLoTime));
    printf("%s = %s\n", _DEW_HI_TIME, TimeConvert(hld.wDewHiTime));
    printf("%s = %d\n", _DEW_HI_MONTH, cvtMetricINT((signed short)hld.wDewHiMonth, _CVT_TEMP) );
    printf("%s = %d\n", _DEW_LO_MONTH, cvtMetricINT((signed short)hld.wDewLoMonth, _CVT_TEMP) );
    printf("%s = %d\n", _DEW_HI_YEAR, cvtMetricINT((signed short)hld.wDewHiYear, _CVT_TEMP) );
    printf("%s = %d\n", _DEW_LO_YEAR, cvtMetricINT((signed short)hld.wDewLoYear, _CVT_TEMP) );

    printf("%s = %d\n", _CHILL_LO_DAY, cvtMetricINT((signed short)hld.wChillLoDay, _CVT_TEMP) );
    printf("%s = %s\n", _CHILL_LO_TIME, TimeConvert(hld.wChillLoTime));
    printf("%s = %d\n", _CHILL_LO_MONTH, cvtMetricINT((signed short)hld.wChillLoMonth, _CVT_TEMP) );
    printf("%s = %d\n", _CHILL_LO_YEAR, cvtMetricINT((signed short)hld.wChillLoYear, _CVT_TEMP) );

    printf("%s = %d\n", _HEAT_HI_DAY, cvtMetricINT(hld.wHeatHiDay, _CVT_TEMP) );
    printf("%s = %s\n", _HEAT_HI_TIME, TimeConvert(hld.wHeatHiTime));
    printf("%s = %d\n", _HEAT_HI_MONTH, cvtMetricINT(hld.wHeatHiMonth, _CVT_TEMP) );
    printf("%s = %d\n", _HEAT_HI_YEAR, cvtMetricINT(hld.wHeatHiYear, _CVT_TEMP) );

    printf("%s = ", _THSW_HI_DAY );
    if((unsigned)hld.wTHSWHiDay == 0x7fff)	// 65535
        printf("n/a\n");
    else
        printf("%d\n", cvtMetricINT((signed short)hld.wTHSWHiDay, _CVT_TEMP) );
    printf("%s = ", _THSW_HI_TIME );
    if((unsigned)hld.wTHSWHiTime == 0xffff)	// 65535
            printf("n/a\n");
    else
        printf("%s\n", TimeConvert(hld.wTHSWHiTime));
    printf("%s = ", _THSW_HI_MONTH );
    if((unsigned)hld.wTHSWHiMonth == 0x7fff)	// 65535
        printf("n/a\n");
    else
        printf("%d\n", cvtMetricINT((signed short)hld.wTHSWHiMonth, _CVT_TEMP) );
    printf("%s = ", _THSW_HI_YEAR );
    if((unsigned)hld.wTHSWHiYear == 0x7fff)	// 65535
        printf("n/a\n");
    else
        printf("%d\n", cvtMetricINT((signed short)hld.wTHSWHiYear, _CVT_TEMP) );

    printf("%s = %d\n", _SOLAR_HI_DAY, hld.wSolarHiDay);

    printf("%s = ", _SOLAR_HI_TIME );
    if((unsigned)hld.wSolarHiTime == 0xffff)	// 65535
        printf("n/a\n");
    else
        printf("%s\n", TimeConvert(hld.wSolarHiTime));
    printf("%s = %d\n", _SOLAR_HI_MONTH, hld.wSolarHiMonth);
    printf("%s = %d\n", _SOLAR_HI_YEAR, hld.wSolarHiYear);
    
    printf("%s = %.1f\n", _UV_HI_DAY, hld.yUVHiDay / 10.0);
    printf("%s = ", _UV_HI_TIME );
    if((unsigned)hld.wUVHiTime == 0xffff)	// 65535
        printf("n/a\n");
    else
        printf("%s\n", TimeConvert(hld.wUVHiTime));
    printf("%s = %.1f\n", _UV_HI_MONTH, hld.yUVHiMonth / 10.0);
    printf("%s = %.1f\n", _UV_HI_YEAR, hld.yUVHiYear / 10.0);

    printf("%s = %.2f\n", _RAIN_RATE_HI_DAY, cvtMetricFLOAT(hld.wRainHiDay / 100.0, _CVT_RAIN) );
    printf("%s = ", _RAIN_RATE_HI_TIME );
    if((unsigned)hld.wRainHiTime == 0xffff)	// 65535
        printf("n/a\n");
    else
        printf("%s\n", TimeConvert(hld.wRainHiTime));
    printf("%s = %.2f\n", _RAIN_RATE_HI_HOUR, cvtMetricFLOAT(hld.wRainHiHour / 100.0, _CVT_RAIN) );
    printf("%s = %.2f\n", _RAIN_RATE_HI_MONTH, cvtMetricFLOAT(hld.wRainHiMonth / 100.0, _CVT_RAIN) );
    printf("%s = %.2f\n", _RAIN_RATE_HI_YEAR, cvtMetricFLOAT(hld.wRainHiYear / 100.0, _CVT_RAIN) );
    printf("%s = %s (%llu)\n", _HL_CAPTURE_TIMESTAMP, hlDateTimeCT, (unsigned long long)tthl);

}


/*--------------------------------------------------------------------------
	PrintGDData
	Dumps the graphics sets to stdout.
----------------------------------------------------------------------------*/
void PrintGDData(char * pData)
{
	if (bVerbose) fprintf(stderr, "Printing Graph data\n");

	PrintTimeRef();					/* Print reference times */

	/* inside temp by hour, last 24 */
	printf("%s = %s\n", _IN_TEMP_BY_HOURS, PrintTempSet24((unsigned char *)pData, TEMP_IN_HOUR, *(pData + NEXT_HOUR_PTR), 90));
	
	/* inside temp highs by day, last 24 */
	printf("%s = %s\n", _IN_TEMP_HI_BY_DAYS, PrintTempSet24((unsigned char *)pData, TEMP_IN_DAY_HIGHS, *(pData + NEXT_DAY_PTR), 90));
	
	/* inside temp highs by day, time, last 24 */
	printf("%s = ", _IN_TEMP_HI_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, TEMP_IN_DAY_HIGH_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* inside temp lows by day, last 24 */
	printf("%s = %s\n", _IN_TEMP_LO_BY_DAYS, PrintTempSet24((unsigned char *)pData, TEMP_IN_DAY_LOWS, *(pData + NEXT_DAY_PTR), 90));
	
	/* inside temp lows by day by time, last 24 */
	printf("%s = ", _IN_TEMP_LO_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, TEMP_IN_DAY_LOW_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* inside temp highs by month, last 25 */
	printf("%s = ", _IN_TEMP_HI_BY_MONTHS);
	PrintTempSet25((unsigned char *)pData, TEMP_IN_MONTH_HIGHS, *(pData + NEXT_MONTH_PTR), 90);

	/* inside temp lows by month, last 25 */
	printf("%s = ", _IN_TEMP_LO_BY_MONTHS);
	PrintTempSet25((unsigned char *)pData, TEMP_IN_MONTH_LOWS, *(pData + NEXT_MONTH_PTR), 90);
	
	/* temp by hour, last 24 */
	printf("%s = %s\n", _TEMP_BY_HOURS, PrintTempSet24((unsigned char *)pData, TEMP_OUT_HOUR, *(pData + NEXT_HOUR_PTR),  90));
	
	/* temp highs by day, last 24 */
	printf("%s = %s\n", _TEMP_HI_BY_DAYS, PrintTempSet24((unsigned char *)pData, TEMP_OUT_DAY_HIGHS, *(pData + NEXT_DAY_PTR), 90));
	
	/* temp highs by day, time, last 24 */
	printf("%s = ", _TEMP_HI_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, TEMP_OUT_DAY_HIGH_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* temp lows by day, last 24 */
	printf("%s = %s\n", _TEMP_LO_BY_DAYS, PrintTempSet24((unsigned char *)pData, TEMP_OUT_DAY_LOWS, *(pData + NEXT_DAY_PTR), 90));
	
	/* temp lows by day by time, last 24 */
	printf("%s = ", _TEMP_LO_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, TEMP_OUT_DAY_LOW_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* temp highs by month, last 24 */
	printf("%s = ", _TEMP_HI_BY_MONTHS);
	PrintTempSet25((unsigned char *)pData, TEMP_OUT_MONTH_HIGHS, *(pData + NEXT_MONTH_PTR), 90);

	/* temp lows by month, last 24 */
	printf("%s = ", _TEMP_LO_BY_MONTHS);
	PrintTempSet25((unsigned char *)pData, TEMP_OUT_MONTH_LOWS, *(pData + NEXT_MONTH_PTR), 90);

	/* dew point by hour, last 24 */
	printf("%s = %s\n", _DEW_BY_HOURS, PrintTempSet24((unsigned char *)pData, DEW_HOUR, *(pData + NEXT_HOUR_PTR), 120));
	
	/* dew point highs by day, last 24 */
	printf("%s = %s\n", _DEW_HI_BY_DAYS, PrintTempSet24((unsigned char *)pData, DEW_DAY_HIGHS, *(pData + NEXT_DAY_PTR), 120));
	
	/* dew point highs by day, time, last 24 */
	printf("%s = ", _DEW_HI_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, DEW_DAY_HIGH_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* dew point lows by day, last 24 */
	printf("%s = %s\n", _DEW_LO_BY_DAYS, PrintTempSet24((unsigned char *)pData, DEW_DAY_LOWS, *(pData + NEXT_DAY_PTR), 120));
	
	/* dew point lows by day, time, last 24 */
	printf("%s = ", _DEW_LO_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, DEW_DAY_LOW_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* dew point highs by month, last 25 */
	printf("%s = ", _DEW_HI_BY_MONTHS);
	PrintTempSet25((unsigned char *)pData, DEW_MONTH_HIGHS, *(pData + NEXT_MONTH_PTR), 120);

	/* dew point lows by month, last 25 */
	printf("%s = ",  _DEW_LO_BY_MONTHS);
	PrintTempSet25((unsigned char *)pData, DEW_MONTH_LOWS, *(pData + NEXT_MONTH_PTR), 120);

	/* wind chill by hour, last 24 */
	printf("%s = %s\n", _CHILL_LO_BY_HOURS, PrintTempSet24((unsigned char *)pData, CHILL_HOUR, *(pData + NEXT_HOUR_PTR), 120));
	
	/* wind chill lows by day, last 24 */
	printf("%s = %s\n", _CHILL_LO_BY_DAYS, PrintTempSet24((unsigned char *)pData, CHILL_DAY_LOWS, *(pData + NEXT_DAY_PTR), 120));
	
	/* wind chill lows by day, time, last 24 */
	printf("%s = ", _CHILL_LO_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, CHILL_DAY_LOW_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* wind chill lows by month, last 25 */
	printf("%s = ", _CHILL_LO_BY_MONTHS);
	PrintTempSet25((unsigned char *)pData, CHILL_MONTH_LOWS, *(pData + NEXT_MONTH_PTR), 120);

	/* THSW by hour, last 24 */
	printf("%s = %s\n", _THSW_HI_BY_HOURS, PrintTempSet24((unsigned char *)pData, THSW_HOUR, *(pData + NEXT_HOUR_PTR), 90));

	/* THSW by day, last 24 */
	printf("%s = %s\n", _THSW_HI_BY_DAYS, PrintTempSet24((unsigned char *)pData, THSW_DAY_HIGHS, *(pData + NEXT_DAY_PTR), 90));

	/* THSW by day, time, last 24 */
	printf("%s = ", _THSW_HI_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, THSW_DAY_HIGH_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* THSW highs by month, last 25 */
	printf("%s = ", _THSW_HI_BY_MONTHS);
	PrintTempSet25((unsigned char *)pData, THSW_MONTH_HIGHS, *(pData + NEXT_MONTH_PTR), 90);

	/* heat index by hour, last 24 */
	printf("%s = %s\n", _HEAT_HI_BY_HOURS, PrintTempSet24((unsigned char *)pData, HEAT_HOUR, *(pData + NEXT_HOUR_PTR), 90));
	
	/* heat index lows by day, last 24 */
	printf("%s = %s\n", _HEAT_HI_BY_DAYS, PrintTempSet24((unsigned char *)pData, HEAT_DAY_HIGHS, *(pData + NEXT_DAY_PTR), 90));
	
	/* heat index highs by day, time, last 24 */
	printf("%s = ", _HEAT_HI_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, HEAT_DAY_HIGH_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* heat index by month, last 25 */
	printf("%s = ", _HEAT_HI_BY_MONTHS);
	PrintTempSet25((unsigned char *)pData, HEAT_MONTH_HIGHS, *(pData + NEXT_MONTH_PTR), 90);

	/* Inside Humidity by hour, last 24 */
	printf("%s = %s\n", _IN_HUM_BY_HOURS, PrintByteSet((unsigned char *)pData, HUM_IN_HOUR, *(pData + NEXT_HOUR_PTR), 24, _CVT_NONE));
	
	/* Inside Humidity highs by day, last 24 */
	printf("%s = %s\n", _IN_HUM_HI_BY_DAYS, PrintByteSet((unsigned char *)pData, HUM_IN_DAY_HIGHS, *(pData + NEXT_DAY_PTR), 24, _CVT_NONE));
	
	/* Inside Humidity highs by day, time, last 24 */
	printf("%s = ", _IN_HUM_HI_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, HUM_IN_DAY_HIGH_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* Inside Humidity lows by day, last 24 */
	printf("%s = %s\n", _IN_HUM_LO_BY_DAYS, PrintByteSet((unsigned char *)pData, HUM_IN_DAY_LOWS, *(pData + NEXT_DAY_PTR), 24, _CVT_NONE));
	
	/* Inside Humidity lows by day, time, last 24 */
	printf("%s = ", _IN_HUM_LO_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, HUM_IN_DAY_LOW_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* Inside Humidity highs by month, last 24 */
	printf("%s = %s\n", _IN_HUM_HI_BY_MONTHS, PrintByteSet((unsigned char *)pData, HUM_IN_MONTH_HIGHS, *(pData + NEXT_MONTH_PTR), 25, _CVT_NONE));

	/* Inside Humidity lows by month, last 24 */
	printf("%s = %s\n", _IN_HUM_LO_BY_MONTHS, PrintByteSet((unsigned char *)pData, HUM_IN_MONTH_LOWS, *(pData + NEXT_MONTH_PTR), 25, _CVT_NONE));

	/* Humidity by hour, last 24 */
	printf("%s = %s\n", _HUM_BY_HOURS, PrintByteSet((unsigned char *)pData, HUM_OUT_HOUR, *(pData + NEXT_HOUR_PTR), 24, _CVT_NONE));
	
	/* Humidity highs by day, last 24 */
	printf("%s = %s\n", _HUM_HI_BY_DAYS, PrintByteSet((unsigned char *)pData, HUM_OUT_DAY_HIGHS, *(pData + NEXT_DAY_PTR), 24, _CVT_NONE));
	
	/* Humidity highs by day, time, last 24 */
	printf("%s = ", _HUM_HI_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, HUM_OUT_DAY_HIGH_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* Humidity lows by day, last 24 */
	printf("%s = %s\n", _HUM_LO_BY_DAYS, PrintByteSet((unsigned char *)pData, HUM_OUT_DAY_LOWS, *(pData + NEXT_DAY_PTR), 24, _CVT_NONE));
	
	/* Humidity lows by day, time, last 24 */
	printf("%s = ", _HUM_LO_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, HUM_OUT_DAY_LOW_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* Humidity highs by month, last 24 */
	printf("%s = %s\n", _HUM_HI_BY_MONTHS, PrintByteSet((unsigned char *)pData, HUM_OUT_MONTH_HIGHS, *(pData + NEXT_MONTH_PTR), 25, _CVT_NONE));

	/* Humidity lows by month, last 24 */
	printf("%s = %s\n", _HUM_LO_BY_MONTHS, PrintByteSet((unsigned char *)pData, HUM_OUT_MONTH_LOWS, *(pData + NEXT_MONTH_PTR), 25, _CVT_NONE));

	/* Barometer 15 min increments, last 24 */
	/* cjb Don't bother using NEXT_15MIN_PTR it never gets updated per Davis due to EEPROM constraints */
	/* tracking which data value is updated in the round robin will have to be done externally         */
/* cjb	printf("%s = %s\n", _BAR_BY_15MIN, PrintBarSet((unsigned char *)pData, BAR_15_MIN, *(pData + NEXT_15MIN_PTR), 24)); */
	printf("%s = %s\n", _BAR_BY_15MIN, PrintBarSet((unsigned char *)pData, BAR_15_MIN, 0, 24));

	/* Barometer by hour, last 24 */
	printf("%s = %s\n", _BAR_BY_HOURS, PrintBarSet((unsigned char *)pData, BAR_HOUR, *(pData + NEXT_HOUR_PTR), 24));

	/* Barometer highs by day, last 24 */
	printf("%s = %s\n", _BAR_HI_BY_DAYS, PrintBarSet((unsigned char *)pData, BAR_DAY_HIGHS, *(pData + NEXT_DAY_PTR), 24));
	
	/* Barometer highs by day, time, last 24 */
	printf("%s = ", _BAR_HI_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, BAR_DAY_HIGH_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* Barometer lows by day, last 24 */
	printf("%s = %s\n", _BAR_LO_BY_DAYS, PrintBarSet((unsigned char *)pData, BAR_DAY_LOWS, *(pData + NEXT_DAY_PTR), 24));
	
	/* Barometer lows by day by time, last 24 */
	printf("%s = ", _BAR_LO_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, BAR_DAY_LOW_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* Barometer highs by month, last 25 */
	printf("%s = %s\n", _BAR_HI_BY_MONTHS, PrintBarSet((unsigned char *)pData, BAR_MONTH_HIGHS, *(pData + NEXT_MONTH_PTR), 25));

	/* Barometer lows by month, last 25 */
	printf("%s = %s\n", _BAR_LO_BY_MONTHS, PrintBarSet((unsigned char *)pData, BAR_MONTH_LOWS, *(pData + NEXT_MONTH_PTR), 25));

	/* Wind speed 10 min averages, last 24 */
	/* cjb Don't bother using NEXT_10MIN_PTR it never gets updated per Davis due to EEPROM constraints */
	/* tracking which data value is updated in the round robin will have to be done externally         */
/*	printf("%s = %s\n", _WIND_AVG_BY_10MIN, PrintByteSet((unsigned char *)pData, WIND_SPEED_10_MIN_AVG, *(pData + NEXT_10MIN_PTR), 24, _CVT_WIND)); */
	printf("%s = %s\n", _WIND_AVG_BY_10MIN, PrintByteSet((unsigned char *)pData, WIND_SPEED_10_MIN_AVG, 0, 24, _CVT_WIND));

	/* Wind speed average by hour, last 24 */
	printf("%s = %s\n", _WIND_AVG_BY_HOURS, PrintByteSet((unsigned char *)pData, WIND_SPEED_HOUR_AVG, *(pData + NEXT_HOUR_PTR), 24, _CVT_WIND));

	/* Wind speed highs by hour, last 24 */
	printf("%s = %s\n", _WIND_HI_BY_HOURS, PrintByteSet((unsigned char *)pData, WIND_SPEED_HOUR_HIGHS, *(pData + NEXT_HOUR_PTR), 24, _CVT_WIND));

	/* Wind speed highs by day, last 24 */
	printf("%s = %s\n", _WIND_HI_BY_DAYS, PrintByteSet((unsigned char *)pData, WIND_SPEED_DAY_HIGHS, *(pData + NEXT_DAY_PTR), 24, _CVT_WIND));
	
	/* Wind speed highs by day, time, last 24 */
	printf("%s = ", _WIND_HI_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, WIND_SPEED_DAY_HIGH_TIMES, *(pData + NEXT_DAY_PTR), 24);

        /* Wind speed highs dir by day, last 24 */
        printf("%s = %s\n", _WIND_HI_DIR_BY_DAYS, PrintByteSet((unsigned char *)pData, WIND_SPEED_DAY_HIGH_DIR, *(pData + NEXT_DAY_PTR), 24, _CVT_NONE));

	/* Wind speed highs by month, last 25 */
	printf("%s = %s\n", _WIND_HI_BY_MONTHS, PrintByteSet((unsigned char *)pData, WIND_SPEED_MONTH_HIGHS, *(pData + NEXT_MONTH_PTR), 25, _CVT_WIND));

        /* Wind speed highs dir by month, last 25 */
        printf("%s = %s\n", _WIND_HI_DIR_BY_MONTHS, PrintByteSet((unsigned char *)pData, WIND_SPEED_MONTH_HIGH_DIR, *(pData + NEXT_MONTH_PTR), 25, _CVT_NONE));

	/* Wind speed highs by year, last 25 */
	printf("%s = %s\n", _WIND_HI_BY_YEARS, PrintByteSet((unsigned char *)pData, WIND_SPEED_YEAR_HIGHS, *(pData + NEXT_YEAR_PTR), 25, _CVT_WIND));

        /* Wind speed highs dir by year, last 25 */
        printf("%s = %s\n", _WIND_HI_DIR_BY_YEARS, PrintByteSet((unsigned char *)pData, WIND_SPEED_YEAR_HIGH_DIR, *(pData + NEXT_YEAR_PTR), 25, _CVT_NONE));

	/* Wind direction by hour, last 24 */
	printf("%s = %s\n", _WIND_DIR_BY_HOURS, PrintByteSet((unsigned char *)pData, WIND_DIR_HOUR, *(pData + NEXT_HOUR_PTR), 24, _CVT_NONE));
	
	/* Wind direction by day, last 24 */
	printf("%s = %s\n", _WIND_DIR_BY_DAYS, PrintByteSet((unsigned char *)pData, WIND_DIR_DAY, *(pData + NEXT_DAY_PTR), 24, _CVT_NONE));
	
	/* Wind direction by month, last 25 */
	printf("%s = %s\n", _WIND_DIR_BY_MONTHS, PrintByteSet((unsigned char *)pData, WIND_DIR_MONTH, *(pData + NEXT_MONTH_PTR), 25, _CVT_NONE));

	/* Rain Rate by minute, last 24 */
	/* cjb Don't bother using NEXT_10MIN_PTR it never gets updated per Davis due to EEPROM constraints */
	/* tracking which data value is updated in the round robin will have to be done externally         */
/*	printf("%s = %s\n", _RAIN_RATE_BY_MINUTES, PrintRainRateSet((unsigned char *)pData, RAIN_RATE_1_MIN, *(pData + NEXT_10MIN_PTR), 24));	*/
	printf("%s = %s\n", _RAIN_RATE_BY_MINUTES, PrintRainRateSet((unsigned char *)pData, RAIN_RATE_1_MIN, 0, 24));
	
	/* Rain Rate by hour, last 24 */
	printf("%s = %s\n", _RAIN_RATE_BY_HOURS, PrintRainRateSet((unsigned char *)pData, RAIN_RATE_HOUR, *(pData + NEXT_HOUR_PTR), 24));
	
	/* Rain Rate highs by day, last 24 */
	printf("%s = %s\n", _RAIN_RATE_HI_BY_DAYS, PrintRainRateSet((unsigned char *)pData, RAIN_RATE_DAY_HIGHS, *(pData + NEXT_DAY_PTR), 24));

	/* Rain Rate highs by day time, last 24 */
	printf("%s = ", _RAIN_RATE_HI_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, RAIN_RATE_DAY_HIGH_TIMES, *(pData + NEXT_DAY_PTR), 24);
	
	/* Rain Rate highs by month, last 25 */
	printf("%s = %s\n", _RAIN_RATE_HI_BY_MONTHS, PrintRainRateSet((unsigned char *)pData, RAIN_RATE_MONTH_HIGHS, *(pData + NEXT_MONTH_PTR), 25));

	/* Rain Rate highs by years, last 25 */
	printf("%s = %s\n", _RAIN_RATE_HI_BY_YEARS,  PrintRainRateSet((unsigned char *)pData, RAIN_RATE_YEAR_HIGHS, *(pData + NEXT_YEAR_PTR), 25));

	/* Rain in 15 min increments, last 24 increments */
	/* cjb Don't bother using NEXT_15MIN_PTR it never gets updated per Davis due to EEPROM constraints */
	/* tracking which data value is updated in the round robin will have to be done externally         */
	/* added cjb 02-jan-2004 Joe omitted this for some reason 
	 * had to add function PrintRainRateByteSet to do it			*/
	printf("%s = %s\n", _RAIN_BY_15MIN, PrintRainRateByteSet((unsigned char *)pData, RAIN_15_MIN, 0, 24));

	/* Rain by hour, last 24 */
	printf("%s = %s\n", _RAIN_BY_HOUR, PrintRainRateSet((unsigned char *)pData, RAIN_HOUR, *(pData + NEXT_HOUR_PTR), 24));

	/* Rain by storm, last 25 */
	/* it appears there's really only 24 of these not 25 -- cjb 28-oct-2007 */
	printf("%s = %s\n", _RAIN_STORMS, PrintRainRateSet((unsigned char *)pData, RAIN_STORM, *(pData + NEXT_RAIN_STORM_PTR), 24));
	
	/* Rain by storm start date, last 25 */
	/* it appears there's really only 24 of these not 25 -- cjb 28-oct-2007 */
	printf("%s = %s\n", _RAIN_STORMS_START_DATE, PrintDateSet((unsigned char *)pData, RAIN_STORM_START, *(pData + NEXT_RAIN_STORM_PTR), 24));

	/* Rain by storm end date, last 25 */
	/* it appears there's really only 24 of these not 25 -- cjb 28-oct-2007 */
	printf("%s = %s\n", _RAIN_STORMS_END_DATE, PrintDateSet((unsigned char *)pData, RAIN_STORM_END, *(pData + NEXT_RAIN_STORM_PTR), 24));

	/* Rain by day, last 25 */
	printf("%s = %s\n", _RAIN_BY_DAY, PrintRainRateSet((unsigned char *)pData, RAIN_DAY_TOTAL, *(pData + NEXT_DAY_PTR), 25));

	/* Rain by month, last 25 */
	printf("%s = %s\n", _RAIN_BY_MONTH, PrintRainRateSet((unsigned char *)pData, RAIN_MONTH_TOTAL, *(pData + NEXT_MONTH_PTR), 25));

	/* Rain by year, last 25 */
	printf("%s = %s\n", _RAIN_BY_YEAR, PrintRainRateSet((unsigned char *)pData, RAIN_YEAR_TOTAL, *(pData + NEXT_RAIN_YEAR_PTR), 25));

	/* ET by hour, last 24 */
	printf("%s = %s\n", _ET_BY_HOUR, PrintETByteSet((unsigned char *)pData, ET_HOUR, *(pData + NEXT_HOUR_PTR), 24));

	/* ET days total, last 24 */
	printf("%s = %s\n", _ET_DAY_TOTAL, PrintETByteSet((unsigned char *)pData, ET_DAY_TOTAL, *(pData + NEXT_DAY_PTR), 24));

        /* ET months total, last 24 */
        printf("%s = %s\n", _ET_MONTH_TOTAL, PrintETSet((unsigned char *)pData, ET_MONTH_TOTAL, *(pData + NEXT_MONTH_PTR), 24));

        /* ET years total, last 24 */
        printf("%s = %s\n", _ET_YEAR_TOTAL, PrintETSet((unsigned char *)pData, ET_YEAR_TOTAL, *(pData + NEXT_YEAR_PTR), 24));

        if (!bMetric) {
            /* ET OZ by hour, last 24 */
            printf("%s = %s\n", _ET_U_BY_HOUR, PrintETOZByteSet((unsigned char *)pData, ET_HOUR, *(pData + NEXT_HOUR_PTR), 24));

	    /* ET OZ days total, last 24 */
	    printf("%s = %s\n", _ET_U_DAY_TOTAL, PrintETOZByteSet((unsigned char *)pData, ET_DAY_TOTAL, *(pData + NEXT_DAY_PTR), 24));

            /* ET OZ months total, last 24 */
            printf("%s = %s\n", _ET_U_MONTH_TOTAL, PrintETQUARTSet((unsigned char *)pData, ET_MONTH_TOTAL, *(pData + NEXT_MONTH_PTR), 24));

            /* ET OZ years total, last 24 */
            printf("%s = %s\n", _ET_U_YEAR_TOTAL, PrintETGALSet((unsigned char *)pData, ET_YEAR_TOTAL, *(pData + NEXT_YEAR_PTR), 24));
        } else {
            /* ET OZ by hour, last 24 */
            printf("%s = %s\n", _ET_M_BY_HOUR, PrintETOZByteSet((unsigned char *)pData, ET_HOUR, *(pData + NEXT_HOUR_PTR), 24));

            /* ET OZ days total, last 24 */
            printf("%s = %s\n", _ET_M_DAY_TOTAL, PrintETOZByteSet((unsigned char *)pData, ET_DAY_TOTAL, *(pData + NEXT_DAY_PTR), 24));

            /* ET OZ months total, last 24 */
            printf("%s = %s\n", _ET_M_MONTH_TOTAL, PrintETQUARTSet((unsigned char *)pData, ET_MONTH_TOTAL, *(pData + NEXT_MONTH_PTR), 24));

            /* ET OZ years total, last 24 */
            printf("%s = %s\n", _ET_M_YEAR_TOTAL, PrintETGALSet((unsigned char *)pData, ET_YEAR_TOTAL, *(pData + NEXT_YEAR_PTR), 24));
        }

	/* Solar  by hour avg, last 24 */
	printf("%s = %s\n", _SOLAR_BY_HOUR_AVG, PrintSolarSet((unsigned char *)pData, SOLAR_HOUR_AVG, *(pData + NEXT_HOUR_PTR), 24));

	/* Solar highs by day, last 24 */
	printf("%s = %s\n", _SOLAR_HI_BY_DAYS, PrintSolarSet((unsigned char *)pData, SOLAR_DAY_HIGHS, *(pData + NEXT_DAY_PTR), 24));

	/* Solar highs by day time, last 24 */
	printf("%s = ", _SOLAR_HI_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, SOLAR_DAY_HIGH_TIMES, *(pData + NEXT_DAY_PTR), 24);

        if (! bVpro2)
	    /* Vantage Pro Solar highs by month, last 25 */
	    printf("%s = %s\n", _SOLAR_HI_BY_MONTHS, PrintSolarSet((unsigned char *)pData, SOLAR_MONTH_HIGHS, *(pData + NEXT_MONTH_PTR), 25));
        else
	    /* Vantage Pro2 Solar highs by month, last 1 */
	    printf("%s = %s\n", _SOLAR_HI_BY_MONTHS, PrintSolarSet((unsigned char *)pData, SOLAR_MONTH_HIGHS, 0, 1));

        /* Solar highs by year, last 1 */
        printf("%s = %s\n", _SOLAR_HI_BY_YEARS, PrintSolarSet((unsigned char *)pData, SOLAR_YEAR_HIGHS, 0, 1));

	/* UV by hour avg, last 24 */
	printf("%s = %s\n", _UV_BY_HOUR_AVG, PrintByteSet((unsigned char *)pData, UV_HOUR_AVG, *(pData + NEXT_HOUR_PTR), 24, _CVT_NONE));

       /* UV MEDS by hour, last 24 */
        printf("%s = %s\n", _UV_MEDS_BY_HOURS, PrintByteSet((unsigned char *)pData, UV_MEDS_HOUR, *(pData + NEXT_HOUR_PTR), 24, _CVT_NONE));

       /* UV MEDS by day, last 24 */
        printf("%s = %s\n", _UV_MEDS_BY_DAYS, PrintByteSet((unsigned char *)pData, UV_MEDS_DAY, *(pData + NEXT_DAY_PTR), 24, _CVT_NONE));

	/* UV highs by day, last 24 */
	printf("%s = %s\n", _UV_HI_BY_DAYS, PrintByteSet((unsigned char *)pData, UV_DAY_HIGHS, *(pData + NEXT_DAY_PTR), 24, _CVT_NONE));

	/* UV highs by day time, last 24 */
	printf("%s = ", _UV_HI_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, UV_DAY_HIGH_TIMES, *(pData + NEXT_DAY_PTR), 24);

        if (! bVpro2)
            /* Vantage Pro UV highs by month, last 25 */
	    printf("%s = %s\n", _UV_HI_BY_MONTHS, PrintByteSet((unsigned char *)pData, UV_MONTH_HIGHS, *(pData + NEXT_MONTH_PTR), 25, _CVT_NONE));
        else
	    /* Vantage Pro2 UV highs by month, last 1 */
	    printf("%s = %s\n", _UV_HI_BY_MONTHS, PrintByteSet((unsigned char *)pData, UV_MONTH_HIGHS, 0, 1, _CVT_NONE));

        /* UV highs by year, last 1 */
        printf("%s = %s\n", _UV_HI_BY_YEARS, PrintByteSet((unsigned char *)pData, UV_YEAR_HIGHS, 0, 1, _CVT_NONE));

       /* Leaf by hour, last 24 */
	printf("%s = %s\n", _LEAF_BY_HOURS, PrintByteSet((unsigned char *)pData, LEAF_HOUR, *(pData + NEXT_HOUR_PTR), 24, _CVT_NONE));

	/* Leaf highs by day, last 24 */
	printf("%s = %s\n", _LEAF_HI_BY_DAYS, PrintByteSet((unsigned char *)pData, LEAF_DAY_HIGHS, *(pData + NEXT_DAY_PTR), 24, _CVT_NONE));

	/* Leaf highs by day, time, last 24 */
	printf("%s = ", _LEAF_HI_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, LEAF_DAY_HIGH_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* Leaf lows by day, last 24 */
	printf("%s = %s\n", _LEAF_LO_BY_DAYS, PrintByteSet((unsigned char *)pData, LEAF_DAY_LOWS, *(pData + NEXT_DAY_PTR), 24, _CVT_NONE));

	/* Leaf lows by day, time, last 24 */
	printf("%s = ", _LEAF_LO_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, LEAF_DAY_LOW_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* Leaf lows by month, last 1 */
	printf("%s = %s\n", _LEAF_LO_BY_MONTHS, PrintByteSet((unsigned char *)pData, LEAF_MONTH_LOWS, 0, 1, _CVT_NONE));

        if (! bVpro2)
	    /* Vantage Pro Leaf highs by month, last 25 */
	    printf("%s = %s\n", _LEAF_HI_BY_MONTHS, PrintByteSet((unsigned char *)pData, LEAF_MONTH_HIGHS, *(pData + NEXT_MONTH_PTR), 25, _CVT_NONE));
        else
	    /* Vantage Pro2 Leaf highs by month, last 1 */
	    printf("%s = %s\n", _LEAF_HI_BY_MONTHS, PrintByteSet((unsigned char *)pData, LEAF_MONTH_HIGHS, 0, 1, _CVT_NONE));

	/* Soil by hour, last 24 */
	printf("%s = %s\n", _SOIL_BY_HOURS, PrintByteSet((unsigned char *)pData, SOIL_HOUR, *(pData + NEXT_HOUR_PTR), 24, _CVT_NONE));

	/* Soil highs by day, last 24 */
	printf("%s = %s\n", _SOIL_HI_BY_DAYS, PrintByteSet((unsigned char *)pData, SOIL_DAY_HIGHS, *(pData + NEXT_DAY_PTR), 24, _CVT_NONE));

	/* Soil highs by day, time, last 24 */
	printf("%s = ", _SOIL_HI_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, SOIL_DAY_HIGH_TIMES, *(pData + NEXT_DAY_PTR), 24);

	/* Soil lows by day, last 24 */
	printf("%s = %s\n", _SOIL_LO_BY_DAYS, PrintByteSet((unsigned char *)pData, SOIL_DAY_LOWS, *(pData + NEXT_DAY_PTR), 24, _CVT_NONE));

	/* Soil lows by day, time, last 24 */
	printf("%s = ", _SOIL_LO_BY_DAYS_TIME);
	PrintTimeSet((unsigned char *)pData, SOIL_DAY_LOW_TIMES, *(pData + NEXT_DAY_PTR), 24);

        if (! bVpro2)
	    /* Soil lows by month, last 25 */
	    printf("%s = %s\n", _SOIL_LO_BY_MONTHS, PrintByteSet((unsigned char *)pData, SOIL_MONTH_LOWS, *(pData + NEXT_MONTH_PTR), 25, _CVT_NONE));
       else
           /* Soil lows by month, last 25 */
	    printf("%s = %s\n", _SOIL_LO_BY_MONTHS, PrintByteSet((unsigned char *)pData, SOIL_MONTH_LOWS, 0, 1, _CVT_NONE));

        if (! bVpro2)
	    /* Vantage Pro Soil highs by month, last 25 */
	    printf("%s = %s\n", _SOIL_HI_BY_MONTHS, PrintByteSet((unsigned char *)pData, SOIL_MONTH_HIGHS, *(pData + NEXT_MONTH_PTR), 25, _CVT_NONE));
        else
	    /* Vantage Pro2 Soil highs by month, last 1 */
	    printf("%s = %s\n", _SOIL_HI_BY_MONTHS, PrintByteSet((unsigned char *)pData, SOIL_MONTH_HIGHS, 0, 1, _CVT_NONE));

	/* graph data capture time stamp */
        printf("%s = %s (%llu)\n", _GR_CAPTURE_TIMESTAMP, grDateTimeCT, (unsigned long long)ttgr);

}	
	


/*--------------------------------------------------------------------------
	PrintTempSet24
	Prints a comma separated list of temperature values.
----------------------------------------------------------------------------*/
char* PrintTempSet24(BYTE *pData, int ptrOffset, BYTE rrOffset, int valOffset)
{
	int i, nEntry;
	signed short sValue;
	static char list[125];
	char value[5];

	nEntry = ptrOffset + rrOffset;
	list[0] = '\0';

	for(i = 0; i < 24; i++) {			/* get data set in proper order */
		sValue = *(pData + nEntry);
                if(sValue == 255)
                    sprintf(value, "n/a");
                else {
                    if (!bMetric)
                        sprintf(value, "%i", sValue - valOffset);
                    else
		        sprintf(value, "%i", cvtMetricINT(sValue - valOffset, _CVT_TEMP));
                }
                if (i < 23 && sizeof(value)-1 < strlen(value)+1) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                } else
                    if (i < 23) strcat(value, ",");
                if (sizeof(list)-1 < strlen(list)+strlen(value)) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                } else
		    strcat(list, value);
		if(++nEntry >= (ptrOffset + 24)) nEntry = ptrOffset;	/* reset to beginning */
	}	
	return list;
}



/*--------------------------------------------------------------------------
	PrintTempSet25
	Prints a comma separated list of temperature values from a 25 data set.
    The Davis EEPROM keeps 24+1 records in some data sets, which the extra one
	is the current value (month or year). We discard this to keep all graphs 
    consistent at 24. In all cases, you can get this data from the highs/lows
    data set anyway.
----------------------------------------------------------------------------*/
void PrintTempSet25(BYTE *pData, int ptrOffset, BYTE rrOffset, int valOffset)
{
	int i, nEntry;
	signed short sValue;

	nEntry = ptrOffset + rrOffset;

	for(i = 0; i < 25; i++) {			/* get data set in proper order */
		sValue = *(pData + nEntry);
		if(++nEntry >= (ptrOffset + 25))
			nEntry = ptrOffset;		/* reset to beginning */
		else {
                        if(sValue == 255)
                            printf("n/a");
                        else {
                            if (!bMetric)
                                printf("%i", sValue - valOffset);
                            else
			        printf("%i", cvtMetricINT(sValue - valOffset, _CVT_TEMP)); /* subtract valOffset to get temp integer */
                        }
			if(i < 24) printf(",");		/* no comma on last entry */
		}
	}	
	printf("\n");
	
}


/*--------------------------------------------------------------------------
	PrintETByteSet
	Prints a comma separated list of byte values.
----------------------------------------------------------------------------*/
char* PrintETByteSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize)
{
	int i, nEntry;
	BYTE yValue;
	static char list[175];
	char value[7];

	nEntry = ptrOffset + rrOffset;
	list[0] = '\0';

	for(i = 0; i < nSetSize; i++) {				/* get data set in proper order */
		yValue = *(pData + nEntry);
                if(yValue == 255)
                    sprintf(value, "n/a");
                else {
                    if (!bMetric)
                        sprintf(value, "%.3f", yValue / 100.0);
                    else
                        sprintf(value, "%.3f", cvtMetricFLOAT(yValue / 100.0, _CVT_RAIN));
                }
                if (i < nSetSize-1 && sizeof(value)-1 < strlen(value)+1) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                } else
                    if (i < nSetSize-1) strcat(value, ",");
                if (sizeof(list)-1 < strlen(list)+strlen(value)) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                }
		if(++nEntry >= (ptrOffset + nSetSize)) {		/* if next is past block */
			nEntry = ptrOffset;			/* reset to first */
			if(nSetSize != 25) strcat(list, value);	/* if set of 25 don't use it because it's current value */
		} else
			strcat(list, value);
	}
	return list;
}

/*--------------------------------------------------------------------------
	PrintETOZByteSet
	Prints a comma separated list of byte values.
----------------------------------------------------------------------------*/
char* PrintETOZByteSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize)
{
	int i, nEntry;
	BYTE yValue;
	static char list[225];
	char value[9];

	nEntry = ptrOffset + rrOffset;
	list[0] = '\0';

	for(i = 0; i < nSetSize; i++) {				/* get data set in proper order */
		yValue = *(pData + nEntry);
                if(yValue == 255)
                    sprintf(value, "n/a");
                else {
                    if (!bMetric)
                        sprintf(value, "%.3f", (yValue / 100.0) * RAIN_OZINCH);
                    else
                        sprintf(value, "%.3f", cvtMetricFLOAT((yValue / 100.0) * RAIN_OZINCH, _CVT_ET_OZ));
                }
                if (i < nSetSize-1 && sizeof(value)-1 < strlen(value)+1) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                } else
                    if (i < nSetSize-1) strcat(value, ",");
                if (sizeof(list)-1 < strlen(list)+strlen(value)) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                }
		if(++nEntry >= (ptrOffset + nSetSize)) {
			nEntry = ptrOffset;			/* reset to beginning */
			if(nSetSize != 25) strcat(list, value);
		} else
			strcat(list, value);
	}
	return list;
}

/*--------------------------------------------------------------------------
	PrintETSet
	Prints a comma separated list of ET values from a 25 data set.
	The Davis EEPROM keeps 24+1 records in some data sets, which the extra one
	is the current value (month or year). We discard this to keep all graphs
	consistent at 24. In all cases, you can get this data from the realtime
	data set anyway.
	NOTE: For 
	ET_YEAR_TOTAL START + 3017 // 25 || 2
	and maybe others
	START + OFFSET + (25 *2) contains the current years total so don't
	do 25 years only do 24 years
----------------------------------------------------------------------------*/
char* PrintETSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize)
{
	int i, nEntry;
	WORD wValue;
	static char list[225];
	char value[9];

	nEntry = ptrOffset + (rrOffset * 2);
	list[0] = '\0';
	for(i = 0; i < nSetSize; i++) {			 /* get data set in proper order */
		wValue = *( (WORD*)(pData + nEntry) );
		if (bSwapEndian) wValue = SwapEndian(wValue);
// don't think so		if(wValue == 32767) wValue = 0;		 /* zero out illegal entries */
//fprintf(stderr, "sizeof(list) %i strlen(list) %i strlen(value) %i strlen(list)+strlen(value) %i value %s\n", (int)sizeof(list), (int)strlen(list), (int)strlen(value), (int)(strlen(list)+strlen(value)), value);
                if(wValue == 65535 || (unsigned)wValue == 0x7fff)	/* illegal data */
                    sprintf(value, "n/a");
                else {
                    if (!bMetric)
                        sprintf(value, "%.3f", wValue / 100.0);
                    else
		        sprintf(value, "%.3f", cvtMetricFLOAT(wValue / 100.0, _CVT_RAIN));
                }
		if (i < nSetSize-1 && sizeof(value)-1 < strlen(value)+1) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                } else
                    if (i < nSetSize-1) strcat(value, ",");
                if (sizeof(list)-1 < strlen(list)+strlen(value)) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                }
		nEntry += 2;
		if(nEntry >= ptrOffset + (nSetSize*2)) {
			nEntry = ptrOffset;		       /* reset to beginning */
			if(nSetSize != 25) strcat(list, value);
		} else
			strcat(list, value);
	}
	return list;
}

/*--------------------------------------------------------------------------
        PrintETQUARTSet
        Prints a comma separated list of ET values from a 25 data set.
        The Davis EEPROM keeps 24+1 records in some data sets, which the extra one
        is the current value (month or year). We discard this to keep all graphs
        consistent at 24. In all cases, you can get this data from the realtime
        data set anyway.
        NOTE: For 
        ET_YEAR_TOTAL START + 3017 // 25 || 2
        and maybe others
        START + OFFSET + (25 *2) contains the current years total so don't
        do 25 years only do 24 years
----------------------------------------------------------------------------*/
char* PrintETQUARTSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize)
{
        int i, nEntry;
        WORD wValue;
        static char list[200];
        char value[8];

        nEntry = ptrOffset + (rrOffset * 2);
        list[0] = '\0';
        for(i = 0; i < nSetSize; i++) {                         /* get data set in proper order */
                wValue = *( (WORD*)(pData + nEntry) );
                if (bSwapEndian) wValue = SwapEndian(wValue);
// don't think so             if(wValue == 32767) wValue = 0;                 /* zero out illegal entries */
//fprintf(stderr, "sizeof(list) %i strlen(list) %i strlen(value) %i strlen(list)+strlen(value) %i value %s\n", (int)sizeof(list), (int)strlen(list), (int)strlen(value), (int)(strlen(list)+strlen(value)), value);
                if(wValue == 65535 || (unsigned)wValue == 0x7fff)	/* illegal data */
                    sprintf(value, "n/a");
                else {
                    if (!bMetric)
                        sprintf(value, "%.3f", ((wValue / 100.0) * RAIN_OZINCH) / 32);
                    else
		        sprintf(value, "%.3f", cvtMetricFLOAT(((wValue / 100.0) * RAIN_OZINCH) / 32, _CVT_ET_QT));
                }
		if (i < nSetSize-1 && sizeof(value)-1 < strlen(value)+1) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                } else
                    if (i < nSetSize-1) strcat(value, ",");
                if (sizeof(list)-1 < strlen(list)+strlen(value)) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                }
                nEntry += 2;
                if(nEntry >= ptrOffset + (nSetSize*2)) {
                        nEntry = ptrOffset;                       /* reset to beginning */
                        if(nSetSize != 25) strcat(list, value);
                } else
                        strcat(list, value);
        }
        return list;
}

/*--------------------------------------------------------------------------
        PrintETGALSet
        Prints a comma separated list of ET values from a 25 data set.
        The Davis EEPROM keeps 24+1 records in some data sets, which the extra one
        is the current value (month or year). We discard this to keep all graphs
        consistent at 24. In all cases, you can get this data from the realtime
        data set anyway.
        NOTE: For 
        ET_YEAR_TOTAL START + 3017 // 25 || 2
        and maybe others
        START + OFFSET + (25 *2) contains the current years total so don't
        do 25 years only do 24 years
----------------------------------------------------------------------------*/
char* PrintETGALSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize)
{
        int i, nEntry;
        WORD wValue;
        static char list[200];
        char value[8];

        nEntry = ptrOffset + (rrOffset * 2);
        list[0] = '\0';
        for(i = 0; i < nSetSize; i++) {                         /* get data set in proper order */
                wValue = *( (WORD*)(pData + nEntry) );
                if (bSwapEndian) wValue = SwapEndian(wValue);
// don't think so              if(wValue == 32767) wValue = 0;                 /* zero out illegal entries */
                if(wValue == 65535 || (unsigned)wValue == 0x7fff)	/* illegal data */
                    sprintf(value, "n/a");
                else {
                    if (!bMetric)
                        sprintf(value, "%.3f", ((wValue / 100.0) * RAIN_OZINCH) / 128);
                    else
		        sprintf(value, "%.3f", cvtMetricFLOAT(((wValue / 100.0) * RAIN_OZINCH) / 128, _CVT_ET_GL));
                }
		if (i < nSetSize-1 && sizeof(value)-1 < strlen(value)+1) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                } else
                    if (i < nSetSize-1) strcat(value, ",");
                if (sizeof(list)-1 < strlen(list)+strlen(value)) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                }
                nEntry += 2;
                if(nEntry >= ptrOffset + (nSetSize*2)) {
                        nEntry = ptrOffset;                       /* reset to beginning */
                        if(nSetSize != 25) strcat(list, value);
                } else
                        strcat(list, value);
        }
        return list;
}

/*--------------------------------------------------------------------------
	PrintSolarSet
	Prints a comma separated list of solar values.
----------------------------------------------------------------------------*/
char* PrintSolarSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize)
{
	int i, nEntry;
	WORD wValue;
	static char list[150];
	char value[6];

	nEntry = ptrOffset + (rrOffset * 2);
	list[0] = '\0';
	for(i = 0; i < nSetSize; i++) {				/* get data set in proper order */
		wValue = *( (WORD*)(pData + nEntry) );
                if (bSwapEndian) wValue = SwapEndian(wValue);
                if((unsigned)wValue == 0x7fff || (unsigned)wValue == 0xffff)	/* illegal data */
                    sprintf(value, "n/a");
                else
		    sprintf(value, "%d", wValue);
		if (i < nSetSize-1 && sizeof(value)-1 < strlen(value)+1) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                } else
                    if (i < nSetSize-1) strcat(value, ",");
                if (sizeof(list)-1 < strlen(list)+strlen(value)) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                }
		nEntry += 2;
		if(nEntry >= ptrOffset + (nSetSize*2)) {
			nEntry = ptrOffset;			/* reset to beginning */
			if(nSetSize != 25) strcat(list, value);
		} else
			strcat(list, value);
	}
	return list;
}


/*--------------------------------------------------------------------------
	PrintByteSet
	Prints a comma separated list of byte values.
----------------------------------------------------------------------------*/
char* PrintByteSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize, int cvtType)
{
	int i, nEntry;
	BYTE yValue;
        static char list[125];
        char value[5];

	nEntry = ptrOffset + rrOffset;
	list[0] = '\0';

	for(i = 0; i < nSetSize; i++) {			/* get data set in proper order */
		yValue = *(pData + nEntry);
                if(yValue == 255)			/* force to n/a */
                    sprintf(value, "n/a");
                else {
                    if (!bMetric)
                        sprintf(value, "%d", yValue);
                    else
		        sprintf(value, "%d", cvtMetricBYTE(yValue,cvtType));
                }
		if (i < nSetSize-1 && sizeof(value)-1 < strlen(value)+1) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                } else
                    if (i < nSetSize-1) strcat(value, ",");		/* no comma on last entry */
                if (sizeof(list)-1 < strlen(list)+strlen(value)) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                }
		if(++nEntry >= (ptrOffset + nSetSize)) {
			nEntry = ptrOffset;			/* reset to beginning */
			if(nSetSize != 25) strcat(list, value);
		} else
			strcat(list, value);
	}	
	return list;
}



/*--------------------------------------------------------------------------
	PrintTimeSet
	Prints the comma separated list of times.
----------------------------------------------------------------------------*/
void PrintTimeSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize)
{
	int i, nEntry;
	WORD wValue;

	nEntry = ptrOffset + (rrOffset * 2);

	for(i = 0; i < nSetSize; i++) {			/* get data set in proper order */
		wValue = *( (WORD*)(pData + nEntry) );
                if (bSwapEndian) wValue = SwapEndian(wValue);
                if(wValue == 65535 || (unsigned)wValue == 0x7fff) wValue = 0;	/* check for empty entry */
		printf("%s", TimeConvert(wValue));
		if(i < nSetSize - 1) printf(",");		/* no comma on last entry */
		nEntry += 2;
		if(nEntry >= ptrOffset + (nSetSize*2)) nEntry = ptrOffset;		/* reset to beginning */
	}	
	printf("\n");
}	
	
	
	
/*--------------------------------------------------------------------------
	PrintBarSet
	Prints a comma separated list of barometers.
----------------------------------------------------------------------------*/
char* PrintBarSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize)
{
	int i, nEntry;
	WORD wValue;
	static char list[200];
	char value[8];

	nEntry = ptrOffset + (rrOffset * 2);
	list[0] = '\0';

	for(i = 0; i < nSetSize; i++) {					/* get data set in proper order */
		wValue = *( (WORD*)(pData + nEntry) );
                if (bSwapEndian) wValue = SwapEndian(wValue);
// don't think so		if(wValue == 65535) wValue = 0;				/* zero out illegal entries */
                if((unsigned)wValue == 0x7fff)				/* check for empty entry */
                    sprintf(value, "n/a");
                else {
                    if (!bMetric)
		        sprintf(value, "%.3f", wValue / 1000.0);		/* print to 3 decimal places - cjb */
                    else
                        sprintf(value, "%.1f", cvtMetricFLOAT(wValue / 1000.0, _CVT_BARO));
                }
		if (i < nSetSize-1 && sizeof(value)-1 < strlen(value)+1) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                } else
                    if (i < nSetSize-1) strcat(value, ",");
                if (sizeof(list)-1 < strlen(list)+strlen(value)) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                }
		nEntry += 2;
		if(nEntry >= ptrOffset + (nSetSize*2)) {
			nEntry = ptrOffset;				/* reset to beginning */
			if(nSetSize != 25) strcat(list, value);
	
		} else
			strcat(list, value);
	}
	return list;
}	



/*--------------------------------------------------------------------------
	PrintRainRateSet
	Prints a comma separated list of rain rates.
	NOTE: For 
	RAIN_YEAR_TOTAL START + 2868 // 25 || 2
	START + OFFSET + (25 *2) contains the current years total so don't
	do 25 years only do 24 years
----------------------------------------------------------------------------*/
char* PrintRainRateSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize)
{
	int i, nEntry;
	WORD wValue;
	static char list[225];
	char value[9];

	nEntry = ptrOffset + (rrOffset * 2);
	list[0] = '\0';

	for(i = 0; i < nSetSize; i++) {				/* get data set in proper order */
		wValue = *( (WORD*)(pData + nEntry) );
                if (bSwapEndian) wValue = SwapEndian(wValue);
//                if (bVpro2)
// don't think so     if (wValue > 32767) wValue = wValue - 32768;	/* for some reason looks like VP2's use this offset */
                if(wValue == 65535 ||(bVpro2 && (unsigned)wValue == 0x7fff))	/* check for empty entry */
                    sprintf(value, "n/a");
                else {
                    if (!bMetric)
		        sprintf(value, "%.2f", wValue / 100.0);
                    else
                        sprintf(value, "%.2f", cvtMetricFLOAT(wValue / 100.0, _CVT_RAIN));
                }
		if (i < nSetSize-1 && sizeof(value)-1 < strlen(value)+1) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                } else
                    if (i < nSetSize-1) strcat(value, ",");
                if (sizeof(list)-1 < strlen(list)+strlen(value)) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                }
		nEntry += 2;
		if(nEntry >= ptrOffset + (nSetSize*2)) {
			nEntry = ptrOffset;			/* reset to beginning */
			if(nSetSize != 25) strcat(list, value);
		} else
			strcat(list, value);
	}
	return list;
}	


/*--------------------------------------------------------------------------
	PrintRainRateByteSet
	Prints a comma separated list of rain rates.
----------------------------------------------------------------------------*/
char* PrintRainRateByteSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize)
{
	int i, nEntry;
	BYTE yValue;
	static char list[150];
	char value[7];

	nEntry = ptrOffset + rrOffset;
	list[0] = '\0';

	for(i = 0; i < nSetSize; i++) {				/* get data set in proper order */
		yValue = *(pData + nEntry);
                if(yValue == 255)				/* check for empty entry */
                    sprintf(value, "n/a");
                else {
                    if (!bMetric)
                        sprintf(value, "%.2f", yValue / 100.0);
                    else
		        sprintf(value, "%.2f", cvtMetricFLOAT(yValue / 100.0,_CVT_RAIN));
                }
		if (i < nSetSize-1 && sizeof(value)-1 < strlen(value)+1) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                } else
                    if (i < nSetSize-1) strcat(value, ",");
                if (sizeof(list)-1 < strlen(list)+strlen(value)) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                }
		if(++nEntry >= (ptrOffset + nSetSize)) {
			nEntry = ptrOffset;			/* reset to beginning */
			if(nSetSize != 25) strcat(list, value);
		} else
			strcat(list, value);
	}
	return list;
}


/*--------------------------------------------------------------------------
        PrintDateDMP
	Reformat the bit pattern to match what PrintDate expects.
	Input: Bit 15 to bit 9 year , bit 8 to 5 is the month, bit 4 to 0 is
	the day. Year offset = 2000.
----------------------------------------------------------------------------*/
char* PrintDateDMP(WORD wDate)
{
	WORD nDate;
	char* myDate;

	nDate = ((wDate << 7) & 0xFF80) + ((wDate >> 9) & 0x007F); 
//	if (bVerbose) fprintf(stderr, "wDate 0x%04x nDate 0x%04x\n",wDate,nDate);

	myDate = PrintDate(nDate);

	return myDate;
}

/*--------------------------------------------------------------------------
	PrintDate
	Prints an encoded date value in the form '23-JUN-2004'. If you don't
	like this format, here is where you need to do your hacking.
	Input: Bit 15 to bit 12 is the month, bit 11 to bit 7 is the day and 
	bit 6 to bit 0. Year offset = 2000.
----------------------------------------------------------------------------*/
char* PrintDate(WORD wDate)
{
	WORD w, myDay;
	static char myDate[12];
	char myMonth[4];
	
	if((unsigned)wDate == 0x7fff || (unsigned)wDate == 0xffff || wDate == 0) {	/* ignore missing data */
		sprintf(myDate, "n/a");
		return myDate;
	}	
	w = (wDate & 0x0f00) >> 7;		/* get msb of date */
	if(wDate & 0x0080) ++w;			/* do crappy Davis format incr */
	myDay = w;
//	myDay = (wDate & 0x0f80) >> 7; 
	
	w = (wDate & 0xf000) >> 12;		/* strip out month */
	switch(w) {
		case 1: sprintf(myMonth, "%s", "JAN"); break;
		case 2: sprintf(myMonth, "%s", "FEB"); break;
		case 3: sprintf(myMonth, "%s", "MAR"); break;
		case 4: sprintf(myMonth, "%s", "APR"); break;
		case 5: sprintf(myMonth, "%s", "MAY"); break;
		case 6: sprintf(myMonth, "%s", "JUN"); break;
		case 7: sprintf(myMonth, "%s", "JUL"); break;
		case 8: sprintf(myMonth, "%s", "AUG"); break;
		case 9: sprintf(myMonth, "%s", "SEP"); break;
		case 10: sprintf(myMonth, "%s", "OCT"); break;
		case 11: sprintf(myMonth, "%s", "NOV"); break;
		case 12: sprintf(myMonth, "%s", "DEC"); break;
		default: sprintf(myMonth, "%s", "???"); break;
	}		
	
        sprintf(myDate, "%02d-%3s-20%02d", myDay, myMonth, (wDate & 0x000f) );

        return myDate;
}


/*--------------------------------------------------------------------------
	PrintDateSet
	Prints a comma separated list of date items.
----------------------------------------------------------------------------*/
char* PrintDateSet(BYTE *pData, int ptrOffset, BYTE rrOffset, int nSetSize)
{
	int i, nEntry;
	WORD wValue;
	static char list[300];
	char value[13];

	nEntry = ptrOffset + (rrOffset * 2);
	list[0] = '\0';

	for(i = 0; i < nSetSize; i++) {				/* get data set in proper order */
		wValue = *( (WORD*)(pData + nEntry) );
                if (bSwapEndian) wValue = SwapEndian(wValue);
		sprintf(value, "%s", PrintDate(wValue));
		if (i < nSetSize-1 && sizeof(value)-1 < strlen(value)+1) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                } else
                    if (i < nSetSize-1) strcat(value, ",");
                if (sizeof(list)-1 < strlen(list)+strlen(value)) {
                    sprintf(list,_OVERFLOW_MSG);
                    return list;
                }
		nEntry += 2;
		if(nEntry >= ptrOffset + (nSetSize*2)) {
			nEntry = ptrOffset;			/* reset to beginning */
			if(nSetSize != 25) strcat(list, value);
		} else
			strcat(list, value);
	}
	return list;
}



/*--------------------------------------------------------------------------
        TimeRef10Min
	Builds the 10 minute time reference
----------------------------------------------------------------------------*/
char* TimeRef10Min(time_t tt)
{
	struct tm stm;
	int i;
	static char list[192];
	char value[8];

	if (bVerbose) fprintf(stderr, "Building TimeRef10Min\n");

	list[0] = '\0';
	
	/* 10min reference */
	stm = *localtime(&tt);
	stm.tm_min = (stm.tm_min/10)*10;		/* round to nearest 10min incr */
	stm.tm_min -= (10 * 24);
	tt = mktime(&stm);
	for(i = 23; i > -1; i--)
	{
		stm.tm_min += 10;
		tt = mktime(&stm);
		sprintf(value, "%s", TimeConvert( (stm.tm_hour * 100)+stm.tm_min) );
		strcat(list, value);
		if(i) strcat(list, ",");
	}
	return list;
}


	
/*--------------------------------------------------------------------------
        TimeRef15Min
        Builds the 15 minutes time reference
----------------------------------------------------------------------------*/
char* TimeRef15Min(time_t tt)
{
        struct tm stm;
        int i;
        static char list[192];
        char value[8];

/* these debugs aren't needed at the moment */
//	if (bVerbose) fprintf(stderr, "Building TimeRef15Min\n");

        list[0] = '\0';

	/* 15min reference */
	stm = *localtime(&tt);
	stm.tm_min = (stm.tm_min/15)*15;		/* round to nearest 15min incr */
	stm.tm_min -= (15 * 24);
	tt = mktime(&stm);
	for(i = 23; i > -1; i--)
	{
		stm.tm_min += 15;
		tt = mktime(&stm);
		sprintf(value, "%s", TimeConvert( (stm.tm_hour * 100)+stm.tm_min) );
		strcat(list, value);
		if(i) strcat(list, ",");
	}
        return list;
}



/*--------------------------------------------------------------------------
        TimeRefHours
        Builds the Hours time reference
----------------------------------------------------------------------------*/
char* TimeRefHours(time_t tt)
{
        struct tm stm;
        int i;
        static char list[192];
        char value[8];

        list[0] = '\0';

	/* hours reference */
        stm = *localtime(&tt);
	stm.tm_hour -= 24;
	tt = mktime(&stm);
	for(i = 23; i > -1; i--)
	{
		stm.tm_hour++;
		tt = mktime(&stm);
		sprintf(value, "%s", TimeConvert(stm.tm_hour * 100));
		strcat(list, value);
		if(i) strcat(list, ",");
	}
        return list;
}



/*--------------------------------------------------------------------------
        TimeRefDays
        Builds the Days time reference
----------------------------------------------------------------------------*/
char* TimeRefDays(time_t tt)
{
        struct tm stm;
        int i;
        static char list[144];
        char value[6];

        list[0] = '\0';

	/* days reference */
        stm = *localtime(&tt);
        stm.tm_mday -= 25;
        tt = mktime(&stm);
	for(i = 24; i; i--)
	{
		stm.tm_mday++;
		tt = mktime(&stm);
		sprintf(value, "%02d/%02d", stm.tm_mon+1, stm.tm_mday);
		strcat(list, value);
		if(i > 1) strcat(list, ",");
	}
        return list;
}



/*--------------------------------------------------------------------------
        TimeRefMonths
        Builds the Months time reference
----------------------------------------------------------------------------*/
char* TimeRefMonths(time_t tt)
{
        struct tm stm;
        int i;
        static char list[144];
        char value[6];

        list[0] = '\0';

	/* Don't keep getting the local time again each time through the    */
	/* FOR loop like Joe did, get it once, set it back 25 months by     */
	/* using current month as that month then increment it each time    */
        /* through the loop. Now I understand why I'd see months duplicated */
	/* in the list at month end when the month changed while this loop  */
	/* was executing. cjb 22-oct-2007 updated 31-may-2011               */

	/* months reference */
	stm = *localtime(&tt);
	for(i = 24; i; i--)
	{
		switch(stm.tm_mon)
		{
			case 0: sprintf(value, "%3s", "JAN"); break;
			case 1: sprintf(value, "%3s", "FEB"); break;
			case 2: sprintf(value, "%3s", "MAR"); break;
			case 3: sprintf(value, "%3s", "APR"); break;
			case 4: sprintf(value, "%3s", "MAY"); break;
			case 5: sprintf(value, "%3s", "JUN"); break;
			case 6: sprintf(value, "%3s", "JUL"); break;
			case 7: sprintf(value, "%3s", "AUG"); break;
			case 8: sprintf(value, "%3s", "SEP"); break;
			case 9: sprintf(value, "%3s", "OCT"); break;
			case 10: sprintf(value, "%3s", "NOV"); break;
			case 11: sprintf(value, "%3s", "DEC"); break;
			default: sprintf(value, "%3s", "???"); break;
		}
		stm.tm_mon++;
		if (stm.tm_mon > 11) stm.tm_mon = 0;
		strcat(list, value);
		if(i > 1) strcat(list, ",");
	}
        return list;
}



/*--------------------------------------------------------------------------
        TimeRefYears
        Builds the Years time reference
	The Davis EEPROM keeps 24+1 records in some data sets, which the extra one
	is the current value (month or year). We discard this to keep all graphs
	consistent at 24. In all cases, you can get this data from the realtime
	data set anyway.
	NOTE: For 
        ET_YEAR_TOTAL START + 3017 // 25 || 2
	RAIN_YEAR_TOTAL START + 2868 // 25 || 2
	and maybe others
	START + OFFSET + (25 *2) contains the current years total so don't
	do 25 years only do 24 years 
----------------------------------------------------------------------------*/
char* TimeRefYears(time_t tt)
{
        struct tm stm;
        int i;
        static char list[144];
        char value[6];

        list[0] = '\0';

	/* years reference */
        stm = *localtime(&tt);
//	stm.tm_year -= 26;
	stm.tm_year -= 25;
        tt = mktime(&stm);
//	for(i = 25; i; i--)
	for(i = 24; i; i--)
	{
		stm.tm_year++;
		tt = mktime(&stm);
		sprintf(value, "%04d", stm.tm_year + 1900);
		strcat(list, value);
		if(i > 1) strcat(list, ",");
	}
	return list;
}



/*--------------------------------------------------------------------------
	PrintTimeRef
	Prints time references for use in graph axis generations.
----------------------------------------------------------------------------*/
void PrintTimeRef(void)
{
	time_t mtt;

	/* get the time once and use it through all these calculations      */
	/* otherwise the day could change the month could change the year   */
	/* could change in the middle of outputting any list or in between  */
	/* outputting a TimeRef list as I've seen happen but never realized */
	/* why. See month explaination for more info. cjb 22-oct-2007       */

	if (bVerbose) fprintf(stderr, "Printing TimeRef\n");

	time(&mtt);

	if (bVerbose) fprintf(stderr, "Got time(&mtt)\n");

	printf("%s = %s\n", _TIME_REF_10MIN, TimeRef10Min(mtt));
	printf("%s = %s\n", _TIME_REF_15MIN, TimeRef15Min(mtt));
	printf("%s = %s\n", _TIME_REF_HOURS, TimeRefHours(mtt));
	printf("%s = %s\n", _TIME_REF_DAYS, TimeRefDays(mtt));
	printf("%s = %s\n", _TIME_REF_MONTHS, TimeRefMonths(mtt));
	printf("%s = %s\n", _TIME_REF_YEARS, TimeRefYears(mtt));

}



/*--------------------------------------------------------------------------
	ForecastString
	Returns the forecast string for the given forecast rule. I used a brute
	force approach here based on the research work done by www.oftedahl.no/.
	The Davis forecast rule byte is actually a compound number where certain
	bits or formulae can be applied to obtain sets of substrings. However, I
	haven't been able to figure out the math behind this, and it remains
	undocumented by Davis. Hence the brute force approach.
----------------------------------------------------------------------------*/
char* ForecastString(WORD wRule)
{
	char *ptr;
	
	if(wRule > 194)
		wRule = 194;				/* trap out of bounds */

	ptr = szForeStrings;
	while(wRule--) {				/* walk thru the null terminators */
		while(*ptr++)
			;
	}		
		
		
	return ptr;
	
}

/*--------------------------------------------------------------------------
        rcdConvertEndian
	Convert all the Big Endian to Little Endian in the RTDATA struct
----------------------------------------------------------------------------*/
void rcdConvertEndian(void)
{
//	rcd.yAlarmOut = SwapEndian( rcd.yAlarmOut );		/* yAlarmOut WORD */
	rcd.wWindDir = SwapEndian( rcd.wWindDir );		/* wWindDir WORD */
	rcd.wSunset = SwapEndian( rcd.wSunset );		/* wSunset WORD */
	rcd.wSunrise = SwapEndian( rcd.wSunrise );		/* wSunrise WORD */
	rcd.wStormStart = SwapEndian( rcd.wStormStart );	/* wStormStart WORD */
	rcd.wStormRain = SwapEndian( rcd.wStormRain );		/* wStormRain WORD */
	rcd.wSolarRad = SwapEndian( rcd.wSolarRad );		/* wSolarRad WORD */
	rcd.wRainYear = SwapEndian( rcd.wRainYear );		/* wRainYear WORD */
	rcd.wRainRate = SwapEndian( rcd.wRainRate );		/* wRainRate WORD */
	rcd.wRainMonth = SwapEndian( rcd.wRainMonth );		/* wRainMonth WORD */
	rcd.wRainDay = SwapEndian( rcd.wRainDay );		/* wRainDay WORD */
	rcd.wOutsideTemp = SwapEndian( rcd.wOutsideTemp );	/* wOutsideTemp WORD */
	rcd.wNextRec = SwapEndian( rcd.wNextRec );		/* wNextRec WORD */
	rcd.wInsideTemp = SwapEndian( rcd.wInsideTemp );	/* wInsideTemp WORD */
	rcd.wETYear = SwapEndian( rcd.wETYear );		/* wETYear WORD */
	rcd.wETMonth = SwapEndian( rcd.wETMonth );		/* wETMonth WORD */
	rcd.wETDay = SwapEndian( rcd.wETDay );			/* wETDay WORD */
	rcd.wBattLevel = SwapEndian( rcd.wBattLevel );		/* wBattLevel WORD */
	rcd.wBarometer = SwapEndian( rcd.wBarometer );		/* wBarometer WORD */
/*	rcd.yAlarmSL = SwapEndian( rcd.yAlarmSL );		* yAlarmSL DWORD */
/*	rcd.wSoilMoist = SwapEndian( rcd.wSoilMoist );		* wSoilMoist DWORD */
/*	rcd.wLeafWet = SwapEndian( rcd.wLeafWet );		* wLeafWet DWORD */
}


/*--------------------------------------------------------------------------
        hldConvertEndian
        Convert all the Big Endian to Little Endian in the HLDATA struct
----------------------------------------------------------------------------*/
void hldConvertEndian(void)
{
	hld.wWindHiTime = SwapEndian( hld.wWindHiTime );		/* wWindHiTime WORD */
	hld.wUVHiTime = SwapEndian( hld.wUVHiTime );			/* wUVHiTime WORD */
	hld.wTempLoYear = SwapEndian( hld.wTempLoYear );		/* wTempLoYear WORD */
	hld.wTempLoTime = SwapEndian( hld.wTempLoTime );		/* wTempLoTime WORD */
	hld.wTempLoMonth = SwapEndian( hld.wTempLoMonth );		/* wTempLoMonth WORD */
	hld.wTempLoDay = SwapEndian( hld.wTempLoDay );			/* wTempLoDay WORD */
	hld.wTempHiYear = SwapEndian( hld.wTempHiYear );		/* wTempHiYear WORD */
	hld.wTempHiTime = SwapEndian( hld.wTempHiTime );		/* wTempHiTime WORD */
	hld.wTempHiMonth = SwapEndian( hld.wTempHiMonth );		/* wTempHiMonth WORD */
	hld.wTempHiDay = SwapEndian( hld.wTempHiDay );			/* wTempHiDay WORD */
	hld.wTHSWHiYear = SwapEndian( hld.wTHSWHiYear );		/* wTHSWHiYear WORD */
	hld.wTHSWHiTime = SwapEndian( hld.wTHSWHiTime );		/* wTHSWHiTime WORD */
	hld.wTHSWHiMonth = SwapEndian( hld.wTHSWHiMonth );		/* wTHSWHiMonth WORD */
	hld.wTHSWHiDay = SwapEndian( hld.wTHSWHiDay );			/* wTHSWHiDay WORD */
	hld.wSolarHiYear = SwapEndian( hld.wSolarHiYear );		/* wSolarHiYear WORD */
	hld.wSolarHiTime = SwapEndian( hld.wSolarHiTime );		/* wSolarHiTime WORD */
	hld.wSolarHiMonth = SwapEndian( hld.wSolarHiMonth );		/* wSolarHiMonth WORD */
	hld.wSolarHiDay = SwapEndian( hld.wSolarHiDay );		/* wSolarHiDay WORD */
	hld.wRainHiYear = SwapEndian( hld.wRainHiYear );		/* wRainHiYear WORD */
	hld.wRainHiTime = SwapEndian( hld.wRainHiTime );		/* wRainHiTime WORD */
	hld.wRainHiMonth = SwapEndian( hld.wRainHiMonth );		/* wRainHiMonth WORD */
	hld.wRainHiHour = SwapEndian( hld.wRainHiHour );		/* wRainHiHour WORD */
	hld.wRainHiDay = SwapEndian( hld.wRainHiDay );			/* wRainHiDay WORD */
	hld.wInTempLoYear = SwapEndian( hld.wInTempLoYear );		/* wInTempLoYear WORD */
	hld.wInTempLoTime = SwapEndian( hld.wInTempLoTime );		/* wInTempLoTime WORD */
	hld.wInTempLoMonth = SwapEndian( hld.wInTempLoMonth );		/* wInTempLoMonth WORD */
	hld.wInTempLoDay = SwapEndian( hld.wInTempLoDay );		/* wInTempLoDay WORD */
	hld.wInTempHiYear = SwapEndian( hld.wInTempHiYear );		/* wInTempHiYear WORD */
	hld.wInTempHiTime = SwapEndian( hld.wInTempHiTime );		/* wInTempHiTime WORD */
	hld.wInTempHiMonth = SwapEndian( hld.wInTempHiMonth );		/* wInTempHiMonth WORD */
	hld.wInTempHiDay = SwapEndian( hld.wInTempHiDay );		/* wInTempHiDay WORD */
	hld.wInHumLoTime = SwapEndian( hld.wInHumLoTime );		/* wInHumLoTime WORD */
	hld.wInHumHiTime = SwapEndian( hld.wInHumHiTime );		/* wInHumHiTime WORD */
	hld.wHeatHiYear = SwapEndian( hld.wHeatHiYear );		/* wHeatHiYear WORD */
	hld.wHeatHiTime = SwapEndian( hld.wHeatHiTime );		/* wHeatHiTime WORD */
	hld.wHeatHiMonth = SwapEndian( hld.wHeatHiMonth );		/* wHeatHiMonth WORD */
	hld.wHeatHiDay = SwapEndian( hld.wHeatHiDay );			/* wHeatHiDay WORD */
	hld.wDewLoYear = SwapEndian( hld.wDewLoYear );			/* wDewLoYear WORD */
	hld.wDewLoTime = SwapEndian( hld.wDewLoTime );			/* wDewLoTime WORD */
	hld.wDewLoMonth = SwapEndian( hld.wDewLoMonth );		/* wDewLoMonth WORD */
	hld.wDewLoDay = SwapEndian( hld.wDewLoDay );			/* wDewLoDay WORD */
	hld.wDewHiYear = SwapEndian( hld.wDewHiYear );			/* wDewHiYear WORD */
	hld.wDewHiTime = SwapEndian( hld.wDewHiTime );			/* wDewHiTime WORD */
	hld.wDewHiMonth = SwapEndian( hld.wDewHiMonth );		/* wDewHiMonth WORD */
	hld.wDewHiDay = SwapEndian( hld.wDewHiDay );			/* wDewHiDay WORD */
	hld.wChillLoYear = SwapEndian( hld.wChillLoYear );		/* wChillLoYear WORD */
	hld.wChillLoTime = SwapEndian( hld.wChillLoTime );		/* wChillLoTime WORD */
	hld.wChillLoMonth = SwapEndian( hld.wChillLoMonth );		/* wChillLoMonth WORD */
	hld.wChillLoDay = SwapEndian( hld.wChillLoDay );		/* wChillLoDay WORD */
	hld.wBaroLoYear = SwapEndian( hld.wBaroLoYear );		/* wBaroLoYear WORD */
	hld.wBaroLoTime = SwapEndian( hld.wBaroLoTime );		/* wBaroLoTime WORD */
	hld.wBaroLoMonth = SwapEndian( hld.wBaroLoMonth );		/* wBaroLoMonth WORD */
	hld.wBaroLoDay = SwapEndian( hld.wBaroLoDay );			/* wBaroLoDay WORD */
	hld.wBaroHiYear = SwapEndian( hld.wBaroHiYear );		/* wBaroHiYear WORD */
	hld.wBaroHiTime = SwapEndian( hld.wBaroHiTime );		/* wBaroHiTime WORD */
	hld.wBaroHiMonth = SwapEndian( hld.wBaroHiMonth );		/* wBaroHiMonth WORD */
	hld.wBaroHiDay = SwapEndian( hld.wBaroHiDay );			/* wBaroHiDay WORD */
}

/*--------------------------------------------------------------------------
        dmpdConvertEndian
        Convert all the Big Endian to Little Endian in the DMPDATA struct
----------------------------------------------------------------------------*/
void dmpdConvertEndian(void)
{
	dmpda.wDate = SwapEndian( dmpda.wDate );			/* dmpda.wDate WORD */
	dmpda.wTime = SwapEndian( dmpda.wTime );			/* dmpda.wTime WORD */
	dmpda.wTempOut = SwapEndian( dmpda.wTempOut );			/* dmpda.wTempOut WORD */
	dmpda.wTempOutHi = SwapEndian( dmpda.wTempOutHi );		/* dmpda.wTempOutHi WORD */
	dmpda.wTempOutLo = SwapEndian( dmpda.wTempOutLo );		/* dmpda.wTempOutLo WORD */
	dmpda.wRainFall = SwapEndian( dmpda.wRainFall );		/* dmpda.wRainFall WORD */
	dmpda.wRainRateHi = SwapEndian( dmpda.wRainRateHi );		/* dmpda.wRainRateHi WORD */
	dmpda.wBarometer = SwapEndian( dmpda.wBarometer );		/* dmpda.wBarometer WORD */
	dmpda.wSolarRad = SwapEndian( dmpda.wSolarRad );		/* dmpda.wSolarRad WORD */
	dmpda.wWindSamples = SwapEndian( dmpda.wWindSamples );		/* dmpda.wWindSamples WORD */
	dmpda.wTempIn = SwapEndian( dmpda.wTempIn );			/* dmpda.wTempIn WORD */
	dmpda.wReedClosed = SwapEndian( dmpda.wReedClosed );		/* dmpda.wReedClosed WORD */
	dmpda.wReedOpened = SwapEndian( dmpda.wReedOpened );		/* dmpda.wReedOpened WORD */
	dmpdb.wSolarRadHi = SwapEndian( dmpdb.wSolarRadHi );		/* dmpdb.wSolarRadHi WORD */
}

/*--------------------------------------------------------------------------
        calcTimeJulianCent
----------------------------------------------------------------------------*/
double calcTimeJulianCent(double jd)
{
    double T = 0.0;
    T = (jd - 2451545.0)/36525.0;
    return T;
}

/*--------------------------------------------------------------------------
        isLeapYear
----------------------------------------------------------------------------*/
int isLeapYear(int yr) 
{
    return ((yr % 4 == 0 && yr % 100 != 0) || yr % 400 == 0);
}

/*--------------------------------------------------------------------------
        calcDoyFromJD
----------------------------------------------------------------------------*/
int calcDoyFromJD(double jd)
{
    double z = 0.0;
    double f = 0.0;
    double alpha = 0.0;
    double A = 0.0;
    double B = 0.0;
    double C = 0.0;
    double D = 0.0;
    double E = 0.0;
    double day = 0.0;
    double month = 0.0;
    double year = 0.0;
    double k = 0.0;
    double doy = 0.0;

    z = floor(jd + 0.5);
    f = (jd + 0.5) - z;
    if (z < 2299161) {
        A = z;
    } else {
        alpha = floor((z - 1867216.25)/36524.25);
        A = z + 1 + alpha - floor(alpha/4);
    }
    B = A + 1524;
    C = floor((B - 122.1)/365.25);
    D = floor(365.25 * C);
    E = floor((B - D)/30.6001);
    day = B - D - floor(30.6001 * E) + f;
    month = (E < 14) ? E - 1 : E - 13;
    year = (month > 2) ? C - 4716 : C - 4715;

    k = (isLeapYear(year) ? 1 : 2);
    doy = floor((275 * month)/9) - k * floor((month + 9)/12) + day -30;
    return (int)doy;
}

/*--------------------------------------------------------------------------
        radToDeg
----------------------------------------------------------------------------*/
double radToDeg(double angleRad) 
{
    return (180.0 * angleRad / M_PI);
}

/*--------------------------------------------------------------------------
        degToRad
----------------------------------------------------------------------------*/
double degToRad(double angleDeg) 
{
    return (M_PI * angleDeg / 180.0);
}

/*--------------------------------------------------------------------------
        calcGeomMeanLongSun
----------------------------------------------------------------------------*/
double calcGeomMeanLongSun(double t)
{
    double L0 = 0.0;

    L0 = 280.46646 + t * (36000.76983 + t*(0.0003032));
    while(L0 > 360.0) { L0 -= 360.0; }
    while(L0 < 0.0) { L0 += 360.0; }
    return L0;           /*  in degrees */
}

/*--------------------------------------------------------------------------
        calcGeomMeanAnomalySun
----------------------------------------------------------------------------*/
double calcGeomMeanAnomalySun(double t)
{
    double M;
    M = 357.52911 + t * (35999.05029 - 0.0001537 * t);
    return M;           /*  in degrees */
}

/*--------------------------------------------------------------------------
        calcEccentricityEarthOrbit
----------------------------------------------------------------------------*/
double calcEccentricityEarthOrbit(double t)
{
    double e = 0.0;
    e = 0.016708634 - t * (0.000042037 + 0.0000001267 * t);
    return e;           /*  unitless */
}

/*--------------------------------------------------------------------------
        calcSunEqOfCenter
----------------------------------------------------------------------------*/
double calcSunEqOfCenter(double t)
{
    double m = 0.0;
    double mrad = 0.0;
    double sinm = 0.0;
    double sin2m = 0.0;
    double sin3m = 0.0;
    double C = 0.0;

    m = calcGeomMeanAnomalySun(t);
    mrad = degToRad(m);
    sinm = sin(mrad);
    sin2m = sin(mrad+mrad);
    sin3m = sin(mrad+mrad+mrad);
    C = sinm * (1.914602 - t * (0.004817 + 0.000014 * t)) + sin2m * (0.019993 - 0.000101 * t) + sin3m * 0.000289;
    return C;           /*  in degrees */
}

/*--------------------------------------------------------------------------
        calcSunTrueLong
----------------------------------------------------------------------------*/
double calcSunTrueLong(double t)
{
    double l0 = 0.0;
    double c = 0.0;
    double O = 0.0;

    l0 = calcGeomMeanLongSun(t);
    c = calcSunEqOfCenter(t);
    O = l0 + c;
    return O;           /*  in degrees */
}

/*--------------------------------------------------------------------------
        calcSunTrueAnomaly
----------------------------------------------------------------------------*/
double calcSunTrueAnomaly(double t)
{
    double m = 0.0;
    double c = 0.0;
    double v = 0.0;

    m = calcGeomMeanAnomalySun(t);
    c = calcSunEqOfCenter(t);
    v = m + c;
    return v;           /*  in degrees */
}

/*--------------------------------------------------------------------------
        calcSunRadVector
----------------------------------------------------------------------------*/
double calcSunRadVector(double t)
{
    double v = 0.0;
    double e = 0.0;
    double R = 0.0;

    v = calcSunTrueAnomaly(t);
    e = calcEccentricityEarthOrbit(t);
    R = (1.000001018 * (1 - e * e)) / (1 + e * cos(degToRad(v)));
    return R;           /*  in AUs */
}

/*--------------------------------------------------------------------------
        calcSunApparentLong
----------------------------------------------------------------------------*/
double calcSunApparentLong(double t)
{
    double o = 0.0;
    double omega = 0.0;
    double lambda = 0.0;

    o = calcSunTrueLong(t);
    omega = 125.04 - 1934.136 * t;
    lambda = o - 0.00569 - 0.00478 * sin(degToRad(omega));
    return lambda;              /*  in degrees */
}

/*--------------------------------------------------------------------------
        calcMeanObliquityOfEcliptic
----------------------------------------------------------------------------*/
double calcMeanObliquityOfEcliptic(double t)
{
    double seconds = 0.0;
    double e0 = 0.0;

    seconds = 21.448 - t*(46.8150 + t*(0.00059 - t*(0.001813)));
    e0 = 23.0 + (26.0 + (seconds/60.0))/60.0;
    return e0;          /*  in degrees */
}

/*--------------------------------------------------------------------------
        calcObliquityCorrection
----------------------------------------------------------------------------*/
double calcObliquityCorrection(double t)
{
    double e0 = 0.0;
    double omega = 0.0;
    double e = 0.0;

    e0 = calcMeanObliquityOfEcliptic(t);
    omega = 125.04 - 1934.136 * t;
    e = e0 + 0.00256 * cos(degToRad(omega));
    return e;           /*  in degrees */
}

/*--------------------------------------------------------------------------
        calcSunDeclination
----------------------------------------------------------------------------*/
double calcSunDeclination(double t)
{
    double e = 0.0;
    double lambda = 0.0;
    double sint = 0.0;
    double theta = 0.0;

    e = calcObliquityCorrection(t);
    lambda = calcSunApparentLong(t);

    sint = sin(degToRad(e)) * sin(degToRad(lambda));
    theta = radToDeg(asin(sint));
    return theta;               /*  in degrees */
}

/*--------------------------------------------------------------------------
        calcEquationOfTime
----------------------------------------------------------------------------*/
double calcEquationOfTime(double t)
{
    double epsilon = 0.0;
    double l0 = 0.0;
    double e = 0.0;
    double m = 0.0;
    double y = 0.0;
    double sin2l0 = 0.0;
    double sinm   = 0.0;
    double cos2l0 = 0.0;
    double sin4l0 = 0.0;
    double sin2m  = 0.0;
    double Etime = 0.0;

    epsilon = calcObliquityCorrection(t);
    l0 = calcGeomMeanLongSun(t);
    e = calcEccentricityEarthOrbit(t);
    m = calcGeomMeanAnomalySun(t);

    y = tan(degToRad(epsilon)/2.0);
    y *= y;

    sin2l0 = sin(2.0 * degToRad(l0));
    sinm   = sin(degToRad(m));
    cos2l0 = cos(2.0 * degToRad(l0));
    sin4l0 = sin(4.0 * degToRad(l0));
    sin2m  = sin(2.0 * degToRad(m));

    Etime = y * sin2l0 - 2.0 * e * sinm + 4.0 * e * y * sinm * cos2l0 - 0.5 * y * y * sin4l0 - 1.25 * e * e * sin2m;
    return radToDeg(Etime)*4.0; /*  in minutes of time */
}

/*--------------------------------------------------------------------------
        calcHourAngleSunrise
----------------------------------------------------------------------------*/
double calcHourAngleSunrise(double lat, double solarDec)
{
    double latRad = 0.0;
    double sdRad  = 0.0;
    double HAarg = 0.0;
    double HA = 0.0;

    latRad = degToRad(lat);
    sdRad  = degToRad(solarDec);
    HAarg = (cos(degToRad(90.833))/(cos(latRad)*cos(sdRad))-tan(latRad) * tan(sdRad));
    HA = acos(HAarg);
    return HA;          /*  in radians (for sunset, use -HA) */
}

/*--------------------------------------------------------------------------
        getJD
----------------------------------------------------------------------------*/
double getJD(int year, int month, int day)
{
    double A = 0.0;
    double B = 0.0;
    double JD = 0.0;
    int monthNum = 0;
    int monthDays = 0;
    char* DAYS = MonthDays;

    monthNum = month - 1;
    while(monthNum--) while(*DAYS++);   /* walk thru the null terminators */
    monthDays = atoi(DAYS);
    if ( (isLeapYear(year)) && (month == 2) ) {
        if (day > 29) {
            day = 29;
        } 
    } else {
        if (day > monthDays) day = monthDays;
    }
    if (month <= 2) {
        year -= 1;
        month += 12;
    }
    A = floor(year/100);
    B = 2 - A + floor(A/4);
    JD = floor(365.25*(year + 4716)) + floor(30.6001*(month+1)) + day + B - 1524.5;
    return JD;
}

/*--------------------------------------------------------------------------
        getTimeLocal
----------------------------------------------------------------------------*/
double getTimeLocal(int hour, int minute, int second, BOOL DST)
{
    double mins = 0.0;

    if (DST) {
        hour -= 1;
    }
    mins = hour * 60 + minute + second/60.0;
    return mins;
}

/*--------------------------------------------------------------------------
        calcAzEl
----------------------------------------------------------------------------*/
double calcAzEl(double T, double localtime, double latitude, double longitude, double zone)
{
    double eqTime = 0.0;
    double theta  = 0.0;
    double solarTimeFix = 0.0;
    double trueSolarTime = 0.0;
    double hourAngle = 0.0;
    double haRad = 0.0;
    double csz = 0.0;
    double zenith = 0.0;
    double azDenom = 0.0;
    double azRad = 0.0;
    double azimuth = 0.0;
    double exoatmElevation = 0.0;
    double refractionCorrection = 0.0;
    double te = 0.0;
    double earthRadVec = 0.0;
    double solarZen = 0.0;

    eqTime = calcEquationOfTime(T);
    theta  = calcSunDeclination(T);
    solarTimeFix = eqTime + 4.0 * longitude - 60.0 * zone;
    earthRadVec = calcSunRadVector(T);
    trueSolarTime = localtime + solarTimeFix;
    while (trueSolarTime > 1440) { trueSolarTime -= 1440; }
    hourAngle = trueSolarTime / 4.0 - 180.0;
    if (hourAngle < -180) { hourAngle += 360.0; }
    haRad = degToRad(hourAngle);
    csz = sin(degToRad(latitude)) * sin(degToRad(theta)) + cos(degToRad(latitude)) * cos(degToRad(theta)) * cos(haRad);
    if (csz > 1.0) {
        csz = 1.0;
    } else if (csz < -1.0) { 
        csz = -1.0;
    }
    zenith = radToDeg(acos(csz));
    azDenom = ( cos(degToRad(latitude)) * sin(degToRad(zenith)) );
    if (abs(azDenom) > 0.001) {
        azRad = (( sin(degToRad(latitude)) * cos(degToRad(zenith)) ) - sin(degToRad(theta))) / azDenom;
        if (abs(azRad) > 1.0) {
            if (azRad < 0) {
                azRad = -1.0;
            } else {
                azRad = 1.0;
            }
        }
        azimuth = 180.0 - radToDeg(acos(azRad));
        if (hourAngle > 0.0) { azimuth = -azimuth; }
    } else {
        if (latitude > 0.0) {
            azimuth = 180.0;
        } else { 
            azimuth = 0.0;
        }
    }
    if (azimuth < 0.0) {
        azimuth += 360.0;
    }
    exoatmElevation = 90.0 - zenith;

    /*  Atmospheric Refraction correction */

    if (exoatmElevation > 85.0) {
        refractionCorrection = 0.0;
    } else {
        te = tan (degToRad(exoatmElevation));
        if (exoatmElevation > 5.0) {
            refractionCorrection = 58.1 / te - 0.07 / (te*te*te) + 0.000086 / (te*te*te*te*te);
        } else if (exoatmElevation > -0.575) {
            refractionCorrection = 1735.0 + exoatmElevation * (-518.2 + exoatmElevation * (103.4 + exoatmElevation * (-12.79 + exoatmElevation * 0.711) ) );
        } else {
            refractionCorrection = -20.774 / te;
        }
        refractionCorrection = refractionCorrection / 3600.0;
    }

    solarZen = zenith - refractionCorrection;

    /* this is stupid don't ask just a placeholder */
    if (solarZen == solarZen && earthRadVec == earthRadVec);

    return azimuth;
}

/*--------------------------------------------------------------------------
        calcSolNoon
----------------------------------------------------------------------------*/
void calcSolNoon(double jd, double longitude, double timezone, int dst)
{
    double tnoon = 0.0;
    double eqTime = 0.0;
    double solNoonOffset = 0.0;
    double newt = 0.0;
    double solNoonLocal = 0.0;

    tnoon = calcTimeJulianCent(jd - longitude/360.0);
    eqTime = calcEquationOfTime(tnoon);
    solNoonOffset = 720.0 - (longitude * 4) - eqTime; /*  in minutes */
    newt = calcTimeJulianCent(jd + solNoonOffset/1440.0);
    eqTime = calcEquationOfTime(newt);
    solNoonLocal = 720 - (longitude * 4) - eqTime + (timezone*60.0);    /*  in minutes */
    if(dst) solNoonLocal += 60.0;
    strcpy(SolarNoon, timeString(solNoonLocal, 3));
}

/*--------------------------------------------------------------------------
        dayString
----------------------------------------------------------------------------*/
char* dayString(double jd, int next, int flag)
{
    double z = 0.0;
    double f = 0.0;
    double alpha = 0.0;
    double A = 0.0;
    double B = 0.0;
    double C = 0.0;
    double D = 0.0;
    double E = 0.0;
    int day = 0;
    int month = 0;
    int year = 0;
    char* MN = MonthNameShort;
    int monthNum = 0;
    static char output[16];

/*  returns a string in the form DDMMMYYYY[ next] to display prev/next rise/set */
/*  flag=2 for DD MMM, 3 for DD MM YYYY, 4 for DDMMYYYY next/prev */
    output[0] = '\0';
    if ( (jd < 900000) || (jd > 2817000) ) {
        strcpy(output, "jd error");
    } else {
        z = floor(jd + 0.5);
        f = (jd + 0.5) - z;
        if (z < 2299161) {
            A = z;
        } else {
            alpha = floor((z - 1867216.25)/36524.25);
            A = z + 1 + alpha - floor(alpha/4);
        }
        B = A + 1524;
        C = floor((B - 122.1)/365.25);
        D = floor(365.25 * C);
        E = floor((B - D)/30.6001);
        day = B - D - floor(30.6001 * E) + f;
        month = (E < 14) ? E - 1 : E - 13;
        year = ((month > 2) ? C - 4716 : C - 4715);
        monthNum = month - 1;
        while(monthNum--) while(*MN++);   /* walk thru the null terminators */
        if (flag == 2) sprintf(output, "%02i%3s", day, MN);
        if (flag == 3) sprintf(output, "%02i%3s%04i", day, MN, year);
        if (flag == 4) sprintf(output, "%02i%3s%04i %4s", day, MN, year, next? "next" : "prev");
    }
    return output;
}

/*--------------------------------------------------------------------------
        timeDateString
----------------------------------------------------------------------------*/
char* timeDateString(double JD, double minutes)
{
    static char output[26];     /* 15 + 10 */
 
    sprintf(output, "%s-%s", timeString(minutes, 2), dayString(JD, 0, 2));
    return output;
}

/*--------------------------------------------------------------------------
        timeString
----------------------------------------------------------------------------*/
char* timeString(double minutes, int flag)
{
/*  timeString returns a zero-padded string (HH:MM:SS) given time in minutes */
/*  flag=2 for HH:MM, 3 for HH:MM:SS */
    double floatHour = 0.0;
    int hour = 0;
    double floatMinute = 0.0;
    int minute = 0;
    double floatSec = 0.0;
    int second = 0;
    static char output[11];
    BOOL PM = FALSE;
    
    if ( (minutes >= 0) && (minutes < 1440) ) {
        floatHour = minutes / 60.0;
        hour = floor(floatHour);
        floatMinute = 60.0 * (floatHour - floor(floatHour));
        minute = floor(floatMinute);
        floatSec = 60.0 * (floatMinute - floor(floatMinute));
        second = floor(floatSec + 0.5);
        if (second > 59) {
            second = 0;
            minute += 1;
        }
        if ((flag == 2) && (second >= 30)) { minute++; }
        if (minute > 59) {
           minute = 0;
           hour += 1;
        }
        if (! bMtime && hour > 12) {
            hour -= 12;
            PM = TRUE;
        }
        if (! bMtime && hour == 12) PM = TRUE;
        output[0] = '\0';
        if (bMtime) {
            sprintf(output, "%02i:%02i", hour, minute);
            if (flag > 2) sprintf(output, "%02i:%02i:%02i", hour, minute, second);
        } else {
            sprintf(output, "%i:%02i%s", hour, minute, (PM)? "PM" : "AM");
            if (flag > 2) sprintf(output, "%i:%02i:%02i%s", hour, minute, second, (PM)? "PM" : "AM");
        }
    } else { 
        strcpy(output, "ts error");
    }
    return output;
}

/*--------------------------------------------------------------------------
        calcSunriseSetUTC
----------------------------------------------------------------------------*/
double calcSunriseSetUTC(BOOL rise, double JD, double latitude, double longitude)
{
    double t = 0.0;
    double eqTime = 0.0;
    double solarDec = 0.0;
    double hourAngle = 0.0;
    double delta = 0.0;
    double timeUTC = 0.0;

    t = calcTimeJulianCent(JD);
    eqTime = calcEquationOfTime(t);
    solarDec = calcSunDeclination(t);
    hourAngle = calcHourAngleSunrise(latitude, solarDec);
    if (!rise) hourAngle = -hourAngle;
    delta = longitude + radToDeg(hourAngle);
    timeUTC = 720 - (4.0 * delta) - eqTime;     /*  in minutes */
    return timeUTC;
}

/*--------------------------------------------------------------------------
        calcSunriseSet
----------------------------------------------------------------------------*/
void calcSunriseSet(BOOL rise, double JD, double latitude, double longitude, double timezone, double dst)
{
    double timeUTC = 0.0;
    double newTimeUTC = 0.0;
    double timeLocal = 0.0;
    double jday = 0.0;
    double increment = 0.0;
    int doy = 0;
    double jdy = 0.0;

    /*  rise = 1 for sunrise, 0 for sunset */
    jdy = JD;
    timeUTC = calcSunriseSetUTC(rise, JD, latitude, longitude);
    newTimeUTC = calcSunriseSetUTC(rise, JD + timeUTC/1440.0, latitude, longitude); 
    if (isnormal(newTimeUTC)) {
        timeLocal = newTimeUTC + (timezone * 60.0);
        timeLocal += ((dst) ? 60.0 : 0.0);
        if ( (timeLocal >= 0.0) && (timeLocal < 1440.0) ) {
            if (rise) 
                strcpy(SunRise, timeString(timeLocal,2));
            else
                strcpy(SunSet, timeString(timeLocal,2));
        } else  {
            jday = JD;
            increment = ((timeLocal < 0) ? 1 : -1);
            while ((timeLocal < 0.0)||(timeLocal >= 1440.0)) {
                timeLocal += increment * 1440.0;
                jday -= increment;
            }
            if (rise) 
                strcpy(SunRise, timeDateString(jday,timeLocal));
            else
                strcpy(SunSet, timeDateString(jday,timeLocal)); 
        }
    } else { /*  no sunrise/set found */
        doy = calcDoyFromJD(JD);
        if ( (latitude > 66.4 && doy > 79 && doy < 267) || (latitude < -66.4 && (doy < 83 || doy > 263)) )
        {   /* previous sunrise/next sunset */
            if (rise) { /*  find previous sunrise */
                jdy = calcJDofNextPrevRiseSet(FALSE, rise, JD, latitude, longitude, timezone, dst);
            } else { /*  find next sunset */
                jdy = calcJDofNextPrevRiseSet(TRUE, rise, JD, latitude, longitude, timezone, dst);
            }
            if (rise)
                strcpy(SunRise, dayString(jdy,0,3));
            else
                strcpy(SunSet, dayString(jdy,0,3));
        } else {   /* previous sunset/next sunrise */
            if (rise == 1) { /*  find previous sunrise */
                jdy = calcJDofNextPrevRiseSet(TRUE, rise, JD, latitude, longitude, timezone, dst);
            } else { /*  find next sunset */
                jdy = calcJDofNextPrevRiseSet(FALSE, rise, JD, latitude, longitude, timezone, dst);
            }
            if (rise)
                strcpy(SunRise, dayString(jdy,0,3));
            else
                strcpy(SunSet, dayString(jdy,0,3));
        }
    }
}

/*--------------------------------------------------------------------------
        calcJDofNextPrevRiseSet
----------------------------------------------------------------------------*/
double calcJDofNextPrevRiseSet(int next, int rise, double JD, double latitude, double longitude, double tz, double dst)
{
    double julianday = 0.0;
    double increment = 0.0;
    double time = 0.0;
    double timeLocal = 0.0;
    double incr = 0.0;

    julianday = JD;
    increment = ((next) ? 1.0 : -1.0);

    time = calcSunriseSetUTC(rise, julianday, latitude, longitude);
    while(!isnormal(time)){
        julianday += increment;
        time = calcSunriseSetUTC(rise, julianday, latitude, longitude);
    }
    timeLocal = time + tz * 60.0 + ((dst) ? 60.0 : 0.0);
    while ((timeLocal < 0.0) || (timeLocal >= 1440.0))
    {
        incr = ((timeLocal < 0) ? 1 : -1);
        timeLocal += (incr * 1440.0);
        julianday -= incr;
    }
    return julianday;
}

/*--------------------------------------------------------------------------
        calcSunRiseSetNoon
----------------------------------------------------------------------------*/
void calcSunRiseSetNoon(float lat, float lng, int year, int month, int day, int hour, int minute, int second, int dst, float tz)
{
    double jday = 0.0;
    double tl = 0.0;
    double total = 0.0;
    double T = 0.0;
    double azimuth = 0.0;

    if(bVerbose) fprintf(stderr, "lat %lf lng %lf year %04i month %02i day %02i hour %02i minute %02i second %02i DST %i zone %.f\n", lat, lng, year, month, day, hour, minute, second, dst, tz);
    if(bVerbose) fprintf(stderr, "Get jday ");
    jday = getJD(year, month, day);
    if(bVerbose) fprintf(stderr, "%lf Get tl ", jday);
    tl = getTimeLocal(hour, minute, second, dst);
    if(bVerbose) fprintf(stderr, "%lf Get total ", tl);
    total = jday + tl/1440.0 - tz/24.0;
    if(bVerbose) fprintf(stderr, "%lf Get T ", tl);
    T = calcTimeJulianCent(total);
    if(bVerbose) fprintf(stderr, "%lf Get azimuth ", T);
    azimuth = calcAzEl(T, tl, lat, lng, tz);
    if(bVerbose) fprintf(stderr, "%lf\ncalcSolNoon... ", azimuth);
    calcSolNoon(jday, lng, tz, dst);
    if(bVerbose) fprintf(stderr, "Done.\ncalcSunriseSet (Rise)... ");
    calcSunriseSet(TRUE, jday, lat, lng, tz, dst);
    if(bVerbose) fprintf(stderr, "Done.\ncalcSunriseSet (Set)... ");
    calcSunriseSet(FALSE, jday, lat, lng, tz, dst);
    if(bVerbose) fprintf(stderr, "Done.\nAll Done.\n\n");
}


/*--------------------------------------------------------------------------
	setupVP1
	setup the graph data pointer offsets for a Vantage Pro
----------------------------------------------------------------------------*/
void setupVP1()
{
    int GRFVP1 = GRAPH_START + 9;			// 186
    if(bVerbose) fprintf(stderr, "\nSetting up for vPro\n");
//							NUMBER  NUMBER
//							OF      OF
//							ENTRYS  BYTES
//							--------------
    TEMP_IN_HOUR		= GRFVP1 + 0;		// 24 x  1
    TEMP_IN_DAY_HIGHS		= GRFVP1 + 24;		// 24 x  1
    TEMP_IN_DAY_HIGH_TIMES	= GRFVP1 + 48;		// 24 x  2
    TEMP_IN_DAY_LOWS		= GRFVP1 + 96;		// 24 x  1
    TEMP_IN_DAY_LOW_TIMES	= GRFVP1 + 120;		// 24 x  2
    TEMP_IN_MONTH_HIGHS		= GRFVP1 + 168;		// 25 x  1
    TEMP_IN_MONTH_LOWS		= GRFVP1 + 193;		// 25 x  1
    TEMP_IN_YEAR_HIGHS		= GRFVP1 + 218;		//  1 x  1
    TEMP_IN_YEAR_LOWS		= GRFVP1 + 219;		//  1 x  1

    TEMP_OUT_HOUR		= GRFVP1 + 220;		// 24 ||  1
    TEMP_OUT_DAY_HIGHS		= GRFVP1 + 244;		// 24 ||  1
    TEMP_OUT_DAY_HIGH_TIMES	= GRFVP1 + 268;		// 24 ||  2
    TEMP_OUT_DAY_LOWS		= GRFVP1 + 316;		// 24 ||  1
    TEMP_OUT_DAY_LOW_TIMES	= GRFVP1 + 340;		// 24 ||  2
    TEMP_OUT_MONTH_HIGHS	= GRFVP1 + 388;		// 25 ||  1
    TEMP_OUT_MONTH_LOWS		= GRFVP1 + 413;		// 25 ||  1
    TEMP_OUT_YEAR_HIGHS		= GRFVP1 + 438;		// 25 ||  1
    TEMP_OUT_YEAR_LOWS		= GRFVP1 + 463;		// 25 ||  1

    DEW_HOUR			= GRFVP1 + 488;		// 24 ||  1
    DEW_DAY_HIGHS		= GRFVP1 + 512;		// 24 ||  1
    DEW_DAY_HIGH_TIMES		= GRFVP1 + 536;		// 24 ||  2
    DEW_DAY_LOWS		= GRFVP1 + 584;		// 24 ||  1
    DEW_DAY_LOW_TIMES		= GRFVP1 + 608;		// 24 ||  2
    DEW_MONTH_HIGHS		= GRFVP1 + 656;		// 25 ||  1
    DEW_MONTH_LOWS		= GRFVP1 + 681;		// 25 ||  1
    DEW_YEAR_HIGHS		= GRFVP1 + 706;		//  1 ||  1
    DEW_YEAR_LOWS		= GRFVP1 + 707;		//  1 ||  1

    CHILL_HOUR			= GRFVP1 + 708;		// 24 ||  1
    CHILL_DAY_LOWS		= GRFVP1 + 732;		// 24 ||  1
    CHILL_DAY_LOW_TIMES		= GRFVP1 + 756;		// 24 ||  2
    CHILL_MONTH_LOWS		= GRFVP1 + 804;		// 25 ||  1
    CHILL_YEAR_LOWS		= GRFVP1 + 829;		//  1 ||  1

    THSW_HOUR			= GRFVP1 + 830;		// 24 ||  1
    THSW_DAY_HIGHS		= GRFVP1 + 854;		// 24 ||  1
    THSW_DAY_HIGH_TIMES		= GRFVP1 + 878;		// 24 ||  2
    THSW_MONTH_HIGHS		= GRFVP1 + 926;		// 25 ||  1
    THSW_YEAR_HIGHS		= GRFVP1 + 951;		//  1 ||  1

    HEAT_HOUR			= GRFVP1 + 952;		// 24 ||  1
    HEAT_DAY_HIGHS		= GRFVP1 + 976;		// 24 ||  1
    HEAT_DAY_HIGH_TIMES		= GRFVP1 + 1000;	// 24 ||  2
    HEAT_MONTH_HIGHS		= GRFVP1 + 1048;	// 25 ||  1
    HEAT_YEAR_HIGHS		= GRFVP1 + 1073;	//  1 ||  1

    HUM_IN_HOUR			= GRFVP1 + 1074;	// 24 ||  1
    HUM_IN_DAY_HIGHS		= GRFVP1 + 1098;	// 24 ||  1
    HUM_IN_DAY_HIGH_TIMES	= GRFVP1 + 1122;	// 24 ||  2
    HUM_IN_DAY_LOWS		= GRFVP1 + 1170;	// 24 ||  1
    HUM_IN_DAY_LOW_TIMES	= GRFVP1 + 1194;	// 24 ||  2
    HUM_IN_MONTH_HIGHS		= GRFVP1 + 1242;	// 25 ||  1
    HUM_IN_MONTH_LOWS		= GRFVP1 + 1267;	// 25 ||  1
    HUM_IN_YEAR_HIGHS		= GRFVP1 + 1292;	//  1 ||  1
    HUM_IN_YEAR_LOWS		= GRFVP1 + 1293;	//  1 ||  1

    HUM_OUT_HOUR		= GRFVP1 + 1294;	// 24 ||  1
    HUM_OUT_DAY_HIGHS		= GRFVP1 + 1318;	// 24 ||  1
    HUM_OUT_DAY_HIGH_TIMES	= GRFVP1 + 1342;	// 24 ||  2
    HUM_OUT_DAY_LOWS		= GRFVP1 + 1390;	// 24 ||  1
    HUM_OUT_DAY_LOW_TIMES	= GRFVP1 + 1414;	// 24 ||  2
    HUM_OUT_MONTH_HIGHS		= GRFVP1 + 1462;	// 25 ||  1
    HUM_OUT_MONTH_LOWS		= GRFVP1 + 1487;	// 25 ||  1
    HUM_OUT_YEAR_HIGHS		= GRFVP1 + 1512;	//  1 ||  1
    HUM_OUT_YEAR_LOWS		= GRFVP1 + 1513;	//  1 ||  1

    BAR_15_MIN			= GRFVP1 + 1514;	// 24 ||  2
    BAR_HOUR			= GRFVP1 + 1562;	// 24 ||  2
    BAR_DAY_HIGHS		= GRFVP1 + 1610;	// 24 ||  2
    BAR_DAY_HIGH_TIMES		= GRFVP1 + 1658;	// 24 ||  2
    BAR_DAY_LOWS		= GRFVP1 + 1706;	// 24 ||  2
    BAR_DAY_LOW_TIMES		= GRFVP1 + 1754;	// 24 ||  2
    BAR_MONTH_HIGHS		= GRFVP1 + 1802;	// 25 ||  2
    BAR_MONTH_LOWS		= GRFVP1 + 1852;	// 25 ||  2
    BAR_YEAR_HIGHS		= GRFVP1 + 1902;	//  1 ||  2
    BAR_YEAR_LOWS		= GRFVP1 + 1904;	//  1 ||  2

    WIND_SPEED_10_MIN_AVG	= GRFVP1 + 1906;	// 24 ||  1
    WIND_SPEED_HOUR_AVG		= GRFVP1 + 1930;	// 24 ||  1
    WIND_SPEED_DAY_HIGHS	= GRFVP1 + 1954;	// 24 ||  1
    WIND_SPEED_DAY_HIGH_TIMES	= GRFVP1 + 1978;	// 24 ||  2
    WIND_SPEED_DAY_HIGH_DIR	= GRFVP1 + 2026;	// 24 ||  1
    WIND_SPEED_MONTH_HIGHS	= GRFVP1 + 2050;	// 25 ||  1
    WIND_SPEED_MONTH_HIGH_DIR	= GRFVP1 + 2075;	// 25 ||  1
    WIND_SPEED_YEAR_HIGHS	= GRFVP1 + 2100;	// 25 ||  1
    WIND_SPEED_YEAR_HIGH_DIR	= GRFVP1 + 2125;	// 25 ||  1

    WIND_DIR_HOUR		= GRFVP1 + 2150;	// 24 ||  1
    WIND_DIR_DAY		= GRFVP1 + 2174;	// 24 ||  1
    WIND_DIR_MONTH		= GRFVP1 + 2198;	// 24 ||  1
    WIND_DIR_DAY_BINS		= GRFVP1 + 2222;	//  8 ||  2
    WIND_DIR_MONTH_BINS		= GRFVP1 + 2238;	//  8 ||  2

    RAIN_RATE_1_MIN		= GRFVP1 + 2254;	// 24 ||  2
    RAIN_RATE_HOUR		= GRFVP1 + 2302;	// 24 ||  2
    RAIN_RATE_DAY_HIGHS		= GRFVP1 + 2350;	// 24 ||  2
    RAIN_RATE_DAY_HIGH_TIMES	= GRFVP1 + 2398;	// 24 ||  2
    RAIN_RATE_MONTH_HIGHS	= GRFVP1 + 2446;	// 25 ||  2
    RAIN_RATE_YEAR_HIGHS	= GRFVP1 + 2496;	// 25 ||  2

    RAIN_15_MIN			= GRFVP1 + 2546;	// 24 ||  1
    RAIN_HOUR			= GRFVP1 + 2570;	// 24 ||  2
    RAIN_STORM			= GRFVP1 + 2618;	// 25 ||  2
    RAIN_STORM_START		= GRFVP1 + 2668;	// 25 ||  2
    RAIN_STORM_END		= GRFVP1 + 2718;	// 25 ||  2
    RAIN_DAY_TOTAL		= GRFVP1 + 2768;	// 25 ||  2
    RAIN_MONTH_TOTAL		= GRFVP1 + 2818;	// 25 ||  2
    RAIN_YEAR_TOTAL		= GRFVP1 + 2868;	// 25 ||  2

    ET_HOUR			= GRFVP1 + 2918;	// 24 ||  1
    ET_DAY_TOTAL		= GRFVP1 + 2942;	// 25 ||  1
    ET_MONTH_TOTAL		= GRFVP1 + 2967;	// 25 ||  2
    ET_YEAR_TOTAL		= GRFVP1 + 3017;	// 25 ||  2

    SOLAR_HOUR_AVG		= GRFVP1 + 3067;	// 24 ||  2
    SOLAR_DAY_HIGHS		= GRFVP1 + 3115;	// 24 ||  2
    SOLAR_DAY_HIGH_TIMES	= GRFVP1 + 3163;	// 24 ||  2
    SOLAR_MONTH_HIGHS		= GRFVP1 + 3211;	// 25 ||  2
    SOLAR_YEAR_HIGHS		= GRFVP1 + 3261;	//  1 ||  2

    UV_HOUR_AVG			= GRFVP1 + 3263;	// 24 ||  1
    UV_MEDS_HOUR		= GRFVP1 + 3287;	// 24 ||  1
    UV_MEDS_DAY			= GRFVP1 + 3311;	// 24 ||  1
    UV_DAY_HIGHS		= GRFVP1 + 3335;	// 24 ||  1
    UV_DAY_HIGH_TIMES		= GRFVP1 + 3359;	// 24 ||  2
    UV_MONTH_HIGHS		= GRFVP1 + 3407;	// 25 ||  1
    UV_YEAR_HIGHS		= GRFVP1 + 3432;	//  1 ||  1

    LEAF_HOUR			= GRFVP1 + 3433;	// 24 ||  1
    LEAF_DAY_LOWS		= GRFVP1 + 3457;	// 24 ||  1
    LEAF_DAY_LOW_TIMES		= GRFVP1 + 3481;	// 24 ||  2
    LEAF_DAY_HIGHS		= GRFVP1 + 3529;	// 24 ||  1
    LEAF_DAY_HIGH_TIMES		= GRFVP1 + 3553;	// 24 ||  2
    WIND_SPEED_HOUR_HIGHS	= GRFVP1 + 3601;	// 24 ||  1
    LEAF_MONTH_LOWS		= GRFVP1 + 3625;	//  1 ||  1
    LEAF_MONTH_HIGHS		= GRFVP1 + 3626;	// 25 ||  1
    LEAF_YEAR_LOWS		= GRFVP1 + 3651;	//  1 ||  1
    LEAF_YEAR_HIGHS		= GRFVP1 + 3652;	//  1 ||  1

    SOIL_HOUR			= GRFVP1 + 3653;	// 24 ||  1
    SOIL_DAY_LOWS		= GRFVP1 + 3677;	// 24 ||  1
    SOIL_DAY_LOW_TIMES		= GRFVP1 + 3701;	// 24 ||  2
    SOIL_DAY_HIGHS		= GRFVP1 + 3749;	// 24 ||  1
    SOIL_DAY_HIGH_TIMES		= GRFVP1 + 3773;	// 24 ||  2
    SOIL_MONTH_LOWS		= GRFVP1 + 3821;	// 25 ||  1
    SOIL_MONTH_HIGHS		= GRFVP1 + 3846;	// 25 ||  1
    SOIL_YEAR_LOWS		= GRFVP1 + 3871;	//  1 ||  1
    SOIL_YEAR_HIGHS		= GRFVP1 + 3872;	//  1 ||  1
    SOIL_YEAR_HIGHS_COMP	= GRFVP1 + 3873;	//  1 ||  1

    RX_PERCENTAGE		= GRFVP1 + 3874;	//  24 ||  1
    SAVE_MIN			= RX_PERCENTAGE + 25;	// 4084
    SAVE_HOUR			= SAVE_MIN + 1;
    SAVE_DAY			= SAVE_HOUR + 1;
    SAVE_MONTH			= SAVE_HOUR + 2;
    SAVE_YEAR			= SAVE_HOUR + 3;
    SAVE_YEAR_COMP		= SAVE_HOUR + 4;
    BAUD_RATE			= SAVE_HOUR + 5;
    DEFAULT_RATE_GRAPH		= SAVE_HOUR + 6;

}


/*--------------------------------------------------------------------------
        setupVP2
        setup the graph data pointer offsets for a Vantage Pro 2
----------------------------------------------------------------------------*/
void setupVP2()
{
    int GRFVP2 = GRAPH_START + 149;			// 326;
    if(bVerbose) fprintf(stderr, "\nSetting up for vPro2\n");
//							NUMBER  NUMBER
//							OF      OF
//							ENTRYS  BYTES
//							--------------
    TEMP_IN_HOUR		= GRFVP2 + 0;		// 24 ||  1
    TEMP_IN_DAY_HIGHS		= GRFVP2 + 24;		// 24 ||  1
    TEMP_IN_DAY_HIGH_TIMES	= GRFVP2 + 48;		// 24 ||  2
    TEMP_IN_DAY_LOWS		= GRFVP2 + 96;		// 24 ||  1
    TEMP_IN_DAY_LOW_TIMES	= GRFVP2 + 120;		// 24 ||  2
    TEMP_IN_MONTH_HIGHS		= GRFVP2 + 168;		// 25 ||  1
    TEMP_IN_MONTH_LOWS		= GRFVP2 + 193;		// 25 ||  1
    TEMP_IN_YEAR_HIGHS		= GRFVP2 + 218;		//  1 ||  1
    TEMP_IN_YEAR_LOWS		= GRFVP2 + 219;		//  1 ||  1

    TEMP_OUT_HOUR		= GRFVP2 + 220;		// 24 ||  1
    TEMP_OUT_DAY_HIGHS		= GRFVP2 + 244;		// 24 ||  1
    TEMP_OUT_DAY_HIGH_TIMES	= GRFVP2 + 268;		// 24 ||  2
    TEMP_OUT_DAY_LOWS		= GRFVP2 + 316;		// 24 ||  1
    TEMP_OUT_DAY_LOW_TIMES	= GRFVP2 + 340;		// 24 ||  2
    TEMP_OUT_MONTH_HIGHS	= GRFVP2 + 388;		// 25 ||  1
    TEMP_OUT_MONTH_LOWS		= GRFVP2 + 413;		// 25 ||  1
    TEMP_OUT_YEAR_HIGHS		= GRFVP2 + 438;		// 25 ||  1
    TEMP_OUT_YEAR_LOWS		= GRFVP2 + 463;		// 25 ||  1

    DEW_HOUR			= GRFVP2 + 488;		// 24 ||  1
    DEW_DAY_HIGHS		= GRFVP2 + 512;		// 24 ||  1
    DEW_DAY_HIGH_TIMES		= GRFVP2 + 536;		// 24 ||  2
    DEW_DAY_LOWS		= GRFVP2 + 584;		// 24 ||  1
    DEW_DAY_LOW_TIMES		= GRFVP2 + 608;		// 24 ||  2
    DEW_MONTH_HIGHS		= GRFVP2 + 656;		// 25 ||  1
    DEW_MONTH_LOWS		= GRFVP2 + 681;		// 25 ||  1
    DEW_YEAR_HIGHS		= GRFVP2 + 706;		//  1 ||  1
    DEW_YEAR_LOWS		= GRFVP2 + 707;		//  1 ||  1

    CHILL_HOUR			= GRFVP2 + 708;		// 24 ||  1
    CHILL_DAY_LOWS		= GRFVP2 + 732;		// 24 ||  1
    CHILL_DAY_LOW_TIMES		= GRFVP2 + 756;		// 24 ||  2
    CHILL_MONTH_LOWS		= GRFVP2 + 804;		// 25 ||  1
    CHILL_YEAR_LOWS		= GRFVP2 + 829;		//  1 ||  1

    THSW_HOUR			= GRFVP2 + 830;		// 24 ||  1
    THSW_DAY_HIGHS		= GRFVP2 + 854;		// 24 ||  1
    THSW_DAY_HIGH_TIMES		= GRFVP2 + 878;		// 24 ||  2
    THSW_MONTH_HIGHS		= GRFVP2 + 926;		// 25 ||  1
    THSW_YEAR_HIGHS		= GRFVP2 + 951;		//  1 ||  1

    HEAT_HOUR			= GRFVP2 + 952;		// 24 ||  1
    HEAT_DAY_HIGHS		= GRFVP2 + 976;		// 24 ||  1
    HEAT_DAY_HIGH_TIMES		= GRFVP2 + 1000;	// 24 ||  2
    HEAT_MONTH_HIGHS		= GRFVP2 + 1048;	// 25 ||  1
    HEAT_YEAR_HIGHS		= GRFVP2 + 1073;	//  1 ||  1

    HUM_IN_HOUR			= GRFVP2 + 1074;	// 24 ||  1
    HUM_IN_DAY_HIGHS		= GRFVP2 + 1098;	// 24 ||  1
    HUM_IN_DAY_HIGH_TIMES	= GRFVP2 + 1122;	// 24 ||  2
    HUM_IN_DAY_LOWS		= GRFVP2 + 1170;	// 24 ||  1
    HUM_IN_DAY_LOW_TIMES	= GRFVP2 + 1194;	// 24 ||  2
    HUM_IN_MONTH_HIGHS		= GRFVP2 + 1242;	// 25 ||  1
    HUM_IN_MONTH_LOWS		= GRFVP2 + 1267;	// 25 ||  1
    HUM_IN_YEAR_HIGHS		= GRFVP2 + 1292;	//  1 ||  1
    HUM_IN_YEAR_LOWS		= GRFVP2 + 1293;	//  1 ||  1

    HUM_OUT_HOUR		= GRFVP2 + 1294;	// 24 ||  1
    HUM_OUT_DAY_HIGHS		= GRFVP2 + 1318;	// 24 ||  1
    HUM_OUT_DAY_HIGH_TIMES	= GRFVP2 + 1342;	// 24 ||  2
    HUM_OUT_DAY_LOWS		= GRFVP2 + 1390;	// 24 ||  1
    HUM_OUT_DAY_LOW_TIMES	= GRFVP2 + 1414;	// 24 ||  2
    HUM_OUT_MONTH_HIGHS		= GRFVP2 + 1462;	// 25 ||  1
    HUM_OUT_MONTH_LOWS		= GRFVP2 + 1487;	// 25 ||  1
    HUM_OUT_YEAR_HIGHS		= GRFVP2 + 1512;	//  1 ||  1
    HUM_OUT_YEAR_LOWS		= GRFVP2 + 1513;	//  1 ||  1

    BAR_15_MIN			= GRFVP2 + 1514;	// 24 ||  2
    BAR_HOUR			= GRFVP2 + 1562;	// 24 ||  2
    BAR_DAY_HIGHS		= GRFVP2 + 1610;	// 24 ||  2
    BAR_DAY_HIGH_TIMES		= GRFVP2 + 1658;	// 24 ||  2
    BAR_DAY_LOWS		= GRFVP2 + 1706;	// 24 ||  2
    BAR_DAY_LOW_TIMES		= GRFVP2 + 1754;	// 24 ||  2
    BAR_MONTH_HIGHS		= GRFVP2 + 1802;	// 25 ||  2
    BAR_MONTH_LOWS		= GRFVP2 + 1852;	// 25 ||  2
    BAR_YEAR_HIGHS		= GRFVP2 + 1902;	//  1 ||  2
    BAR_YEAR_LOWS		= GRFVP2 + 1904;	//  1 ||  2

    WIND_SPEED_10_MIN_AVG	= GRFVP2 + 1906;	// 24 ||  1
    WIND_SPEED_HOUR_AVG		= GRFVP2 + 1930;	// 24 ||  1
    WIND_SPEED_HOUR_HIGHS	= GRFVP2 + 1954;	// 24 ||  1
    WIND_SPEED_DAY_HIGHS	= GRFVP2 + 1978;	// 24 ||  1
    WIND_SPEED_DAY_HIGH_TIMES	= GRFVP2 + 2002;	// 24 ||  2
    WIND_SPEED_DAY_HIGH_DIR	= GRFVP2 + 2050;	// 24 ||  1
    WIND_SPEED_MONTH_HIGHS	= GRFVP2 + 2074;	// 25 ||  1
    WIND_SPEED_MONTH_HIGH_DIR	= GRFVP2 + 2099;	// 25 ||  1
    WIND_SPEED_YEAR_HIGHS	= GRFVP2 + 2124;	// 25 ||  1
    WIND_SPEED_YEAR_HIGH_DIR	= GRFVP2 + 2149;	// 25 ||  1

    WIND_DIR_HOUR		= GRFVP2 + 2174;	// 24 ||  1
    WIND_DIR_DAY		= GRFVP2 + 2198;	// 24 ||  1
    WIND_DIR_MONTH		= GRFVP2 + 2222;	// 24 ||  1
    WIND_DIR_DAY_BINS		= GRFVP2 + 2246;	//  8 ||  2
    WIND_DIR_MONTH_BINS		= GRFVP2 + 2262;	//  8 ||  2

    RAIN_RATE_1_MIN		= GRFVP2 + 2278;	// 24 ||  2
    RAIN_RATE_HOUR		= GRFVP2 + 2326;	// 24 ||  2
    RAIN_RATE_DAY_HIGHS		= GRFVP2 + 2374;	// 24 ||  2
    RAIN_RATE_DAY_HIGH_TIMES	= GRFVP2 + 2422;	// 24 ||  2
    RAIN_RATE_MONTH_HIGHS	= GRFVP2 + 2470;	// 25 ||  2
    RAIN_RATE_YEAR_HIGHS	= GRFVP2 + 2520;	// 25 ||  2

    RAIN_15_MIN			= GRFVP2 + 2570;	// 24 ||  1
    RAIN_HOUR			= GRFVP2 + 2594;	// 24 ||  2
    RAIN_STORM			= GRFVP2 + 2642;	// 25 ||  2
    RAIN_STORM_START		= GRFVP2 + 2692;	// 25 ||  2
    RAIN_STORM_END		= GRFVP2 + 2742;	// 25 ||  2
    RAIN_DAY_TOTAL		= GRFVP2 + 2792;	// 25 ||  2
    RAIN_MONTH_TOTAL		= GRFVP2 + 2842;	// 25 ||  2
    RAIN_YEAR_TOTAL		= GRFVP2 + 2892;	// 25 ||  2

    ET_HOUR			= GRFVP2 + 2942;	// 24 ||  1
    ET_DAY_TOTAL		= GRFVP2 + 2966;	// 25 ||  1
    ET_MONTH_TOTAL		= GRFVP2 + 2991;	// 25 ||  2
    ET_YEAR_TOTAL		= GRFVP2 + 3041;	// 25 ||  2

    SOLAR_HOUR_AVG		= GRFVP2 + 3091;	// 24 ||  2
    SOLAR_DAY_HIGHS		= GRFVP2 + 3139;	// 24 ||  2
    SOLAR_DAY_HIGH_TIMES	= GRFVP2 + 3187;	// 24 ||  2
    SOLAR_MONTH_HIGHS		= GRFVP2 + 3235;	//  1 ||  2
    SOLAR_YEAR_HIGHS		= GRFVP2 + 3237;	//  1 ||  2

    UV_HOUR_AVG			= GRFVP2 + 3239;	// 24 ||  1
    UV_MEDS_HOUR		= GRFVP2 + 3263;	// 24 ||  1
    UV_MEDS_DAY			= GRFVP2 + 3287;	// 24 ||  1
    UV_DAY_HIGHS		= GRFVP2 + 3311;	// 24 ||  1
    UV_DAY_HIGH_TIMES		= GRFVP2 + 3335;	// 24 ||  2
    UV_MONTH_HIGHS		= GRFVP2 + 3383;	//  1 ||  1
    UV_YEAR_HIGHS		= GRFVP2 + 3384;	//  1 ||  1

    LEAF_HOUR			= GRFVP2 + 3385;	// 24 ||  1
    LEAF_DAY_LOWS		= GRFVP2 + 3409;	// 24 ||  1
    LEAF_DAY_LOW_TIMES		= GRFVP2 + 3433;	// 24 ||  2
    LEAF_DAY_HIGHS		= GRFVP2 + 3481;	// 24 ||  1
    LEAF_DAY_HIGH_TIMES		= GRFVP2 + 3505;	// 24 ||  2
    LEAF_MONTH_LOWS		= GRFVP2 + 3553;	//  1 ||  1
    LEAF_MONTH_HIGHS		= GRFVP2 + 3554;	//  1 ||  1
    LEAF_YEAR_LOWS		= GRFVP2 + 3555;	//  1 ||  1
    LEAF_YEAR_HIGHS		= GRFVP2 + 3556;	//  1 ||  1

    SOIL_HOUR			= GRFVP2 + 3557;	// 24 ||  1
    SOIL_DAY_LOWS		= GRFVP2 + 3581;	// 24 ||  1
    SOIL_DAY_LOW_TIMES		= GRFVP2 + 3605;	// 24 ||  2
    SOIL_DAY_HIGHS		= GRFVP2 + 3653;	// 24 ||  1
    SOIL_DAY_HIGH_TIMES		= GRFVP2 + 3677;	// 24 ||  2
    SOIL_MONTH_LOWS		= GRFVP2 + 3825;	//  1 ||  1
    SOIL_MONTH_HIGHS		= GRFVP2 + 3726;	//  1 ||  1
    SOIL_YEAR_LOWS		= GRFVP2 + 3727;	//  1 ||  1
    SOIL_YEAR_HIGHS		= GRFVP2 + 3728;	//  1 ||  1
    SOIL_YEAR_HIGHS_COMP	= GRFVP2 + 3729;	//  1 ||  1

    RX_PERCENTAGE		= GRFVP2 + 3730;	//  24 ||  1
    SAVE_MIN			= RX_PERCENTAGE + 25;   // 4080
    SAVE_HOUR			= SAVE_MIN + 1;
    SAVE_DAY			= SAVE_HOUR + 1;
    SAVE_MONTH			= SAVE_HOUR + 2;
    SAVE_YEAR			= SAVE_HOUR + 3;
    SAVE_YEAR_COMP		= SAVE_HOUR + 4;
    BAUD_RATE			= SAVE_HOUR + 5;
    DEFAULT_RATE_GRAPH		= SAVE_HOUR + 6;
}

#
