/**********************************************************************************/
/*  vproweather 1.0.9                                                             */
/*    A program for acquiring data from a Davis Vantage Pro Weather Station.      */
/*                                                                                */
/* (c)2007-2013 Curt Blank email: curt@curtronics.com                             */
/* VPROWEATHER is covered under the GNU general license. See the attached         */
/* LICENSE file for details.                                                      */
/*                                                                                */
/* This software is provided as-is, without any expressed or implied              */
/* warranties, including, without limitation, the implied warranties of           */
/* merchantibility and fitness for any particular purpose.                        */
/*                                                                                */
/* mySQLhandler.c - connect to MySQL server, using connection parameters          */
/* specified in an option file or on the command line                             */
/*                                                                                */
/*     v0.8-0 cjb  - new                                                          */
/*     v0.8-9 cjb  - clean up multiple compiler warnings that weren't there       */
/*                   before                                                       */
/*     v0.8-43 cjb - add DST to INSERT INTO DT                                    */
/*     v0.9.9  cjb - update StormStarted column in Rain table as soon as Real     */
/*                   Time data indicates a storm has started                      */
/*                 - if a MySQL error occurs and is reported add the query and    */
/*                   location to the report                                       */
/*                 - missing double quote setting ET in the Rain table in         */
/*                   RainUpdate                                                   */
/*                 - improved error reporting when a MySQL query fails for        */
/*                   easier troubleshooting                                       */
/*                 - in RainUpdate only include StormStart on INSERT if the       */
/*                   StormStart date is not currently in the Rain table           */
/*      v1.0.0 cjb - if possible calculate and store _CURRENT_RAIN_COL (New in    */
/*                   DB 11.1) in Rain table using previous and current            */
/*                   StormAmount                                                  */
/*                 - changed _CURRENT_RAIN_COL to INCREMENTAL_RAIN_COL - there    */
/*                   is no way to insure it is the current rain, only that it     */
/*                   is the incremental amount since the last reading             */
/*                 - added _RAIN_STORMS_START_TIME_COL, _RAIN_STORMS_END_TIME_COL */
/*                   in the Storms table so populate them when vPro indicates the */
/*                   storm has ended                                              */
/*                 - fixed a Rain trigger that updates TotalRain in the MHL & YHL */
/*                   tables when a row is deleted from the Rain table             */
/*                 - fixed IncRain calculation. Again. Integer and float math are */
/*                   a PITA but I think I got it this time.                       */
/*                 - fixed IncRain calculation. Again. Intermixed integer and     */
/*                   float math can be PITA but I think (hope) I got it this time */
/*      v1.0.1 cjb - use selected High/Low values because the vPro is collecting  */
/*                   every few seconds and we may only be collecting every minute */
/*      v1.0.3 cjb - add missing string terminator to hlDate                      */
/*      v1.0.5 cjb - added getPW function to replace MySQL get_tty_password       */
/*                   function which some distros strip from the MySQL supplied    */
/*                   library functions                                            */
/*                 - watch out for overflow message when getting graph set lists  */
/*      v1.0.6 cjb - adjust char sizes in HourlyUpdate & MinPtrUpdate due to      */
/*                   Metric conversion option                                     */
/*                 - flag data in the DB that can be US or Metric as such         */
/*                 - removed MySQL password max length restriction when entering  */
/*                   password interactively                                       */
/*      v1.0.8 cjb - modified to handle multiple Vantage Pro weather staions      */
/*                   storing their data in a single MySQL DB                      */
/*                 - MinutesRad & MinutesUV now DECIMAL(6,2) in the DB formatted  */
/*                   as mmm.ss where mmm - minutes & ss = seconds                 */
/*                 - added PerMinute column to Rain table that contains the       */
/*                   calculated value for amount of rain per minute for the       */
/*                   minutes from the current time stored in the DB to the last   */
/*                   time stored                                                  */
/*      v1.0.9 cjb - get and use Storm Start Time to in getStormEndedRecID        */
/*                 - added space for &&T1 in SELECT statement                     */
/*                 - some rain related strings in StormsUpdate were too small for */
/*                   metric conversion                                            */
/*                 - modify maskPassword                                          */
/*                                                                                */
/**********************************************************************************/

#include <stdio.h>
#include <string.h>                     /* for strdup() */
#include <ctype.h>
#include <termios.h>
#include <my_global.h>
#include <my_sys.h>
#include <mysql.h>
#include <my_getopt.h>
#include "include/names.h"
#include "include/dhandler.h"
#include "include/mySQLnames.h"
#include "include/mySQLhandler.h"

/* local functions */
static char *readPW(int nfd, BOOL echo);
char *varAddChar(char *pBuf, char new);
char *varAddString(char *pBuf, char *sNew);
static BOOL mySQLsetup();
static void printSQLerror (MYSQL *mySQLconn, char *message, int loc, char *Query);
static int TemperatureUpdate(unsigned long long int IndexID);
static int BarometerUpdate(unsigned long long int IndexID);
static int WindUpdate(unsigned long long int IndexID);
static int RainUpdate(unsigned long long int IndexID, int vpID);
static int HumidityUpdate(unsigned long long int IndexID);
static int SolarUpdate(unsigned long long int IndexID, int cid);
static int getHLtableIntValue(int cid, char date[], char dateCol[], char table[], char column[]);
static void getHLtableString(int cid, char date[], char dateCol[], char table[], char column[], char value[]);
static unsigned long long int getRecIDDHM(int cid, char hlDate[], char Time[]);
static float getFloatValue(unsigned long long int RecID, char table[], char column[], char index[], int *F);
static int getIntValue(unsigned long long int RecID, char table[], char column[], char index[], int *F);
static unsigned long int getLongIntValue(unsigned long long int RecID, char table[], char column[], char index[], int *F);
static int setFloatValue(unsigned long long int RecID, char table[], char column[], float value);
static int setHLtableIntValue(int cid, char date[], char dateCol[], char table[], char column[], int value);
static int setIntValue(unsigned long long int RecID, char table[], char column[], int value);
static int HighLowUpdate();
static void getDateTimeDT(unsigned long long int RecID, char *Date, char *Time);
static BOOL getIsRaining(unsigned long long int IndexID);
static float getDayAmount(unsigned long long int IndexID);
static float getStormAmount(unsigned long long int IndexID);
static BOOL getStormStartDT(int cid, char forDate[], char *Date, char *Time);
static BOOL getStormEndDT(int cid, char startDate[], char forDate[], char StormAmount[], char *Date, char *Time);
static unsigned long long int checkForStormStart(char *StormStart, int vpID);
static unsigned long long int getStormEndedRecID(int cid, char startDate[], char forDate[], char StormAmount[]);
static BOOL enableProfileSQL(char whereFrom[]);
static BOOL disableProfileSQL(char whereFrom[]);
static void logProfileSQL(char whereFrom[]);
//static int cvtEnglishINT(int value, int cvtType, int F);
static float cvtEnglishFLOAT(float value, int cvtType, int F);

/* local Data */
static MYSQL *mySQLconn = NULL;
static int LoadCnt = 0;
int LastProfileID;
static int star[5] = { '|','/','-','\\','*' };
static char *IndexDT[] = {"RecID", "vpID", "rtCaptureTime", "wxCaptureTime", "rtTT", "wxTT", "bTT", "sTT", "eTT", "rt", '\0'};
static char *DT[] = {"RecID", "vpID", "bDateTime", "DST", "YearMonthDay", "YearMonth", "bDate", "bTime", "HHMM", "bYear", "bMonth", "bDay", "bHour", "bMinute", "bSecond", "bTenths", "bTT", "vpIDorgRecID", '\0'};
static char *YMD[] = {"vpID", "YearMonthDay", "YearMonth", "bDate", "bYear", "bMonth", "bDay", "DayName", _SUNRISE_COL, _SUNSET_COL, _SOLAR_NOON_COL,'\0'};
static char *DHL[] = {"vpID", "YearMonthDay", _THSW_DHL_HI_COL, _THSW_DHL_HI_TIME_COL, _UV_DHL_HI_COL, _UV_DHL_HI_TIME_COL, _DEW_DHL_HI_COL, _DEW_DHL_HI_TIME_COL, _DEW_DHL_LO_COL, _DEW_DHL_LO_TIME_COL,'\0'};
static char *Location[] = {"vpID", _LOCATION_MODEL_COL, _LOCATION_SN_COL, _LOCATION_DESC_COL, _LOCATION_ADDRESS_COL, _LOCATION_CITY_COL, _LOCATION_COUNTY_COL, _LOCATION_STATE_COL, _LOCATION_COUNTRY_COL, _LOCATION_LONG_COL, _LOCATION_LAT_COL, _LOCATION_CONTACT_COL,'\0'};
static char *Temperature[] = {"RecID", _INSIDE_TEMP_COL, _OUTSIDE_TEMP_COL, _WINDCHILL_COL, _HEATINDEX_COL, "F", '\0'};
static char *Humidity[] = {"RecID", _INSIDE_HUM_COL, _OUTSIDE_HUM_COL, '\0'};
static char *Wind[] = {"RecID", _WIND_SPEED_COL, _WIND_DIR_COL, _WIND_DIR_ROSE_COL, "F", '\0'};
static char *Rain[] = {"RecID", _STORM_START_DATE_COL, _IS_RAINING_COL, _INCREMENTAL_RAIN_COL, _PER_MINUTE_RAIN_COL, _DAY_RAIN_COL, _RAIN_STORM_COL, _RAIN_RATE_COL, _DAY_ET_COL, "F", '\0'};
static char *Barometer[] = {"RecID", _BARO_CURR_COL, _BARO_TREND_COL, _BARO_TREND_IMG_COL, "F", '\0'};
static char *Solar[] = {"RecID", _SOLAR_RAD_COL, _SOLAR_RAD_MIN_COL, _UV_COL, _UV_MIN_COL,'\0'};
static char *Storms[] = {"vpID", _RAIN_STORMS_START_DATE_COL, _RAIN_STORMS_START_TIME_COL, _RAIN_STORMS_END_DATE_COL, _RAIN_STORMS_END_TIME_COL, _RAIN_STORMS_AMOUNT_COL, _RAIN_STORMS_HIGH_RATE_COL, _RAIN_STORMS_HIGH_RATE_DT_COL, "F",'\0'};
static char *Hourly[] = {"RecID", _FORECAST_RULE_NUM_COL, _FORECAST_ICON_NUM_COL, _TEMP_BY_HOURS_COL, _IN_TEMP_BY_HOURS_COL, _HUM_BY_HOURS_COL, _IN_HUM_BY_HOURS_COL, _WIND_AVG_BY_HOURS_COL, _WIND_DIR_BY_HOURS_COL, _WIND_HI_BY_HOURS_COL, _BAR_BY_HOURS_COL, _RAIN_BY_HOUR_COL, _RAIN_RATE_BY_HOURS_COL, _CHILL_LO_BY_HOURS_COL, _HEAT_HI_BY_HOURS_COL, _THSW_HI_BY_HOURS_COL, _DEW_BY_HOURS_COL, _SOLAR_BY_HOUR_AVG_COL, _ET_BY_HOUR_COL, "F",'\0'};


/*--------------------------------------------------------------------------
        printSQLerror
----------------------------------------------------------------------------*/
void printSQLerror (MYSQL *mySQLconn, char *message, int loc, char *Query)
{
    ClrSerLock(PID);
    fprintf (stderr, "%s\nQuery %2i: %s\n", message, loc, Query);
    if (mySQLconn != NULL)
    {
#if MYSQL_VERSION_ID >= 40101
    fprintf (stderr, "Error %u (%s): %s\n",
    mysql_errno (mySQLconn), mysql_sqlstate(mySQLconn), mysql_error (mySQLconn));
#else
    fprintf (stderr, "Error %u: %s\n",
    mysql_errno (mySQLconn), mysql_error (mySQLconn));
#endif
    }
}


/*--------------------------------------------------------------------------
        readPW
----------------------------------------------------------------------------*/
char *readPW(int nfd, BOOL echo)
{
    static char *pBuf = NULL;
    char *pTmp = NULL;
    char tmp;

    for (;;) {
        if (read(nfd, &tmp, 1)  != 1) break;
        if (tmp == 3) {
            if (pBuf != NULL) free(pBuf);
            return((char *)-3);
        }
        if (tmp == '\b' || (int)tmp == 127) {
            if (echo) {
                fputs("\b \b",stdout);
                fflush(stdout);
            }
            continue;
        }
        if (tmp == '\n' || tmp == '\r' || tmp == 3) break;
        if (iscntrl(tmp)) continue;
        if (echo) {
            fputc('*',stdout);
            fflush(stdout);
        }
        if (pBuf == NULL) {
           pBuf = getMemPtr(2);
           *pBuf = tmp;
           *(pBuf+1) = '\0';
        } else {
           pTmp = getMemPtr(strlen(pBuf)+1);
           sprintf(pTmp, "%s%c%c", pBuf, tmp, '\0');
           free(pBuf);
           pBuf = pTmp;
        }
    }
    if (pBuf != NULL) *(pBuf+strlen(pBuf)) = '\0';
    return(pBuf);
}


/*--------------------------------------------------------------------------
        getPW
----------------------------------------------------------------------------*/
char *getPW()
{
    struct termios oldtio, newtio;
    char *pwPtr = NULL;

    fputs("Enter password: ",stdout);
    fflush(stdout);

    tcgetattr(fileno(stdin), &oldtio);
    newtio = oldtio;
    newtio.c_lflag &= ~(ECHO | ISIG | ICANON);
    newtio.c_cc[VMIN] = 1;
    newtio.c_cc[VTIME] = 0;
    tcsetattr(fileno(stdin), TCSADRAIN, &newtio);
    pwPtr = readPW(fileno(stdin), isatty(fileno(stdout)));
    tcsetattr(fileno(stdin), TCSADRAIN, &oldtio);
    if (isatty(fileno(stdout))) fputc('\n',stdout);
    if ((long int)pwPtr == -3) controlC(2);

    return(pwPtr);
}


/*--------------------------------------------------------------------------
        DateSQL
----------------------------------------------------------------------------*/
int DateSQL(char * s1)
{
    char myDay[3] = "";
    char myMonth[4] = "";
    char myNewMonth[3] ="";
    char myYear[5] = "";

    sscanf(s1, "%2s-%3s-%4s", myDay, myMonth, myYear);

    if (strcmp(myMonth, "JAN") == 0)
        sprintf(myNewMonth, "%s", "01");
    else if (strcmp(myMonth, "FEB") == 0)
        sprintf(myNewMonth, "%s", "02");
    else if (strcmp(myMonth, "MAR") == 0)
        sprintf(myNewMonth, "%s", "03");
    else if (strcmp(myMonth, "APR") == 0)
        sprintf(myNewMonth, "%s", "04");
    else if (strcmp(myMonth, "MAY") == 0)
        sprintf(myNewMonth, "%s", "05");
    else if (strcmp(myMonth, "JUN") == 0)
        sprintf(myNewMonth, "%s", "06");
    else if (strcmp(myMonth, "JUL") == 0)
        sprintf(myNewMonth, "%s", "07");
    else if (strcmp(myMonth, "AUG") == 0)
        sprintf(myNewMonth, "%s", "08");
    else if (strcmp(myMonth, "SEP") == 0)
        sprintf(myNewMonth, "%s", "09");
    else if (strcmp(myMonth, "OCT") == 0)
        sprintf(myNewMonth, "%s", "10");
    else if (strcmp(myMonth, "NOV") == 0)
        sprintf(myNewMonth, "%s", "11");
    else if (strcmp(myMonth, "DEC") == 0)
        sprintf(myNewMonth, "%s", "12");
    else
        sprintf(myNewMonth, "%s", "??");

    sprintf( s1, "%4s-%2s-%2s", myYear, myNewMonth, myDay);
    s1[strlen(s1)] = '\0';

    if(strcmp(myNewMonth, "??") == 0)
        return FALSE;
    else
        return TRUE;
}


/*--------------------------------------------------------------------------
        getLastIndex
----------------------------------------------------------------------------*/
unsigned long long int getLastIndex(int cid)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    unsigned long long int LastPtr =  0;

    if (mySQLconn == NULL) {
        if (bVerboseSQL) fprintf(stderr, "\ngetLastIndex: setting up mySQLconn\n");
        if (! mySQLsetup()) {
            if (bVerboseSQL) fprintf(stderr, "getLastIndex: setting up mySQLconn failed\n");
            return LastPtr;
        }
        if (bVerboseSQL) fprintf(stderr, "\n");
    }

    sprintf(Query, "SELECT RecID FROM IndexDT WHERE vpID = \"%i\" ORDER BY RecID DESC LIMIT 1", cid);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 1, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 2, Query);
        else {
            LastPtr = 0;
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sscanf(row[0], "%llu", &LastPtr);
            mysql_free_result(res_set);
        }
    }
    if (bVerboseSQL) fprintf(stderr, "Returning RecID %llu\n", LastPtr);
    if (bProfiling) logProfileSQL("getLastIndex");
    return LastPtr;
}


/*--------------------------------------------------------------------------
        chkLocation
----------------------------------------------------------------------------*/
void chkLocation(int cid)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    char Updates[255];
    char value[128];
    char *ErrMsg = NULL;
    int cpos;
    unsigned long int rowsReturned;
    double retLongitude = -1000;
    double retLatitude = -1000;
    extern double latitude;
    extern double longitude;

    if (bVerboseSQL) fprintf(stderr, "\nChecking Location %s\n", getCurTime());

    sprintf(Query, "SELECT vpID, Longitude, Latitude FROM Location WHERE vpID = \"%i\"", cid);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 3, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 4, Query);
        else {
            rowsReturned = mysql_num_rows(res_set);
            if (rowsReturned > 1) {
                mysql_free_result(res_set);
                ErrMsg = varAddString(ErrMsg, "too many rows returned looking for RecID (");
                sprintf(value, "%lu", rowsReturned);
                ErrMsg = varAddString(ErrMsg, value);
                ErrMsg = varAddString(ErrMsg, ")");
                printSQLerror(mySQLconn, ErrMsg, 5, Query);
                free(ErrMsg);
                ErrMsg = NULL;
            }
            if (res_set != NULL) {
                if ((row = mysql_fetch_row (res_set)) != NULL) {
                    if (row[1] != NULL) sscanf(row[1], "%lf", &retLongitude);
                    if (row[2] != NULL) sscanf(row[2], "%lf", &retLatitude);
                    Updates[0] = '\0';
                    if (longitude!= -1000.0 && longitude != retLongitude) {
                        sprintf(value, "Longitude = \"%lf\", ", longitude);
                        strcat(Updates, value);
                        LoadCnt++;
                    }
                    if (latitude != -1000.0 && latitude != retLatitude) {
                        sprintf(value, "Latitude = \"%lf\"", latitude);
                        strcat(Updates, value);
                        LoadCnt++;
                    }
                    if (Updates[0] != '\0') {
                        cpos = strlen(Updates) - 1;
                        while(cpos >= 0 && (Updates[cpos] == ' ' || Updates[cpos] == ','))
                            Updates[cpos--] = '\0';
                        sprintf(Query, "UPDATE Location SET %s WHERE vpID = \"%i\"", Updates, cid);
                        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                        if (mysql_query(mySQLconn, Query) != 0)
                            printSQLerror(mySQLconn, "mysql_query() failed", 6, Query);
                        else if (bVerboseSQL)
                            fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                    }
                    mysql_free_result(res_set);
                } else {
                    if (latitude != -1000.0 && longitude!= -1000.0)
                        sprintf(Query,"INSERT INTO Location (vpID, Longitude, Latitude) VALUES (%i, %lf, %lf)", cid, longitude, latitude);
                    else
                        sprintf(Query,"INSERT INTO Location (vpID) VALUES (%i)", cid);
                    LoadCnt++;
                    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                    if (mysql_query(mySQLconn, Query) != 0)
                        printSQLerror(mySQLconn, "mysql_query() failed", 7, Query);
                    else if (bVerboseSQL)
                        fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                }
            }
        }
    }
}


/*--------------------------------------------------------------------------
        setupRecID
----------------------------------------------------------------------------*/
unsigned long long int setupRecID(int cid, time_t TT)
{

    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char DateTime[20];
    char Query[1024];
    char Update[512];
    char Attr[255];
    char Values[255];
    char value[128];
    char retDateTime[20];
    char dateSQL[11];
    char timeSQL[9];
    char DayName[10];
    char Sunrise[26];
    char Sunset[26];
    char Solarnoon[11];
    char retDayName[10];
    char retSunrise[6];
    char retSunset[6];
    char retSolarnoon[9];
    char ttC[20];
    char cidC[4];
    char *pos;
    char *ErrMsg = NULL;
    char *Wheres = NULL;
    char *Indexes = NULL;
    int cpos;
    int DST = -1;
    BOOL usedWheres = FALSE;
    unsigned long long int InsertPtr = 0;
    unsigned long int rowsReturned;
    struct tm DT;

    if (bVerboseSQL) fprintf(stderr, "\nsetupRecID %s\n", getCurTime());

    Query[0] = '\0';
    Attr[0] = '\0';
    Values[0] = '\0';

    DT = *(localtime(&TT));
    DT.tm_year += 1900;
    DT.tm_mon += 1;

    DST = DT.tm_isdst;
    strftime(DayName, 10, "%A", &DT);
    sprintf(dateSQL, "%04u-%02u-%02u", DT.tm_year, DT.tm_mon, DT.tm_mday);
    sprintf(timeSQL, "%02u:%02u:%02u", DT.tm_hour, DT.tm_min, DT.tm_sec);
    sprintf(DateTime, "%10s %8s", dateSQL, timeSQL);

//    if (bVerboseSQL) fprintf(stderr, "DT.tm_year %4i DT.tm_mon %02i DT.tm_mday %02i DayName \"%s\"\nDT.tm_hour %02i DT.tm_min %02i DT.tm_sec %02i\nDateTime \"%s\"\ndateSQL %s timeSQL %s\n", DT.tm_year, DT.tm_mon, DT.tm_mday, DayName, DT.tm_hour, DT.tm_min, DT.tm_sec, DateTime, dateSQL, timeSQL);

    sprintf(Query, "SELECT bDateTime, RecID FROM DT USE INDEX (PRIMARY) WHERE bDateTime = \"%s\" && DST = \"%i\" && vpID = \"%i\"", DateTime, DST, cid);
    if (bVerboseSQL) fprintf(stderr, "\nQuery: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 8, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 9, Query);
        else {
            rowsReturned = mysql_num_rows(res_set);
            if (rowsReturned > 1) {
                mysql_free_result(res_set);
                ErrMsg = varAddString(ErrMsg, "too many rows returned looking for RecID (");
                sprintf(value, "%lu", rowsReturned);
                ErrMsg = varAddString(ErrMsg, value);
                ErrMsg = varAddString(ErrMsg, ")");
                printSQLerror(mySQLconn, ErrMsg, 10, Query);
                free(ErrMsg);
                ErrMsg = NULL;
            }
            retDateTime[0] = '\0';
            InsertPtr = 0;
            if (res_set != NULL) {
                if ((row = mysql_fetch_row (res_set)) != NULL) {
                    if (row[0] != NULL) sprintf(retDateTime, "%s", row[0]);	/* need to use sprintf if there's spaces in the string */
                    if (row[1] != NULL) sscanf(row[1], "%llu", &InsertPtr);
                }
                mysql_free_result(res_set);
            }
            if (bVerboseSQL && InsertPtr != 0) fprintf(stderr, "got (1) ");
        }
    }

    if (InsertPtr == 0 && rtDateTimeCT[0] != '\0') {
        sprintf(Query, "SELECT T1.RecID FROM IndexDT AS T1 USE INDEX (rtCaptureTime) INNER JOIN DT AS T2 USE INDEX (PRIMARY,HHMM) USING (RecID) WHERE T1.rtCaptureTime = \"%s\" && T2.HHMM = \"%02u:%02u\" && T2.DST = \"%i\" && T2.vpID = \"%i\"", DateTime, DT.tm_hour, DT.tm_min, DST, cid);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 11, Query);
        else {
            res_set = mysql_store_result(mySQLconn);
            if (res_set == NULL)
                printSQLerror(mySQLconn, "mysql_store_result() failed", 12, Query);
            else {
                rowsReturned = mysql_num_rows(res_set);
                if (rowsReturned > 1) {
                    mysql_free_result(res_set);
                    ErrMsg = varAddString(ErrMsg, "too many rows returned looking for RecID (");
                    sprintf(value, "%lu", rowsReturned);
                    ErrMsg = varAddString(ErrMsg, value);
                    ErrMsg = varAddString(ErrMsg, ")");
                    printSQLerror(mySQLconn, ErrMsg, 13, Query);
                    free(ErrMsg);
                    ErrMsg = NULL;
                }
                retDateTime[0] = '\0';
                InsertPtr = 0;
                if (res_set != NULL) {
                    if ((row = mysql_fetch_row (res_set)) != NULL)
                        if (row[0] != NULL) sscanf(row[0], "%llu", &InsertPtr);
                    mysql_free_result(res_set);
                }
                if (bVerboseSQL && InsertPtr != 0) fprintf(stderr, "got (2) ");
            }
        }

    }

    if (InsertPtr == 0 && grDateTimeCT[0] != '\0') {
        sprintf(Query, "SELECT T1.RecID FROM IndexDT AS T1 USE INDEX (wxCaptureTime) INNER JOIN DT AS T2 USE INDEX (PRIMARY,HHMM) USING (RecID) WHERE T1.wxCaptureTime = \"%s\" && T2.HHMM = \"%02u:%02u\" && T2.DST = \"%i\" && T2.vpID = \"%i\"", DateTime, DT.tm_hour, DT.tm_min, DST, cid);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 14, Query);
        else {
            res_set = mysql_store_result(mySQLconn);
            if (res_set == NULL)
                printSQLerror(mySQLconn, "mysql_store_result() failed", 15, Query);
            else {
                rowsReturned = mysql_num_rows(res_set);
                if (rowsReturned > 1) {
                    mysql_free_result(res_set);
                    ErrMsg = varAddString(ErrMsg, "too many rows returned looking for RecID (");
                    sprintf(value, "%lu", rowsReturned);
                    ErrMsg = varAddString(ErrMsg, value);
                    ErrMsg = varAddString(ErrMsg, ")");
                    printSQLerror(mySQLconn, ErrMsg, 16, Query);
                    free(ErrMsg);
                    ErrMsg = NULL;
                }
                retDateTime[0] = '\0';
                InsertPtr = 0;
                if (res_set != NULL) {
                    if ((row = mysql_fetch_row (res_set)) != NULL)
                        if (row[0] != NULL) sscanf(row[0], "%llu", &InsertPtr);
                    mysql_free_result(res_set);
                }
                if (bVerboseSQL && InsertPtr != 0) fprintf(stderr, "got (3) ");
            }
        }
    }

    if (bVerboseSQL) {
        if (InsertPtr == 0) fprintf(stderr, "\n");
        fprintf(stderr, "InsertPtr %llu DateTime %s\n", InsertPtr, DateTime);
    }
    if (InsertPtr == 0 && DateTime != NULL) {
        if (rtDateTimeCT[0] == '\0' && grDateTimeCT[0] == '\0') {
            if (TT > 0) {
                sprintf(Query, "INSERT INTO IndexDT (rtCaptureTime, wxCaptureTime, bTT, sTT) VALUES (NULL, NULL, %lu, %lu)", TT, ttRun);
                sprintf(ttC, "%llu", (unsigned long long)TT);
                pos = strstr( ttC, " ");
                if (pos != NULL) *pos = '\0';
                Wheres = varAddString(Wheres, "bTT = \"");
                Wheres = varAddString(Wheres, ttC);
                Wheres = varAddString(Wheres, "\" && ");
                Indexes = varAddString(Indexes, "bTT,");
            } else {
                sprintf(Query, "INSERT INTO IndexDT (rtCaptureTime, wxCaptureTime, sTT) VALUES (NULL, NULL, %lu)", ttRun);
                sprintf(ttC, "%llu", (unsigned long long)ttRun);
                pos = strstr( ttC, " ");
                if (pos != NULL) *pos = '\0';
                Wheres = varAddString(Wheres, "sTT = \"");
                Wheres = varAddString(Wheres, ttC);
                Wheres = varAddString(Wheres, "\"");
                Indexes = varAddString(Indexes, "sTT");
            }
        } else {
            Attr[0] = '\0';
            Values[0] = '\0';

            strcat(Attr, "vpID, ");
            strcat(Values, "\"");
            sprintf(cidC, "%i", cid);
            pos = strstr( cidC, " ");
            if (pos != NULL) *pos = '\0';
            strcat(Values, cidC);
            strcat(Values, "\", ");

            if (rtDateTimeCT[0] != '\0') {
                strcat(Attr, "rtCaptureTime, ");
                strcat(Values, "\"");
                strcat(Values, rtDateTimeCT);
                strcat(Values, "\", ");
                Wheres = varAddString(Wheres, "rtCaptureTime = \"");
                Wheres = varAddString(Wheres, rtDateTimeCT);
                Wheres = varAddString(Wheres, "\" && ");
                Indexes = varAddString(Indexes, "rtCaptureTime,");
            }
            if (grDateTimeCT[0] != '\0') {
                strcat(Attr, "wxCaptureTime, ");
                strcat(Values, "\"");
                strcat(Values, grDateTimeCT);
                strcat(Values, "\", ");
                Wheres = varAddString(Wheres, "wxCaptureTime = \"");
                Wheres = varAddString(Wheres, grDateTimeCT);
                Wheres = varAddString(Wheres, "\" && ");
                Indexes = varAddString(Indexes, "wxCaptureTime,");
            }
            if (ttrt > 0) {
                strcat(Attr, "rtTT, ");
                strcat(Values, "\"");
                sprintf(ttC, "%llu", (unsigned long long)ttrt);
                pos = strstr(ttC, " ");
                if (pos != NULL) *pos = '\0';
                strcat(Values, ttC);
                strcat(Values, "\", ");
                Wheres = varAddString(Wheres, "rtTT = \"");
                Wheres = varAddString(Wheres, ttC);
                Wheres = varAddString(Wheres, "\" && ");
                Indexes = varAddString(Indexes, "rtTT,");
            }
            if (ttgr > 0) {
                strcat(Attr, "wxTT, ");
                strcat(Values, "\"");
                sprintf(ttC, "%llu", (unsigned long long)ttgr);
                pos = strstr(ttC, " ");
                if (pos != NULL) *pos = '\0';
                strcat(Values, ttC);
                strcat(Values, "\", ");
                Wheres = varAddString(Wheres, "wxTT = \"");
                Wheres = varAddString(Wheres, ttC);
                Wheres = varAddString(Wheres, "\" && ");
                Indexes = varAddString(Indexes, "wxTT,");
            }
            if (TT > 0) {
                strcat(Attr, "bTT, ");
                strcat(Values, "\"");
                sprintf(ttC, "%llu", (unsigned long long)TT);
                pos = strstr( ttC, " ");
                if (pos != NULL) *pos = '\0';
                strcat(Values, ttC);
                strcat(Values, "\", ");
                Wheres = varAddString(Wheres, "bTT = \"");
                Wheres = varAddString(Wheres, ttC);
                Wheres = varAddString(Wheres, "\" && ");
                Indexes = varAddString(Indexes, "bTT,");
            }
            strcat(Attr, "sTT");
            strcat(Values, "\"");
            sprintf(ttC, "%llu", (unsigned long long)ttRun);
            pos = strstr( ttC, " ");
            if (pos != NULL) *pos = '\0';
            strcat(Values, ttC);
            strcat(Values, "\"");
            Wheres = varAddString(Wheres, "sTT = \"");
            Wheres = varAddString(Wheres, ttC);
            Wheres = varAddString(Wheres, "\"");
            Indexes = varAddString(Indexes, "sTT,");

            cpos = strlen(Attr) - 1;
            while(cpos >= 0 && (Attr[cpos] == ' ' || Attr[cpos] == ','))
                Attr[cpos--] = '\0';
            cpos = strlen(Values) - 1;
            while(cpos >= 0 && (Values[cpos] == ' ' || Values[cpos] == ','))
                Values[cpos--] = '\0';
            sprintf(Query, "INSERT INTO IndexDT (%s) VALUES (%s)", Attr, Values);
        }
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
        if (mysql_query(mySQLconn, Query) != 0) {
            printSQLerror(mySQLconn, "mysql_query() failed", 17, Query);
        } else if (bVerboseSQL) {
            fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
        }

        /* setup time */
        res_set = mysql_store_result(mySQLconn);
        if (res_set != NULL) mysql_free_result(res_set);

        cpos = strlen(Wheres) - 1;
        while(cpos >= 0 && (Wheres[cpos] == ' ' || Wheres[cpos] == '&'))
            Wheres[cpos--] = '\0';
        cpos = strlen(Indexes) - 1;
        while(cpos >= 0 && (Indexes[cpos] == ' ' || Indexes[cpos] == ','))
            Indexes[cpos--] = '\0';
        if (Wheres != NULL) {
            usedWheres = TRUE;
            sprintf(Query, "SELECT RecID FROM IndexDT USE INDEX (%s) WHERE vpID = \"%i\" && %s", Indexes, cid, Wheres);
            if (bVerboseSQL) {
                fprintf(stderr, "Query: %s\n", Query);
                usedWheres = TRUE;
            }
            if (mysql_query(mySQLconn, Query) != 0)
                printSQLerror(mySQLconn, "mysql_query() failed", 18, Query);
            else {
                res_set = mysql_store_result(mySQLconn);
                if (res_set == NULL)
                    printSQLerror(mySQLconn, "mysql_store_result() failed", 19, Query);
                else {
                    rowsReturned = mysql_num_rows(res_set);
                    InsertPtr = 0;
                    if (rowsReturned > 1) {
                        mysql_free_result(res_set);
                        ErrMsg = varAddString(ErrMsg, "too many rows returned looking for new RecID (");
                        sprintf(value, "%lu", rowsReturned);
                        ErrMsg = varAddString(ErrMsg, value);
                        ErrMsg = varAddString(ErrMsg, ")");
                        printSQLerror(mySQLconn, ErrMsg, 20, Query);
                        free(ErrMsg);
                        ErrMsg = NULL;
                        fprintf(stderr, "Continuing...\n");
                    } else if (res_set != NULL) {
                        if ((row = mysql_fetch_row (res_set)) != NULL) {
                            if (row[0] != NULL) sscanf(row[0], "%llu", &InsertPtr);
                        }
                        mysql_free_result(res_set);
                    }
                    if (bVerboseSQL && InsertPtr != 0) fprintf(stderr, "got InsertPtr for DT %llu\n", InsertPtr);
                }
            }
        }
        if (Wheres != NULL) {
            free(Wheres);
            Wheres = NULL;
        }
        if (Indexes != NULL) {
            free(Indexes);
            Indexes = NULL;
        }

        if (bVerboseSQL && ! usedWheres) fprintf(stderr, "InsertPtr %llu\n", InsertPtr);
        if (InsertPtr > 0) {
            sprintf(Query, "INSERT INTO DT (bDateTime, DST, RecID, vpID, YearMonthDay, YearMonth, bDate, bTime, HHMM, bYear, bMonth, bDay, bHour, bMinute, bSecond, bTT) VALUES (\"%s\", \"%i\", \"%llu\", \"%i\", \"%4u%02u%02u\", \"%4u%02u\", \"%s\", \"%s\", \"%02u:%02u\", \"%4u\", \"%02u\", \"%02u\", \"%02u\", \"%02u\", \"%02u\", \"%llu\")", DateTime, DST, InsertPtr, cid, DT.tm_year, DT.tm_mon, DT.tm_mday, DT.tm_year, DT.tm_mon, dateSQL, timeSQL, DT.tm_hour, DT.tm_min, DT.tm_year, DT.tm_mon, DT.tm_mday, DT.tm_hour, DT.tm_min, DT.tm_sec, (unsigned long long)TT);
            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
            if (mysql_query(mySQLconn, Query) != 0)
                 printSQLerror(mySQLconn, "mysql_query() failed", 21, Query);
            else if (bVerboseSQL)
                fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));

            Sunrise[0] = '\0';
            Sunset[0] = '\0';
            Solarnoon[0] = '\0';
            if (bCalcSun) {
                if (SunRise != 0) sprintf(Sunrise, "%5s", SunRise);
                if (SunSet != 0) sprintf(Sunset, "%5s", SunSet);
                if (SolarNoon != 0) sprintf(Solarnoon, "%8s", SolarNoon);
                pos = strstr(Sunrise, ":");		/* may only contain the date, no time, we don't want that */
                if (pos == NULL)
                    Sunrise[0] = '\0';
                else {
                    pos = strstr(Sunrise, "-");		/* may include the date, we only want the time */
                    if (pos != NULL) *pos = '\0';
                }					/* check use VP's value if necessary */
                if (Sunrise[0] == '\0') sprintf(Sunrise, "%5s", TimeConvert(rcd.wSunrise));
                pos = strstr(Sunset, ":");		/* may only contain the date, no time, we don't want that */
                if (pos == NULL)
                    Sunset[0] = '\0';
                else {
                    pos = strstr(Sunset, "-");		/* may include the date, we only want the time */
                    if (pos != NULL) *pos = '\0';
                }					/* check use VP's value if necessary */
                if (Sunset[0] == '\0') sprintf(Sunset, "%5s", TimeConvert(rcd.wSunset));
            }
            if (Sunrise[0] == '\0' && rcd.wSunrise != 0) sprintf(Sunrise, "%5s", TimeConvert(rcd.wSunrise));
            if (Sunset[0] == '\0' && rcd.wSunset != 0) sprintf(Sunset, "%5s", TimeConvert(rcd.wSunset));

            sprintf(Query, "SELECT DayName, %s, %s, %s FROM YMD USE INDEX (PRIMARY) WHERE YearMonthDay = \"%4i%02i%02i\" && vpID = \"%i\"", _SUNRISE_COL, _SUNSET_COL, _SOLAR_NOON_COL, DT.tm_year, DT.tm_mon, DT.tm_mday, cid);
            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
            if (mysql_query(mySQLconn, Query) != 0)
                printSQLerror(mySQLconn, "mysql_query() failed", 22, Query);
            else {
                res_set = mysql_store_result(mySQLconn);
                if (res_set == NULL)
                    printSQLerror(mySQLconn, "mysql_store_result() failed", 23, Query);
                else {
                    retDayName[0] = '\0';
                    retSunrise[0] = '\0';
                    retSunset[0] = '\0';
                    retSolarnoon[0] = '\0';
                    if ((row = mysql_fetch_row (res_set)) != NULL) {
                        if (row[0] != NULL) sscanf(row[0], "%9s", retDayName);
                        if (row[1] != NULL) sscanf(row[1], "%5s", retSunrise);
                        if (row[2] != NULL) sscanf(row[2], "%5s", retSunset);
                        if (row[3] != NULL) sscanf(row[3], "%8s", retSolarnoon);
                    }
                    mysql_free_result(res_set);
                }
            }
            Update[0] = '\0';
            if (retDayName[0] == '\0' && DayName[0] != '\0') {
                strcat(Update, "DayName = \"");
                strcat(Update, DayName);
                strcat(Update, "\", ");
            }
#ifdef _SUNRISE_COL
            if (Sunrise[0] != '\0' && strcmp(Sunrise, retSunrise) != 0) {
                strcat(Update, _SUNRISE_COL);
                strcat(Update, " = \"");
                strcat(Update, Sunrise);
                strcat(Update, "\", ");
            }
#endif
#ifdef _SUNSET_COL
            if (Sunset[0] != '\0' && strcmp(Sunset, retSunset) != 0) {
                strcat(Update, _SUNSET_COL);
                strcat(Update, " = \"");
                strcat(Update, Sunset);
                strcat(Update, "\", ");
            }
#endif
#ifdef _SOLAR_NOON_COL
            if (bCalcSun && Solarnoon[0] != '\0' && strcmp(Solarnoon, retSolarnoon) != 0) {
                strcat(Update, _SOLAR_NOON_COL);
                strcat(Update, " = \"");
                strcat(Update, Solarnoon);
                strcat(Update, "\", ");
            }
#endif
            cpos = strlen(Update) - 1;
            while(cpos >= 0 && (Update[cpos] == ' ' || Update[cpos] == ','))
                Update[cpos--] = '\0';

            if (Update[0] != '\0') {
                sprintf(Query, "UPDATE YMD SET %s WHERE YearMonthDay = \"%4i%02i%02i\" && vpID = \"%i\"", Update, DT.tm_year, DT.tm_mon, DT.tm_mday, cid);
                if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                if (mysql_query(mySQLconn, Query) != 0)
                    printSQLerror(mySQLconn, "mysql_query() failed", 24, Query);
                else if (bVerboseSQL)
                    fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
            }
        }
    }
    if (bVerboseSQL && InsertPtr == 0) fprintf(stderr, "InsertPtr %llu\n", InsertPtr);
    if (bProfiling) logProfileSQL("setupRecID");
    if (InsertPtr == 0) {
        fprintf(stderr, "\nERROR: can't get RecID, exiting....\n\n");
        ClrSerLock(PID);
        mysql_close (mySQLconn);
        exit(2);
    }
    return InsertPtr;
}


/*--------------------------------------------------------------------------
        HourlyUpdate
----------------------------------------------------------------------------*/
int HourlyUpdate(unsigned long long int IndexID, char * pData, int cid)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    time_t curSec;
    struct tm DT;
    char myYMD[9] = "";
    char Query[1024];
    char sAttr[384];
    char Attr[384];
    char Values[384];
    char Updates[786];
    unsigned long long int retRecID = 0;
    unsigned long long int newRecID = 0;
    char InsertDT[20];
    char value[128];
    time_t mtt;
    char Times[144] = "";
    char InTempS[125] = "";
    char OutTempS[125] = "";
    char WindChillS[125] = "";
    char HeatIndexS[125] = "";
    char InHumS[125] = "";
    char OutHumS[125] = "";
    char BaroS[200] = "";
    char WindSpeedS[125] = "";
    char WindDirS[125] = "";
    char WindHighS[125] = "";
    char RainS[225] = "";
    char RainRateS[225] = "";
    char DewPointS[125] = "";
    char THSWS[125] = "";
    char SolarS[150] = "";
    char ETS[175] = "";
    char Time[6] = "";
    char InTemp[6] = "";
    char OutTemp[6] = "";
    char WindChill[6] = "";
    char HeatIndex[6] = "";
    char InHum[6] = "";
    char OutHum[6] = "";
    char Baro[9] = "";
    char WindSpeed[6] = "";
    char WindDir[6] = "";
    char WindHigh[6] = "";
    char Rain[10] = "";
    char RainRate[10] = "";
    char DewPoint[6] = "";
    char THSW[6] = "";
    char Solar[7] ="";
    char ET[8] = "";
    char retInTemp[6] = "";
    char retOutTemp[6] = "";
    char retWindChill[6] = "";
    char retHeatIndex[6] = "";
    char retInHum[6] = "";
    char retOutHum[6] = "";
    char retBaro[9] = "";
    char retWindSpeed[6] = "";
    char retWindDir[6] = "";
    char retWindHigh[6] = "";
    char retRain[10] = "";
    char retRainRate[10] = "";
    char retDewPoint[6] = "";
    char retTHSW[6] = "";
    char retSolar[7] ="";
    char retET[8] = "";
    int TTpos, ITpos, OTpos, WCpos, HIpos, IHpos, OHpos, BRpos, WSpos, WDpos, WHpos, RApos, RRpos, DPpos, TIpos, SRpos, ETpos;
    int cpos, ROWpos;

    if (bVerboseSQL) fprintf(stderr, "\nProcessing Hourly... %s\n", getCurTime());
    sprintf(Query, "SELECT YearMonthDay FROM DT USE INDEX (RecID) WHERE RecID = \"%llu\"", IndexID);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 25, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 26, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sscanf(row[0], "%s", myYMD);
            mysql_free_result(res_set);
        }
    }

    if (*myYMD == '\0' || myYMD == NULL) {
        fprintf(stderr, "ERROR! from HourlyUpdate could not get YearMonthDay for %llu\n", IndexID);
        return FALSE;
    }
    if (sscanf(myYMD, "%4u%2u%2u", &DT.tm_year, &DT.tm_mon, &DT.tm_mday) == EOF) {
        fprintf(stderr, "ERROR! parsing %s\n", myYMD);
        return 0;
    }

    DT = *(localtime(&ttRun));

    DT.tm_hour = 0;
    DT.tm_min = 0;
    DT.tm_sec = 0;

    DT.tm_mday--;		/* for starters we want yesterdays date */

    /* The time in the Times list is yesterday until 00:00 is encountered, from then on it is today */
    time(&mtt);
    sprintf(Times, "%s", TimeRefHours(mtt));

    Attr[0] = '\0';
    sAttr[0] = '\0';
#ifdef _IN_TEMP_BY_HOURS_COL
    sprintf(InTempS, "%s", PrintTempSet24((unsigned char *)pData, TEMP_IN_HOUR, *(pData + NEXT_HOUR_PTR), 90));
    BOOL InTempSValid = TRUE;
    if (strcmp(InTempS, _OVERFLOW_MSG) == 0) {
        InTempSValid = FALSE;
        fprintf(stderr, "graph data %-10s %s\n", "InTempS", _OVERFLOW_MSG);
    } else {
        sprintf(Attr, "%s, ", _IN_TEMP_BY_HOURS_COL);
        strcat(sAttr, Attr);
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","InTempS");
    }
#endif
#ifdef _TEMP_BY_HOURS_COL
    sprintf(OutTempS, "%s", PrintTempSet24((unsigned char *)pData, TEMP_OUT_HOUR, *(pData + NEXT_HOUR_PTR), 90));
    BOOL OutTempSValid = TRUE;
    if (strcmp(OutTempS, _OVERFLOW_MSG) == 0) {
        OutTempSValid = FALSE;
        fprintf(stderr, "graph data %-10s %s\n", "OutTempS", _OVERFLOW_MSG);
    } else {
        sprintf(Attr, "%s, ", _TEMP_BY_HOURS_COL);
        strcat(sAttr, Attr);
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","OutTempS");
    }
#endif
#ifdef _CHILL_LO_BY_HOURS_COL
    sprintf(WindChillS, "%s", PrintTempSet24((unsigned char *)pData, CHILL_HOUR, *(pData + NEXT_HOUR_PTR), 120));
    BOOL WindChillSValid = TRUE;
    if (strcmp(WindChillS, _OVERFLOW_MSG) == 0) {
        WindChillSValid = FALSE;
        fprintf(stderr, "graph data %-10s %s\n", "WindChillS", _OVERFLOW_MSG);
    } else {
        sprintf(Attr, "%s, ", _CHILL_LO_BY_HOURS_COL);
        strcat(sAttr, Attr);
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","WindChillS");
    }
#endif
#ifdef _HEAT_HI_BY_HOURS_COL
    sprintf(HeatIndexS, "%s", PrintTempSet24((unsigned char *)pData, HEAT_HOUR, *(pData + NEXT_HOUR_PTR), 90));
    BOOL HeatIndexSValid = TRUE;
    if (strcmp(HeatIndexS, _OVERFLOW_MSG) == 0) {
        HeatIndexSValid = FALSE;
        fprintf(stderr, "graph data %-10s %s\n", "HeatIndexS", _OVERFLOW_MSG);
    } else {
        sprintf(Attr, "%s, ", _HEAT_HI_BY_HOURS_COL);
        strcat(sAttr, Attr);
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","HeatIndexS");
    }
#endif
#ifdef _IN_HUM_BY_HOURS_COL
    sprintf(InHumS, "%s", PrintByteSet((unsigned char *)pData, HUM_IN_HOUR, *(pData + NEXT_HOUR_PTR), 24, _CVT_NONE));
    BOOL InHumSValid = TRUE;
    if (strcmp(InHumS, _OVERFLOW_MSG) == 0) {
        InHumSValid = FALSE;
        fprintf(stderr, "graph data %-10s %s\n", "InHumS", _OVERFLOW_MSG);
    } else {
        sprintf(Attr, "%s, ", _IN_HUM_BY_HOURS_COL);
        strcat(sAttr, Attr);
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","InHumS");
    }
#endif
#ifdef _HUM_BY_HOURS_COL
    sprintf(OutHumS, "%s", PrintByteSet((unsigned char *)pData, HUM_OUT_HOUR, *(pData + NEXT_HOUR_PTR), 24, _CVT_NONE));
    BOOL OutHumSValid = TRUE;
    if (strcmp(OutHumS, _OVERFLOW_MSG) == 0) {
        OutHumSValid = FALSE;
        fprintf(stderr, "graph data %-10s %s\n", "OutHumS", _OVERFLOW_MSG);
    } else {
        sprintf(Attr, "%s, ", _HUM_BY_HOURS_COL);
        strcat(sAttr, Attr);
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","OutHumS");
    }
#endif
#ifdef _WIND_AVG_BY_HOURS_COL
    sprintf(WindSpeedS, "%s", PrintByteSet((unsigned char *)pData, WIND_SPEED_HOUR_AVG, *(pData + NEXT_HOUR_PTR), 24, _CVT_WIND));
    BOOL WindSpeedSValid = TRUE;
    if (strcmp(WindSpeedS, _OVERFLOW_MSG) == 0) {
        WindSpeedSValid = FALSE;
        fprintf(stderr, "graph data %-10s %s\n", "WindSpeedS", _OVERFLOW_MSG);
    } else {
        sprintf(Attr, "%s, ", _WIND_AVG_BY_HOURS_COL);
        strcat(sAttr, Attr);
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","WindSpeedS");
    }
#endif
#ifdef _WIND_DIR_BY_HOURS_COL
    sprintf(WindDirS, "%s", PrintByteSet((unsigned char *)pData, WIND_DIR_HOUR, *(pData + NEXT_HOUR_PTR), 24, _CVT_NONE));
    BOOL WindDirSValid = TRUE;
    if (strcmp(WindDirS, _OVERFLOW_MSG) == 0) {
        WindDirSValid = FALSE;
        fprintf(stderr, "graph data %-10s %s\n", "WindDirS", _OVERFLOW_MSG);
    } else {
        sprintf(Attr, "%s, ", _WIND_DIR_BY_HOURS_COL);
        strcat(sAttr, Attr);
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","WindDirS");
    }
#endif
#ifdef _WIND_HI_BY_HOURS_COL
    sprintf(WindHighS, "%s", PrintByteSet((unsigned char *)pData, WIND_SPEED_HOUR_HIGHS, *(pData + NEXT_HOUR_PTR), 24, _CVT_WIND));
    BOOL WindHighSValid = TRUE;
    if (strcmp(WindHighS, _OVERFLOW_MSG) == 0) {
        WindHighSValid = FALSE;
        fprintf(stderr, "graph data %-10s %s\n", "WindHighS", _OVERFLOW_MSG);
    } else {
        sprintf(Attr, "%s, ", _WIND_HI_BY_HOURS_COL);
        strcat(sAttr, Attr);
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","WindHighS");
    }
#endif
#ifdef _DEW_BY_HOURS_COL
    sprintf(DewPointS, "%s", PrintTempSet24((unsigned char *)pData, DEW_HOUR, *(pData + NEXT_HOUR_PTR), 120));
    BOOL DewPointSValid = TRUE;
    if (strcmp(DewPointS, _OVERFLOW_MSG) == 0) {
        DewPointSValid = FALSE;
        fprintf(stderr, "graph data %-10s %s\n", "DewPointS", _OVERFLOW_MSG);
    } else {
        sprintf(Attr, "%s, ", _DEW_BY_HOURS_COL);
        strcat(sAttr, Attr);
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","DewPointS");
    }
#endif
#ifdef _THSW_HI_BY_HOURS_COL
    sprintf(THSWS, "%s", PrintTempSet24((unsigned char *)pData, THSW_HOUR, *(pData + NEXT_HOUR_PTR), 90));
    BOOL THSWSValid = TRUE;
    if (strcmp(THSWS, _OVERFLOW_MSG) == 0) {
        THSWSValid = FALSE;
        fprintf(stderr, "graph data %-10s %s\n", "THSWS", _OVERFLOW_MSG);
    } else {
        sprintf(Attr, "%s, ", _THSW_HI_BY_HOURS_COL);
        strcat(sAttr, Attr);
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","THSWS");
    }
#endif
#ifdef _SOLAR_BY_HOUR_AVG_COL
    sprintf(SolarS, "%s", PrintSolarSet((unsigned char *)pData, SOLAR_HOUR_AVG, *(pData + NEXT_HOUR_PTR), 24));
    BOOL SolarSValid = TRUE;
    if (strcmp(SolarS, _OVERFLOW_MSG) == 0) {
        SolarSValid = FALSE;
        fprintf(stderr, "graph data %-10s %s\n", "SolarS", _OVERFLOW_MSG);
    } else {
        sprintf(Attr, "%s, ", _SOLAR_BY_HOUR_AVG_COL);
        strcat(sAttr, Attr);
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","SolarS");
    }
#endif
#ifdef _BAR_BY_HOURS_COL
    sprintf(BaroS, "%s", PrintBarSet((unsigned char *)pData, BAR_HOUR, *(pData + NEXT_HOUR_PTR), 24));
    BOOL BaroSValid = TRUE;
    if (strcmp(BaroS, _OVERFLOW_MSG) == 0) {
        BaroSValid = FALSE;
        fprintf(stderr, "graph data %-10s %s\n", "BaroS", _OVERFLOW_MSG);
    } else {
        sprintf(Attr, "%s, ", _BAR_BY_HOURS_COL);
        strcat(sAttr, Attr);
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","BaroS");
    }
#endif
#ifdef _ET_BY_HOUR_COL
    sprintf(ETS, "%s", PrintETByteSet((unsigned char *)pData, ET_HOUR, *(pData + NEXT_HOUR_PTR), 24));
    BOOL ETSValid = TRUE;
    if (strcmp(ETS, _OVERFLOW_MSG) == 0) {
        ETSValid = FALSE;
        fprintf(stderr, "graph data %-10s %s\n", "ETS", _OVERFLOW_MSG);
    } else {
        sprintf(Attr, "%s, ", _ET_BY_HOUR_COL);
        strcat(sAttr, Attr);
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","ETS");
    }
#endif
#ifdef _RAIN_BY_HOUR_COL
    sprintf(RainS, "%s", PrintRainRateSet((unsigned char *)pData, RAIN_HOUR, *(pData + NEXT_HOUR_PTR), 24));
    BOOL RainSValid = TRUE;
    if (strcmp(RainS, _OVERFLOW_MSG) == 0) {
        RainSValid = FALSE;
        fprintf(stderr, "graph data %-10s %s\n", "RainS", _OVERFLOW_MSG);
    } else {
        sprintf(Attr, "%s, ", _RAIN_BY_HOUR_COL);
        strcat(sAttr, Attr);
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","RainS");
    }
#endif
#ifdef _RAIN_RATE_BY_HOURS_COL
    sprintf(RainRateS, "%s", PrintRainRateSet((unsigned char *)pData, RAIN_RATE_HOUR, *(pData + NEXT_HOUR_PTR), 24));
    BOOL RainRateSValid = TRUE;
    if (strcmp(RainRateS, _OVERFLOW_MSG) == 0) {
        RainRateSValid = FALSE;
        fprintf(stderr, "graph data %-10s %s\n", "RainRateS", _OVERFLOW_MSG);
    } else {
        sprintf(Attr, "%s, ", _RAIN_RATE_BY_HOURS_COL);
        strcat(sAttr, Attr);
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","RainRateS");
    }
#endif
    cpos = strlen(sAttr) - 1;
    while(cpos >= 0 && (sAttr[cpos] == ' ' || sAttr[cpos] == ','))
        sAttr[cpos--] = '\0';
    if (bVerboseSQL) fprintf(stderr, "\nCheck Attributes: %s\n\n", sAttr);

    if (sAttr[0] != '\0') {
        TTpos = ITpos = OTpos = WCpos = HIpos = IHpos = OHpos = BRpos = WSpos = WDpos = WHpos = RApos = RRpos = DPpos = TIpos = SRpos = ETpos = 0;
        while((sprintf(Time, "%s", SubString(Times, ',', &TTpos))) && TTpos <= strlen(Times)) {
            if (strcmp(Time, "00:00") == 0) DT.tm_mday++;
            if (strcmp(Time, "n/a") == 0) {
                if (bVerboseSQL) fprintf(stderr, "Time %s\n", Time); 
            } else {
                if (sscanf(Time, "%2u:%2u", &DT.tm_hour, &DT.tm_min) == EOF) {
                    fprintf(stderr, "ERROR! parsing %s\n", Time);
                    return 0;
                }
                curSec = mktime(&DT);
                sprintf(myYMD, "%4u%02u%02u", DT.tm_year + 1900, DT.tm_mon + 1, DT.tm_mday);
    
                InTemp[0] = retInTemp[0] = OutTemp[0] = retOutTemp[0] = WindChill[0] = retWindChill[0] = HeatIndex[0] = retHeatIndex[0] = '\0';
                InHum[0] = retInHum[0] = OutHum[0] = retOutHum[0] = WindSpeed[0] = retWindSpeed[0] = WindDir[0] = retWindDir[0] = '\0';
                WindHigh[0] = retWindHigh[0] = DewPoint[0] = retDewPoint[0] = THSW[0] = retTHSW[0] = Solar[0] = retSolar[0] = '\0';
                Baro[0] = retBaro[0] = ET[0] = retET[0] = Rain[0] = retRain[0] = RainRate[0] = retRainRate[0] = '\0';
    
                retRecID = 0;
                sprintf(Query, "SELECT T1.RecID, %s FROM %s AS T1 INNER JOIN DT AS T2 USE INDEX (YearMonthDay,HHMM,vpID) USING (RecID) WHERE T2.YearMonthDay = \"%s\" && T2.HHMM = \"%s\" && T2.vpID = \"%i\" LIMIT 1", sAttr, _HOURLY_TABLE_NAME, myYMD, Time, cid);
                if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                if (mysql_query(mySQLconn, Query) != 0)
                    printSQLerror(mySQLconn, "mysql_query() failed", 27, Query);
                else {
                    res_set = mysql_store_result(mySQLconn);
                    if (res_set == NULL)
                        printSQLerror(mySQLconn, "mysql_store_result() failed", 28, Query);
                    else {
                        if ((row = mysql_fetch_row (res_set)) != NULL) {
                            ROWpos = 0;
                            if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%llu", &retRecID);
                            ROWpos++;
#ifdef _IN_TEMP_BY_HOURS_COL
                            if (InTempSValid) {
                                if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retInTemp);
                                ROWpos++;
                            }
#endif
#ifdef _TEMP_BY_HOURS_COL
                            if (OutTempSValid) {
                                if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retOutTemp);
                                ROWpos++;
                            }
#endif
#ifdef _CHILL_LO_BY_HOURS_COL
                            if (WindChillSValid) {
                                if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retWindChill);
                                ROWpos++;
                            }
#endif
#ifdef _HEAT_HI_BY_HOURS_COL
                            if (HeatIndexSValid) {
                                if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retHeatIndex);
                                ROWpos++;
                            }
#endif
#ifdef _IN_HUM_BY_HOURS_COL
                            if (InHumSValid) {
                                if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retInHum);
                                ROWpos++;
                            }
#endif
#ifdef _HUM_BY_HOURS_COL
                            if (OutHumSValid) {
                                if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retOutHum);
                                ROWpos++;
                            }
#endif
#ifdef _WIND_AVG_BY_HOURS_COL
                            if (WindSpeedSValid) {
                                if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retWindSpeed);
                                ROWpos++;
                            }
#endif
#ifdef _WIND_DIR_BY_HOURS_COL
                            if (WindDirSValid) {
                                if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retWindDir);
                                ROWpos++;
                            }
#endif
#ifdef _WIND_HI_BY_HOURS_COL
                            if (WindHighSValid) {
                                if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retWindHigh);
                                ROWpos++;
                            }
#endif
#ifdef _DEW_BY_HOURS_COL
                            if (DewPointSValid) {
                                if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retDewPoint);
                                ROWpos++;
                            }
#endif
#ifdef _THSW_HI_BY_HOURS_COL
                            if (THSWSValid) {
                                if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retTHSW);
                                ROWpos++;
                            }
#endif
#ifdef _SOLAR_BY_HOUR_AVG_COL
                            if (SolarSValid) {
                                if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retSolar);
                                ROWpos++;
                            }
#endif
#ifdef _BAR_BY_HOURS_COL
                            if (BaroSValid) {
                                if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retBaro);
                                ROWpos++;
                            }
#endif
#ifdef _ET_BY_HOUR_COL
                            if (ETSValid) {
                                if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retET);
                                ROWpos++;
                            }
#endif
#ifdef _RAIN_BY_HOUR_COL
                            if (RainSValid) {
                                if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retRain);
                                ROWpos++;
                            }
#endif
#ifdef _RAIN_RATE_BY_HOURS_COL
                            if (RainRateSValid) {
                                if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retRainRate);
                                ROWpos++;
                            }
#endif
                        }
                        mysql_free_result(res_set);
    
#ifdef _IN_TEMP_BY_HOURS_COL
                        if (InTempSValid) {
                            sprintf(InTemp, "%s", SubString(InTempS, ',', &ITpos));
                            if (strcmp(InTemp, "n/a") == 0) sprintf(InTemp, "0");
                        }
#endif
#ifdef _TEMP_BY_HOURS_COL
                        if (OutTempSValid) {
                            sprintf(OutTemp, "%s", SubString(OutTempS, ',', &OTpos));
                            if (strcmp(OutTemp, "n/a") == 0) sprintf(OutTemp, "0");
                        }
#endif
#ifdef _CHILL_LO_BY_HOURS_COL
                        if (WindChillSValid) {
                            sprintf(WindChill, "%s", SubString(WindChillS, ',', &WCpos));
                            if (strcmp(WindChill, "n/a") == 0) sprintf(WindChill, "0");
                        }
#endif
#ifdef _HEAT_HI_BY_HOURS_COL
                        if (HeatIndexSValid) {
                            sprintf(HeatIndex, "%s", SubString(HeatIndexS, ',', &HIpos));
                            if (strcmp(HeatIndex, "n/a") == 0) sprintf(HeatIndex, "0");
                        }
#endif
#ifdef _IN_HUM_BY_HOURS_COL
                        if (InHumSValid) {
                            sprintf(InHum, "%s", SubString(InHumS, ',', &IHpos));
                            if (strcmp(InHum, "n/a") == 0) sprintf(InHum, "0");
                        }
#endif
#ifdef _HUM_BY_HOURS_COL
                        if (OutHumSValid) {
                            sprintf(OutHum, "%s", SubString(OutHumS, ',', &OHpos));
                            if (strcmp(OutHum, "n/a") == 0) sprintf(OutHum, "0");
                        }
#endif
#ifdef _WIND_AVG_BY_HOURS_COL
                        if (WindSpeedSValid) {
                            sprintf(WindSpeed, "%s", SubString(WindSpeedS, ',', &WSpos));
                            if (strcmp(WindSpeed, "n/a") == 0) sprintf(WindSpeed, "0");
                        }
#endif
#ifdef _WIND_DIR_BY_HOURS_COL
                        if (WindDirSValid) {
                            sprintf(WindDir, "%s", SubString(WindDirS, ',', &WDpos));
                            if (strcmp(WindDir, "n/a") == 0) sprintf(WindDir, "0");
                        }
#endif
#ifdef _WIND_HI_BY_HOURS_COL
                        if (WindHighSValid) {
                            sprintf(WindHigh, "%s", SubString(WindHighS, ',', &WHpos));
                            if (strcmp(WindHigh, "n/a") == 0) sprintf(WindHigh, "0");
                        }
#endif
#ifdef _DEW_BY_HOURS_COL
                        if (DewPointSValid) {
                            sprintf(DewPoint, "%s", SubString(DewPointS, ',', &DPpos));
                            if (strcmp(DewPoint, "n/a") == 0) sprintf(DewPoint, "0");
                        }
#endif
#ifdef _THSW_HI_BY_HOURS_COL
                        if (THSWSValid) {
                            sprintf(THSW, "%s", SubString(THSWS, ',', &TIpos));
                            if (strcmp(THSW, "n/a") == 0) sprintf(THSW, "0");
                        }
#endif
#ifdef _SOLAR_BY_HOUR_AVG_COL
                        if (SolarSValid) {
                            sprintf(Solar, "%s", SubString(SolarS, ',', &SRpos));
                            if (strcmp(Solar, "n/a") == 0) sprintf(Solar, "0");
                        }
#endif
#ifdef _BAR_BY_HOURS_COL
                        if (BaroSValid) {
                            sprintf(Baro, "%s", SubString(BaroS, ',', &BRpos));
                            if (strcmp(Baro, "n/a") == 0) sprintf(Baro, "0");
                        }
#endif
#ifdef _ET_BY_HOUR_COL
                        if (ETSValid) {
                            sprintf(ET, "%s", SubString(ETS, ',', &ETpos));
                            if (strcmp(ET, "n/a") == 0) sprintf(ET, "0");
                        }
#endif
#ifdef _RAIN_BY_HOUR_COL
                        if (RainSValid) {
                            sprintf(Rain, "%s", SubString(RainS, ',', &RApos));
                            if (strcmp(Rain, "n/a") == 0) sprintf(Rain, "0");
                        }
#endif
#ifdef _RAIN_RATE_BY_HOURS_COL
                        if (RainRateSValid) {
                            sprintf(RainRate, "%s", SubString(RainRateS, ',', &RRpos));
                            if (strcmp(RainRate, "n/a") == 0) sprintf(RainRate, "0");
                        }
#endif
    
//                        if (bVerboseSQL) fprintf(stderr, "retRecID \"%llu\"\nDate \"%8s\" Time \"%5s\" InTemp   \"%3s\" retInTemp   \"%3s\" OutTemp  \"%3s\" retOutTemp  \"%3s\" WindChill \"%3s\" retWindChill \"%3s\" HeatIndex  \"%3s\" retHeatIndex  \"%3s\"\nDate \"%8s\" Time \"%5s\" Baro  \"%6s\" retBaro  \"%6s\" WindSpeed \"%2s\" retWindSpeed \"%2s\" WindDir    \"%2s\"    retWindDir \"%2s\" WindHigh    \"%2s\" retWindHigh    \"%2s\"\nDate \"%8s\" Time \"%5s\" InHum    \"%3s\" retInHum    \"%3s\" OutHum   \"%3s\" retOutHum   \"%3s\" Rain    \"%5s\" retRain    \"%5s\" RainRate \"%5s\" retRainRate \"%5s\"\nDate \"%8s\" Time \"%5s\" DewPoint \"%3s\" retDewPoint \"%3s\" THSW     \"%3s\" retTHSW     \"%3s\" Solar    \"%4s\" retSolar    \"%4s\" ET      \"%6s\" retET      \"%6s\"\n", retRecID, myYMD, Time, InTemp, retInTemp, OutTemp, retOutTemp, WindChill, retWindChill, HeatIndex, retHeatIndex, myYMD, Time, Baro, retBaro, WindSpeed, retWindSpeed, WindDir, retWindDir, WindHigh, retWindHigh, myYMD, Time, InHum, retInHum, OutHum, retOutHum, Rain, retRain, RainRate, retRainRate, myYMD, Time, DewPoint, retDewPoint, THSW, retTHSW, Solar, retSolar, ET, retET);
    
                        /* whew! we gots the data so now do the comparisons inserts & updates */
    
                        Attr[0] = '\0';
                        Values[0] = '\0';
                        Updates[0] = '\0';
#ifdef _IN_TEMP_BY_HOURS_COL
                        if (InTempSValid) {
                            if (retRecID == 0) {
                                strcat(Attr, _IN_TEMP_BY_HOURS_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", InTemp);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (*InTemp != '\0' && InTemp != NULL && strcmp(InTemp, retInTemp) != 0) {
                                sprintf(value, "%s = \"%s\", ", _IN_TEMP_BY_HOURS_COL, InTemp);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
#ifdef _TEMP_BY_HOURS_COL
                        if (OutTempSValid) {
                            if (retRecID == 0) {
                                strcat(Attr, _TEMP_BY_HOURS_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", OutTemp);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (*OutTemp != '\0' && OutTemp != NULL && strcmp(OutTemp, retOutTemp) != 0) {
                                sprintf(value, "%s = \"%s\", ", _TEMP_BY_HOURS_COL, OutTemp);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
#ifdef _CHILL_LO_BY_HOURS_COL
                        if (WindChillSValid) {
                            if (retRecID == 0) {
                                strcat(Attr, _CHILL_LO_BY_HOURS_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", WindChill);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (*WindChill != '\0' && WindChill != NULL && strcmp(WindChill, retWindChill) != 0) {
                                sprintf(value, "%s = \"%s\", ", _CHILL_LO_BY_HOURS_COL, WindChill);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
#ifdef _HEAT_HI_BY_HOURS_COL
                        if (HeatIndexSValid) {
                            if (retRecID == 0) {
                                strcat(Attr, _HEAT_HI_BY_HOURS_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", HeatIndex);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (*HeatIndex != '\0' && HeatIndex != NULL && strcmp(HeatIndex, retHeatIndex) != 0) {
                                sprintf(value, "%s = \"%s\", ", _HEAT_HI_BY_HOURS_COL, HeatIndex);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
#ifdef _IN_HUM_BY_HOURS_COL
                        if (InHumSValid) {
                            if (retRecID == 0) {
                                strcat(Attr, _IN_HUM_BY_HOURS_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", InHum);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (*InHum != '\0' && InHum != NULL && strcmp(InHum, retInHum) != 0) {
                                sprintf(value, "%s = \"%s\", ", _IN_HUM_BY_HOURS_COL, InHum);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
#ifdef _HUM_BY_HOURS_COL
                        if (OutHumSValid) {
                            if (retRecID == 0) {
                                strcat(Attr, _HUM_BY_HOURS_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", OutHum);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (*OutHum != '\0' && OutHum != NULL && strcmp(OutHum, retOutHum) != 0) {
                                sprintf(value, "%s = \"%s\", ", _HUM_BY_HOURS_COL, OutHum);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
#ifdef _WIND_AVG_BY_HOURS_COL
                        if (WindSpeedSValid) {
                            if (retRecID == 0) {
                                strcat(Attr, _WIND_AVG_BY_HOURS_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", WindSpeed);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (*WindSpeed != '\0' && WindSpeed != NULL && strcmp(WindSpeed, retWindSpeed) != 0) {
                                sprintf(value, "%s = \"%s\", ", _WIND_AVG_BY_HOURS_COL, WindSpeed);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
#ifdef _WIND_DIR_BY_HOURS_COL
                        if (WindDirSValid) {
                            if (retRecID == 0) {
                                strcat(Attr, _WIND_DIR_BY_HOURS_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", WindDir);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (*WindDir != '\0' && WindDir != NULL && strcmp(WindDir, retWindDir) != 0) {
                                sprintf(value, "%s = \"%s\", ", _WIND_DIR_BY_HOURS_COL, WindDir);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
#ifdef _WIND_HI_BY_HOURS_COL
                        if (WindHighSValid) {
                            if (retRecID == 0) {
                                strcat(Attr, _WIND_HI_BY_HOURS_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", WindHigh);
                                strcat(Values, value);
                            } else if (*WindHigh != '\0' && WindHigh != NULL && strcmp(WindHigh, retWindHigh) != 0) {
                                sprintf(value, "%s = \"%s\", ", _WIND_HI_BY_HOURS_COL, WindHigh);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
#ifdef _DEW_BY_HOURS_COL
                        if (DewPointSValid) {
                            if (retRecID == 0) {
                                strcat(Attr, _DEW_BY_HOURS_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", DewPoint);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (*DewPoint != '\0' && DewPoint != NULL && strcmp(DewPoint, retDewPoint) != 0) {
                                sprintf(value, "%s = \"%s\", ", _DEW_BY_HOURS_COL, DewPoint);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
#ifdef _THSW_HI_BY_HOURS_COL
                        if (THSWSValid) {
                            if (retRecID == 0) {
                                strcat(Attr, _THSW_HI_BY_HOURS_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", THSW);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (*THSW != '\0' && THSW != NULL && strcmp(THSW, retTHSW) != 0) {
                                sprintf(value, "%s = \"%s\", ", _THSW_HI_BY_HOURS_COL, THSW);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
#ifdef _SOLAR_BY_HOUR_AVG_COL
                        if (SolarSValid) {
                            if (retRecID == 0) {
                                strcat(Attr, _SOLAR_BY_HOUR_AVG_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", Solar);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (*Solar != '\0' && Solar != NULL && strcmp(Solar, retSolar) != 0) {
                                sprintf(value, "%s = \"%s\", ", _SOLAR_BY_HOUR_AVG_COL, Solar);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
#ifdef _BAR_BY_HOURS_COL
                        if (BaroSValid) {
                            if (retRecID == 0) {
                                strcat(Attr, _BAR_BY_HOURS_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", Baro);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (*Baro != '\0' && Baro != NULL && strcmp(Baro, retBaro) != 0) {
                                sprintf(value, "%s = \"%s\", ", _BAR_BY_HOURS_COL, Baro);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
#ifdef _ET_BY_HOUR_COL
                        if (ETSValid) {
                            if (retRecID == 0) {
                                strcat(Attr, _ET_BY_HOUR_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", ET);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (*ET != '\0' && ET != NULL && strcmp(ET, retET) != 0) {
                                sprintf(value, "%s = \"%s\", ", _ET_BY_HOUR_COL, ET);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
#ifdef _RAIN_BY_HOUR_COL
                        if (RainSValid) {
                            if (retRecID == 0) {
                                strcat(Attr, _RAIN_BY_HOUR_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", Rain);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (*Rain != '\0' && Rain != NULL && strcmp(Rain, retRain) != 0) {
                                sprintf(value, "%s = \"%s\", ", _RAIN_BY_HOUR_COL, Rain);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
#ifdef _RAIN_RATE_BY_HOURS_COL
                        if (RainRateSValid) {
                            if (retRecID == 0) {
                                strcat(Attr, _RAIN_RATE_BY_HOURS_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", RainRate);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (*RainRate != '\0' && RainRate != NULL && strcmp(RainRate, retRainRate) != 0) {
                                sprintf(value, "%s = \"%s\", ", _RAIN_RATE_BY_HOURS_COL, RainRate);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
                        if (Attr[0] != '\0') {
                            strcat(Attr, "F, ");
                            strcat(Values, bMetric ? "\"m\", " : "\"u\", ");
                            cpos = strlen(Attr) - 1;
                            while(cpos >= 0 && (Attr[cpos] == ' ' || Attr[cpos] == ','))
                                Attr[cpos--] = '\0';
                            cpos = strlen(Values) - 1;
                            while(cpos >= 0 && (Values[cpos] == ' ' || Values[cpos] == ','))
                                Values[cpos--] = '\0';
                            sprintf( InsertDT, "%4u-%02u-%02u %s:00", DT.tm_year + 1900, DT.tm_mon + 1, DT.tm_mday, Time);
                            newRecID = setupRecID(cid, curSec);   /* Hourly */
                            sprintf(Query, "INSERT INTO %s (RecID, %s) VALUES (\"%llu\", %s)", _HOURLY_TABLE_NAME, Attr, newRecID, Values);
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            if (mysql_query(mySQLconn, Query) != 0)
                                printSQLerror(mySQLconn, "mysql_query() failed", 29, Query);
                            else if (bVerboseSQL)
                                fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
    
                        }
                        if (Updates[0] != '\0') {
                            strcat(Updates, bMetric ? "F = \"m\", " : "F = \"u\", ");
                            cpos = strlen(Updates) - 1;
                            while(cpos >= 0 && (Updates[cpos] == ' ' || Updates[cpos] == ','))
                                Updates[cpos--] = '\0';
                            sprintf(Query, "UPDATE %s SET %s WHERE RecID = \"%llu\"", _HOURLY_TABLE_NAME, Updates, retRecID);
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            if (mysql_query(mySQLconn, Query) != 0)
                                printSQLerror(mySQLconn, "mysql_query() failed", 30, Query);
                            else if (bVerboseSQL)
                                fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
    
                        }
                        if (bVerboseSQL) fprintf(stderr, "\n");
                    }
                }
            }
        }
    }
    if (bProfiling) logProfileSQL("HourlyUpdate");
    return TRUE;
}


/*--------------------------------------------------------------------------
        ForecastUpdate
----------------------------------------------------------------------------*/
int ForecastUpdate(unsigned long long int IndexID, int cid)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    char Attr[255];
    char Values[255];
    char Updates[255];
    unsigned long long int DTRecID = 0;
    unsigned long long int HourlyRecID = 0;
    char myDateTime[20];
    char myDate[11];
    char myHour[3];
    unsigned int retRule = 0;
    unsigned int retIcon = 0;
    char value[128];
    int cpos;
    time_t curSec;
    struct tm DT;

    if (bVerboseSQL) fprintf(stderr, "\nProcessing Forecast... %s\n", getCurTime());
    myDateTime[0] = '\0';
    myDate[0] = '\0';
    myHour[0] = '\0';
    sprintf(Query, "SELECT bDateTime, bDate, bHour FROM DT USE INDEX (RecID) WHERE RecID = \"%llu\"", IndexID);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 31, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 32, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL) {
                if (row[0] != NULL) sprintf(myDateTime, "%s", row[0]);	/* need to use sprintf if there's spaces in the string */
                if (row[1] != NULL) sscanf(row[1], "%s", myDate);
                if (row[2] != NULL) sscanf(row[2], "%s", myHour);
            }
            mysql_free_result(res_set);
        }
    }

    if (myDateTime[0] == '\0' || myHour[0] == '\0') 
        return FALSE;

    sprintf(Query, "SELECT RecID FROM DT USE INDEX (bDate,HHMM,vpID) WHERE bDate = \"%s\" && HHMM = \"%s:00\" && vpID = \"%i\"", myDate, myHour, cid);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 33, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 34, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL) {
                if (row[0] != NULL) sscanf(row[0], "%llu", &DTRecID);
            }
            mysql_free_result(res_set);
        }
    }

    if (DTRecID > 0) {
        sprintf(Query, "SELECT RecID FROM %s USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", _FORECAST_TABLE_NAME, DTRecID);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 35, Query);
        else {
            res_set = mysql_store_result(mySQLconn);
           if (res_set == NULL)
                printSQLerror(mySQLconn, "mysql_store_result() failed", 36, Query);
            else {
                if ((row = mysql_fetch_row (res_set)) != NULL) {
                    if (row[0] != NULL) sscanf(row[0], "%llu", &HourlyRecID);
                }
                mysql_free_result(res_set);
            }
        }
    }

    if (DTRecID == 0) {
        sprintf(myDateTime, "%s %s:00:00", myDate, myHour);	/* 2011-03-15 */

        DT = *(localtime(&ttRun));
        if (sscanf(myDate, "%4u-%2u-%2u", &DT.tm_year, &DT.tm_mon, &DT.tm_mday) == EOF) {
            fprintf(stderr, "ERROR! parsing %s\n", myDate);
            return 0;
        }
        DT.tm_hour = atoi( myHour );
        DT.tm_min = 0;
        DT.tm_sec = 0;
        DT.tm_year -= 1900;
        DT.tm_mon -= 1;
        curSec = mktime(&DT);
        if (bVerboseSQL) fprintf(stderr, "\nForecast getting RecID for %04d-%02d-%02d %02d:%02d:%02d\n", DT.tm_year+1900, DT.tm_mon+1, DT.tm_mday, DT.tm_hour, DT.tm_min, DT.tm_sec);
        DTRecID = setupRecID(cid, curSec);		/* ForecastUpdate */
        if (bVerboseSQL) fprintf(stderr, "\n");
    }

    if (HourlyRecID == 0) {
        Attr[0] = '\0';
        Values[0] = '\0';
#ifdef _FORE_ICON_COL
        if (&rcd.yForeIcon != NULL) {
            strcat(Attr, ", ");
            strcat(Attr, _FORE_ICON_COL);
            sprintf(value, ", \"%u\"", rcd.yForeIcon);
            strcat(Values, value);
            LoadCnt++;
        }
#endif
#ifdef _FORE_RULE_COL
        if (&rcd.yRule != NULL) {
            strcat(Attr, ", ");
            strcat(Attr, _FORE_RULE_COL);
            sprintf(value, ", \"%u\"", rcd.yRule);
            strcat(Values, value);
            LoadCnt++;
        }
#endif
        if (Attr[0] != '\0') {
            sprintf(Query, "INSERT INTO %s (RecID%s) VALUES (\"%llu\"%s)", _FORECAST_TABLE_NAME, Attr, DTRecID, Values);
            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
            if (mysql_query(mySQLconn, Query) != 0)
                printSQLerror(mySQLconn, "mysql_query() failed", 37, Query);
            else if (bVerboseSQL)
                fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
        }
    }
    if (HourlyRecID != 0) {
        Updates[0] = '\0';
        sprintf(Query, "SELECT %s, %s FROM %s USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", _FORE_ICON_COL, _FORE_RULE_COL, _FORECAST_TABLE_NAME, HourlyRecID);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 38, Query);
        else {
            res_set = mysql_store_result(mySQLconn);
            if (res_set == NULL)
                printSQLerror(mySQLconn, "mysql_store_result() failed", 39, Query);
            else {
                if ((row = mysql_fetch_row (res_set)) != NULL) {
                    if (row[0] != NULL) sscanf(row[0], "%u", &retIcon);
                    if (row[1] != NULL) sscanf(row[1], "%u", &retRule);
                }
                mysql_free_result(res_set);
            }
        }
#ifdef _FORE_ICON_COL
        if (rcd.yForeIcon != retIcon && &rcd.yForeIcon != NULL) {
            sprintf(value, "%s = \"%u\", ", _FORE_ICON_COL, rcd.yForeIcon);
            strcat(Updates, value);
            LoadCnt++;
        }
#endif
#ifdef _FORE_RULE_COL
        if (rcd.yRule != retRule && &rcd.yRule != NULL) {
            sprintf(value, "%s = \"%u\", ", _FORE_RULE_COL, rcd.yRule);
            strcat(Updates, value);
            LoadCnt++;
        }
#endif
        if (Updates[0] != '\0') {
            cpos = strlen(Updates) - 1;
            while(cpos >= 0 && (Updates[cpos] == ' ' || Updates[cpos] == ','))
                Updates[cpos--] = '\0';
            sprintf(Query, "UPDATE %s SET %s WHERE RecID = \"%llu\"", _FORECAST_TABLE_NAME, Updates, HourlyRecID);
            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
            if (mysql_query(mySQLconn, Query) != 0)
                printSQLerror(mySQLconn, "mysql_query() failed", 40, Query);
            else if (bVerboseSQL)
                fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
        }

    }
    if (bProfiling) logProfileSQL(" ForecastUpdate");
    return TRUE;
}


/*--------------------------------------------------------------------------
        getIsRaining
----------------------------------------------------------------------------*/
BOOL getIsRaining(unsigned long long int IndexID)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    BOOL IsRaining = FALSE;

    sprintf(Query, "SELECT %s FROM %s USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", _IS_RAINING_COL, _RAIN_TABLE_NAME, IndexID);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 41, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set != NULL) {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sscanf(row[0], "%i", &IsRaining );
            mysql_free_result(res_set);
        }
    }
    if (bProfiling) logProfileSQL("getIsRaining");
    return IsRaining;
}


/*--------------------------------------------------------------------------
        getStormAmount
----------------------------------------------------------------------------*/
float getStormAmount(unsigned long long int IndexID)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    float StormAmount = -1.0;

#ifndef _RAIN_STORM_COL
    return StormAmount;
#endif
    sprintf(Query, "SELECT %s FROM %s USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", _RAIN_STORM_COL, _RAIN_TABLE_NAME, IndexID);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 42, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set != NULL) {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sscanf(row[0], "%f", &StormAmount );
            mysql_free_result(res_set);
        }
    }
    if (bProfiling) logProfileSQL("getStormAmount");
    return StormAmount;
}


/*--------------------------------------------------------------------------
        getDayAmount
----------------------------------------------------------------------------*/
float getDayAmount(unsigned long long int IndexID)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    float DayAmount = -1.0;

#ifndef _DAY_RAIN_COL
    return DayAmount;
#endif
    sprintf(Query, "SELECT %s FROM %s USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", _DAY_RAIN_COL, _RAIN_TABLE_NAME, IndexID);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 43, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set != NULL) {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sscanf(row[0], "%f", &DayAmount );
            mysql_free_result(res_set);
        }
    }
    if (bProfiling) logProfileSQL("getDayAmount");
    return DayAmount;
}


/*--------------------------------------------------------------------------
        getDateTimeDT
----------------------------------------------------------------------------*/
void getDateTimeDT(unsigned long long int RecID, char *Date, char *Time)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];

    Date[0] = '\0';
    Time[0] = '\0';
    sprintf(Query, "SELECT bDate, bTime FROM DT USE INDEX (RecID) WHERE RecID = \"%llu\"", RecID);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 44, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 45, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL) {
                if (row[0] != NULL) sprintf(Date, "%s", row[0]);
                if (row[1] != NULL) sprintf(Time, "%s", row[1]);
            }
            mysql_free_result(res_set);
        }
    }
    if (bProfiling) logProfileSQL("getDateTimeDT");
}


/*--------------------------------------------------------------------------
        getStormStartDT
----------------------------------------------------------------------------*/
BOOL getStormStartDT(int cid, char forDate[], char *Date, char *Time)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    unsigned long long int retRecID = 0;

    Date[0] = '\0';
    Time[0] = '\0';
    sprintf(Query, "SELECT T1.RecID FROM %s AS T1 INNER JOIN DT AS T2 USING (RecID) WHERE T1.%s = \"%s\" && T2.vpID = \"%i\"", _RAIN_TABLE_NAME, _STORM_START_DATE_COL, forDate, cid);	/* do not use (StormAmount) index here */
    if (bVerboseSQL) fprintf(stderr, "Query: (getStormStartDT) %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 46, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 47, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sscanf(row[0], "%llu", &retRecID);
            mysql_free_result(res_set);
        }
    }
    if (retRecID > 0) {
        getDateTimeDT(retRecID, Date, Time);
        if (Date[0] != '\0' && Time[0] != '\0') return TRUE;
    }
    if (bProfiling) logProfileSQL("getStormStartDT");
    return FALSE;
}


/*--------------------------------------------------------------------------
        getStormEndedRecID
----------------------------------------------------------------------------*/
unsigned long long int getStormEndedRecID(int cid, char startDate[], char forDate[], char StormAmount[])
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    char startTime[9];
    unsigned long long int retRecID = 0;

    startTime[0] = '\0';
    sprintf(Query, "SELECT T1.bTime from DT AS T1 INNER JOIN %s AS T2 USE INDEX (StormStartedDate) USING (RecID) WHERE T2.%s = \"%s\" && T1.vpID = \"%i\"", _RAIN_TABLE_NAME, _STORM_START_DATE_COL, startDate, cid);
    if (bVerboseSQL) fprintf(stderr, "Query: (getStormEndedRecID) %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 48, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 49, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sprintf(startTime, "%s", row[0]);
            mysql_free_result(res_set);
        }
    }
    if (startTime[0] == '\0') strcpy(startTime, "00:00:00");

    sprintf(Query, "SELECT T1.RecID from DT AS T1 USE INDEX (PRIMARY,bDate) INNER JOIN %s AS T2 USING (RecID) WHERE T1.bDate = \"%s\" && T1.bDateTime >= \"%s %s\" && T1.vpID = \"%i\" && T2.%s = \"%s\" ORDER BY T1.bDateTime LIMIT 1", _RAIN_TABLE_NAME, forDate, startDate, startTime, cid, _RAIN_STORM_COL, StormAmount);   /* do not use (StormAmount) index here */
    if (bVerboseSQL) fprintf(stderr, "Query: (getStormEndedRecID) %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 50, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 51, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sscanf(row[0], "%llu", &retRecID);
            mysql_free_result(res_set);
        }
    }
    if (bProfiling) logProfileSQL("getStormEndedRecID");
    return retRecID;
}


/*--------------------------------------------------------------------------
        getStormEndDT
----------------------------------------------------------------------------*/
BOOL getStormEndDT(int cid, char startDate[], char forDate[], char StormAmount[], char *Date, char *Time)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    char newStormAmount[6];
    unsigned long long int retRecID = 0;

    Date[0] = '\0';
    Time[0] = '\0';

    retRecID = getStormEndedRecID(cid, startDate, forDate, StormAmount);	/* sometimes the vPro lies about the StormAmount so if it did   */
    if (retRecID == 0) {					/* and the end was not found for that end date get the highest  */
        newStormAmount[0] = '\0';				/* StormAmount for that date it said the storm ended and use it */ 
        sprintf(Query, "SELECT T1.%s FROM %s AS T1 INNER JOIN DT AS T2 USE INDEX (RecIDvpID,bDate) USING (RecID) WHERE T2.bDate = \"%s\" && T2.vpID = \"%i\" ORDER BY T1.%s DESC LIMIT 1", _RAIN_STORM_COL, _RAIN_TABLE_NAME, forDate, cid, _RAIN_STORM_COL);	/* MAX() */

        if (bVerboseSQL) fprintf(stderr, "Query: (getStormEndDT) %s\n", Query);
        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 52, Query);
        else {
            res_set = mysql_store_result(mySQLconn);
            if (res_set == NULL)
                printSQLerror(mySQLconn, "mysql_store_result() failed", 53, Query);
            else {
                if ((row = mysql_fetch_row (res_set)) != NULL)
                    if (row[0] != NULL) sprintf(newStormAmount, "%s", row[0]);
                mysql_free_result(res_set);
            }
        }
        if (strlen(newStormAmount) > 0) retRecID = getStormEndedRecID(cid, startDate, forDate, newStormAmount);
    }
    if (retRecID > 0) {
        getDateTimeDT(retRecID, Date, Time);
        if (Date[0] != '\0' && Time[0] != '\0') return TRUE;
    }
    if (bProfiling) logProfileSQL("getStormEndDT");
    return FALSE;
}


/*--------------------------------------------------------------------------
        StormsUpdate
----------------------------------------------------------------------------*/
int StormsUpdate(unsigned long long int IndexID, char * pData, int cid)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    char sAttr[255];
    char Attr[384];
    char Values[384];
    char Updates[786];
    char value[128];
    char StormsStarted[300] = "";
    char StormsEnded[300] = "";
    char StormsAmountRpt[225] = "";
    char StormStartDate[12] = "";
    char StormEndDate[12] = "";
    char StormAmountRpt[9] = "";
//    char StormAmountCalc[7] = "";
    char HighRate[9] = "";
    char HighRateDT[20] = "";
    char retStormStartDate[12] = "";
    char retStormStartTime[10];
    char retStormEndDate[12] = "";
    char retStormEndTime[10];
    char retStormAmountRpt[9] = "";
//    char retStormAmountCalc[7] = "";
    char retHighRate[9] = "";
    char retHighRateDT[20] = "";
    char DTStormStartDate[12] = "";
    char DTStormStartTime[10] = "";
    char DTStormEndDate[12] = "";
    char DTStormEndTime[10] = "";
    char useDTStormEndTime[10] = "";
    char Units[2] = "";
    char retUnits[2] = "";
    char *spos;
    int cpos, ROWpos, SSpos, SEpos, SApos;
    BOOL DoStormTable = TRUE;

    if (bVerboseSQL) fprintf(stderr, "StormsUpdate %s\n", getCurTime());
    sAttr[0] = '\0';
#ifndef _RAIN_STORMS_START_DATE_COL 		/* can't do nothin' if we don't have the key column */
    DoStormTable = FALSE;
#endif
    if(DoStormTable) {
        sprintf(StormsStarted, "%s", PrintDateSet((unsigned char *)pData, RAIN_STORM_START, *(pData + NEXT_RAIN_STORM_PTR), 25));
        BOOL StormsStartedValid = TRUE;
        if (strcmp(StormsStarted, _OVERFLOW_MSG) == 0) {
            StormsStartedValid = FALSE;
            fprintf(stderr, "graph data %-15s %s\n", "StormsStarted", _OVERFLOW_MSG);
        } else {
            sprintf(Attr, "%s, ", _RAIN_STORMS_START_DATE_COL);
            strcat(sAttr, Attr);
#ifdef _RAIN_STORMS_START_TIME_COL
            sprintf(Attr, "%s, ", _RAIN_STORMS_START_TIME_COL);
            strcat(sAttr, Attr);
#endif
//            if (bVerboseSQL) fprintf(stderr, "graph data %-15s Valid\n","StormsStarted");
        }
#ifdef _RAIN_STORMS_END_DATE_COL
        sprintf(StormsEnded, "%s", PrintDateSet((unsigned char *)pData, RAIN_STORM_END, *(pData + NEXT_RAIN_STORM_PTR), 25));
        BOOL StormsEndedValid = TRUE;
        if (strcmp(StormsEnded, _OVERFLOW_MSG) == 0) {
            StormsEndedValid = FALSE;
            fprintf(stderr, "graph data %-15s %s\n", "StormsEnded", _OVERFLOW_MSG);
        } else {
            sprintf(Attr, "%s, ", _RAIN_STORMS_END_DATE_COL);
            strcat(sAttr, Attr);
#endif
#ifdef _RAIN_STORMS_END_TIME_COL
            sprintf(Attr, "%s, ", _RAIN_STORMS_END_TIME_COL);
            strcat(sAttr, Attr);
#endif
//            if (bVerboseSQL) fprintf(stderr, "graph data %-15s Valid\n","StormsEnded");
        }
#ifdef _RAIN_STORMS_AMOUNT_COL
        sprintf(StormsAmountRpt, "%s", PrintRainRateSet((unsigned char *)pData, RAIN_STORM, *(pData + NEXT_RAIN_STORM_PTR), 25));
        BOOL StormsAmountRptValid = TRUE;
        if (strcmp(StormsAmountRpt, _OVERFLOW_MSG) == 0) {
            StormsAmountRptValid = FALSE;
            fprintf(stderr, "graph data %-15s %s\n", "StormsAmountRpt", _OVERFLOW_MSG);
        } else {
            sprintf(Attr, "%s, ", _RAIN_STORMS_AMOUNT_COL);
            strcat(sAttr, Attr);
#endif
#ifdef _RAIN_STORMS_HIGH_RATE_COL
            sprintf(Attr, "%s, ", _RAIN_STORMS_HIGH_RATE_COL);
            strcat(sAttr, Attr);
#endif
#ifdef _RAIN_STORMS_HIGH_RATE_DT_COL
            sprintf(Attr, "%s, ", _RAIN_STORMS_HIGH_RATE_DT_COL);
            strcat(sAttr, Attr);
#endif
//            if (bVerboseSQL) fprintf(stderr, "graph data %-15s Valid\n","StormsAmountRpt");
        }
        strcat(sAttr, "F");
        cpos = strlen(sAttr) - 1;
        while(cpos >= 0 && (sAttr[cpos] == ' ' || sAttr[cpos] == ','))
            sAttr[cpos--] = '\0';
        if (bVerboseSQL) fprintf(stderr, "\nCheck Attributes: %s\n\n", sAttr);


        SSpos = SEpos = SApos = 0;
        while(StormsStartedValid && (sprintf(StormStartDate, "%s", SubString(StormsStarted, ',', &SSpos))) && SSpos <= strlen(StormsStarted)) {
            if (! DateSQL(StormStartDate)) return FALSE;

            retStormStartDate[0] = StormEndDate[0] = retStormEndDate[0] = retStormStartTime[0] = retStormEndTime[0] = '\0';
            retStormStartTime[0] = retStormEndTime[0] = '\0';
            StormAmountRpt[0] = retStormAmountRpt[0] = HighRate[0] = retHighRate[0] = HighRateDT[0] = retHighRateDT[0] = Units[0] = retUnits[0] = '\0';
//            StormAmountCalc[0] = retStormAmountCalc[0] = HighRate[0] = retHighRate[0] = HighRateDT[0] = retHighRateDT[0] = '\0';

            sprintf(Query, "SELECT %s FROM %s USE INDEX (PRIMARY) WHERE %s = \"%s\" && vpID = \"%i\"", sAttr, _STORMS_TABLE_NAME, _RAIN_STORMS_START_DATE_COL, StormStartDate, cid);
            if (bVerboseSQL) fprintf(stderr, "%s checking for StormStartDate %s\nQuery: %s\n", getCurTime(), StormStartDate, Query);
            if (mysql_query(mySQLconn, Query) != 0)
                printSQLerror(mySQLconn, "mysql_query() failed", 54, Query);
            else {
                res_set = mysql_store_result(mySQLconn);
                if (res_set == NULL)
                    printSQLerror(mySQLconn, "mysql_store_result() failed", 55, Query);
                else {
                    if ((row = mysql_fetch_row (res_set)) != NULL) {
                        ROWpos = 0;
                        if (StormsStartedValid) {
                            if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retStormStartDate);
                            ROWpos++;
                        }

#ifdef _RAIN_STORMS_START_TIME_COL
                        if (StormsStartedValid) {
                            if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retStormStartTime);
                            ROWpos++;
                        }
#endif
#ifdef _RAIN_STORMS_END_DATE_COL
                        if (StormsEndedValid) {
                            if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retStormEndDate);
                            ROWpos++;
                        }
#endif
#ifdef _RAIN_STORMS_END_TIME_COL
                        if (StormsEndedValid) {
                            if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retStormEndTime);
                            ROWpos++;
                        }
#endif
#ifdef _RAIN_STORMS_AMOUNT_COL
                        if (StormsAmountRptValid) {
                            if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retStormAmountRpt);
                            ROWpos++;
                        }
#endif
#ifdef _RAIN_STORMS_HIGH_RATE_COL
                        if (StormsAmountRptValid) {
                            if (row[ROWpos] != NULL) sscanf(row[ROWpos], "%s", retHighRate);
                            ROWpos++;
                        }
#endif
#ifdef _RAIN_STORMS_HIGH_RATE_DT_COL
                        if (StormsAmountRptValid) {
                            if (row[ROWpos] != NULL) sprintf(retHighRateDT, "%s", row[ROWpos]);
                            ROWpos++;
                        }
#endif
                        if (StormsAmountRptValid) {
                            if (row[ROWpos] != NULL) sprintf(retUnits, "%s", row[ROWpos]);
                            ROWpos++;
                        }
                    }
                    mysql_free_result(res_set);

#ifdef _RAIN_STORMS_END_DATE_COL
                    sprintf(StormEndDate, "%s", SubString(StormsEnded, ',', &SEpos));
                    if (! DateSQL(StormEndDate)) return FALSE;
#endif
                    strcpy(useDTStormEndTime, "23:59:59");
#ifdef _RAIN_STORMS_AMOUNT_COL
                    sprintf(StormAmountRpt, "%s", SubString(StormsAmountRpt, ',', &SApos));
#if defined(_RAIN_STORMS_END_DATE_COL) && defined(_RAIN_STORMS_END_TIME_COL)
                    if (getStormEndDT(cid, StormStartDate, StormEndDate, StormAmountRpt, DTStormEndDate, DTStormEndTime))
                        strcpy(useDTStormEndTime, DTStormEndTime);
#endif
#endif
                    if (bVerboseSQL) fprintf(stderr, "StormStartDate \"%-10s\"  StormEndDate \"%-10s\"  useDTStormEndTime \"%-8s\" StormAmountRpt \"%s\"\n", StormStartDate, StormEndDate, useDTStormEndTime, StormAmountRpt);
#if defined(_RAIN_STORMS_HIGH_RATE_COL) && defined(_RAIN_TABLE_NAME) && defined(_RAIN_RATE_COL)
                    sprintf(Query, "SELECT T1.%s FROM %s AS T1 INNER JOIN DT AS T2 USE INDEX (RecIDvpID,bDateTime) USING (RecID) WHERE T2.bDateTime >= \"%s 00:00:00\" && T2.bDateTime <= \"%s %s\" && T2.vpID = \"%i\" ORDER BY T1.%s DESC LIMIT 1", _RAIN_RATE_COL, _RAIN_TABLE_NAME, StormStartDate, StormEndDate, useDTStormEndTime, cid, _RAIN_RATE_COL);	/* MAX() */
                    if (bVerboseSQL) fprintf(stderr, "%s\nQuery: %s\n", getCurTime(), Query);
                    if (mysql_query(mySQLconn, Query) != 0)
                        printSQLerror(mySQLconn, "mysql_query() failed", 56, Query);
                    else {
                        res_set = mysql_store_result(mySQLconn);
                        if (res_set == NULL)
                            printSQLerror(mySQLconn, "mysql_store_result() failed", 57, Query);
                        else {
                            if ((row = mysql_fetch_row (res_set)) != NULL)
                                if (row[0] != NULL) sscanf(row[0], "%s", HighRate);
                            mysql_free_result(res_set);
                        }
                    }

                    if (HighRate[0] != '\0') {
                        sprintf(Query, "SELECT T1.bDateTime, T2.F FROM DT AS T1 USE INDEX (bDateTime) INNER JOIN %s AS T2 USING (RecID) WHERE T1.bDateTime >= \"%s 00:00:00\" && T1.bDateTime <= \"%s %s\" && T1.vpID = \"%i\" && T2.%s = \"%s\" ORDER BY T1.bDateTime LIMIT 1", _RAIN_TABLE_NAME, StormStartDate, StormEndDate, useDTStormEndTime, cid, _RAIN_RATE_COL, HighRate);
                        if (bVerboseSQL) fprintf(stderr, "%s\nQuery: %s\n", getCurTime(), Query);
                        if (mysql_query(mySQLconn, Query) != 0)
                            printSQLerror(mySQLconn, "mysql_query() failed", 58, Query);
                        else {
                            res_set = mysql_store_result(mySQLconn);
                            if (res_set == NULL)
                                printSQLerror(mySQLconn, "mysql_store_result() failed", 59, Query);
                            else {
                                if ((row = mysql_fetch_row (res_set)) != NULL)
                                    if (row[0] != NULL) sprintf(HighRateDT, "%s", row[0]);
                                    if (row[1] != NULL) sprintf(Units, "%s", row[1]);
                                mysql_free_result(res_set);
                            }
                        }
                    }
#endif

                    Attr[0] = '\0';
                    Values[0] = '\0';
                    Updates[0] = '\0';
                    if (StormsStartedValid) {
                        if (strlen(retStormStartDate) == 0){
                            strcat(Attr, _RAIN_STORMS_START_DATE_COL);
                            strcat(Attr, ", ");
                            sprintf(value, "\"%s\", ", StormStartDate);
                            strcat(Values, value);
                            LoadCnt++;
                        }
#ifdef _RAIN_STORMS_START_TIME_COL
                        if (strlen(StormStartDate) > 0) {
                            if (getStormStartDT(cid, StormStartDate, DTStormStartDate, DTStormStartTime)) {
                                if (strlen(retStormStartDate) == 0) {
                                    strcat(Attr, _RAIN_STORMS_START_TIME_COL);
                                    strcat(Attr, ", ");
                                    sprintf(value, "\"%s\", ", DTStormStartTime);
                                    strcat(Values, value);
                                    LoadCnt++;
                                } else if (strcmp(DTStormStartTime, retStormStartTime) != 0) {
                                    sprintf(value, "%s = \"%s\", ", _RAIN_STORMS_START_TIME_COL, DTStormStartTime);
                                    strcat(Updates, value);
                                    LoadCnt++;
                                }
                            }
                        }
#endif
                    }
#ifdef _RAIN_STORMS_END_DATE_COL
                    if (StormsEndedValid) {
                        if (StormEndDate[0] != '\0' && StormEndDate != NULL) {
                            if (strlen(retStormStartDate) == 0) {
                                strcat(Attr, _RAIN_STORMS_END_DATE_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", StormEndDate);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (strcmp(StormEndDate, retStormEndDate) != 0) {
                                sprintf(value, "%s = \"%s\", ", _RAIN_STORMS_END_DATE_COL, StormEndDate);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
#ifdef _RAIN_STORMS_END_TIME_COL
                            if (getStormEndDT(cid, StormStartDate, StormEndDate, StormAmountRpt, DTStormEndDate, DTStormEndTime)) {
                                if (strlen(retStormStartDate) == 0) {
                                    strcat(Attr, _RAIN_STORMS_END_TIME_COL);
                                    strcat(Attr, ", ");
                                    sprintf(value, "\"%s\", ", DTStormEndTime);
                                    strcat(Values, value);
                                    LoadCnt++;
                                } else if (strcmp(DTStormEndTime, retStormEndTime) != 0) {
                                    sprintf(value, "%s = \"%s\", ", _RAIN_STORMS_END_TIME_COL, DTStormEndTime);
                                    strcat(Updates, value);
                                    LoadCnt++;
                                }
                            }
#endif
                        }
                    }
#endif
                    if (StormsAmountRptValid) {
#ifdef _RAIN_STORMS_AMOUNT_COL
                        if (StormAmountRpt[0] != '\0' && StormAmountRpt != NULL) {
                            if (strlen(retStormStartDate) == 0) {
                                strcat(Attr, _RAIN_STORMS_AMOUNT_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", StormAmountRpt);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (strcmp(StormAmountRpt, retStormAmountRpt) != 0) {
                                sprintf(value, "%s = \"%s\", ", _RAIN_STORMS_AMOUNT_COL, StormAmountRpt);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
#ifdef _RAIN_STORMS_HIGH_RATE_COL
                        if (HighRate[0] != '\0' && HighRate != NULL) {
                            if (strlen(retStormStartDate) == 0) {
                                strcat(Attr, _RAIN_STORMS_HIGH_RATE_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", HighRate);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (strcmp(HighRate, retHighRate) != 0) {
                                sprintf(value, "%s = \"%s\", ", _RAIN_STORMS_HIGH_RATE_COL, HighRate);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
#ifdef _RAIN_STORMS_HIGH_RATE_DT_COL
                        if (HighRateDT[0] != '\0' && HighRateDT != NULL) {
                            if (strlen(retStormStartDate) == 0) {
                                strcat(Attr, _RAIN_STORMS_HIGH_RATE_DT_COL);
                                strcat(Attr, ", ");
                                sprintf(value, "\"%s\", ", HighRateDT);
                                strcat(Values, value);
                                LoadCnt++;
                            } else if (strcmp(HighRate, retHighRate) != 0 || strcmp(HighRateDT, retHighRateDT) != 0) {
                                sprintf(value, "%s = \"%s\", ", _RAIN_STORMS_HIGH_RATE_DT_COL, HighRateDT);
                                strcat(Updates, value);
                                LoadCnt++;
                            }
                        }
#endif
                    }
                    if (retUnits[0] == '\0') {
                        if (Units[0] == '\0')
                            strcpy(retUnits, bMetric ? "m" : "u");
                        else
                            strcpy(retUnits, Units);
                    }
                    if (Attr[0] != '\0') {
                        strcat(Attr, "F, ");
                        sprintf(value, "\"%s\", ", retUnits);
                        strcat(Values, value);
                        strcat(Attr, "vpID, ");
                        sprintf(value, "\"%i\", ", cid);
                        strcat(Values, value);
                        cpos = strlen(Attr) - 1;
                        while(cpos >= 0 && (Attr[cpos] == ' ' || Attr[cpos] == ','))
                            Attr[cpos--] = '\0';
                        cpos = strlen(Values) - 1;
                        while(cpos >= 0 && (Values[cpos] == ' ' || Values[cpos] == ','))
                            Values[cpos--] = '\0';
                        spos = strstr( Attr, _RAIN_STORMS_START_DATE_COL);
                        sprintf(Query, "INSERT INTO %s (%s) VALUES (%s)", _STORMS_TABLE_NAME, Attr, Values);
                        if (spos != NULL) {
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            if (mysql_query(mySQLconn, Query) != 0)
                                printSQLerror(mySQLconn, "mysql_query() failed", 60, Query);
                            else if (bVerboseSQL)
                                fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                        } else
                            fprintf(stderr, "Can't do INSERT on %s without primary key (%s)\n(%s)\n", _STORMS_TABLE_NAME, _RAIN_STORMS_START_DATE_COL, Query);
                    }
                    if (Updates[0] != '\0') {
                        sprintf(value, "F = \"%s\", ", retUnits);
                        strcat(Updates, value);
                        cpos = strlen(Updates) - 1;
                        while(cpos >= 0 && (Updates[cpos] == ' ' || Updates[cpos] == ','))
                            Updates[cpos--] = '\0';
                        sprintf(Query, "UPDATE %s SET %s WHERE %s = \"%s\" && vpID = \"%i\"", _STORMS_TABLE_NAME, Updates, _RAIN_STORMS_START_DATE_COL, StormStartDate, cid);
                        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                        if (mysql_query(mySQLconn, Query) != 0)
                            printSQLerror(mySQLconn, "mysql_query() failed", 61, Query);
                        else if (bVerboseSQL)
                        fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                    }
                    if (bVerboseSQL) fprintf(stderr, "\n");
                }
            }
        }
    }
    if (bProfiling) logProfileSQL("StormsUpdate");
    return TRUE;
}


/*--------------------------------------------------------------------------
        cvtEnglishINT
----------------------------------------------------------------------------*/
int cvtEnglishINT(int value, int cvtType, int F)
{

    if (cvtType == _CVT_NONE || F != 'm') return value;

    switch (cvtType) {
        case _CVT_WIND:
            value = roundf(value / 1.609344);		/* Kilometers */
            break;
        case '?':
        default:
            break;
    }
    return value;
}


/*--------------------------------------------------------------------------
        cvtEnglishFLOAT
----------------------------------------------------------------------------*/
float cvtEnglishFLOAT(float value, int cvtType, int F)
{

    if (cvtType == _CVT_NONE || F != 'm') return value;

    switch (cvtType) {
        case _CVT_TEMP:
            value = (value * (9.0/5.0)) + 32.0; /* Fahrenheit */
            value = (roundf(value * 10.0))/10.0;
            break;
        case _CVT_WIND:
            value = value / 1.609344;		/* Kilometers */
            value = (roundf(value * 10.0))/10.0;
            break;
        case '?':
        default:
            break;
    }
    return value;
}


/*--------------------------------------------------------------------------
        TemperatureUpdate
----------------------------------------------------------------------------*/
int TemperatureUpdate(unsigned long long int IndexID)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    char Attr[255];
    char Values[255];
    float OutTemp, OutTempF, OutHum, HeatIndex, WindChill;
    unsigned long long int retRecID = 0;
    char value[10];

    if (bVerboseSQL) fprintf(stderr, "\nTemperatureUpdate %s\n", getCurTime());
    sprintf(Query, "SELECT RecID FROM %s USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", _TEMP_TABLE_NAME, IndexID);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 62, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 63, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sscanf(row[0], "%llu", &retRecID);
            mysql_free_result(res_set);
        }
    }

    if (retRecID != 0) {
        fprintf(stderr, "ERROR! from TemperatureUpdate found RecID %llu when expecting none\n", retRecID);
        return FALSE;
    } else {
        Attr[0] = '\0';
        Values[0] = '\0';
#ifdef _INSIDE_TEMP_COL
        if (&rcd.wInsideTemp != NULL) {
            strcat(Attr, ", ");
            strcat(Attr, _INSIDE_TEMP_COL);
            sprintf(value, ", \"%.1f\"", cvtMetricFLOAT(((signed short)rcd.wInsideTemp) / 10.0, _CVT_TEMP) );
            strcat(Values, value);
            LoadCnt++;
        }
#endif
#ifdef _OUTSIDE_TEMP_COL
        if (&rcd.wOutsideTemp != NULL) {
            OutTempF = (signed short)rcd.wOutsideTemp / 10.0;
            OutTemp = cvtMetricFLOAT(OutTempF, _CVT_TEMP);
            strcat(Attr, ", ");
            strcat(Attr, _OUTSIDE_TEMP_COL);
            sprintf(value, ", \"%.1f\"", OutTemp );
            strcat(Values, value);
            LoadCnt++;
#endif
#ifdef _WINDCHILL_COL
            if (&rcd.yWindSpeed != NULL) {
                WindChill = cvtMetricFLOAT(calcWindChill( OutTempF, rcd.yWindSpeed), _CVT_TEMP);
                strcat(Attr, ", ");
                strcat(Attr, _WINDCHILL_COL);
                sprintf(value, ", \"%.1f\"", WindChill );
                strcat(Values, value);
                LoadCnt++;
            }
#endif
#ifdef _HEATINDEX_COL
            if (&rcd.yOutsideHum != NULL) {
                OutHum = rcd.yOutsideHum;
                HeatIndex = cvtMetricFLOAT(calcHeatIndex( OutTempF, OutHum ), _CVT_TEMP);
                strcat(Attr, ", ");
                strcat(Attr, _HEATINDEX_COL);
                sprintf(value, ", \"%.1f\"", HeatIndex );
                strcat(Values, value);
                LoadCnt++;
            }
#endif
        }
        strcat(Attr, ", F");
        strcat(Values, bMetric ? ", \"m\"" : ", \"u\"");
        sprintf(Query, "INSERT INTO %s (RecID%s) VALUES (\"%llu\"%s)", _TEMP_TABLE_NAME, Attr, IndexID, Values);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 64, Query);
        else if (bVerboseSQL)
            fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
    }
    if (bProfiling) logProfileSQL("TemperatureUpdate");
    return TRUE;
}


/*--------------------------------------------------------------------------
        BarometerUpdate
----------------------------------------------------------------------------*/
int BarometerUpdate(unsigned long long int IndexID)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    char Attr[255];
    char Values[255];
    unsigned long long int retRecID = 0;
    char value[20];
    char curBaroTrend[16];
    char curBaroTrendImg[10];

    if (bVerboseSQL) fprintf(stderr, "\nBarometerUpdate %s\n", getCurTime());
    sprintf(Query, "SELECT RecID FROM %s USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", _BARO_TABLE_NAME, IndexID);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 65, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 66, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sscanf(row[0], "%llu", &retRecID);
            mysql_free_result(res_set);
        }
    }

    if (retRecID != 0) {
        fprintf(stderr, "ERROR! from BarometerUpdate found RecID %llu when expecting none\n", retRecID);
        return FALSE;
    } else {
        Attr[0] = '\0';
        Values[0] = '\0';
        sprintf(curBaroTrend, "%s", BaroTrend(0));
        sprintf(curBaroTrendImg, "%s", BaroTrendImage());
#ifdef _BARO_CURR_COL
        if (rcd.wBarometer > 0) {
            strcat(Attr, ", ");
            strcat(Attr, _BARO_CURR_COL);
            if (!bMetric)
                sprintf(value, ", \"%.3f\"", rcd.wBarometer / 1000.0 );
            else
                sprintf(value, ", \"%.1f\"", cvtMetricFLOAT(rcd.wBarometer / 1000.0, _CVT_BARO) );
            strcat(Values, value);
            LoadCnt++;
        }
#endif
#ifdef _BARO_TREND_COL
        if (*curBaroTrend != '\0' && curBaroTrend != NULL) {
            strcat(Attr, ", ");
            strcat(Attr, _BARO_TREND_COL);
            sprintf(value, ", \"%s\"", curBaroTrend);
            strcat(Values, value);
            LoadCnt++;
        }
#endif
#ifdef _BARO_TREND_IMG_COL
        if (*curBaroTrendImg != '\0' && curBaroTrendImg != NULL) {
            strcat(Attr, ", ");
            strcat(Attr, _BARO_TREND_IMG_COL);
            sprintf(value, ", \"%s\"", curBaroTrendImg);
            strcat(Values, value);
            LoadCnt++;
        }
#endif
        strcat(Attr, ", F");
        strcat(Values, bMetric ? ", \"m\"" : ", \"u\"");
        sprintf(Query, "INSERT INTO %s (RecID%s) VALUES (\"%llu\"%s)", _BARO_TABLE_NAME, Attr, IndexID, Values);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);

        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 67, Query);
        else if (bVerboseSQL)
            fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
    }
    if (bProfiling) logProfileSQL("BarometerUpdate");
    return TRUE;
}


/*--------------------------------------------------------------------------
        WindUpdate
----------------------------------------------------------------------------*/
int WindUpdate(unsigned long long int IndexID)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    char Attr[255];
    char Values[255];
    unsigned long long int retRecID = 0;
    char value[10];
    char curWindRose[4];

    if (bVerboseSQL) fprintf(stderr, "\nWindUpdate %s\n", getCurTime());
    sprintf(Query, "SELECT RecID FROM %s USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", _WIND_TABLE_NAME, IndexID);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 68, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 69, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sscanf(row[0], "%llu", &retRecID);
            mysql_free_result(res_set);
        }
    }

    if (retRecID != 0) {
        fprintf(stderr, "ERROR! from WindUpdate found RecID %llu when expecting none\n", retRecID);
        return FALSE;
    } else {
        Attr[0] = '\0';
        Values[0] = '\0';

        sprintf(curWindRose, "%s", WindRoseDir());
#ifdef _WIND_SPEED_COL
        if (&rcd.yWindSpeed != NULL) {
            strcat(Attr, ", ");
            strcat(Attr, _WIND_SPEED_COL);
            sprintf(value, ", \"%u\"", cvtMetricBYTE(rcd.yWindSpeed, _CVT_WIND) );
            strcat(Values, value);
            LoadCnt++;
        }
#endif
#ifdef _WIND_DIR_COL
        if (&rcd.wWindDir != NULL) {
            strcat(Attr, ", ");
            strcat(Attr, _WIND_DIR_COL);
            sprintf(value, ", \"%u\"", rcd.wWindDir);
            strcat(Values, value);
            LoadCnt++;
        }
#endif
#ifdef _WIND_DIR_ROSE_COL
        if (*curWindRose != '\0' && curWindRose != NULL) {
            strcat(Attr, ", ");
            strcat(Attr, _WIND_DIR_ROSE_COL);
            sprintf(value, ", \"%s\"", curWindRose);
            strcat(Values, value);
            LoadCnt++;
        }
#endif
        strcat(Attr, ", F");
        strcat(Values, bMetric ? ", \"m\"" : ", \"u\"");
        sprintf(Query, "INSERT INTO %s (RecID%s) VALUES (\"%llu\"%s)", _WIND_TABLE_NAME, Attr, IndexID, Values);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);

        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 70, Query);
        else if (bVerboseSQL)
            fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
    }
    if (bProfiling) logProfileSQL("WindUpdate");
    return TRUE;
}


/*--------------------------------------------------------------------------
        checkForStormStart
----------------------------------------------------------------------------*/
unsigned long long int checkForStormStart(char *StormStart, int vpID)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    unsigned long long int RecID = 0;

    sprintf(Query, "SELECT t1.RecID FROM DT AS t1 INNER JOIN %s AS t2 USING (RecID) WHERE t2.%s = \"%s\" && t1.vpID = \"%i\"", _RAIN_TABLE_NAME, _STORM_START_DATE_COL, StormStart,vpID);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 71, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set != NULL) {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sscanf(row[0], "%llu", &RecID);
            mysql_free_result(res_set);
        }
    }
    if (bProfiling) logProfileSQL("checkForStormStart");
    return RecID;
}


/*--------------------------------------------------------------------------
        getPrevRainRecID
----------------------------------------------------------------------------*/
unsigned long long int getPrevRainRecID(unsigned long long int IndexID, int vpID)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    unsigned long long int prevRecID = 0;
    time_t curTT = 0;

    time(&curTT);
    curTT = max(0,curTT-86400);
    sprintf(Query, "SELECT t1.RecID FROM DT AS t1 USE INDEX (RecIDvpID) INNER JOIN %s AS t2 USING (RecID) WHERE t1.vpID = \"%i\" && t1.RecID < \"%llu\" ORDER BY t1.RecID DESC LIMIT 1", _RAIN_TABLE_NAME, vpID, IndexID);	/* MAX() */
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 72, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set != NULL) {
            if ((row = mysql_fetch_row (res_set)) != NULL) {
                if (row[0] != NULL) sscanf(row[0], "%llu", &prevRecID);
            }
            mysql_free_result(res_set);
        }
    }
    if (bVerboseSQL) fprintf(stderr, "Previous RecID %llu\n", prevRecID);
    if (bProfiling) logProfileSQL("getPrevRainRecID");
    return(prevRecID);
}


/*--------------------------------------------------------------------------
        getIncRain
----------------------------------------------------------------------------*/
float getIncRain(unsigned long long int IndexID, int vpID, float *PerMinute)
{
    float IncRain = -1.0;
    int F;
    unsigned long int bTTbeg = 0;
    unsigned long int bTTend = 0;
    unsigned long long int prevRecID = 0;

    *PerMinute = -1.0;
#ifdef _RAIN_TABLE_NAME
    if (mySQLconn == NULL) {
        if (bVerboseSQL) fprintf(stderr, "\ngetIncRain: setting up mySQLconn\n");
        if (! mySQLsetup()) {
            if (bVerboseSQL) fprintf(stderr, "getIncRain: setting up mySQLconn failed\n");
            return IncRain;
        }
    }

#ifdef _RAIN_STORM_COL
    float StormAmount = -1.0;
    float fStormRain  = -1.0;
    float eTime = 0.0;
#ifdef _DAY_RAIN_COL
    float DayAmount = -1.0;
    float fRainDay = -1.0;
    float IncRainDay = -1.0;
#endif
#endif
    if (bVerboseSQL) fprintf(stderr, "\ngetIncRain: IndexID %llu %s\n", IndexID, getCurTime());
#ifdef _RAIN_STORM_COL
    if (IndexID > 1) {
        bTTend = getLongIntValue(IndexID, "DT", "bTT", "RecID", &F);
        if (bTTend == 0xffffffff) bTTend = ttrt;
        prevRecID = getPrevRainRecID(IndexID, vpID);
        bTTbeg = getLongIntValue(prevRecID, "DT", "bTT", "RecID", &F);
        if (bTTbeg == 0xffffffff) bTTbeg = 0;
        if (bTTbeg > 0 && bTTend > 0) eTime = (float)(bTTend - bTTbeg) / 60.0;
        if (eTime < 0) eTime = 0;
        if (bVerboseSQL) fprintf(stderr, "bTTbeg %lu bTTend %lu eTime %.6f\n", bTTbeg, bTTend, eTime);
        if (prevRecID > 0) {
            StormAmount = getStormAmount(prevRecID);	 /* try this to prevent a -0.00 value */
            if (StormAmount >= 0.0) {
                fStormRain = cvtMetricFLOAT(rcd.wStormRain / 100.0, _CVT_RAIN);
                IncRain = fStormRain - StormAmount;
            }
        }
        if (bVerboseSQL) fprintf(stderr, "getIncRain: rcd.wStormRain %i fStormRain %f StormAmount %f IncRain    %f\n", rcd.wStormRain, fStormRain, StormAmount, IncRain);
#ifdef _DAY_RAIN_COL
        if (prevRecID > 0) {
            DayAmount = getDayAmount(prevRecID);		/* try this to prevent a -0.00 value */
            if (DayAmount >= 0.0) {
                fRainDay = cvtMetricFLOAT(rcd.wRainDay / 100.0, _CVT_RAIN);
                IncRainDay = fRainDay - DayAmount;
            }
        }
        if (IncRain < 0.0 && IncRainDay >= 0.0) IncRain = IncRainDay;
/* keep this around for now in case -0.0 starts showing up again
        if (IncRain < 0.0)
          if (IncRainDay >= 0.0)
              IncRain = IncRainDay;
          else
              IncRain = 0.0;
*/
        if (eTime > 0.0 && IncRain >= 0.0) *PerMinute = IncRain / eTime;
        if (bVerboseSQL) fprintf(stderr, "getIncRain: rcd.wRainDay   %i fRainDay   %f DayAmount   %f IncRainDay %f PerMinute %f\n", rcd.wRainDay, fRainDay, DayAmount, IncRainDay, *PerMinute);
#endif
    }
#endif
#endif
    if (*PerMinute < 0.0 && IncRain >= 0.0) *PerMinute = IncRain;
    if (bVerboseSQL) fprintf(stderr, "getIncRain: Returning IncRain %.2f PerMinute %.3f\n\n", IncRain, *PerMinute);
    if (bProfiling) logProfileSQL("getIncRain");
    return IncRain;
}


/*--------------------------------------------------------------------------
        RainUpdate
----------------------------------------------------------------------------*/
int RainUpdate(unsigned long long int IndexID, int vpID)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    char Attr[255];
    char Values[255];
    unsigned long long int retRecID = 0;
    char Time[6];
    char value[16];
    char StormStart[12];
    unsigned long long int StormStartPtr = 0;
    unsigned long long int LastStormStartPtr = 0;
    BOOL IsRaining = FALSE;
    BOOL UpdateStormStarted = TRUE;
    float StormAmount = -1.0;
    float PerMinute = -1.0;
#ifdef _INCREMENTAL_RAIN_COL
    float IncRain = -1.0;
#endif

    if (bVerboseSQL) fprintf(stderr, "\nRainUpdate %s\n", getCurTime());
    sprintf(Query, "SELECT RecID FROM %s USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", _RAIN_TABLE_NAME, IndexID);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 73, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 74, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sscanf(row[0], "%llu", &retRecID);
            mysql_free_result(res_set);
        }
    }

    sprintf(Query, "SELECT HHMM FROM DT USE INDEX (RecID) WHERE RecID = \"%llu\"", IndexID);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 75, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 76, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sscanf(row[0], "%s", Time);
            mysql_free_result(res_set);
        }
    }

    if (retRecID != 0) {
        fprintf(stderr, "ERROR! from RainUpdate found RecID %llu when expecting none\n", retRecID);
        return FALSE;
    } else {
        Attr[0] = '\0';
        Values[0] = '\0';
        StormStart[0] = '\0';

        sprintf(StormStart, "%s", PrintDate(rcd.wStormStart));
        if (strcmp(StormStart, "n/a") == 0)
            StormStart[0] = '\0';
        else
            DateSQL(StormStart);

#ifdef _DAY_RAIN_COL
        if (rcd.wRainDay > 0 && &rcd.wRainDay != NULL) {
            strcat(Attr, ", ");
            strcat(Attr, _DAY_RAIN_COL);
            if (strcmp(Time, "00:00") != 0) 				/* if it's midnight and there is a Days Rain amount */
                sprintf(value, ", \"%.2f\"", cvtMetricFLOAT(rcd.wRainDay / 100.0, _CVT_RAIN) );	/* it's yesterday's amount i.e. the Vantage Pro     */
            else							/* hasn't zeroed it yet so...                       */
                sprintf(value, ", 0.00\"");				/* zero it                                          */
            strcat(Values, value);
            LoadCnt++;
        }
#endif
#ifdef _RAIN_STORM_COL
        if (rcd.wStormRain > 0 && &rcd.wStormRain != NULL) {
            strcat(Attr, ", ");
            strcat(Attr, _RAIN_STORM_COL);
            sprintf(value, ", \"%.2f\"", cvtMetricFLOAT(rcd.wStormRain / 100.0, _CVT_RAIN) );
            strcat(Values, value);
            LoadCnt++;
        }
#endif
#ifdef _INCREMENTAL_RAIN_COL
        if (IndexID > 1) {
            IncRain = getIncRain(IndexID, vpID, &PerMinute);
            if (IncRain >= 0.0) {
                strcat(Attr, ", ");
                strcat(Attr, _INCREMENTAL_RAIN_COL);
                sprintf(value, ", \"%.2f\"", IncRain);
                strcat(Values, value);
                LoadCnt++;
            }
#ifdef _PER_MINUTE_RAIN_COL
            if (PerMinute >= 0.0) {
                strcat(Attr, ", ");
                strcat(Attr, _PER_MINUTE_RAIN_COL);
                sprintf(value, ", \"%.3f\"", PerMinute);
                strcat(Values, value);
                LoadCnt++;
            }
#endif
        }
#endif
#ifdef _RAIN_RATE_COL
        if ((rcd.wRainDay > 0 || rcd.wStormRain > 0) && &rcd.wRainRate != NULL) {
            strcat(Attr, ", ");
            strcat(Attr, _RAIN_RATE_COL);
            sprintf(value, ", \"%.2f\"", cvtMetricFLOAT(rcd.wRainRate / 100.0, _CVT_RAIN) );
            strcat(Values, value);
            LoadCnt++;
        }
#endif
#ifdef _STORM_START_DATE_COL
        if (StormStart[0] != '\0' && StormStart != NULL) {
            if (checkForStormStart(StormStart, vpID) == 0) {
                strcat(Attr, ", ");
                strcat(Attr, _STORM_START_DATE_COL);
                sprintf(value, ", \"%s\"", StormStart);
                strcat(Values, value);
                LoadCnt++;
            }
        }
#endif
#ifdef _IS_RAINING_COL
        if (&rcd.wRainRate != NULL) {
            strcat(Attr, ", ");
            strcat(Attr, _IS_RAINING_COL);
            sprintf(value, ", \"%u\"", rcd.wRainRate ? TRUE : FALSE);
            strcat(Values, value);
            LoadCnt++;
        }
#endif
#ifdef _DAY_ET_COL
        if (&rcd.wETDay != NULL) {
            strcat(Attr, ", ");
            strcat(Attr, _DAY_ET_COL);
            if (strcmp(Time, "00:00") != 0)				/* if it's midnight and there is a ET amount    */
                sprintf(value, ", \"%.3f\"", cvtMetricFLOAT(rcd.wETDay / 1000.0, _CVT_RAIN) );	/* it's yesterday's amount i.e. the Vantage Pro */
            else							/* hasn't zeroed it yet so...                   */
                sprintf(value, ", \"0.000\"");				/* zero it                                      */
            strcat(Values, value);
            LoadCnt++;
        }
#endif
        strcat(Attr, ", F");
        strcat(Values, bMetric ? ", \"m\"" : ", \"u\"");
        sprintf(Query, "INSERT INTO %s (RecID%s) VALUES (\"%llu\"%s)", _RAIN_TABLE_NAME, Attr, IndexID, Values);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);

        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 77, Query);
        else if (bVerboseSQL)
            fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
    }
#if defined(_RAIN_TABLE_NAME) && defined(_IS_RAINING_COL) && defined(_STORM_START_DATE_COL) && defined(_RAIN_STORM_COL)
    /* do this as an update in case the vPro is slow in flagging a storm, which it is, */
    /* and to avoid multiple rows in the Rain table indicating the start of the storm  */
    if((unsigned)rcd.wStormStart != 0x7fff && (unsigned)rcd.wStormStart != 0xffff && rcd.wStormStart != 0) {
        sprintf(StormStart, "%s", PrintDate(rcd.wStormStart));
        if (DateSQL(StormStart)) {
            if (checkForStormStart(StormStart, vpID) == 0) {
                StormStartPtr = LastStormStartPtr = IndexID;
                if ((IsRaining = getIsRaining(StormStartPtr)) || (StormAmount = getStormAmount(StormStartPtr)) > 0.0){
                    while(StormStartPtr > 0 && (IsRaining || StormAmount > 0.0)) {
                        if ((IsRaining = getIsRaining(StormStartPtr)) || (StormAmount = getStormAmount(StormStartPtr)) > 0.0) {
                            LastStormStartPtr = StormStartPtr;
                            StormStartPtr--;
                        }
                    }
                    if (bVerboseSQL)
                        fprintf(stderr, "CurStormStartPtr %llu StormStartPtr %llu CurIsRaining %i\n", StormStartPtr, LastStormStartPtr, IsRaining);
                    sprintf(Query, "SELECT RecID FROM %s USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", _RAIN_TABLE_NAME, LastStormStartPtr);
                    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                    if (mysql_query(mySQLconn, Query) != 0)
                        printSQLerror(mySQLconn, "mysql_query() failed", 78, Query);
                    else {
                        res_set = mysql_store_result(mySQLconn);
                        if (res_set == NULL) {
                            fprintf(stderr, "WARN: Can't find RecID (%llu) in %s table to update %s\n", LastStormStartPtr, _RAIN_TABLE_NAME, _STORM_START_DATE_COL);
                            UpdateStormStarted = FALSE;
                        } else {
                            if (UpdateStormStarted) {
                                if ((row = mysql_fetch_row (res_set)) != NULL) {
                                    if (row[0] != NULL) sscanf(row[0], "%llu", &StormStartPtr);
                                } else {
                                    UpdateStormStarted = FALSE;
                                }
                                mysql_free_result(res_set);
                                if (! UpdateStormStarted || LastStormStartPtr != StormStartPtr) {
                                    fprintf(stderr, "WARN: RecID mismatch attempting to update %s in %s - wanted (%llu) got (%llu)\n", _STORM_START_DATE_COL, _RAIN_TABLE_NAME, LastStormStartPtr, StormStartPtr);
                                    UpdateStormStarted = FALSE;
                                }
                            }
                        }
                    }
                    if (UpdateStormStarted) {
                        sprintf(Query, "UPDATE %s SET %s = \"%s\" WHERE RecID = \"%llu\"", _RAIN_TABLE_NAME, _STORM_START_DATE_COL, StormStart,  LastStormStartPtr);
                        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                        if (mysql_query(mySQLconn, Query) != 0)
                            fprintf(stderr, "WARN: Could not update %s in %s in %s table (%llu)\n", StormStart, _STORM_START_DATE_COL, _RAIN_TABLE_NAME, LastStormStartPtr);
                        else if (bVerboseSQL)
                            fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                    }
                }
            }
        }
    }
#endif
    if (bProfiling) logProfileSQL("RainUpdate");
    return TRUE;
}


/*--------------------------------------------------------------------------
        HumidityUpdate
----------------------------------------------------------------------------*/
int HumidityUpdate(unsigned long long int IndexID)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    char Attr[255];
    char Values[255];
    unsigned long long int retRecID = 0;
    char value[10];

    if (bVerboseSQL) fprintf(stderr, "\nHumidityUpdate %s\n", getCurTime());
    sprintf(Query, "SELECT RecID FROM %s USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", _HUM_TABLE_NAME, IndexID);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 79, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 80, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sscanf(row[0], "%llu", &retRecID);
            mysql_free_result(res_set);
        }
    }

    if (retRecID != 0) {
        fprintf(stderr, "ERROR! from HumidityUpdate found RecID %llu when expecting none\n", retRecID);
        return FALSE;
    } else {
        Attr[0] = '\0';
        Values[0] = '\0';
#ifdef _INSIDE_HUM_COL
        if (&rcd.yInsideHum != NULL) {
            strcat(Attr, ", ");
            strcat(Attr, _INSIDE_HUM_COL);
            sprintf(value, ", \"%u\"", rcd.yInsideHum  );
            strcat(Values, value);
            LoadCnt++;
        }
#endif
#ifdef _OUTSIDE_HUM_COL
        if (&rcd.yOutsideHum != NULL) {
            strcat(Attr, ", ");
            strcat(Attr, _OUTSIDE_HUM_COL);
            sprintf(value, ", \"%u\"", rcd.yOutsideHum  );
            strcat(Values, value);
            LoadCnt++;
        }
#endif
        sprintf(Query, "INSERT INTO %s (RecID%s) VALUES (\"%llu\"%s)", _HUM_TABLE_NAME, Attr, IndexID, Values);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 81, Query);
        else if (bVerboseSQL)
            fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
    }
    if (bProfiling) logProfileSQL("HumidityUpdate");
    return TRUE;
}


/*--------------------------------------------------------------------------
        SolarUpdate
----------------------------------------------------------------------------*/
int SolarUpdate(unsigned long long int IndexID, int cid)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    char Attr[255];
    char Values[255];
    int myYMD = 0;
    unsigned long long int retRecID = 0;
    int Minutes = 0;
    int Seconds = 0;
    float eTime = 0.0;
    unsigned long int bTTbeg = 0;
    unsigned long int bTTend = 0;
    char value[32];

    if (bVerboseSQL) fprintf(stderr, "\nSolarUpdate %s\n", getCurTime());
    sprintf(Query, "SELECT RecID FROM %s USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", _SOLAR_TABLE_NAME, IndexID);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 82, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 83, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sscanf(row[0], "%llu", &retRecID);
            mysql_free_result(res_set);
        }
    }

    if (retRecID != 0) {
        fprintf(stderr, "ERROR! from SolarUpdate found RecID %llu when expecting none\n", retRecID);
        return FALSE;
    } else {
        if(rcd.wSolarRad == 32767)
            return TRUE;
        sprintf(Query, "SELECT YearMonthDay, bTT FROM DT USE INDEX (RecIDvpID) WHERE RecID = \"%llu\" && vpID = \"%i\"", IndexID, cid);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 84, Query);
        else {
            res_set = mysql_store_result(mySQLconn);
            if (res_set == NULL)
                printSQLerror(mySQLconn, "mysql_store_result() failed", 85, Query);
            else {
                if ((row = mysql_fetch_row (res_set)) != NULL)
                    if (row[0] != NULL) sscanf(row[0], "%i", &myYMD);
                    if (row[1] != NULL) sscanf(row[1], "%lu", &bTTend);
                mysql_free_result(res_set);
            }
        }
        if (myYMD > 0) {
            sprintf(Query, "SELECT T1.bTT from DT AS T1 USE INDEX (YMDvpID) INNER JOIN %s AS T2 USING (RecID) WHERE T1.YearMonthDay = \"%i\" && T1.vpID = \"%i\" && T2.%s > 0 ORDER BY T1.bTT LIMIT 1", _SOLAR_TABLE_NAME, myYMD, cid, _SOLAR_RAD_COL);
            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
            if (mysql_query(mySQLconn, Query) != 0)
                printSQLerror(mySQLconn, "mysql_query() failed", 86, Query);
            else {
                res_set = mysql_store_result(mySQLconn);
                if (res_set == NULL)
                    printSQLerror(mySQLconn, "mysql_store_result() failed", 87, Query);
                else {
                    if ((row = mysql_fetch_row (res_set)) != NULL)
                        if (row[0] != NULL) sscanf(row[0], "%lu", &bTTbeg);
                    mysql_free_result(res_set);
                }
            }
        }
        if (bTTbeg > 0 && bTTend > 0) {
            eTime = (float)((bTTend - bTTbeg)/60);
            Minutes = (int)eTime;
            Seconds = (bTTend-bTTbeg)%60;
            eTime += (float)Seconds/100.0;
        }
        if (bVerboseSQL) fprintf(stderr, "bTTbeg %lu bTTend %lu Minutes %i Seconds %i eTime %.2f\n", bTTbeg, bTTend, Minutes, Seconds, eTime);
        Attr[0] = '\0';
        Values[0] = '\0';
#ifdef _SOLAR_RAD_COL
        if (&rcd.wSolarRad != NULL) {
            strcat(Attr, ", ");
            strcat(Attr, _SOLAR_RAD_COL);
            sprintf(value, ", \"%u\"", rcd.wSolarRad);
            strcat(Values, value);
            LoadCnt++;
        }
#endif
#ifdef _SOLAR_RAD_MIN_COL
        if (rcd.wSolarRad > 0 && bTTbeg > 0) {
            strcat(Attr, ", ");
            strcat(Attr, _SOLAR_RAD_MIN_COL);
            sprintf(value, ", \"%.02f\"", eTime);
            strcat(Values, value);
            LoadCnt++;
        }
#endif
        sprintf(Query, "INSERT INTO %s (RecID%s) VALUES (\"%llu\"%s)", _SOLAR_TABLE_NAME, Attr, IndexID, Values);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 88, Query);
        else if (bVerboseSQL)
            fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
    }
    if (bProfiling) logProfileSQL("SolarUpdate");
    return TRUE;
}


/*--------------------------------------------------------------------------
        getHLtableIntValue
----------------------------------------------------------------------------*/
int getHLtableIntValue(int cid, char date[], char dateCol[], char table[], char column[])
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    int value;

    sprintf(Query, "SELECT %s FROM %s USE INDEX (PRIMARY) WHERE %s = \"%s\" && vpID = \"%i\" LIMIT 1", column, table, dateCol, date, cid);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 89, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 90, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL) {
                if (row[0] != NULL) 
                    sscanf(row[0], "%i", &value);
                else
                    value = 0xffff;
            }
            mysql_free_result(res_set);
        }
    }
    return value;
}


/*--------------------------------------------------------------------------
        getHLtableString
----------------------------------------------------------------------------*/
void getHLtableString(int cid, char date[], char dateCol[], char table[], char column[], char value[])
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];

    sprintf(Query, "SELECT %s FROM %s USE INDEX (PRIMARY) WHERE %s = \"%s\" && vpID = \"%i\" LIMIT 1", column, table, dateCol, date, cid);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 91, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 92, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL)
                    sprintf(value, "%s", row[0]);
            mysql_free_result(res_set);
        }
    }
    return;
}


/*--------------------------------------------------------------------------
        getRecIDDHM
----------------------------------------------------------------------------*/
unsigned long long int getRecIDDHM(int cid, char hlDate[], char Time[])
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    unsigned long long int RecID = 0;

    sprintf(Query, "SELECT RecID FROM DT USE INDEX (bDate,HHMM,vpID) WHERE bDate = \"%s\" && HHMM = \"%s\" && vpID = \"%i\" LIMIT 1", hlDate, Time, cid);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 93, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 94, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sscanf(row[0], "%llu", &RecID);
            mysql_free_result(res_set);
        }
    }
    return RecID;
}


/*--------------------------------------------------------------------------
        getFloatValue
----------------------------------------------------------------------------*/
float getFloatValue(unsigned long long int RecID, char table[], char column[], char index[], int *F)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    char Value[128];
    float value = 0xffff;

    *F = '-';
    sprintf(Query, "SELECT %s, F FROM %s ", column, table);
    if (strlen(index) > 0) {
        sprintf(Value, "USE INDEX (%s) ", index);
        strcat(Query, Value);
    }
    sprintf(Value, "WHERE RecID = %llu", RecID);
    strcat(Query, Value);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 95, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 96, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL) {
                if (row[0] != NULL) sscanf(row[0], "%f", &value);
                if (row[1] != NULL) sscanf(row[1], "%c", (char *)F);
            }
            mysql_free_result(res_set);
        }
    }
    return value;
}


/*--------------------------------------------------------------------------
        getIntValue
----------------------------------------------------------------------------*/
int getIntValue(unsigned long long int RecID, char table[], char column[], char index[], int *F)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    char Value[128];
    int value = 0xffff;

    *F = '-';
    sprintf(Query, "SELECT %s", column);
    if (strcmp(table, _HUM_TABLE_NAME) != 0 && strcmp(table, _SOLAR_TABLE_NAME) != 0 && strcmp(table, "DT") != 0) strcat(Query, ", F");
    sprintf(Value, " FROM %s ", table);
    strcat(Query, Value);
    if (strlen(index) > 0) {
        sprintf(Value, "USE INDEX (%s) ", index);
        strcat(Query, Value);
    }
    sprintf(Value, "WHERE RecID = %llu", RecID);
    strcat(Query, Value);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 97, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 98, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL) {
                if (row[0] != NULL) sscanf(row[0], "%i", &value);
                if (strcmp(table, _HUM_TABLE_NAME) != 0 && strcmp(table, _SOLAR_TABLE_NAME) != 0)
                    if (row[1] != NULL) sscanf(row[1], "%c", (char *)F);
            }
            mysql_free_result(res_set);
        }
    }
    return value;
}


/*--------------------------------------------------------------------------
        getLongIntValue
----------------------------------------------------------------------------*/
unsigned long int getLongIntValue(unsigned long long int RecID, char table[], char column[], char index[], int *F)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    char Query[1024];
    char Value[128];
    unsigned long int value = 0xffffffff;

    *F = '-';
    sprintf(Query, "SELECT %s", column);
    if (strcmp(table, _HUM_TABLE_NAME) != 0 && strcmp(table, _SOLAR_TABLE_NAME) != 0 && strcmp(table, "DT") != 0) strcat(Query, ", F");
    sprintf(Value, " FROM %s ", table);
    strcat(Query, Value);
    if (strlen(index) > 0) {
        sprintf(Value, "USE INDEX (%s) ", index);
        strcat(Query, Value);
    }
    sprintf(Value, "WHERE RecID = %llu", RecID);
    strcat(Query, Value);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 99, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 100, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL) {
                if (row[0] != NULL) sscanf(row[0], "%lu", &value);
                if (strcmp(table, _HUM_TABLE_NAME) != 0 && strcmp(table, _SOLAR_TABLE_NAME) != 0)
                    if (row[1] != NULL) sscanf(row[1], "%c", (char *)F);
            }
            mysql_free_result(res_set);
        }
    }
    return value;
}


/*--------------------------------------------------------------------------
        setFloatValue
----------------------------------------------------------------------------*/
int setFloatValue(unsigned long long int RecID, char table[], char column[], float value)
{
    char Query[1024];

    sprintf(Query, "UPDATE %s SET %s = %.3f WHERE RecID = \"%llu\"", table, column, value,  RecID);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0) {
        printSQLerror(mySQLconn, "mysql_query() failed", 101, Query);
        return FALSE;
    } else if (bVerboseSQL)
        fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));

    return TRUE;
}

/*--------------------------------------------------------------------------
	setHLtableIntValue
----------------------------------------------------------------------------*/
int setHLtableIntValue(int cid, char date[], char dateCol[], char table[], char column[], int value)
{
    char Query[1024];

    sprintf(Query, "UPDATE %s SET %s = %i WHERE %s = \"%s\" && vpID = \"%i\"", table, column, value, dateCol, date, cid);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0) {
        printSQLerror(mySQLconn, "mysql_query() failed", 102, Query);
        return FALSE;
    } else if (bVerboseSQL)
        fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));

    return TRUE;
}


/*--------------------------------------------------------------------------
        setHLtableString
----------------------------------------------------------------------------*/
int setHLtableString(int cid, char date[], char dateCol[], char table[], char column[], char value[])
{
    char Query[1024];

    sprintf(Query, "UPDATE %s SET %s = \"%s\" WHERE %s = \"%s\" && vpID = \"%i\"", table, column, value, dateCol, date, cid);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0) {
        printSQLerror(mySQLconn, "mysql_query() failed", 103, Query);
        return FALSE;
    } else if (bVerboseSQL)
        fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));

    return TRUE;
}


/*--------------------------------------------------------------------------
        setIntValue
----------------------------------------------------------------------------*/
int setIntValue(unsigned long long int RecID, char table[], char column[], int value)
{
    char Query[1024];

    sprintf(Query, "UPDATE %s SET %s = %i WHERE RecID = \"%llu\"", table, column, value,  RecID);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0) {
        printSQLerror(mySQLconn, "mysql_query() failed", 104, Query);
        return FALSE;
    } else if (bVerboseSQL)
        fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));

    return TRUE;
}


/*--------------------------------------------------------------------------
        HighLowUpdate
----------------------------------------------------------------------------*/
int HighLowUpdate(int cid)
{
    char hlDate[11];
    float InTempHiDay, InTempLoDay, TempHiDay, TempLoDay, TempLoDayU, TempHiDayU;
    float BaroHiDay, BaroLoDay, ChillLoCalc, HeatHiCalc, RainHiDay;
    int WindHiDay, WindHiDayU, InHumHiDay, InHumLoDay, SolarHiDay, THSWHiDay, THSWHiDayU, UVHiDay, UVHiDayU, DewHiDay, DewHiDayU, DewLoDay, DewLoDayU;
    char BaroHiTime[6], BaroLoTime[6], WindHiTime[6], InTempHiTime[6], InTempLoTime[6], InHumHiTime[6], InHumLoTime[6], TempHiTime[6], TempLoTime[6], SolarHiTime[6], RainHiTime[6], THSWHiTime[6], retTHSWHiTime[20], UVHiTime[6], retUVHiTime[20], DewHiTime[6], retDewHiTime[20], DewLoTime[6], retDewLoTime[20];
    float retBaroHiMinute, retBaroLoMinute, retInTempHiMinute, retInTempLoMinute, retTempLoMinuteU, retTempHiMinute, retTempLoMinute, retChillLoMinute, retHeatHiMinute, retRainRateHiMinute;
    int retWindHiMinute, retInHumHiMinute, retInHumLoMinute, retHumHiDay, retSolarHiDay, retTHSWHiDay, retUVHiDay, retDewHiDay, retDewLoDay;
    float retWindHiMinuteU;
    int F;
    unsigned long long int retRecID = 0;
    int status = TRUE;
    BOOL bMtimeSave;
    BOOL bVerboseSQLsave = bVerboseSQL;
    struct tm DT;
    char myYMD[9] = "";
    char myDT[20] = "";

    if (bVerboseSQL) fprintf(stderr, "\nHighLowUpdate --> RealTime %s\n", getCurTime());

    bMtimeSave = bMtime;
    bMtime = TRUE;
    strncpy(hlDate, hlDateTimeCT, 10);
    hlDate[10] = '\0';

#if defined(_BARO_TABLE_NAME) && defined(_BARO_CURR_COL)
    if (bVerboseSQL) fprintf(stderr, "\nChecking BaroHiDay\n");
    BaroHiDay = cvtMetricFLOAT(hld.wBaroHiDay / 1000.0, _CVT_BARO);
    strcpy(BaroHiTime , TimeConvert(hld.wBaroHiTime));
    if (strcmp(BaroHiTime, "n/a") == 0) {
        if (bVerboseSQL) fprintf(stderr, "BaroHiTime %s\n", BaroHiTime);
    } else {
        retRecID = getRecIDDHM(cid, hlDate, BaroHiTime);
        if (retRecID > 0) {
            retBaroHiMinute = getFloatValue(retRecID, _BARO_TABLE_NAME, _BARO_CURR_COL, "PRIMARY", &F);
            if (retBaroHiMinute != 65535 && BaroHiDay > retBaroHiMinute) {
                if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s BaroHiTime %s BaroHiDay %.3f retBaroHiMinute %.3f Format %s\n", hlDate, BaroHiTime, BaroHiDay, retBaroHiMinute, ((int)F == 'm') ? "Metric" : "English");
                status &= setFloatValue(retRecID, _BARO_TABLE_NAME, _BARO_CURR_COL, BaroHiDay);
            }
        }
    }

    if (bVerboseSQL) fprintf(stderr, "\nChecking BaroLoDay\n");
    BaroLoDay = cvtMetricFLOAT(hld.wBaroLoDay / 1000.0, _CVT_BARO);
    strcpy(BaroLoTime , TimeConvert(hld.wBaroLoTime));
    if (strcmp(BaroLoTime, "n/a") == 0) {
        if (bVerboseSQL) fprintf(stderr, "BaroLoTime %s\n", BaroLoTime);
    } else {
        retRecID = getRecIDDHM(cid, hlDate, BaroLoTime);
        if (retRecID > 0) {
            retBaroLoMinute = getFloatValue(retRecID, _BARO_TABLE_NAME, _BARO_CURR_COL, "PRIMARY", &F);
            if (retBaroLoMinute != 65535 && BaroLoDay < retBaroLoMinute) {
                if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s BaroLoTime %s BaroLoDay %.3f retBaroLoMinute %.3f Format %s\n", hlDate, BaroLoTime, BaroLoDay, retBaroLoMinute, ((int)F == 'm') ? "Metric" : "English");
                status &= setFloatValue(retRecID, _BARO_TABLE_NAME, _BARO_CURR_COL, BaroLoDay);
            }
        }
    }
#endif

#if defined(_WIND_TABLE_NAME) && defined(_WIND_SPEED_COL)
    if (bVerboseSQL) fprintf(stderr, "\nChecking WindHiDay\n");
    WindHiDayU = hld.yWindHiDay;
    WindHiDay = cvtMetricINT(WindHiDayU, _CVT_WIND);
    strcpy(WindHiTime , TimeConvert(hld.wWindHiTime));
    if (strcmp(WindHiTime, "n/a") == 0) {
        if (bVerboseSQL) fprintf(stderr, "WindHiTime %s\n", WindHiTime);
    } else {
        retRecID = getRecIDDHM(cid, hlDate, WindHiTime);
        if (retRecID > 0) {
            retWindHiMinute = getIntValue(retRecID, _WIND_TABLE_NAME, _WIND_SPEED_COL, "PRIMARY", &F);
            if (retWindHiMinute != 65535 && WindHiDay > retWindHiMinute) {
                if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s WindHiTime %s WindHiDay %i WindHiDayU %i retWindHiMinute %i Format %s\n", hlDate, WindHiTime, WindHiDay, WindHiDayU, retWindHiMinute, ((int)F == 'm') ? "Metric" : "English");
                status &= setIntValue(retRecID, _WIND_TABLE_NAME, _WIND_SPEED_COL, WindHiDay);
#if defined(_TEMP_TABLE_NAME) && defined(_OUTSIDE_TEMP_COL) && defined(_WINDCHILL_COL)
                retWindHiMinute = getIntValue(retRecID, _WIND_TABLE_NAME, _WIND_SPEED_COL, "PRIMARY", &F);
                retTempLoMinute = getFloatValue(retRecID, _TEMP_TABLE_NAME, _OUTSIDE_TEMP_COL, "PRIMARY", &F);
                if (retTempLoMinute != 65535) {
                    retWindHiMinuteU =cvtEnglishFLOAT(retWindHiMinute, _CVT_WIND, F);
                    retTempLoMinuteU = cvtEnglishFLOAT(retTempLoMinute, _CVT_TEMP, F);
                    ChillLoCalc = cvtMetricFLOAT(calcWindChill(retTempLoMinuteU, retWindHiMinuteU), _CVT_TEMP);
                    retChillLoMinute = getFloatValue(retRecID, _TEMP_TABLE_NAME, _WINDCHILL_COL, "PRIMARY", &F);
                    if (retChillLoMinute != 65535 && ChillLoCalc != retChillLoMinute) {
                        if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s WindHiTime %s retTempLoMinute %.1f retTempLoMinuteU %.1f WindHiDay %i WindHiDayU %i retWindHiMinute %i retWindHiMinuteU %.1f ChillLoCalc %.1f retChillLoMinute %.1f Format %s\n", hlDate, WindHiTime, retTempLoMinute, retTempLoMinuteU, WindHiDay, WindHiDayU, retWindHiMinute, retWindHiMinuteU, ChillLoCalc, retChillLoMinute, ((int)F == 'm') ? "Metric" : "English");
                        status &= setFloatValue(retRecID, _TEMP_TABLE_NAME, _WINDCHILL_COL, ChillLoCalc);
                    }
                }
#endif
            }
        }
    }
#endif

#if defined(_TEMP_TABLE_NAME) && defined(_INSIDE_TEMP_COL)
    if (bVerboseSQL) fprintf(stderr, "\nChecking InTempHiDay\n");
    InTempHiDay = cvtMetricFLOAT(((signed short)hld.wInTempHiDay) / 10.0, _CVT_TEMP);
    strcpy(InTempHiTime , TimeConvert(hld.wInTempHiTime));
    if (strcmp(InTempHiTime, "n/a") == 0) {
        if (bVerboseSQL) fprintf(stderr, "InTempHiTime %s\n", InTempHiTime);
    } else {
        retRecID = getRecIDDHM(cid, hlDate, InTempHiTime);
        if (retRecID > 0) {
            retInTempHiMinute = getFloatValue(retRecID, _TEMP_TABLE_NAME, _INSIDE_TEMP_COL, "PRIMARY", &F);
            if (retInTempHiMinute != 65535 && InTempHiDay > retInTempHiMinute) {
                if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s InTempHiTime %s InTempHiDay %.1f retInTempHiMinute %.1f Format %s\n", hlDate, InTempHiTime, InTempHiDay, retInTempHiMinute, ((int)F == 'm') ? "Metric" : "English");
                status &= setFloatValue(retRecID, _TEMP_TABLE_NAME, _INSIDE_TEMP_COL, InTempHiDay);
            }
        }
    }

    if (bVerboseSQL) fprintf(stderr, "\nChecking InTempLoDay\n");
    InTempLoDay = cvtMetricFLOAT(((signed short)hld.wInTempLoDay) / 10.0, _CVT_TEMP);
    strcpy(InTempLoTime , TimeConvert(hld.wInTempLoTime));
    if (strcmp(InTempLoTime, "n/a") == 0) {
        if (bVerboseSQL) fprintf(stderr, "InTempLoTime %s\n", InTempLoTime);
    } else {
        retRecID = getRecIDDHM(cid, hlDate, InTempLoTime);
        if (retRecID > 0) {
            retInTempLoMinute = getFloatValue(retRecID, _TEMP_TABLE_NAME, _INSIDE_TEMP_COL, "PRIMARY", &F);
            if (retInTempLoMinute != 65535 && InTempLoDay < retInTempLoMinute) {
                if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s InTempLoTime %s InTempLoDay %.1f retInTempLoMinute %.1f Format %s\n", hlDate, InTempLoTime, InTempLoDay, retInTempLoMinute, ((int)F == 'm') ? "Metric" : "English");
                status &= setFloatValue(retRecID, _TEMP_TABLE_NAME, _INSIDE_TEMP_COL, InTempLoDay);
            }
        }
    }
#endif

#if defined(_HUM_TABLE_NAME) && defined(_INSIDE_HUM_COL)
    if (bVerboseSQL) fprintf(stderr, "\nChecking InHumHiDay\n");
    InHumHiDay = hld.yInHumHiDay;
    strcpy(InHumHiTime , TimeConvert(hld.wInHumHiTime));
    if (strcmp(InHumHiTime, "n/a") == 0) {
        if (bVerboseSQL) fprintf(stderr, "InHumHiTime %s\n", InHumHiTime);
    } else {
        retRecID = getRecIDDHM(cid, hlDate, InHumHiTime);
        if (retRecID > 0) {
            retInHumHiMinute = getIntValue(retRecID, _HUM_TABLE_NAME, _INSIDE_HUM_COL, "PRIMARY", &F);
            if (retInHumHiMinute != 65535 && InHumHiDay > retInHumHiMinute) {
                if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s InHumHiTime %s InHumHiDay %i retInHumHiMinute %i\n", hlDate, InHumHiTime, InHumHiDay, retInHumHiMinute);
                status &= setIntValue(retRecID, _HUM_TABLE_NAME, _INSIDE_HUM_COL, InHumHiDay);
            }
        }
    }

    if (bVerboseSQL) fprintf(stderr, "\nChecking InHumLoDay\n");
    InHumLoDay = hld.yInHumLoDay;
    strcpy(InHumLoTime , TimeConvert(hld.wInHumLoTime));
    if (strcmp(InHumLoTime, "n/a") == 0) {
        if (bVerboseSQL) fprintf(stderr, "InHumLoTime %s\n", InHumLoTime);
    } else {
        retRecID = getRecIDDHM(cid, hlDate, InHumLoTime);
        if (retRecID > 0) {
            retInHumLoMinute = getIntValue(retRecID, _HUM_TABLE_NAME, _INSIDE_HUM_COL, "PRIMARY", &F);
            if (retInHumLoMinute != 65535 && InHumLoDay < retInHumLoMinute) {
                if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s InHumLoTime %s InHumLoDay %i retInHumLoMinute %i\n", hlDate, InHumLoTime, InHumLoDay, retInHumLoMinute);
                status &= setIntValue(retRecID, _HUM_TABLE_NAME, _INSIDE_HUM_COL, InHumLoDay);
            }
        }
    }
#endif

#if defined(_TEMP_TABLE_NAME) && defined(_OUTSIDE_TEMP_COL)
    if (bVerboseSQL) fprintf(stderr, "\nChecking TempHiDay\n");
    TempHiDayU = (signed short)hld.wTempHiDay / 10.0;
    TempHiDay = cvtMetricFLOAT(TempHiDayU, _CVT_TEMP);
    strcpy(TempHiTime , TimeConvert(hld.wTempHiTime));
    if (strcmp(TempHiTime, "n/a") == 0) {
        if (bVerboseSQL) fprintf(stderr, "TempHiTime %s\n", TempHiTime);
    } else {
        retRecID = getRecIDDHM(cid, hlDate, TempHiTime);
        if (retRecID > 0) {
            retTempHiMinute = getFloatValue(retRecID, _TEMP_TABLE_NAME, _OUTSIDE_TEMP_COL, "PRIMARY", &F);
            if (retTempHiMinute != 65535 && TempHiDay > retTempHiMinute) {
                if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s TempHiTime %s TempHiDay %.1f TempHiDayU %.1f retTempHiMinute %.1f Format %s\n", hlDate, TempHiTime, TempHiDay, TempHiDayU, retTempHiMinute, ((int)F == 'm') ? "Metric" : "English");
                status &= setFloatValue(retRecID, _TEMP_TABLE_NAME, _OUTSIDE_TEMP_COL, TempHiDay);
#if defined(_WIND_TABLE_NAME) && defined(_WIND_SPEED_COL) && defined(_WINDCHILL_COL)
                retTempHiMinute = getFloatValue(retRecID, _TEMP_TABLE_NAME, _OUTSIDE_TEMP_COL, "PRIMARY", &F);
                retWindHiMinute = getIntValue(retRecID, _WIND_TABLE_NAME, _WIND_SPEED_COL, "PRIMARY", &F);
                if (retWindHiMinute != 65535) {
                    retWindHiMinuteU = cvtEnglishFLOAT(retWindHiMinute, _CVT_WIND, F);
                    ChillLoCalc = cvtMetricFLOAT(calcWindChill(TempHiDayU, retWindHiMinuteU), _CVT_TEMP);
                    retChillLoMinute = getFloatValue(retRecID, _TEMP_TABLE_NAME, _WINDCHILL_COL, "PRIMARY", &F);
                    if (retChillLoMinute != 65535 && ChillLoCalc != retChillLoMinute) {
                        if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s TempHiTime %s TempHiDay %.1f TempHiDayU %.1f retWindHiMinute %i retWindHiMinuteU %.1f ChillLoCalc %.1f retChillLoMinute %.1f Format %s\n", hlDate, TempHiTime, TempHiDay, TempHiDayU, retWindHiMinute, retWindHiMinuteU, ChillLoCalc, retChillLoMinute, ((int)F == 'm') ? "Metric" : "English");
                        status &= setFloatValue(retRecID, _TEMP_TABLE_NAME, _WINDCHILL_COL, ChillLoCalc);
                    }
                }
#endif
#if defined(_HUM_TABLE_NAME) && defined(_OUTSIDE_HUM_COL) && defined(_HEATINDEX_COL)
                retHumHiDay = getIntValue(retRecID, _HUM_TABLE_NAME, _OUTSIDE_HUM_COL, "PRIMARY", &F);
                if (retHumHiDay != 65535) {
                    HeatHiCalc = cvtMetricFLOAT(calcHeatIndex( TempHiDayU, retHumHiDay), _CVT_TEMP);
                    retHeatHiMinute = getFloatValue(retRecID, _TEMP_TABLE_NAME, _HEATINDEX_COL, "PRIMARY", &F);
                    if (retHeatHiMinute != 65535 && HeatHiCalc != retHeatHiMinute) {
                        if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s TempHiTime %s TempHiDay %.1f TempHiDayU %.1f retHumHiDay %i HeatHiCalc %.1f retHeatHiMinute %.1f Format %s\n", hlDate, TempHiTime, TempHiDay, TempHiDayU, retHumHiDay, HeatHiCalc, retHeatHiMinute, ((int)F == 'm') ? "Metric" : "English");
                        status &= setFloatValue(retRecID, _TEMP_TABLE_NAME, _HEATINDEX_COL, HeatHiCalc);
                    }
                }
#endif
            }
        }
    }

    if (bVerboseSQL) fprintf(stderr, "\nChecking TempLoDay\n");
    TempLoDayU = (signed short)hld.wTempLoDay / 10.0;
    TempLoDay = cvtMetricFLOAT(TempLoDayU, _CVT_TEMP);
    strcpy(TempLoTime , TimeConvert(hld.wTempLoTime));
    if (strcmp(TempLoTime, "n/a") == 0) {
        if (bVerboseSQL) fprintf(stderr, "TempLoTime %s\n", TempLoTime);
    } else {
        retRecID = getRecIDDHM(cid, hlDate, TempLoTime);
        if (retRecID > 0) {
            retTempLoMinute = getFloatValue(retRecID, _TEMP_TABLE_NAME, _OUTSIDE_TEMP_COL, "PRIMARY", &F);
            if (retTempLoMinute != 65535 && TempLoDay < retTempLoMinute) {
                if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s TempLoTime %s TempLoDay %.1f TempLoDayU %.1f retTempLoMinute %.1f Format %s\n", hlDate, TempLoTime, TempLoDay, TempLoDayU, retTempLoMinute, ((int)F == 'm') ? "Metric" : "English");
                status &= setFloatValue(retRecID, _TEMP_TABLE_NAME, _OUTSIDE_TEMP_COL, TempLoDay);
#if defined(_WIND_TABLE_NAME) && defined(_WIND_SPEED_COL) & defined(_WINDCHILL_COL)
                retTempLoMinute = getFloatValue(retRecID, _TEMP_TABLE_NAME, _OUTSIDE_TEMP_COL, "PRIMARY", &F);
                retWindHiMinute = getIntValue(retRecID, _WIND_TABLE_NAME, _WIND_SPEED_COL, "PRIMARY", &F);
                if (retWindHiMinute != 65535) {
                    retWindHiMinuteU = cvtEnglishFLOAT(retWindHiMinute, _CVT_WIND, F);
                    ChillLoCalc = cvtMetricFLOAT(calcWindChill(TempLoDayU, retWindHiMinuteU), _CVT_TEMP);
                    retChillLoMinute = getFloatValue(retRecID, _TEMP_TABLE_NAME, _WINDCHILL_COL, "PRIMARY", &F);
                    if (retChillLoMinute != 65535 && ChillLoCalc != retChillLoMinute) {
                        if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s TempLoTime %s TempLoDay %.1f TempLoDayU %.1f retWindHiMinute %i retWindHiMinuteU %.1f ChillLoCalc %.1f retChillLoMinute %.1f Format %s\n", hlDate, TempLoTime, TempLoDay, TempLoDayU, retWindHiMinute, retWindHiMinuteU, ChillLoCalc, retChillLoMinute, ((int)F == 'm') ? "Metric" : "English");
                        status &= setFloatValue(retRecID, _TEMP_TABLE_NAME, _WINDCHILL_COL, ChillLoCalc);
                    }
                }
#endif
#if defined(_HUM_TABLE_NAME) && defined(_OUTSIDE_HUM_COL) && defined(_HEATINDEX_COL)
                retHumHiDay = getIntValue(retRecID, _HUM_TABLE_NAME, _OUTSIDE_HUM_COL, "PRIMARY", &F);
                if (retHumHiDay != 65535) {
                    HeatHiCalc = cvtMetricFLOAT(calcHeatIndex( TempLoDayU, retHumHiDay), _CVT_TEMP);
                    retHeatHiMinute = getFloatValue(retRecID, _TEMP_TABLE_NAME, _HEATINDEX_COL, "PRIMARY", &F);
                    if (retHeatHiMinute != 65535 && HeatHiCalc != retHeatHiMinute) {
                        if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s TempLoTime %s TempLoDay %.1f TempLoDayU %.1f retHumHiDay %i HeatHiCalc %.1f retHeatHiMinute %.1f Format %s\n", hlDate, TempLoTime, TempLoDay, TempLoDayU, retHumHiDay, HeatHiCalc, retHeatHiMinute, ((int)F == 'm') ? "Metric" : "English");
                        status &= setFloatValue(retRecID, _TEMP_TABLE_NAME, _HEATINDEX_COL, HeatHiCalc);
                    }
                }
#endif
            }
        }
    }
#endif

#if defined(_SOLAR_TABLE_NAME) && defined(_SOLAR_RAD_COL)
    if (bVerboseSQL) fprintf(stderr, "\nChecking SolarHiDay\n");
    SolarHiDay = hld.wSolarHiDay;
    strcpy(SolarHiTime , TimeConvert(hld.wSolarHiTime));
    if (strcmp(SolarHiTime, "n/a") == 0) {
        if (bVerboseSQL) fprintf(stderr, "SolarHiTime %s\n", SolarHiTime);
    } else {
        retRecID = getRecIDDHM(cid, hlDate, SolarHiTime);
        if (retRecID > 0) {
            retSolarHiDay = getIntValue(retRecID, _SOLAR_TABLE_NAME, _SOLAR_RAD_COL, "PRIMARY", &F);
            if (retSolarHiDay != 65535 && SolarHiDay > retSolarHiDay) {
                if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s SolarHiTime %s SolarHiDay %i retSolarHiDay %i\n", hlDate, SolarHiTime, SolarHiDay, retSolarHiDay);
                status &= setIntValue(retRecID, _SOLAR_TABLE_NAME, _SOLAR_RAD_COL, SolarHiDay);
            }
        }
    }
#endif

#if defined(_RAIN_TABLE_NAME) && defined(_RAIN_RATE_COL)
    if (bVerboseSQL) fprintf(stderr, "\nChecking RainHiDay\n");
    RainHiDay = cvtMetricFLOAT(hld.wRainHiDay / 100.0, _CVT_RAIN);
    strcpy(RainHiTime , TimeConvert(hld.wRainHiTime));
    if (strcmp(RainHiTime, "n/a") == 0) {
        if (bVerboseSQL) fprintf(stderr, "RainHiTime %s\n", RainHiTime);
    } else {
        retRecID = getRecIDDHM(cid, hlDate, RainHiTime);
        if (retRecID > 0) {
            retRainRateHiMinute = getFloatValue(retRecID, _RAIN_TABLE_NAME, _RAIN_RATE_COL, "PRIMARY", &F);
            if (retRainRateHiMinute != 65535 && RainHiDay > 0.0 && retRainRateHiMinute > 0.0 && RainHiDay > retRainRateHiMinute) {
                if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s RainHiTime %s RainHiDay %.2f retRainRateHiMinute %.2f Format %s\n", hlDate, RainHiTime, RainHiDay, retRainRateHiMinute, ((int)F == 'm') ? "Metric" : "English");
                status &= setFloatValue(retRecID, _RAIN_TABLE_NAME, _RAIN_RATE_COL, RainHiDay);
            }
        }
    }
#endif

#if defined(_THSW_DHL_TABLE_NAME) && defined(_THSW_DHL_HI_COL)
    if (bVerboseSQL) fprintf(stderr, "\nChecking DHL THSWHiDay\n");
    THSWHiDayU = (signed short)hld.wTHSWHiDay;
    THSWHiDay = cvtMetricINT(THSWHiDayU, _CVT_TEMP);
    strcpy(THSWHiTime , TimeConvert(hld.wTHSWHiTime));
#
    DT = *(localtime(&ttRun));
    sprintf(myYMD, "%4u%02u%02u", DT.tm_year + 1900, DT.tm_mon + 1, DT.tm_mday);
    sprintf(myDT, "%4u-%02u-%02u %s:00", DT.tm_year + 1900, DT.tm_mon + 1, DT.tm_mday, THSWHiTime);

    if (strcmp(THSWHiTime, "n/a") == 0) {
        if (bVerboseSQL) fprintf(stderr, "THSWHiTime %s\n", THSWHiTime);
    } else {
        retTHSWHiTime[0] = '\0';
        retTHSWHiDay = getHLtableIntValue(cid, myYMD, "YearMonthDay", _THSW_DHL_TABLE_NAME, _THSW_DHL_HI_COL);
        getHLtableString(cid, myYMD, "YearMonthDay", _THSW_DHL_TABLE_NAME, _THSW_DHL_HI_TIME_COL, retTHSWHiTime);
        if (retTHSWHiDay == 65535 || THSWHiDay > retTHSWHiDay) {
            if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s THSWHiTime %s THSWHiDay %i THSWHiDayU %i retTHSWHiDay %i retTHSWHiTime %s Format %s\n", hlDate, THSWHiTime, THSWHiDay, THSWHiDayU, retTHSWHiDay, retTHSWHiTime, ((int)F == 'm') ? "Metric" : "English");
            status &= setHLtableIntValue(cid, myYMD, "YearMonthDay", _THSW_DHL_TABLE_NAME, _THSW_DHL_HI_COL, THSWHiDay);
            status &= setHLtableString(cid, myYMD, "YearMonthDay", _THSW_DHL_TABLE_NAME, _THSW_DHL_HI_TIME_COL, myDT);
        }
    }
#endif

#if defined(_UV_DHL_TABLE_NAME) && defined(_UV_DHL_HI_COL)
    if (bVerboseSQL) fprintf(stderr, "\nChecking DHL UVHiDay\n");
    UVHiDayU = (signed short)hld.yUVHiDay;
    UVHiDay = cvtMetricINT(UVHiDayU, _CVT_TEMP);
    strcpy(UVHiTime , TimeConvert(hld.wUVHiTime));
#
    DT = *(localtime(&ttRun));
    sprintf(myYMD, "%4u%02u%02u", DT.tm_year + 1900, DT.tm_mon + 1, DT.tm_mday);
    sprintf(myDT, "%4u-%02u-%02u %s:00", DT.tm_year + 1900, DT.tm_mon + 1, DT.tm_mday, UVHiTime);

    if (strcmp(UVHiTime, "n/a") == 0) {
        if (bVerboseSQL) fprintf(stderr, "UVHiTime %s\n", UVHiTime);
    } else {
        retUVHiTime[0] = '\0';
        retUVHiDay = getHLtableIntValue(cid, myYMD, "YearMonthDay", _UV_DHL_TABLE_NAME, _UV_DHL_HI_COL);
        getHLtableString(cid, myYMD, "YearMonthDay", _UV_DHL_TABLE_NAME, _UV_DHL_HI_TIME_COL, retUVHiTime);
        if (retUVHiDay == 65535 || UVHiDay > retUVHiDay) {
            if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s UVHiTime %s UVHiDay %i UVHiDayU %i retUVHiDay %i retUVHiTime %s Format %s\n", hlDate, UVHiTime, UVHiDay, UVHiDayU, retUVHiDay, retUVHiTime, ((int)F == 'm') ? "Metric" : "English");
            status &= setHLtableIntValue(cid, myYMD, "YearMonthDay", _UV_DHL_TABLE_NAME, _UV_DHL_HI_COL, UVHiDay);
            status &= setHLtableString(cid, myYMD, "YearMonthDay", _UV_DHL_TABLE_NAME, _UV_DHL_HI_TIME_COL, myDT);
        }
    }
#endif

#if defined(_DEW_DHL_TABLE_NAME) && defined(_DEW_DHL_HI_COL)
    if (bVerboseSQL) fprintf(stderr, "\nChecking DHL DewHiDay\n");
    DewHiDayU = (signed short)hld.wDewHiDay;
    DewHiDay = cvtMetricINT(DewHiDayU, _CVT_TEMP);
    strcpy(DewHiTime , TimeConvert(hld.wDewHiTime));
#
    DT = *(localtime(&ttRun));
    sprintf(myYMD, "%4u%02u%02u", DT.tm_year + 1900, DT.tm_mon + 1, DT.tm_mday);
    sprintf(myDT, "%4u-%02u-%02u %s:00", DT.tm_year + 1900, DT.tm_mon + 1, DT.tm_mday, DewHiTime);

    if (strcmp(DewHiTime, "n/a") == 0) {
        if (bVerboseSQL) fprintf(stderr, "DewHiTime %s\n", DewHiTime);
    } else {
        retDewHiTime[0] = '\0';
        retDewHiDay = getHLtableIntValue(cid, myYMD, "YearMonthDay", _DEW_DHL_TABLE_NAME, _DEW_DHL_HI_COL);
        getHLtableString(cid, myYMD, "YearMonthDay", _DEW_DHL_TABLE_NAME, _DEW_DHL_HI_TIME_COL, retDewHiTime);
        if (retDewHiDay == 65535 || DewHiDay > retDewHiDay) {
            if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s DewHiTime %s DewHiDay %i DewHiDayU %i retDewHiDay %i retDewHiTime %s Format %s\n", hlDate, DewHiTime, DewHiDay, DewHiDayU, retDewHiDay, retDewHiTime, ((int)F == 'm') ? "Metric" : "English");
            status &= setHLtableIntValue(cid, myYMD, "YearMonthDay", _DEW_DHL_TABLE_NAME, _DEW_DHL_HI_COL, DewHiDay);
            status &= setHLtableString(cid, myYMD, "YearMonthDay", _DEW_DHL_TABLE_NAME, _DEW_DHL_HI_TIME_COL, myDT);
        }
    }
#endif

#if defined(_DEW_DHL_TABLE_NAME) && defined(_DEW_DHL_LO_COL)
    if (bVerboseSQL) fprintf(stderr, "\nChecking DHL DewLoDay\n");
    DewLoDayU = (signed short)hld.wDewLoDay;
    DewLoDay = cvtMetricINT(DewLoDayU, _CVT_TEMP);
    strcpy(DewLoTime , TimeConvert(hld.wDewLoTime));
#
    DT = *(localtime(&ttRun));
    sprintf(myYMD, "%4u%02u%02u", DT.tm_year + 1900, DT.tm_mon + 1, DT.tm_mday);
    sprintf(myDT, "%4u-%02u-%02u %s:00", DT.tm_year + 1900, DT.tm_mon + 1, DT.tm_mday, DewLoTime);

    if (strcmp(DewLoTime, "n/a") == 0) {
        if (bVerboseSQL) fprintf(stderr, "DewLoTime %s\n", DewLoTime);
    } else {
        retDewLoTime[0] = '\0';
        retDewLoDay = getHLtableIntValue(cid, myYMD, "YearMonthDay", _DEW_DHL_TABLE_NAME, _DEW_DHL_LO_COL);
        getHLtableString(cid, myYMD, "YearMonthDay", _DEW_DHL_TABLE_NAME, _DEW_DHL_LO_TIME_COL, retDewLoTime);
        if (retDewLoDay == 65535 || DewLoDay > retDewLoDay) {
            if (bVerboseSQL) fprintf(stderr, "Data:  hlDate %s DewLoTime %s DewLoDay %i DewLoDayU %i retDewLoDay %i retDewLoTime %s Format %s\n", hlDate, DewLoTime, DewLoDay, DewLoDayU, retDewLoDay, retDewLoTime, ((int)F == 'm') ? "Metric" : "English");
            status &= setHLtableIntValue(cid, myYMD, "YearMonthDay", _DEW_DHL_TABLE_NAME, _DEW_DHL_LO_COL, DewLoDay);
            status &= setHLtableString(cid, myYMD, "YearMonthDay", _DEW_DHL_TABLE_NAME, _DEW_DHL_LO_TIME_COL, myDT);
        }
    }
#endif

    bVerboseSQL = bVerboseSQLsave;
    bMtime = bMtimeSave;
    if (bProfiling) logProfileSQL("HighLowUpdate");
    return status;
}

/*--------------------------------------------------------------------------
        MinPtrUpdate
----------------------------------------------------------------------------*/
int MinPtrUpdate(unsigned long long int IndexID, char * pData, int cid)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    time_t mtt;
    char myDateTime[20];
    unsigned int myMinutes;
    long long unsigned BaroLastID;
    long long unsigned WindLastID;
    long long unsigned RainLastID;
    char Query[1024];
    char Attr[255];
    char Values[512];
    char value[303];
    char Ref10MinS[144];
    char Ref15MinS[144];
    char Baro15MinS[200];
    char Wind10MinS[125];
    char Rain15MinS[150];
    char Ref10MinVal[6] = "";
    char Ref15MinVal[6] = "";
    char retRef10MinVal[6] = "";
    char retRef15MinVal[6] = "";
    char Baro15MinVal[9] = "";
    char Wind10MinVal[6] = "";
    char Rain15MinVal[7] = "";
    char retBaro15MinVal[9] = "";
    char retWind10MinVal[6] = "";
    char retRain15MinVal[7] = "";
    char retPtr[3];
    time_t retBaroTimeT;
    time_t retWindTimeT;
    time_t retRainTimeT;
    int retBaroPtr;
    int retWindPtr;
    int retRainPtr;
    int ret10RefMin;
    int ret15RefMin;
    int retBaroMin;
    int retWindMin;
    int retRainMin;
    char retItemName[21];
    char retItemData[301];
    char curBaro[7];
    char curWind[4];
    char curRain[6];
    int Ref10MinPtr = -1;
    int Ref15MinPtr = -1;
    int Baro15MinPtr = -1;
    int Wind10MinPtr = -1;
    int Rain15MinPtr = -1;
    BOOL BaroPseudoPtr;
    BOOL WindPseudoPtr;
    BOOL RainPseudoPtr;
    BOOL Loop;
    int PtrPos, SSpos, retSSpos;

    if (bVerboseSQL) fprintf(stderr, "\nMinPtrUpdate %s\n", getCurTime());
    if (_MIN_PTR_POINTER_COL == NULL || _MIN_PTR_ITEM_NAME_COL == NULL || 	/* if any of these aren't defined */
        _MIN_PTR_ITEM_DATA_COL == NULL)					/* we can't do our job so...      */
            return FALSE;						/* we're outta here               */

    myDateTime[0] = '\0';
    sprintf(Query, "SELECT bDateTime, bMinute FROM DT USE INDEX (RecID) WHERE RecID = \"%llu\"", IndexID);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 105, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 106, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL)
                if (row[0] != NULL) sprintf(myDateTime, "%s", row[0]);
                if (row[1] != NULL) sscanf(row[1], "%u", &myMinutes);
            mysql_free_result(res_set);
        }
    }

    if (myDateTime[0] == '\0') return FALSE;

    time(&mtt);

    Ref10MinS[0] = Ref15MinS[0] = Baro15MinS[0] = Wind10MinS[0] = Rain15MinS[0] ='\0';
    curBaro[0] = curWind[0] = curRain[0] = '\0';

    /* <*>-<*>-<*>-<*>-<*>-<*> REF10MIN <*>-<*>-<*>-<*>-<*>-<*> */

    retPtr[0] = retItemName[0] = retItemData[0] = '\0';
    Ref10MinPtr = ret10RefMin = -1;

#ifdef _MIN_PTR_REF10MIN

    sprintf(Ref10MinS, "%s", TimeRef10Min(mtt));

    sprintf(Query, "SELECT bMinute, %s, %s, %s FROM %s WHERE %s = \"%s\" && Save != \"%u\"", _MIN_PTR_POINTER_COL, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_ITEM_DATA_COL, _MINUTE_PTR_DATA_TABLE_NAME, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_REF10MIN, TRUE);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 107, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 108, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL) {
                if (row[0] != NULL) sscanf(row[0], "%i", &ret10RefMin);
                if (row[1] != NULL) sscanf(row[1], "%s", retPtr);
                if (row[2] != NULL) sscanf(row[2], "%s", retItemName);
                if (row[3] != NULL) sscanf(row[3], "%s", retItemData);
            }
            mysql_free_result(res_set);
        }
    }

    if (retItemData[0] != '\0') {
        SSpos = retSSpos = 0;
        PtrPos = 0;
        Loop = TRUE;
        while(Loop && sprintf(Ref10MinVal, "%s", SubString(Ref10MinS, ',', &SSpos)) && SSpos <= strlen(Ref10MinS) &&
          sprintf(retRef10MinVal, "%s", SubString(retItemData, ',', &retSSpos)) && retSSpos <= strlen(retItemData)) {
            if (strcmp(Ref10MinVal, retRef10MinVal) != 0) {
                if (Ref10MinPtr < 0 )
                    Ref10MinPtr = PtrPos;
                else {
                    Ref10MinPtr = -1;
                    Loop = FALSE;
                }
            }
            PtrPos++;
        }
        sprintf(Query, "DELETE FROM %s WHERE vpID = \"%i\" && %s = \"%s\"  && Save != \"%u\"", _MINUTE_PTR_DATA_TABLE_NAME, cid, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_REF10MIN, TRUE);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 109, Query);
        else if (bVerboseSQL)
            fprintf(stderr, "       DELETE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
    }

    Attr[0] = '\0';
    Values[0] = '\0';

    strcat(Attr, "RecID");
    sprintf(value, "\"%llu\"", IndexID);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", vpID");
    sprintf(value, ", \"%i\"", cid);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", bDateTime");
    sprintf(value, ", \"%s\"", myDateTime);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", bMinute");
    if ((myMinutes % 10) == 0)
        sprintf(value, ", \"%02u\"", myMinutes);
    else if (ret10RefMin >= 0)
        sprintf(value, ", \"%02i\"", ret10RefMin);
    else
        sprintf(value, ", NULL");
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", time_t");
    sprintf(value, ", \"%u\"", (unsigned int)mtt);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", ");
    strcat(Attr, _MIN_PTR_ITEM_NAME_COL);
    sprintf(value, ", \"%s\"", _MIN_PTR_REF10MIN);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", ");
    strcat(Attr, _MIN_PTR_ITEM_DATA_COL);
    sprintf(value, ", \"%s\"", Ref10MinS);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", ");
    strcat(Attr, _MIN_PTR_POINTER_COL);
    if (Ref10MinPtr >= 0 )
        sprintf(value, ", \"%i\"", Ref10MinPtr);
    else
        sprintf(value, ", NULL" );
    strcat(Values, value);
    LoadCnt++;

    sprintf(Query, "INSERT INTO %s (%s) VALUES (%s)", _MINUTE_PTR_DATA_TABLE_NAME, Attr, Values);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 110, Query);
    else if (bVerboseSQL)
        fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
#endif

    /* <*>-<*>-<*>-<*>-<*>-<*> REF15MIN <*>-<*>-<*>-<*>-<*>-<*> */

    retPtr[0] = retItemName[0] = retItemData[0] = '\0';
    Ref15MinPtr = ret15RefMin = -1;

#ifdef _MIN_PTR_REF15MIN

    sprintf(Ref15MinS, "%s", TimeRef15Min(mtt));

    sprintf(Query, "SELECT bMinute, %s, %s, %s FROM %s WHERE %s = \"%s\" && Save != \"%u\"", _MIN_PTR_POINTER_COL, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_ITEM_DATA_COL, _MINUTE_PTR_DATA_TABLE_NAME, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_REF15MIN, TRUE);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 111, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 112, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL) {
                if (row[0] != NULL) sscanf(row[0], "%i", &ret15RefMin);
                if (row[1] != NULL) sscanf(row[1], "%s", retPtr);
                if (row[2] != NULL) sscanf(row[2], "%s", retItemName);
                if (row[3] != NULL) sscanf(row[3], "%s", retItemData);
            }
            mysql_free_result(res_set);
        }
    }

    if (retItemData[0] != '\0') {
        SSpos = retSSpos = 0;
        PtrPos = 0;
        Ref15MinPtr = -1;
        Loop = TRUE;
        while(Loop && sprintf(Ref15MinVal, "%s", SubString(Ref15MinS, ',', &SSpos)) && SSpos <= strlen(Ref15MinS) &&
          sprintf(retRef15MinVal, "%s", SubString(retItemData, ',', &retSSpos)) && retSSpos <= strlen(retItemData)) {
            if (strcmp(Ref15MinVal, retRef15MinVal) != 0) {
                if (Ref15MinPtr < 0 )
                    Ref15MinPtr = PtrPos;
                else {
                    Ref15MinPtr = -1;
                    Loop = FALSE;
                }
            }
            PtrPos++;
        }
        sprintf(Query, "DELETE FROM %s WHERE vpID = \"%i\" && %s = \"%s\" && Save != \"%u\"", _MINUTE_PTR_DATA_TABLE_NAME, cid, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_REF15MIN, TRUE);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 113, Query);
        else if (bVerboseSQL)
            fprintf(stderr, "       DELETE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
    }

    Attr[0] = '\0';
    Values[0] = '\0';

    strcat(Attr, "RecID");
    sprintf(value, "\"%llu\"", IndexID);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", vpID");
    sprintf(value, ", \"%i\"", cid);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", bDateTime");
    sprintf(value, ", \"%s\"", myDateTime);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", bMinute");
    if ((myMinutes % 15) == 0)
        sprintf(value, ", \"%02u\"", myMinutes);
    else if (ret15RefMin >= 0)
        sprintf(value, ", \"%02i\"", ret15RefMin);
    else
        sprintf(value, ", NULL");
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", time_t");
    sprintf(value, ", \"%u\"", (unsigned int)mtt);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", ");
    strcat(Attr, _MIN_PTR_ITEM_NAME_COL);
    sprintf(value, ", \"%s\"", _MIN_PTR_REF15MIN);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", ");
    strcat(Attr, _MIN_PTR_ITEM_DATA_COL);
    sprintf(value, ", \"%s\"", Ref15MinS);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", ");
    strcat(Attr, _MIN_PTR_POINTER_COL);
    if (Ref10MinPtr >= 0 )
        sprintf(value, ", \"%i\"", Ref15MinPtr);
    else
        sprintf(value, ", NULL" );
    strcat(Values, value);
    LoadCnt++;

    sprintf(Query, "INSERT INTO %s (%s) VALUES (%s)", _MINUTE_PTR_DATA_TABLE_NAME, Attr, Values);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 114, Query);
    else if (bVerboseSQL)
        fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
#endif

    /* <*>-<*>-<*>-<*>-<*>-<*> BARO15MIN <*>-<*>-<*>-<*>-<*>-<*> */

    retItemName[0] = retItemData[0] = '\0';
    Baro15MinPtr = retBaroPtr = retBaroMin = -1;
    BaroLastID = retBaroTimeT = 0;
    BaroPseudoPtr =  FALSE;

#ifdef _MIN_PTR_BARO15MIN
    sprintf(Baro15MinS, "%s", PrintBarSet((unsigned char *)pData, BAR_15_MIN, 0, 24));
    BOOL Baro15MinSValid = TRUE;
    if (strcmp(Baro15MinS, _OVERFLOW_MSG) == 0) {
        Baro15MinSValid = FALSE;
        /* use Baro15MinSValid for now so as to not get set but not used compiler warning */
        if (! Baro15MinSValid) fprintf(stderr, "graph data %-10s %s\n", "Baro15MinS", _OVERFLOW_MSG);
    } else
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","Baro15MinS");

    sprintf(Query, "SELECT bMinute, time_t, RecID, %s, %s, %s FROM %s WHERE vpID = \"%i\" && %s = \"%s\" && Last = \"%u\"", _MIN_PTR_POINTER_COL, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_ITEM_DATA_COL, _MINUTE_PTR_DATA_TABLE_NAME, cid, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_BARO15MIN, TRUE);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 115, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 116, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL) {
                if (row[0] != NULL) sscanf(row[0], "%i", &retBaroMin);
                if (row[1] != NULL) sscanf(row[1], "%u", (unsigned int *)&retBaroTimeT);
                if (row[2] != NULL) sscanf(row[2], "%llu", &BaroLastID);
                if (row[3] != NULL) sscanf(row[3], "%i", &retBaroPtr);
                if (row[4] != NULL) sscanf(row[4], "%s", retItemName);
                if (row[5] != NULL) sscanf(row[5], "%s", retItemData);
            }
            mysql_free_result(res_set);
        }
    }

    sprintf(Query, "UPDATE %s SET Last = \"%u\" WHERE vpID = \"%i\" && %s = \"%s\" && Last = \"%u\"", _MINUTE_PTR_DATA_TABLE_NAME, FALSE, cid, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_BARO15MIN, TRUE);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 117, Query);
    else if (bVerboseSQL)
        fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));

    if (retItemData[0] != '\0') {
        SSpos = retSSpos = 0;
        PtrPos = 0;
        Loop = TRUE;
        while(Loop && sprintf(Baro15MinVal, "%s", SubString(Baro15MinS, ',', &SSpos)) && SSpos <= strlen(Baro15MinS) &&
          sprintf(retBaro15MinVal, "%s", SubString(retItemData, ',', &retSSpos)) && retSSpos <= strlen(retItemData)) {
            if (strcmp(Baro15MinVal, retBaro15MinVal) != 0) {
                if (Baro15MinPtr < 0 ) {
                    Baro15MinPtr = PtrPos;
                    strcpy(curBaro, Baro15MinVal);
                } else {
                    Baro15MinPtr = -1;
                    curBaro[0] = '\0';
                    Loop = FALSE;
                }
            }
            PtrPos++;
        }

        sprintf(Query, "DELETE FROM %s WHERE vpID = \"%i\" && %s = \"%s\" && Save != \"%u\"", _MINUTE_PTR_DATA_TABLE_NAME, cid, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_BARO15MIN, TRUE);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 118, Query);
        else if (bVerboseSQL)
            fprintf(stderr, "       DELETE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));

        if (Baro15MinPtr < 0 && retBaroPtr >= 0 && (myMinutes % 15) == 0) {
            Baro15MinPtr = (retBaroPtr + (((mtt - retBaroTimeT) + 59) / 900)) % 24;
            BaroPseudoPtr = TRUE;
            SSpos = 0;
            PtrPos = Baro15MinPtr;
            while (sprintf(Baro15MinVal, "%s", SubString(Baro15MinS, ',', &SSpos)) && SSpos <= strlen(Baro15MinS) && PtrPos-- > 0);
            sprintf(curBaro, "%s", Baro15MinVal);
        }

    }

    Attr[0] = '\0';
    Values[0] = '\0';

    strcat(Attr, "vpID");
    sprintf(value, "\"%i\"", cid);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", bDateTime");
    sprintf(value, ", \"%s\"", myDateTime);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", bMinute");
    if (Baro15MinPtr >= 0)
        sprintf(value, ", \"%02u\"", myMinutes);
    else if (retBaroMin >= 0)
        sprintf(value, ", \"%02i\"", retBaroMin);
    else
        sprintf(value, ", NULL");
    strcat(Values, value);
    LoadCnt++;
   
    strcat(Attr, ", time_t");
    if (Baro15MinPtr >= 0)
        sprintf(value, ", \"%u\"", (unsigned int)mtt);
    else if (retBaroTimeT >= 0)
        sprintf(value, ", \"%u\"", (unsigned int)retBaroTimeT);
    else
        sprintf(value, ", NULL");
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", RecID");
    if (Baro15MinPtr >= 0)
        sprintf(value, ", \"%llu\"", IndexID);
    else if (BaroLastID > 0)
        sprintf(value, ", \"%llu\"", BaroLastID);
    else
        sprintf(value, ", NULL");
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", Last");
    sprintf(value, ", \"%u\"", TRUE);
    strcat(Values, value);
    LoadCnt++;

    if (BaroPseudoPtr) {
        strcat(Attr, ", Save");
        sprintf(value, ", \"%u\"", TRUE);
        strcat(Values, value);
        LoadCnt++;
    }

    strcat(Attr, ", ");
    strcat(Attr, _MIN_PTR_ITEM_NAME_COL);
    sprintf(value, ", \"%s\"", _MIN_PTR_BARO15MIN);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", ");
    strcat(Attr, _MIN_PTR_ITEM_DATA_COL);
    sprintf(value, ", \"%s\"", Baro15MinS);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", ");
    strcat(Attr, _MIN_PTR_POINTER_COL);
    if (Baro15MinPtr >= 0 )
        sprintf(value, ", \"%i\"", Baro15MinPtr);
    else if (retBaroPtr >= 0 && strcmp(Baro15MinS, retItemData) == 0)
        sprintf(value, ", \"%i\"", retBaroPtr);
    else
        sprintf(value, ", NULL");
    strcat(Values, value);
    LoadCnt++;

    if (Baro15MinPtr >= 0 ) {

    }

    sprintf(Query, "INSERT INTO %s (%s) VALUES (%s)", _MINUTE_PTR_DATA_TABLE_NAME, Attr, Values);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 119, Query);
    else if (bVerboseSQL)
        fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
#endif

    /* <*>-<*>-<*>-<*>-<*>-<*> WIND10MIN <*>-<*>-<*>-<*>-<*>-<*> */

    retItemName[0] = retItemData[0] = '\0';
    Wind10MinPtr = retWindPtr = retWindMin = -1;
    WindLastID = retWindTimeT = 0;
    WindPseudoPtr =  FALSE;

#ifdef _MIN_PTR_WIND10MIN

    sprintf(Wind10MinS, "%s", PrintByteSet((unsigned char *)pData, WIND_SPEED_10_MIN_AVG, 0, 24, _CVT_WIND));
    BOOL Wind10MinSValid = TRUE;
    if (strcmp(Wind10MinS, _OVERFLOW_MSG) == 0) {
        Wind10MinSValid = FALSE;
        /* use Wind10MinSValid for now so as to not get set but not used compiler warning */
        if (! Wind10MinSValid) fprintf(stderr, "graph data %-10s %s\n", "Wind10MinS", _OVERFLOW_MSG);
    } else
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","Wind10MinS");

    sprintf(Query, "SELECT bMinute, time_t, RecID, %s, %s, %s FROM %s WHERE vpID = \"%i\" && %s = \"%s\" && Last = \"%u\"", _MIN_PTR_POINTER_COL, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_ITEM_DATA_COL, _MINUTE_PTR_DATA_TABLE_NAME, cid, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_WIND10MIN, TRUE);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 120, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 121, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL) {
                if (row[0] != NULL) sscanf(row[0], "%i", &retWindMin);
                if (row[1] != NULL) sscanf(row[1], "%u", (unsigned int *)&retWindTimeT);
                if (row[2] != NULL) sscanf(row[2], "%llu", &WindLastID);
                if (row[3] != NULL) sscanf(row[3], "%i", &retWindPtr);
                if (row[4] != NULL) sscanf(row[4], "%s", retItemName);
                if (row[5] != NULL) sscanf(row[5], "%s", retItemData);
            }
            mysql_free_result(res_set);
        }
    }

    sprintf(Query, "UPDATE %s SET Last = \"%u\" WHERE vpID = \"%i\" && %s = \"%s\" && Last = \"%u\"", _MINUTE_PTR_DATA_TABLE_NAME, FALSE, cid, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_WIND10MIN, TRUE);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 122, Query);
    else if (bVerboseSQL)
        fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));

    if (retItemData[0] != '\0') {
        SSpos = retSSpos = 0;
        PtrPos = 0;
        Loop = TRUE;
        while(Loop && sprintf(Wind10MinVal, "%s", SubString(Wind10MinS, ',', &SSpos)) && SSpos <= strlen(Wind10MinS) &&
          sprintf(retWind10MinVal, "%s", SubString(retItemData, ',', &retSSpos)) && retSSpos <= strlen(retItemData)) {
            if (strcmp(Wind10MinVal, retWind10MinVal) != 0) {
                if (Wind10MinPtr < 0 ) {
                    Wind10MinPtr = PtrPos;
                    strcpy(curWind, Wind10MinVal);
                } else {
                    Wind10MinPtr = -1;
                    curWind[0] = '\0';
                    Loop = FALSE;
                }
            }
            PtrPos++;
        }

        sprintf(Query, "DELETE FROM %s WHERE vpID = \"%i\" && %s = \"%s\" && Save != \"%u\"", _MINUTE_PTR_DATA_TABLE_NAME, cid, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_WIND10MIN, TRUE);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 123, Query);
        else if (bVerboseSQL)
            fprintf(stderr, "       DELETE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));

        if (Wind10MinPtr < 0 && retWindPtr >= 0 && (myMinutes % 10) == 0) {
            Wind10MinPtr = (retWindPtr + (((mtt - retWindTimeT) + 59) / 600)) % 24;
            WindPseudoPtr = TRUE;
            SSpos = 0;
            PtrPos = Wind10MinPtr;
            while (sprintf(Wind10MinVal, "%s", SubString(Wind10MinS, ',', &SSpos)) && SSpos <= strlen(Wind10MinS) && PtrPos-- > 0);
            sprintf(curWind, "%s", Wind10MinVal);
        }
    }

    Attr[0] = '\0';
    Values[0] = '\0';

    strcat(Attr, "vpID");
    sprintf(value, "\"%i\"", cid);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", bDateTime");
    sprintf(value, ", \"%s\"", myDateTime);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", bMinute");
    if (Wind10MinPtr >= 0)
        sprintf(value, ", \"%02u\"", myMinutes);
    else if (retWindMin >= 0)
        sprintf(value, ", \"%02i\"", retWindMin);
    else
        sprintf(value, ", NULL");
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", time_t");
    if (Wind10MinPtr >= 0)
        sprintf(value, ", \"%u\"", (unsigned int)mtt);
    else if (retWindTimeT >= 0)
        sprintf(value, ", \"%u\"", (unsigned int)retWindTimeT);
    else
        sprintf(value, ", NULL");
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", RecID");
    if (Wind10MinPtr >= 0)
        sprintf(value, ", \"%llu\"", IndexID);
    else if (WindLastID > 0)
        sprintf(value, ", \"%llu\"", WindLastID);
    else
        sprintf(value, ", NULL");
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", Last");
    sprintf(value, ", \"%u\"", TRUE);
    strcat(Values, value);
    LoadCnt++;

    if (WindPseudoPtr) {
        strcat(Attr, ", Save");
        sprintf(value, ", \"%u\"", TRUE);
        strcat(Values, value);
        LoadCnt++;
    }

    strcat(Attr, ", ");
    strcat(Attr, _MIN_PTR_ITEM_NAME_COL);
    sprintf(value, ", \"%s\"", _MIN_PTR_WIND10MIN);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", ");
    strcat(Attr, _MIN_PTR_ITEM_DATA_COL);
    sprintf(value, ", \"%s\"", Wind10MinS);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", ");
    strcat(Attr, _MIN_PTR_POINTER_COL);
    if (Wind10MinPtr >= 0 )
        sprintf(value, ", \"%i\"", Wind10MinPtr);
    else if (retWindPtr >= 0 && strcmp(Wind10MinS, retItemData) == 0)
        sprintf(value, ", \"%i\"", retWindPtr);
    else
        sprintf(value, ", NULL");
    strcat(Values, value);
    LoadCnt++;

    sprintf(Query, "INSERT INTO %s (%s) VALUES (%s)", _MINUTE_PTR_DATA_TABLE_NAME, Attr, Values);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 124, Query);
    else if (bVerboseSQL)
        fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
#endif

    /* <*>-<*>-<*>-<*>-<*>-<*> RAIN15MIN <*>-<*>-<*>-<*>-<*>-<*> */

    retItemName[0] = retItemData[0] = '\0';
    Rain15MinPtr = retRainPtr = retRainMin = -1;
    RainLastID = retRainTimeT = 0;
    RainPseudoPtr =  FALSE;

#ifdef _MIN_PTR_RAIN15MIN

    sprintf(Rain15MinS, "%s", PrintRainRateByteSet((unsigned char *)pData, RAIN_15_MIN, 0, 24));
    BOOL Rain15MinSValid = TRUE;
    if (strcmp(Rain15MinS, _OVERFLOW_MSG) == 0) {
        Rain15MinSValid = FALSE;
        /* use Rain15MinSValid for now so as to not get set but not used compiler warning */
        if (! Rain15MinSValid) fprintf(stderr, "graph data %-10s %s\n", "Rain15MinS", _OVERFLOW_MSG); 
    } else
        if (bVerboseSQL) fprintf(stderr, "graph data %-10s Valid\n","Rain15MinS");

    sprintf(Query, "SELECT bMinute, time_t, RecID, %s, %s, %s FROM %s WHERE vpID = \"%i\" && %s = \"%s\" && Last = \"%u\"", _MIN_PTR_POINTER_COL, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_ITEM_DATA_COL, _MINUTE_PTR_DATA_TABLE_NAME, cid, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_RAIN15MIN, TRUE);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 125, Query);
    else {
        res_set = mysql_store_result(mySQLconn);
        if (res_set == NULL)
            printSQLerror(mySQLconn, "mysql_store_result() failed", 126, Query);
        else {
            if ((row = mysql_fetch_row (res_set)) != NULL) {
                if (row[0] != NULL) sscanf(row[0], "%i", &retRainMin);
                if (row[1] != NULL) sscanf(row[1], "%u", (unsigned int *)&retRainTimeT);
                if (row[2] != NULL) sscanf(row[2], "%llu", &RainLastID);
                if (row[3] != NULL) sscanf(row[3], "%i", &retRainPtr);
                if (row[4] != NULL) sscanf(row[4], "%s", retItemName);
                if (row[5] != NULL) sscanf(row[5], "%s", retItemData);
            }
            mysql_free_result(res_set);
        }
    }

    sprintf(Query, "UPDATE %s SET Last = \"%u\" WHERE vpID = \"%i\" && %s = \"%s\" && Last = \"%u\"", _MINUTE_PTR_DATA_TABLE_NAME, FALSE, cid, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_RAIN15MIN, TRUE);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 127, Query);
    else if (bVerboseSQL)
        fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));

    if (retItemData[0] != '\0') {
        SSpos = retSSpos = 0;
        PtrPos = 0;
        Loop = TRUE;
        while(Loop && sprintf(Rain15MinVal, "%s", SubString(Rain15MinS, ',', &SSpos)) && SSpos <= strlen(Rain15MinS) &&
          sprintf(retBaro15MinVal, "%s", SubString(retItemData, ',', &retSSpos)) && retSSpos <= strlen(retItemData)) {
            if (strcmp(Rain15MinVal, retRain15MinVal) != 0) {
                if (Rain15MinPtr < 0 ) {
                    Rain15MinPtr = PtrPos;
                    strcpy(curRain, Rain15MinVal);
                } else {
                    Rain15MinPtr = -1;
                    curRain[0] = '\0';
                    Loop = FALSE;
                }
            }
            PtrPos++;
        }

        sprintf(Query, "DELETE FROM %s WHERE vpID = \"%i\" && %s = \"%s\" && Save != \"%u\"", _MINUTE_PTR_DATA_TABLE_NAME, cid, _MIN_PTR_ITEM_NAME_COL, _MIN_PTR_RAIN15MIN, TRUE);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 128, Query);
        else if (bVerboseSQL)
            fprintf(stderr, "       DELETE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));

        if (Rain15MinPtr < 0 && Baro15MinPtr >= 0) {
            if (Baro15MinPtr >= 0)
                Rain15MinPtr = Baro15MinPtr;
            else
                Rain15MinPtr = (retRainPtr + (((mtt - retRainTimeT) + 59) / 900)) % 24;
            RainPseudoPtr = TRUE;
            SSpos = 0;
            PtrPos = Rain15MinPtr;
            while (sprintf(Rain15MinVal, "%s", SubString(Rain15MinS, ',', &SSpos)) && SSpos <= strlen(Rain15MinS) && PtrPos-- > 0);
            sprintf(curRain, "%s", Rain15MinVal);
        }
    }

    Attr[0] = '\0';
    Values[0] = '\0';

    strcat(Attr, "vpID");
    sprintf(value, "\"%i\"", cid);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", bDateTime");
    sprintf(value, ", \"%s\"", myDateTime);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", bMinute");
    if (Rain15MinPtr >= 0)
        sprintf(value, ", \"%02u\"", myMinutes);
    else if (retRainMin >= 0)
        sprintf(value, ", \"%02i\"", retRainMin);
    else
        sprintf(value, ", NULL");
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", time_t");
    if (Rain15MinPtr >= 0)
        sprintf(value, ", \"%u\"", (unsigned int)mtt);
    else if (retRainTimeT >= 0)
        sprintf(value, ", \"%u\"", (unsigned int)retRainTimeT);
    else
        sprintf(value, ", NULL");
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", RecID");
    if (Rain15MinPtr >= 0)
        sprintf(value, ", \"%llu\"", IndexID);
    else if (RainLastID > 0)
        sprintf(value, ", \"%llu\"", RainLastID);
    else
        sprintf(value, ", NULL");
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", Last");
    sprintf(value, ", \"%u\"", TRUE);
    strcat(Values, value);
    LoadCnt++;

#ifdef _MIN_PTR_BARO15MIN
    if (BaroPseudoPtr && RainPseudoPtr) {
#else
    if (RainPseudoPtr) {
#endif
        strcat(Attr, ", Save");
        sprintf(value, ", \"%u\"", TRUE);
        strcat(Values, value);
        LoadCnt++;
    }

    strcat(Attr, ", ");
    strcat(Attr, _MIN_PTR_ITEM_NAME_COL);
    sprintf(value, ", \"%s\"", _MIN_PTR_RAIN15MIN);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", ");
    strcat(Attr, _MIN_PTR_ITEM_DATA_COL);
    sprintf(value, ", \"%s\"", Rain15MinS);
    strcat(Values, value);
    LoadCnt++;

    strcat(Attr, ", ");
    strcat(Attr, _MIN_PTR_POINTER_COL);
    if (Rain15MinPtr >= 0 )
        sprintf(value, ", \"%i\"", Rain15MinPtr);
    else if (retRainPtr >= 0 && strcmp(Rain15MinS, retItemData) == 0)
        sprintf(value, ", \"%i\"", retRainPtr);
    else
        sprintf(value, ", NULL");
    strcat(Values, value);
    LoadCnt++;

    sprintf(Query, "INSERT INTO %s (%s) VALUES (%s)", _MINUTE_PTR_DATA_TABLE_NAME, Attr, Values);
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 129, Query);
    else if (bVerboseSQL)
        fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
#endif

#if defined(_10MIN_TABLE_NAME) && defined(_10MIN_CUR_PTR_COL)
    if ((myMinutes % 10) == 0) {
        if (bVerboseSQL) fprintf(stderr, "\ndoing %s\n", _10MIN_TABLE_NAME);
        Attr[0] = '\0';
        Values[0] = '\0';

        strcat(Attr, "RecID");
        sprintf(value, "\"%llu\"", IndexID);
        strcat(Values, value);
        LoadCnt++;

        if (WindLastID > 0) {
            strcat(Attr, ", ");
            strcat(Attr, "LastID");
            sprintf(value, ", \"%llu\"", WindLastID);
            strcat(Values, value);
            LoadCnt++;
        }

#ifdef _10MIN_LAST_PTR_COL
        if (retWindPtr >= 0) {
            strcat(Attr, ", ");
            strcat(Attr, _10MIN_LAST_PTR_COL);
            sprintf(value, ", \"%i\"", retWindPtr);
            strcat(Values, value);
            LoadCnt++;
        }
#endif

        if (Wind10MinPtr >= 0 || retWindPtr >= 0) {
            strcat(Attr, ", ");
            strcat(Attr, _10MIN_CUR_PTR_COL);
            if (Wind10MinPtr >= 0)
                sprintf(value, ", \"%i\"", Wind10MinPtr);
            else if (retWindPtr >= 0)
                sprintf(value, ", \"%i\"", (retWindPtr + 1) % 24);
            strcat(Values, value);
            LoadCnt++;
        }

        strcat(Attr, ", ");
        strcat(Attr, "WindPseudoCurPtr");
        sprintf(value, ", \"%u\"", WindPseudoPtr);
        strcat(Values, value);
        LoadCnt++;

#ifdef _10MIN_WIND_AVG_BY_RPT_COL
        if (Wind10MinPtr >= 0 ) {
            if (curWind[0] != '\0') {
                strcat(Attr, ", ");
                strcat(Attr, _10MIN_WIND_AVG_BY_RPT_COL);
                sprintf(value, ", \"%s\"", curWind);
                strcat(Values, value);
                LoadCnt++;
            }
        }
#endif
        sprintf(Query, "INSERT INTO %s (%s) VALUES (%s)", _10MIN_TABLE_NAME, Attr, Values);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 130, Query);
        else if (bVerboseSQL)
            fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
    }
#endif

#if defined(_15MIN_TABLE_NAME) && defined(_15MIN_CUR_PTR_COL)
    if (Baro15MinPtr >= 0 || Rain15MinPtr >= 0) {
        if (bVerboseSQL) fprintf(stderr, "\ndoing %s\n", _15MIN_TABLE_NAME);
        Attr[0] = '\0';
        Values[0] = '\0';

        strcat(Attr, "RecID");
        sprintf(value, "\"%llu\"", IndexID);
        strcat(Values, value);
        LoadCnt++;

        if (BaroLastID > 0 || RainLastID > 0) {
            strcat(Attr, ", ");
            strcat(Attr, "LastID");
            if (BaroLastID > 0) 
                sprintf(value, ", \"%llu\"", BaroLastID);
            else if (RainLastID >0)
                sprintf(value, ", \"%llu\"", RainLastID);
            else
                sprintf(value, ", NULL");
            strcat(Values, value);
            LoadCnt++;
        }

#ifdef _15MIN_LAST_PTR_COL
        if (retBaroPtr >= 0 || retRainPtr >= 0) {
            strcat(Attr, ", ");
            strcat(Attr, _15MIN_LAST_PTR_COL);
            if (retBaroPtr >= 0)
                sprintf(value, ", \"%i\"", retBaroPtr);
            else if (retRainPtr >= 0)
                sprintf(value, ", \"%i\"", retRainPtr);
            else
                sprintf(value, ", NULL");
            strcat(Values, value);
            LoadCnt++;
        }
#endif

        strcat(Attr, ", ");
        strcat(Attr, _15MIN_CUR_PTR_COL);
        if (Baro15MinPtr >= 0)
            sprintf(value, ", \"%i\"", Baro15MinPtr);
        else if (Rain15MinPtr >= 0)
            sprintf(value, ", \"%i\"", Rain15MinPtr);
        else
            sprintf(value, ", NULL");
        strcat(Values, value);
        LoadCnt++;

        strcat(Attr, ", ");
        strcat(Attr, "BaroPseudoCurPtr");
        sprintf(value, ", \"%u\"", BaroPseudoPtr);
        strcat(Values, value);
        LoadCnt++;

        strcat(Attr, ", ");
        strcat(Attr, "RainPseudoCurPtr");
        sprintf(value, ", \"%u\"", RainPseudoPtr);
        strcat(Values, value);
        LoadCnt++;

#ifdef _15MIN_BAR_BY_RPT_COL
        if (curBaro[0] != '\0') {
            strcat(Attr, ", ");
            strcat(Attr, _15MIN_BAR_BY_RPT_COL);
            sprintf(value, ", \"%s\"", curBaro);
            strcat(Values, value);
            LoadCnt++;
        }
#endif
#ifdef _15MIN_RAIN_BY_RPT_COL
        if (curRain[0] != '\0') {
            strcat(Attr, ", ");
            strcat(Attr, _15MIN_RAIN_BY_RPT_COL);
            sprintf(value, ", \"%s\"", curRain);
            strcat(Values, value);
            LoadCnt++;
        }
#endif
        sprintf(Query, "INSERT INTO %s (%s) VALUES (%s)", _15MIN_TABLE_NAME, Attr, Values);
        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
        if (mysql_query(mySQLconn, Query) != 0)
            printSQLerror(mySQLconn, "mysql_query() failed", 131, Query);
        else if (bVerboseSQL)
            fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
    }
#endif

    if (bProfiling) logProfileSQL("MinPtrUpdate");
    return TRUE;
}


/*--------------------------------------------------------------------------
        maskPassword
----------------------------------------------------------------------------*/
char *maskPassword(char *pw)
{
    int i;

    if (pw != NULL) {
        for (i = 0; i < strlen(pw); i++) pw[i] = '*';
        pw[i] = '\0';
    }

    return(pw);
}


/*--------------------------------------------------------------------------
        showSQLopts
----------------------------------------------------------------------------*/
void showSQLopts()
{
    static char *opt_password_t = NULL;

    if (opt_password != NULL) {
        opt_password_t = getMemPtr(strlen(opt_password));
        strcpy(opt_password_t, opt_password);
    }
    if (bVerboseSQL) fprintf(stderr, "\nmySQL params:\n\topt_host_name   \"%s\"\n\topt_user_name   \"%s\"\n\topt_password    \"%s\"\n\topt_db_name     \"%s\"\n\topt_port_num    \"%i\"\n\topt_socket_name \"%s\"\n",opt_host_name,opt_user_name,maskPassword(opt_password_t),opt_db_name,opt_port_num,opt_socket_name);
   if (opt_password_t != NULL) free(opt_password_t);

    return;
}


/*--------------------------------------------------------------------------
        mySQLsetup
----------------------------------------------------------------------------*/
BOOL mySQLsetup()
{

    /* initialize connection handler */
    mySQLconn = mysql_init (NULL);
    if (mySQLconn == NULL)
    {
        printSQLerror (NULL, "mysql_init() failed (probably out of memory)", 132, "");
        return FALSE;
    }

   if (bVerboseSQL) showSQLopts();
    /* connect to server */
    if (mysql_real_connect (mySQLconn, opt_host_name, opt_user_name, opt_password,
        opt_db_name, opt_port_num, opt_socket_name, opt_flags) == NULL)
    {
        printSQLerror (mySQLconn, "mysql_real_connect() failed", 133, "");
        mysql_close (mySQLconn);
        return FALSE;
    }

    if (bProfiling) enableProfileSQL("mySQLsetup");

    return TRUE;
}

/*--------------------------------------------------------------------------
        enableProfileSQL
----------------------------------------------------------------------------*/
BOOL enableProfileSQL(char whereFrom[])
{
    FILE *fp;
    char Query[1024];

    if (strlen(sProfileDataFN) > 0) {
        fp = fopen(sProfileDataFN, "a");
        if(fp == NULL) {
            fprintf(stderr, "can't open %s. Profiling disabled.\n", sProfileDataFN);
            bProfiling = FALSE;
        } else {
            fprintf(fp, "\nProfiling enabled for %s %s\n", whereFrom, getCurTime());
            fclose(fp);
        }
    }
    strcpy(Query, "SET PROFILING = 1");
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0) {
        printSQLerror(mySQLconn, "mysql_query() failed", 134, Query);
        return FALSE;
    } else
        return TRUE;
}

/*--------------------------------------------------------------------------
        disableProfileSQL
----------------------------------------------------------------------------*/
BOOL disableProfileSQL(char whereFrom[])
{
    FILE *fp;
    char Query[1024];

    if (strlen(sProfileDataFN) > 0) {
        fp = fopen(sProfileDataFN, "a");
        if(fp == NULL)
            fprintf(stderr, "can't open %s\n", sProfileDataFN);
        else {
            fprintf(fp, "\nProfiling disabled for %s %s\n\n", whereFrom, getCurTime());
            fclose(fp);
        }
    }
    strcpy(Query, "SET PROFILING = 0");
    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0) {
        printSQLerror(mySQLconn, "mysql_query() failed", 135, Query);
        return FALSE;
    } else
        return TRUE;
}


/*--------------------------------------------------------------------------
        varAddChar
----------------------------------------------------------------------------*/
char *varAddChar(char *pBuf, char new)
{
    char *pNew = NULL;

    if (pBuf == NULL) {
       pNew = getMemPtr(2);
       *pNew = new;
       *(pNew+1) = '\0';
    } else {
       pNew = getMemPtr(strlen(pBuf)+1);
       *(pNew) = '\0';
       sprintf(pNew, "%s%c%c", pBuf, new, '\0');
       free(pBuf);
    }

    return(pNew);
}


/*--------------------------------------------------------------------------
        varAddString
----------------------------------------------------------------------------*/
char *varAddString(char *pBuf, char *sNew)
{
    char *pNew = NULL;

    if (pBuf == NULL) {
       pNew = getMemPtr(strlen(sNew)+2);
       *(pNew) = '\0';
       strcpy(pNew, sNew);
    } else {
       pNew = getMemPtr(strlen(pBuf)+strlen(sNew)+2);
       *(pNew) = '\0';
       strcpy(pNew, pBuf);
       strcat(pNew, sNew);
       free(pBuf);
    }

    return(pNew);
}


/*--------------------------------------------------------------------------
        ImportWarning
----------------------------------------------------------------------------*/
BOOL ImportWarning(int vpID)
{
    extern BOOL bImportOR;

    fprintf(stderr, " \nWARNING:\n");
    if (! bImportOR)
        fprintf(stderr, "\tIt appears you are attempting to import data that is from this\n\tdatabase or from another database that has a Vantage Pro Weather\n\tConsole with the same Console ID = %i\n\n\tIf you really want to import this data append \":OVERRIDE\" to the\n\tfilename supplied with the \"-i\" option\n\ti.e. \"%s:OVERRIDE\"\n", vpID, sImportDataFN);
    else
        fprintf(stderr, "\tYou have choosen to import data that may be from this database\n\tor from a database with a duplicate Vantage Pro Console ID = %i\n", vpID);

    return TRUE;
}


/*--------------------------------------------------------------------------
        doImport
----------------------------------------------------------------------------*/
void doImport()
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    BOOL MySQLconnect = FALSE;
    unsigned long int rowsReturned = 0;
    FILE *fp;
    int ch = 0;
    int rt = 0;
#define	maxCols	51
    char *values[maxCols];
    char *Query = NULL;
    char *Updates = NULL;
    char *ErrMsg = NULL;
    char value[128];
    int cpos = 0;
    int i;
    int cols;
    int retLocvpID = -1;
    unsigned long int iCount = 0;
    unsigned long long int retRecID = 0;
    BOOL foundRecID = FALSE;
    BOOL foundvpID = FALSE;
    BOOL foundStorm = FALSE;
    BOOL foundYMD = FALSE;
    BOOL foundDHL = FALSE;
    BOOL LOOP = TRUE;
    BOOL iok = FALSE;
    BOOL warned = FALSE;
    extern BOOL bImportOR;
    extern char VersionM[];

    if (bVerboseSQL) fprintf(stderr, "\nImporting from \"%s\"\n\n", sImportDataFN);
    if (strlen(sImportDataFN) > 0) {
        fp = fopen(sImportDataFN, "r");
        if(fp == NULL) {
            fprintf(stderr, "can't open %s\n", sImportDataFN);
            if (MySQLconnect) mysql_close (mySQLconn);
            return;
        } else {
            if (mySQLconn == NULL) { 
                if (bVerboseSQL) fprintf(stderr, "doImport: setting up mySQLconn to %s\n",opt_db_name);
                if (! mySQLsetup()) {
                    if (bVerboseSQL) fprintf(stderr, "doImport: setting up mySQLconn failed\n");
                    return;
                }
                MySQLconnect = TRUE;
                if (bVerboseSQL) fprintf(stderr, "\n");
            }
            for (i = 0; i <= maxCols; i++) values[i] = NULL;
            while (LOOP && rt  != EOF && ch != EOF) {
                rt = fgetc(fp);
                switch (rt) {
                    case 'V':
                        if (bVerboseSQL) fprintf(stderr, "%c ", rt);
                        i = 0;
                        if (values[i] != NULL) {
                            free(values[i]);
                            values[i] = NULL;
                        }
                        ch = fgetc(fp);
                        ch = fgetc(fp);
                        while(ch != '\0') {
                            while(ch != '\3' && ch != '\0') {
                                values[i] = varAddChar(values[i], ch);
                                ch = fgetc(fp);
                            }
                            if (ch == '\3') {
                                i++;
                                if (values[i] != NULL) {
                                    free(values[i]);
                                    values[i] = NULL;
                                }
                            }
                            if (ch != '\0') ch = fgetc(fp);
                        }
                        cols = i;
                        if (bVerboseSQL) {
                            for (i = 0; i <= cols; i++) fprintf(stderr, "\"%s\" ", values[i]);
                            fprintf(stderr, "\n");
                        }
                        if (strcmp(values[0], VersionM) != 0) {
                            fprintf(stderr, "\nERROR! Version mismatch Export v%s Import v%s aborting...\n", values[0], VersionM);
                            LOOP = FALSE;
                        } else
                            iok = TRUE;
                        break;
                    case 'L':
                        if (! iok) break;
                        if (bVerboseSQL) fprintf(stderr, "%c ", rt);
                        i = 0;
                        if (values[i] != NULL) {
                            free(values[i]);
                            values[i] = NULL;
                        }
                        ch = fgetc(fp);
                        ch = fgetc(fp);
                        while(ch != '\0') {
                            while(ch != '\3' && ch != '\0') {
                                values[i] = varAddChar(values[i], ch);
                                ch = fgetc(fp);
                            }
                            if (ch == '\3') {
                                i++;
                                if (values[i] != NULL) {
                                    free(values[i]);
                                    values[i] = NULL;
                                }
                            }
                            if (ch != '\0') ch = fgetc(fp);
                        }
                        if (!bVerboseSQL) fprintf(stderr, "%c%c", star[(iCount-1)%5], 0x0d);
                        iCount++;
                        cols = i;
                        if (bVerboseSQL) {
                            for (i = 0; i <= cols; i++) fprintf(stderr, "\"%s\" ", values[i]);
                            fprintf(stderr, "\n");
                        }
                        if (Query != NULL) {
                            free(Query);
                            Query = NULL;
                        }
                        Query = varAddString(Query, "SELECT ");
                        i = 0;
                        while (Location[i] != '\0') {
                            Query = varAddString(Query, Location[i++]);
                            if (Location[i] != '\0') Query = varAddString(Query, ", ");
                        }
                        Query = varAddString(Query, " FROM Location WHERE vpID = \"");
                        Query = varAddString(Query, values[0]);
                        Query = varAddString(Query, "\"");
                        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                        retLocvpID = 0;
                        foundvpID = FALSE;
                        if (mysql_query(mySQLconn, Query) != 0) {
                            printSQLerror(mySQLconn, "mysql_query() failed", 136, Query);
                            return;
                        } else {
                            res_set = mysql_store_result(mySQLconn);
                            if (res_set == NULL) {
                                printSQLerror(mySQLconn, "mysql_store_result() failed", 137, Query);
                                return;
                            } else {
                                row = mysql_fetch_row (res_set);
                                if (row != NULL && row[0] != NULL) {
                                    sscanf(row[0], "%i", &retLocvpID);
                                    sprintf(value, "%i", retLocvpID);
                                    if (strcmp(row[0], value) == 0) foundvpID = TRUE;
                                }
                            }
//                            if (bVerboseSQL) fprintf(stderr, "Location found vpID %s\n", foundvpID ? "True" : "False");
                        }
                        if (! foundvpID) {
                            mysql_free_result(res_set);
                            if (Query != NULL) {
                                free(Query);
                                Query = NULL;
                            }
                            Query = varAddString(Query, "INSERT INTO Location (");
                            i = 0;
                            while (Location[i] != '\0') {
                                if (values[i] != NULL) {
                                    Query = varAddString(Query, Location[i]);
                                    Query = varAddString(Query, ", ");
                                }
                                i++;
                            }
                            cpos = strlen(Query) - 1;
                            while(cpos >= 0 && (Query[cpos] == ' ' || Query[cpos] == ','))
                                Query[cpos--] = '\0';
                            Query = varAddString(Query, ") VALUES (");
                            i = 0;
                            while (Location[i] != '\0') {
                                if (values[i] != NULL) {
                                    Query = varAddString(Query, "\"");
                                    Query = varAddString(Query, values[i]);
                                    Query = varAddString(Query, "\", ");
                                }
                                i++;
                            }
                            cpos = strlen(Query) - 1;
                            while(cpos >= 0 && (Query[cpos] == ' ' || Query[cpos] == ','))
                                Query[cpos--] = '\0';
                            Query = varAddString(Query, ")");
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            if (mysql_query(mySQLconn, Query) != 0)
                                printSQLerror(mySQLconn, "mysql_query() failed", 138, Query);
                            else if (bVerboseSQL)
                                fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                        } else {
                            if (Updates != NULL) {
                                free(Updates);
                                Updates = NULL;
                            }
                            i = cols = 1;
                            while (Location[i] != '\0') {
                                if ((values[cols] == NULL && row[i] != NULL) || (values[cols] != NULL && row[i] == NULL) || (strcmp(values[cols], row[i]) != 0)) {
                                    Updates = varAddString(Updates,  Location[i]);
                                    Updates = varAddString(Updates,  " = ");
                                    if (values[cols] != NULL) {
                                        Updates = varAddString(Updates,  "\"");
                                        Updates = varAddString(Updates,  values[cols]);
                                        Updates = varAddString(Updates,  "\"");
                                    } else
                                        Updates = varAddString(Updates, "NULL");
                                    Updates = varAddString(Updates,  ", ");
                                }
                                i++;
                                cols++;
                            }
                            mysql_free_result(res_set);
                            if (Updates != NULL) {
                                cpos = strlen(Updates) - 1;
                                while(cpos >= 0 && (Updates[cpos] == ' ' || Updates[cpos] == ','))
                                    Updates[cpos--] = '\0';
                                if (Query != NULL) {
                                    free(Query);
                                    Query = NULL;
                                }
                                Query = varAddString(Query, "UPDATE Location SET ");
                                Query = varAddString(Query, Updates);
                                sprintf(value, " WHERE vpID = \"%i\"", retLocvpID);
                                Query = varAddString(Query, value);
                                if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                if (mysql_query(mySQLconn, Query) != 0)
                                    printSQLerror(mySQLconn, "mysql_query() failed", 139, Query);
                                else if (bVerboseSQL)
                                    fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                            }
                        }
                        if (bVerboseSQL) fprintf(stderr, "\n");
                        for (i = 0; i < cols; i++) {
                            if (values[i] != NULL) {
                                free(values[i]);
                                values[i] = NULL;
                            }
                        }
                        break;
                    case 'I':
                        if (! iok) break;
                        if (bVerboseSQL) fprintf(stderr, "%c ", rt);
                        i = 0;
                        if (values[i] != NULL) {
                            free(values[i]);
                            values[i] = NULL;
                        }
                        ch = fgetc(fp);
                        ch = fgetc(fp);
                        while(ch != '\0') {
                            while(ch != '\3' && ch != '\0') {
                                values[i] = varAddChar(values[i], ch);
                                ch = fgetc(fp);
                            }
                            if (ch == '\3') {
                                i++;
                                if (values[i] != NULL) {
                                    free(values[i]);
                                    values[i] = NULL;
                                }
                            }
                            if (ch != '\0') ch = fgetc(fp);
                        }
                        cols = i;
                        if (!bVerboseSQL) fprintf(stderr, "%c%c", star[(iCount-1)%5], 0x0d);
                        if (bVerboseSQL) {
                            for (i = 0; i <= cols; i++) fprintf(stderr, "\"%s\" ", values[i]);
                            fprintf(stderr, "\n");
                        }
                        if (Query != NULL) {
                            free(Query);
                            Query = NULL;
                        }
                        retRecID = 0;
                        Query = varAddString(Query, "SELECT ");
                        i = 0;
                        while (IndexDT[i] != '\0') {
                            Query = varAddString(Query, IndexDT[i++]);
                            if (IndexDT[i] != '\0') Query = varAddString(Query, ", ");
                        }
                        Query = varAddString(Query, " FROM IndexDT USE INDEX (PRIMARY) WHERE RecID = \"");
                        Query = varAddString(Query, values[0]);
                        Query = varAddString(Query, "\" && ");
                        i = 1;
                        while (IndexDT[i] != '\0') {
                            Query = varAddString(Query, IndexDT[i]);
                            if (values[i] != NULL ) {
                                Query = varAddString(Query, " = \"");
                                Query = varAddString(Query, values[i]);
                                Query = varAddString(Query, "\"");
                            } else
                                Query = varAddString(Query, " IS NULL");
                            i++;
                            if (IndexDT[i] != '\0') Query = varAddString(Query, " && ");
                        }
                        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                        if (mysql_query(mySQLconn, Query) != 0) {
                            printSQLerror(mySQLconn, "mysql_query() failed", 140, Query);
                            return;
                        } else {
                            res_set = mysql_store_result(mySQLconn);
                            rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                            if (rowsReturned > 1) {
                                mysql_free_result(res_set);
                                ErrMsg = varAddString(ErrMsg, "too many rows returned looking for RecID (");
                                sprintf(value, "%lu", rowsReturned);
                                ErrMsg = varAddString(ErrMsg, value);
                                ErrMsg = varAddString(ErrMsg, ")");
                                printSQLerror(mySQLconn, ErrMsg, 141, Query);
                                free(ErrMsg);
                                ErrMsg = NULL;
                            }
                            if (res_set == NULL) {
                                printSQLerror(mySQLconn, "mysql_store_result() failed", 142, Query);
                                return;
                            } else {
                                row = mysql_fetch_row (res_set);
                                if (row != NULL && row[0] != NULL) {
                                    sscanf(row[0], "%llu", &retRecID);
                                    if (bVerboseSQL) fprintf(stderr, "retRecID  %llu\n", retRecID);
                                }
                            }
                        }
                        if (! warned && retRecID != 0) {
                            (ImportWarning(atoi(values[1]))) && (warned = TRUE);
                            if (! bImportOR) {
                                LOOP = FALSE;
                                break;
                            }
                        }
                        iCount++;
                        if (retRecID == 0) {
                            mysql_free_result(res_set);
                            if (Query != NULL) {
                                free(Query);
                                Query = NULL;
                            }
                            Query = varAddString(Query, "SELECT ");
                            i = 0;
                            while (IndexDT[i] != '\0') {
                                Query = varAddString(Query, IndexDT[i++]);
                                if (IndexDT[i] != '\0') Query = varAddString(Query, ", ");
                            }
                            Query = varAddString(Query, " FROM IndexDT USE INDEX (vpIDorgRecID) WHERE vpIDorgRecID = \"");
                            Query = varAddString(Query, values[0]);
                            Query = varAddString(Query, "\" && ");
                            i = 1;
                            while (IndexDT[i] != '\0') {
                                Query = varAddString(Query, IndexDT[i]);
                                if (values[i] != NULL ) {
                                    Query = varAddString(Query, " = \"");
                                    Query = varAddString(Query, values[i]);
                                    Query = varAddString(Query, "\"");
                                } else
                                    Query = varAddString(Query, " IS NULL");
                                i++;
                                if (IndexDT[i] != '\0') Query = varAddString(Query, " && ");
                            }
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            if (mysql_query(mySQLconn, Query) != 0) {
                                printSQLerror(mySQLconn, "mysql_query() failed", 143, Query);
                                return;
                            } else {
                                res_set = mysql_store_result(mySQLconn);
                                rowsReturned = mysql_num_rows(res_set);	/* 0 = not found */
                                if (rowsReturned > 1) {
                                    mysql_free_result(res_set);
                                    ErrMsg = varAddString(ErrMsg, "too many rows returned looking for RecID (");
                                    sprintf(value, "%lu", rowsReturned);
                                    ErrMsg = varAddString(ErrMsg, value);
                                    ErrMsg = varAddString(ErrMsg, ")");
                                    printSQLerror(mySQLconn, ErrMsg, 144, Query);
                                    free(ErrMsg);
                                    ErrMsg = NULL;
                                }
                                if (res_set == NULL) {
                                    printSQLerror(mySQLconn, "mysql_store_result() failed", 145, Query);
                                    return;
                                } else {
                                    row = mysql_fetch_row (res_set);
                                    if (row != NULL && row[0] != NULL) {
                                        sscanf(row[0], "%llu", &retRecID);
                                        if (bVerboseSQL) fprintf(stderr, "retRecID  %llu\n", retRecID);
                                    }
                                }
                            }
                        }
                        if (retRecID == 0) {
                            mysql_free_result(res_set);
                            if (Query != NULL) {
                                free(Query);
                                Query = NULL;
                            }
                            Query = varAddString(Query, "SELECT ");
                            i = 0;
                            while (IndexDT[i] != '\0') {
                                Query = varAddString(Query, "t1.");
                                Query = varAddString(Query, IndexDT[i++]);
                                if (IndexDT[i] != '\0') Query = varAddString(Query, ", ");
                            }
                            Query = varAddString(Query, " FROM IndexDT AS t1 INNER JOIN DT AS t2 USING (RecID,vpID,vpIDorgRecID) WHERE t1.rtTT = t2.bTT && ");
                            i = 1;
                            while (IndexDT[i] != '\0') {
                                Query = varAddString(Query, "t1.");
                                Query = varAddString(Query, IndexDT[i]);
                                if (values[i] != NULL ) {
                                    Query = varAddString(Query, " = \"");
                                    Query = varAddString(Query, values[i]);
                                    Query = varAddString(Query, "\"");
                                } else
                                    Query = varAddString(Query, " IS NULL");
                                i++;
                                if (IndexDT[i] != '\0') Query = varAddString(Query, " && ");
                            }
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            if (mysql_query(mySQLconn, Query) != 0) {
                                printSQLerror(mySQLconn, "mysql_query() failed", 146, Query);
                                return;
                            } else {
                                res_set = mysql_store_result(mySQLconn);
                                rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                                if (rowsReturned > 1) {
                                    mysql_free_result(res_set);
                                    ErrMsg = varAddString(ErrMsg, "too many rows returned looking for RecID (");
                                    sprintf(value, "%lu", rowsReturned);
                                    ErrMsg = varAddString(ErrMsg, value);
                                    ErrMsg = varAddString(ErrMsg, ")");
                                    printSQLerror(mySQLconn, ErrMsg, 147, Query);
                                    free(ErrMsg);
                                    ErrMsg = NULL;
                                }
                                if (res_set == NULL) {
                                    printSQLerror(mySQLconn, "mysql_store_result() failed", 148, Query);
                                    return;
                                } else {
                                    row = mysql_fetch_row (res_set);
                                    if (row != NULL && row[0] != NULL) {
                                        sscanf(row[0], "%llu", &retRecID);
                                        if (bVerboseSQL) fprintf(stderr, "retRecID  %llu\n", retRecID);
                                    }
                                }
                            }
                        }
                        if (retRecID == 0) {
                            mysql_free_result(res_set);
                            if (Query != NULL) {
                                free(Query);
                                Query = NULL;
                            }
                            Query = varAddString(Query, "SELECT RecID FROM IndexDT USE INDEX (vpIDorgRecID) WHERE vpIDorgRecID = \"");
                            Query = varAddString(Query, values[0]);
                            Query = varAddString(Query, "\" && vpID = \"");
                            Query = varAddString(Query, values[1]);
                            Query = varAddString(Query, "\"");
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            if (mysql_query(mySQLconn, Query) != 0) {
                                printSQLerror(mySQLconn, "mysql_query() failed", 149, Query);
                                return;
                            } else {
                                res_set = mysql_store_result(mySQLconn);
                                rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                                if (rowsReturned > 1) {
                                    mysql_free_result(res_set);
                                    ErrMsg = varAddString(ErrMsg, "too many rows returned looking for RecID (");
                                    sprintf(value, "%lu", rowsReturned);
                                    ErrMsg = varAddString(ErrMsg, value);
                                    ErrMsg = varAddString(ErrMsg, ")");
                                    printSQLerror(mySQLconn, ErrMsg, 150, Query);
                                    free(ErrMsg);
                                    ErrMsg = NULL;
                                }
                                if (res_set == NULL) {
                                    printSQLerror(mySQLconn, "mysql_store_result() failed", 151, Query);
                                    return;
                                } else {
                                    row = mysql_fetch_row (res_set);
                                    if (row != NULL && row[0] != NULL) {
                                        sscanf(row[0], "%llu", &retRecID);
                                        if (bVerboseSQL) fprintf(stderr, "retRecID  %llu\n", retRecID);
                                    }   
                                }
                            }
                        }
                        if (retRecID == 0) {
                            mysql_free_result(res_set);
                            if (Query != NULL) {
                                free(Query);
                                Query = NULL;
                            }
                            Query = varAddString(Query, "INSERT INTO IndexDT (vpIDorgRecID, ");
                            i = 1;
                            while (IndexDT[i] != '\0') {
                                Query = varAddString(Query, IndexDT[i]);
                                i++;
                                if (IndexDT[i] != '\0') Query = varAddString(Query, ", ");
                            }
                            Query = varAddString(Query, ") VALUES (");
                            i = 0;
                            while (IndexDT[i] != '\0') {
                                if (values[i] != NULL) {
                                    Query = varAddString(Query, "\"");
                                    Query = varAddString(Query, values[i]);
                                    Query = varAddString(Query, "\"");
                                } else
                                    Query = varAddString(Query, "NULL");
                                i++;
                                if (IndexDT[i] != '\0') Query = varAddString(Query, ", ");
                            }
                            Query = varAddString(Query, ")");
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            if (mysql_query(mySQLconn, Query) != 0)
                                printSQLerror(mySQLconn, "mysql_query() failed", 152, Query);
                            else if (bVerboseSQL)
                                fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                        } else {
                            if (Updates != NULL) {
                                free(Updates);
                                Updates = NULL;
                            }
                            i = cols = 1;
                            while (IndexDT[i] != '\0') {
                                if ((values[cols] == NULL && row[i] != NULL) || (values[cols] != NULL && row[i] == NULL) || (strcmp(values[cols], row[i]) != 0)) {
                                    Updates = varAddString(Updates,  IndexDT[i]);
                                    Updates = varAddString(Updates,  " = ");
                                    if (values[cols] != NULL) {
                                        Updates = varAddString(Updates,  "\"");
                                        Updates = varAddString(Updates,  values[cols]);
                                        Updates = varAddString(Updates,  "\"");
                                    } else {
                                        Updates = varAddString(Updates, "NULL");
                                    }
                                    Updates = varAddString(Updates,  ", ");
                                }
                                i++;
                                cols++;
                            }
                            mysql_free_result(res_set);
                            if (Updates != NULL) {
                                cpos = strlen(Updates) - 1;
                                while(cpos >= 0 && (Updates[cpos] == ' ' || Updates[cpos] == ','))
                                    Updates[cpos--] = '\0';
                                if (Query != NULL) {
                                    free(Query);
                                    Query = NULL;
                                }
                                Query = varAddString(Query, "UPDATE IndexDT SET ");
                                Query = varAddString(Query, Updates);
                                sprintf(value, " WHERE RecID = \"%llu\"", retRecID);
                                Query = varAddString(Query, value);
                                if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                if (mysql_query(mySQLconn, Query) != 0)
                                    printSQLerror(mySQLconn, "mysql_query() failed", 153, Query);
                                else if (bVerboseSQL)
                                    fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                            }
                        }
                        if (bVerboseSQL) fprintf(stderr, "\n");
                        for (i = 0; i < cols; i++) {
                            if (values[i] != NULL) {
                                free(values[i]);
                                values[i] = NULL;
                            }
                        }
                        break;
                    case 'T':
                        if (! iok) break;
                        if (bVerboseSQL) fprintf(stderr, "%c ", rt);
                        i = 0;
                        if (values[i] != NULL) {
                            free(values[i]);
                            values[i] = NULL;
                        }
                        ch = fgetc(fp);
                        ch = fgetc(fp);
                        while(ch != '\0') {
                            while(ch != '\3' && ch != '\0') {
                                values[i] = varAddChar(values[i], ch);
                                ch = fgetc(fp);
                            }
                            if (ch == '\3') {
                                i++;
                                if (values[i] != NULL) {
                                    free(values[i]);
                                    values[i] = NULL;
                                }
                            }
                            if (ch != '\0') ch = fgetc(fp);
                        }
                        cols = i;
                        if (!bVerboseSQL) fprintf(stderr, "%c%c", star[(iCount-1)%5], 0x0d);
                        if (bVerboseSQL) {
                            for (i = 0; i <= cols; i++) fprintf(stderr, "\"%s\" ", values[i]);
                            fprintf(stderr, "\n");
                        }
                        if (Query != NULL) {
                            free(Query);
                            Query = NULL;
                        }
                        retRecID = 0;
                        Query = varAddString(Query, "SELECT RecID FROM IndexDT USE INDEX (PRIMARY) WHERE RecID = \"");
                        Query = varAddString(Query, values[0]);
                        Query = varAddString(Query, "\" && vpID = \"");
                        Query = varAddString(Query, values[1]);
                        Query = varAddString(Query, "\"");
                        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                        if (mysql_query(mySQLconn, Query) != 0) {
                            printSQLerror(mySQLconn, "mysql_query() failed", 154, Query);
                            return;
                        } else {
                            res_set = mysql_store_result(mySQLconn);
                            rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                            if (rowsReturned > 1) {
                                mysql_free_result(res_set);
                                ErrMsg = varAddString(ErrMsg, "too many rows returned looking for RecID (");
                                sprintf(value, "%lu", rowsReturned);
                                ErrMsg = varAddString(ErrMsg, value);
                                ErrMsg = varAddString(ErrMsg, ")");
                                printSQLerror(mySQLconn, ErrMsg, 155, Query);
                                free(ErrMsg);
                                ErrMsg = NULL;
                            }
                            if (res_set == NULL) {
                                printSQLerror(mySQLconn, "mysql_store_result() failed", 156, Query);
                                return;
                            } else {
                                row = mysql_fetch_row (res_set);
                                if (row != NULL) {
                                    if (row[0] != NULL) sscanf(row[0], "%llu", &retRecID);
                                    if (bVerboseSQL) fprintf(stderr, "retRecID \"%llu\"\n", retRecID);
                                }
                                mysql_free_result(res_set);
                            }
                        }
                        if (! warned && retRecID != 0) {
                            (ImportWarning(atoi(values[1]))) && (warned = TRUE);
                            if (! bImportOR) {
                                LOOP = FALSE;
                                break;
                            }
                        }
                        iCount++;
                        if (retRecID == 0) {
                            if (Query != NULL) {
                                free(Query);
                                Query = NULL;
                            }
                            Query = varAddString(Query, "SELECT RecID FROM IndexDT USE INDEX (vpIDorgRecID) WHERE vpIDorgRecID = \"");
                            Query = varAddString(Query, values[0]);
                            Query = varAddString(Query, "\" && vpID = \"");
                            Query = varAddString(Query, values[1]);
                            Query = varAddString(Query, "\"");
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            if (mysql_query(mySQLconn, Query) != 0) {
                                printSQLerror(mySQLconn, "mysql_query() failed", 157, Query);
                                return;
                            } else {
                                res_set = mysql_store_result(mySQLconn);
                                rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                                if (rowsReturned > 1) {
                                    mysql_free_result(res_set);
                                    ErrMsg = varAddString(ErrMsg, "too many rows returned looking for RecID (");
                                    sprintf(value, "%lu", rowsReturned);
                                    ErrMsg = varAddString(ErrMsg, value);
                                    ErrMsg = varAddString(ErrMsg, ")");
                                    printSQLerror(mySQLconn, ErrMsg, 158, Query);
                                    free(ErrMsg);
                                    ErrMsg = NULL;
                                }
                                if (res_set == NULL) {
                                    printSQLerror(mySQLconn, "mysql_store_result() failed", 159, Query);
                                    return;
                                } else {
                                    row = mysql_fetch_row (res_set);
                                    if (row != NULL) {
                                        if (row[0] != NULL) sscanf(row[0], "%llu", &retRecID);
                                        if (bVerboseSQL) fprintf(stderr, "retRecID \"%llu\"\n", retRecID);
                                    }
                                    mysql_free_result(res_set);
                                }
                            }
                        }
                        if (retRecID == 0) {
                            if (Query != NULL) {
                                free(Query);
                                Query = NULL;
                            }
                            Query = varAddString(Query, "INSERT INTO IndexDT (vpIDorgRecID, vpID) VALUES (\"");
                            Query = varAddString(Query, values[0]);
                            Query = varAddString(Query, "\", \"");
                            Query = varAddString(Query, values[1]);
                            Query = varAddString(Query, "\")");
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            if (mysql_query(mySQLconn, Query) != 0)
                                printSQLerror(mySQLconn, "mysql_query() failed", 160, Query);
                            else if (bVerboseSQL)
                                fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                            if (Query != NULL) {
                                free(Query);
                                Query = NULL;
                            }
                            Query = varAddString(Query, "SELECT RecID FROM IndexDT USE INDEX (vpIDorgRecID) WHERE vpIDorgRecID = \"");
                            Query = varAddString(Query, values[0]);
                            Query = varAddString(Query, "\" && vpID = \"");
                            Query = varAddString(Query, values[1]);
                            Query = varAddString(Query, "\"");
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            if (mysql_query(mySQLconn, Query) != 0) {
                                printSQLerror(mySQLconn, "mysql_query() failed", 161, Query);
                                return;
                            } else {
                                res_set = mysql_store_result(mySQLconn);
                                rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                                if (res_set == NULL) {
                                    printSQLerror(mySQLconn, "mysql_store_result() failed", 162, Query);
                                    return;
                                } else {
                                    while ((row = mysql_fetch_row (res_set)) != NULL) {
                                       if (row[0] != NULL) sscanf(row[0], "%llu", &retRecID);
                                       if (bVerboseSQL) fprintf(stderr, "retRecID \"%llu\"\n", retRecID);
                                    }
                                    mysql_free_result(res_set);
                                }
                            }
                        }
                        if (retRecID != 0) {		/* this needs to be != for real */
                            if (Query != NULL) {
                                free(Query);
                                Query = NULL;
                            }
                            Query = varAddString(Query, "SELECT ");
                            i = 0;
                            while (DT[i] != '\0') {
                                  if (i > 0) Query = varAddString(Query, ", ");
                                  Query = varAddString(Query, DT[i]);
                                  i++;
                            }
                            sprintf(value, " FROM DT USE INDEX (RecIDvpID) WHERE RecID = \"%llu\" && vpID = \"%s\"", retRecID, values[1]);
                            Query = varAddString(Query, value);
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            foundRecID = FALSE;
                            if (mysql_query(mySQLconn, Query) != 0) {
                                printSQLerror(mySQLconn, "mysql_query() failed", 163, Query);
                                return;
                            } else {
                                res_set = mysql_store_result(mySQLconn);
                                rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                                if (res_set == NULL) {
                                    printSQLerror(mySQLconn, "mysql_store_result() failed", 164, Query);
                                    return;
                                } else if (rowsReturned > 0) {
                                    row = mysql_fetch_row (res_set);
                                    if (row != NULL && row[0] != NULL) {
                                        sprintf(value, "%llu", retRecID);
                                        if (strcmp(row[0], value) == 0) foundRecID = TRUE;
                                    }
                                }
//                                if (bVerboseSQL) fprintf(stderr, "DT found RecID %s\n", foundRecID ? "True" : "False");
                            }
                            if (! foundRecID) {
                                if (Query != NULL) {
                                    free(Query);
                                    Query = NULL;
                                }
                                Query = varAddString(Query, "INSERT INTO DT (");
                                i = 0;
                                while (DT[i] != '\0') {
                                      Query = varAddString(Query, DT[i]);
                                      i++;
                                      if (DT[i] != '\0') Query = varAddString(Query, ", ");
                                }
                                sprintf(value, ") VALUES (\"%llu\", ",retRecID);
                                Query = varAddString(Query, value);
                                i = cols = 1;
                                while (DT[i] != '\0') {
                                    if (DT[i+1] != '\0') {
                                        if (values[cols] != NULL) {
                                            Query = varAddString(Query, "\"");
                                            Query = varAddString(Query, values[cols]);
                                            Query = varAddString(Query, "\", ");
                                        }
                                    }
                                    i++;
                                    cols++;
                                }
                                Query = varAddString(Query, "\"");
                                Query = varAddString(Query, values[0]);
                                Query = varAddString(Query, "\")");
                                if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                if (mysql_query(mySQLconn, Query) != 0)
                                    printSQLerror(mySQLconn, "mysql_query() failed", 165, Query);
                                else if (bVerboseSQL)
                                    fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                            } else {
                                if (Updates != NULL) {
                                    free(Updates);
                                    Updates = NULL;
                                }
                                i = cols = 1;
                                while (DT[i+1] != '\0') {
                                    if ((values[cols] == NULL && row[i] != NULL) || (values[cols] != NULL && row[i] == NULL) || (strcmp(values[cols], row[i]) != 0)) {
                                        Updates = varAddString(Updates,  DT[i]);
                                        Updates = varAddString(Updates,  " = ");
                                        if (values[cols] != NULL) {
                                            Updates = varAddString(Updates,  "\"");
                                            Updates = varAddString(Updates,  values[cols]);
                                            Updates = varAddString(Updates,  "\"");
                                        } else
                                            Updates = varAddString(Updates, "NULL");
                                        Updates = varAddString(Updates,  ", ");
                                    }
                                    i++;
                                    cols++;
                                }
                                if ((values[0] == NULL && row[i] != NULL) || (values[0] != NULL && row[i] == NULL) || atoi(row[i]) != atoi(values[0])) {
                                    if (retRecID != atoi(values[0])) {
                                        Updates = varAddString(Updates, DT[i]);
                                        Updates = varAddString(Updates, " = \"");
                                        Updates = varAddString(Updates, values[0]);
                                        Updates = varAddString(Updates, "\", ");
                                    }
                                }
                                cols++;
                                mysql_free_result(res_set);
                                if (Updates != NULL) {
                                    cpos = strlen(Updates) - 1;
                                    while(cpos >= 0 && (Updates[cpos] == ' ' || Updates[cpos] == ','))
                                        Updates[cpos--] = '\0';
                                    if (Query != NULL) {
                                        free(Query);
                                        Query = NULL;
                                    }
                                    Query = varAddString(Query, "UPDATE DT SET ");
                                    Query = varAddString(Query, Updates);
                                    sprintf(value, " WHERE RecID = \"%llu\"", retRecID);
                                    Query = varAddString(Query, value);
                                    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                    if (mysql_query(mySQLconn, Query) != 0)
                                        printSQLerror(mySQLconn, "mysql_query() failed", 166, Query);
                                    else if (bVerboseSQL)
                                        fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                                }
                            }

                            /* Temperature */
                           if (values[cols] == NULL) {
                                i = 0;
                                while (Temperature[i++] != '\0') cols++;
                            } else {
                                if (Query != NULL) {
                                    free(Query);
                                    Query = NULL;
                                }
                                Query = varAddString(Query, "SELECT ");
                                i = 0;
                                while (Temperature[i] != '\0') {
                                      if (i > 0) Query = varAddString(Query, ", ");
                                      Query = varAddString(Query, Temperature[i]);
                                      i++;
                                }
                                Query = varAddString(Query, " FROM ");
                                Query = varAddString(Query, _TEMP_TABLE_NAME);
                                sprintf(value, " USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", retRecID);
                                Query = varAddString(Query, value);
                                if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                foundRecID = FALSE;
                                if (mysql_query(mySQLconn, Query) != 0) {
                                    printSQLerror(mySQLconn, "mysql_query() failed", 167, Query);
                                    return;
                                } else {
                                    res_set = mysql_store_result(mySQLconn);
                                    rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                                    if (res_set == NULL) {
                                        printSQLerror(mySQLconn, "mysql_store_result() failed", 168, Query);
                                        return;
                                    } else if (rowsReturned > 0) {
                                        row = mysql_fetch_row (res_set);
                                        if (row != NULL && row[0] != NULL) {
                                            sprintf(value, "%llu", retRecID);
                                            if (strcmp(row[0], value) == 0) foundRecID = TRUE;
                                        }
                                    }
//                                    if (bVerboseSQL) fprintf(stderr, "Temperature found RecID %s\n", foundRecID ? "True" : "False");
                                }
                                if (! foundRecID) {
                                    mysql_free_result(res_set);
                                    if (Query != NULL) {
                                        free(Query);
                                        Query = NULL;
                                    }
                                    Query = varAddString(Query, "INSERT INTO ");
                                    Query = varAddString(Query, _TEMP_TABLE_NAME);
                                    Query = varAddString(Query, " (");
                                    i = 0;
                                    while (Temperature[i] != '\0') {
                                          if (values[cols+i] != NULL) {
                                              Query = varAddString(Query, Temperature[i]);
                                              Query = varAddString(Query, ", ");
                                          }
                                          i++;
                                    }
                                    cpos = strlen(Query) - 1;
                                    while(cpos >= 0 && (Query[cpos] == ' ' || Query[cpos] == ','))
                                        Query[cpos--] = '\0';
                                    sprintf(value, ") VALUES (\"%llu\", ",retRecID);
                                    Query = varAddString(Query, value);
                                    i = 1;
                                    cols++;
                                    while (Temperature[i] != '\0') {
                                        if (values[cols] != NULL) {
                                            Query = varAddString(Query, "\"");
                                            Query = varAddString(Query, values[cols]);
                                            Query = varAddString(Query, "\", ");
                                        }
                                        i++;
                                        cols++;
                                    }
                                    cpos = strlen(Query) - 1;
                                    while(cpos >= 0 && (Query[cpos] == ' ' || Query[cpos] == ','))
                                        Query[cpos--] = '\0';
                                    Query = varAddString(Query, ")");
                                    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                    if (mysql_query(mySQLconn, Query) != 0)
                                        printSQLerror(mySQLconn, "mysql_query() failed", 169, Query);
                                    else if (bVerboseSQL)
                                        fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                                } else {
                                    if (Updates != NULL) {
                                        free(Updates);
                                        Updates = NULL;
                                    }
                                    i = 1;
                                    cols++;
                                    while (Temperature[i] != '\0') {
                                        if ((values[cols] == NULL && row[i] != NULL) || (values[cols] != NULL && row[i] == NULL) || (strcmp(values[cols], row[i]) != 0)) {
                                            Updates = varAddString(Updates,  Temperature[i]);
                                            Updates = varAddString(Updates,  " = ");
                                            if (values[cols] != NULL) {
                                                Updates = varAddString(Updates,  "\"");
                                                Updates = varAddString(Updates,  values[cols]);
                                                Updates = varAddString(Updates,  "\"");
                                            } else
                                                Updates = varAddString(Updates, "NULL");
                                            Updates = varAddString(Updates,  ", ");
                                        }
                                        i++;
                                        cols++;
                                    }
                                    mysql_free_result(res_set);
                                    if (Updates != NULL) {
                                        cpos = strlen(Updates) - 1;
                                        while(cpos >= 0 && (Updates[cpos] == ' ' || Updates[cpos] == ','))
                                            Updates[cpos--] = '\0';
                                        if (Query != NULL) {
                                            free(Query);
                                            Query = NULL;
                                        }
                                        Query = varAddString(Query, "UPDATE ");
                                        Query = varAddString(Query, _TEMP_TABLE_NAME);
                                        Query = varAddString(Query, " SET ");
                                        Query = varAddString(Query, Updates);
                                        sprintf(value, " WHERE RecID = \"%llu\"", retRecID);
                                        Query = varAddString(Query, value);
                                        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                        if (mysql_query(mySQLconn, Query) != 0)
                                            printSQLerror(mySQLconn, "mysql_query() failed", 170, Query);
                                        else if (bVerboseSQL)
                                            fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                                    }
                                }
                            }

                            /* Humidity */
                            if (values[cols] == NULL) {
                                i = 0;
                                while (Humidity[i++] != '\0') cols++;
                            } else {
                                if (Query != NULL) {
                                    free(Query);
                                    Query = NULL;
                                }
                                Query = varAddString(Query, "SELECT ");
                                i = 0;
                                while (Humidity[i] != '\0') {
                                      if (i > 0) Query = varAddString(Query, ", ");
                                      Query = varAddString(Query, Humidity[i]);
                                      i++;
                                }
                                Query = varAddString(Query, " FROM ");
                                Query = varAddString(Query, _HUM_TABLE_NAME);
                                sprintf(value, " USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", retRecID);
                                Query = varAddString(Query, value);
                                if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                foundRecID = FALSE;
                                if (mysql_query(mySQLconn, Query) != 0) {
                                    printSQLerror(mySQLconn, "mysql_query() failed", 171, Query);
                                    return;
                                } else {
                                    res_set = mysql_store_result(mySQLconn);
                                    rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                                    if (res_set == NULL) {
                                        printSQLerror(mySQLconn, "mysql_store_result() failed", 172, Query);
                                        return;
                                    } else if (rowsReturned > 0) {
                                        row = mysql_fetch_row (res_set);
                                        if (row != NULL && row[0] != NULL) {
                                            sprintf(value, "%llu", retRecID);
                                            if (strcmp(row[0], value) == 0) foundRecID = TRUE;
                                        }
                                    }
//                                    if (bVerboseSQL) fprintf(stderr, "Humidity found RecID %s\n", foundRecID ? "True" : "False");
                                }
                                if (! foundRecID) {
                                    mysql_free_result(res_set);
                                    if (Query != NULL) {
                                        free(Query);
                                        Query = NULL;
                                    }
                                    Query = varAddString(Query, "INSERT INTO ");
                                    Query = varAddString(Query, _HUM_TABLE_NAME);
                                    Query = varAddString(Query, " (");
                                    i = 0;
                                    while (Humidity[i] != '\0') {
                                          if (values[cols+i] != NULL) {
                                              Query = varAddString(Query, Humidity[i]);
                                              Query = varAddString(Query, ", ");
                                          }
                                          i++;
                                    }
                                    cpos = strlen(Query) - 1;
                                    while(cpos >= 0 && (Query[cpos] == ' ' || Query[cpos] == ','))
                                        Query[cpos--] = '\0';
                                    sprintf(value, ") VALUES (\"%llu\", ",retRecID);
                                    Query = varAddString(Query, value);
                                    i = 1;
                                    cols++;
                                    while (Humidity[i] != '\0') {
                                        if (values[cols] != NULL) {
                                            Query = varAddString(Query, "\"");
                                            Query = varAddString(Query, values[cols]);
                                            Query = varAddString(Query, "\", ");
                                        }
                                        i++;
                                        cols++;
                                    }
                                    cpos = strlen(Query) - 1;
                                    while(cpos >= 0 && (Query[cpos] == ' ' || Query[cpos] == ','))
                                        Query[cpos--] = '\0';
                                    Query = varAddString(Query, ")");
                                    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                    if (mysql_query(mySQLconn, Query) != 0)
                                        printSQLerror(mySQLconn, "mysql_query() failed", 173, Query);
                                    else if (bVerboseSQL)
                                        fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                                } else {
                                    if (Updates != NULL) {
                                        free(Updates);
                                        Updates = NULL;
                                    }
                                    i = 1;
                                    cols++;
                                    while (Humidity[i] != '\0') {
                                        if ((values[cols] == NULL && row[i] != NULL) || (values[cols] != NULL && row[i] == NULL) || (strcmp(values[cols], row[i]) != 0)) {
                                            Updates = varAddString(Updates,  Humidity[i]);
                                            Updates = varAddString(Updates,  " = ");
                                            if (values[cols] != NULL) {
                                                Updates = varAddString(Updates,  "\"");
                                                Updates = varAddString(Updates,  values[cols]);
                                                Updates = varAddString(Updates,  "\"");
                                            } else
                                                Updates = varAddString(Updates, "NULL");
                                            Updates = varAddString(Updates,  ", ");
                                        }
                                        i++;
                                        cols++;
                                    }
                                    mysql_free_result(res_set);
                                    if (Updates != NULL) {
                                        cpos = strlen(Updates) - 1;
                                        while(cpos >= 0 && (Updates[cpos] == ' ' || Updates[cpos] == ','))
                                            Updates[cpos--] = '\0';
                                        if (Query != NULL) {
                                            free(Query);
                                            Query = NULL;
                                        }
                                        Query = varAddString(Query, "UPDATE ");
                                        Query = varAddString(Query, _HUM_TABLE_NAME);
                                        Query = varAddString(Query, " SET ");
                                        Query = varAddString(Query, Updates);
                                        sprintf(value, " WHERE RecID = \"%llu\"", retRecID);
                                        Query = varAddString(Query, value);
                                        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                        if (mysql_query(mySQLconn, Query) != 0)
                                            printSQLerror(mySQLconn, "mysql_query() failed", 174, Query);
                                        else if (bVerboseSQL)
                                            fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                                    }
                                }
                            }

                            /* Wind */
                            if (values[cols] == NULL) {
                                i = 0;
                                while (Wind[i++] != '\0') cols++;
                            } else {
                                if (Query != NULL) {
                                    free(Query);
                                    Query = NULL;
                                }
                                Query = varAddString(Query, "SELECT ");
                                i = 0;
                                while (Wind[i] != '\0') {
                                      if (i > 0) Query = varAddString(Query, ", ");
                                      Query = varAddString(Query, Wind[i]);
                                      i++;
                                }
                                Query = varAddString(Query, " FROM ");
                                Query = varAddString(Query, _WIND_TABLE_NAME);
                                sprintf(value, " USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", retRecID);
                                Query = varAddString(Query,  value);
                                if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                foundRecID = FALSE;
                                if (mysql_query(mySQLconn, Query) != 0) {
                                    printSQLerror(mySQLconn, "mysql_query() failed", 175, Query);
                                    return;
                                } else {
                                    res_set = mysql_store_result(mySQLconn);
                                    rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                                    if (res_set == NULL) {
                                        printSQLerror(mySQLconn, "mysql_store_result() failed", 176, Query);
                                        return;
                                    } else if (rowsReturned > 0) {
                                        row = mysql_fetch_row (res_set);
                                        if (row != NULL && row[0] != NULL) {
                                            sprintf(value, "%llu", retRecID);
                                            if (strcmp(row[0], value) == 0) foundRecID = TRUE;
                                        }
                                    }
//                                    if (bVerboseSQL) fprintf(stderr, "Wind found RecID %s\n", foundRecID ? "True" : "False");
                                }
                                if (! foundRecID) {
                                    mysql_free_result(res_set);
                                    if (Query != NULL) {
                                        free(Query);
                                        Query = NULL;
                                    }
                                    Query = varAddString(Query, "INSERT INTO ");
                                    Query = varAddString(Query, _WIND_TABLE_NAME);
                                    Query = varAddString(Query, " (");
                                    i = 0;
                                    while (Wind[i] != '\0') {
                                          if (values[cols+i] != NULL) {
                                              Query = varAddString(Query, Wind[i]);
                                              Query = varAddString(Query,  ", ");
                                          }
                                          i++;
                                    }
                                    cpos = strlen(Query) - 1;
                                    while(cpos >= 0 && (Query[cpos] == ' ' || Query[cpos] == ','))
                                        Query[cpos--] = '\0';
                                    sprintf(value, ") VALUES (\"%llu\", ",retRecID);
                                    Query = varAddString(Query, value);
                                    i = 1;
                                    cols++;
                                    while (Wind[i] != '\0') {
                                        if (values[cols] != NULL) {
                                            Query = varAddString(Query, "\"");
                                            Query = varAddString(Query, values[cols]);
                                            Query = varAddString(Query, "\", ");
                                        }
                                        i++;
                                        cols++;
                                    }
                                    cpos = strlen(Query) - 1;
                                    while(cpos >= 0 && (Query[cpos] == ' ' || Query[cpos] == ','))
                                        Query[cpos--] = '\0';
                                    Query = varAddString(Query, ")");
                                    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                    if (mysql_query(mySQLconn, Query) != 0)
                                        printSQLerror(mySQLconn, "mysql_query() failed", 177, Query);
                                    else if (bVerboseSQL)
                                        fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                                } else {
                                    if (Updates != NULL) {
                                        free(Updates);
                                        Updates = NULL;
                                    }
                                    i = 1;
                                    cols++;
                                    while (Wind[i] != '\0') {
                                        if ((values[cols] == NULL && row[i] != NULL) || (values[cols] != NULL && row[i] == NULL) || (strcmp(values[cols], row[i]) != 0)) {
                                            Updates = varAddString(Updates,  Wind[i]);
                                            Updates = varAddString(Updates,  " = ");
                                            if (values[cols] != NULL) {
                                                Updates = varAddString(Updates,  "\"");
                                                Updates = varAddString(Updates,  values[cols]);
                                                Updates = varAddString(Updates,  "\"");
                                            } else
                                                Updates = varAddString(Updates, "NULL");
                                            Updates = varAddString(Updates,  ", ");
                                        }
                                        i++;
                                        cols++;
                                    }
                                    mysql_free_result(res_set);
                                    if (Updates != NULL) {
                                        cpos = strlen(Updates) - 1;
                                        while(cpos >= 0 && (Updates[cpos] == ' ' || Updates[cpos] == ','))
                                            Updates[cpos--] = '\0';
                                        if (Query != NULL) {
                                            free(Query);
                                            Query = NULL;
                                        }
                                        Query = varAddString(Query, "UPDATE ");
                                        Query = varAddString(Query, _WIND_TABLE_NAME);
                                        Query = varAddString(Query, " SET ");
                                        Query = varAddString(Query, Updates);
                                        sprintf(value, " WHERE RecID = \"%llu\"", retRecID);
                                        Query = varAddString(Query, value);
                                        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                        if (mysql_query(mySQLconn, Query) != 0)
                                            printSQLerror(mySQLconn, "mysql_query() failed", 178, Query);
                                        else if (bVerboseSQL)
                                            fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                                    }
                                }
                            }

                            /* Rain */
                            if (values[cols] == NULL) {
                                i = 0;
                                while (Rain[i++] != '\0') cols++;
                            } else {
                                if (Query != NULL) {
                                    free(Query);
                                    Query = NULL;
                                }
                                Query = varAddString(Query, "SELECT ");
                                i = 0;
                                while (Rain[i] != '\0') {
                                      if (i > 0) Query = varAddString(Query, ", ");
                                      Query = varAddString(Query, Rain[i]);
                                      i++;
                                }
                                Query = varAddString(Query, " FROM ");
                                Query = varAddString(Query, _RAIN_TABLE_NAME);
                                sprintf(value, " USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", retRecID);
                                Query = varAddString(Query, value);
                                if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                foundRecID = FALSE;
                                if (mysql_query(mySQLconn, Query) != 0) {
                                    printSQLerror(mySQLconn, "mysql_query() failed", 179, Query);
                                    return;
                                } else {
                                    res_set = mysql_store_result(mySQLconn);
                                    rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                                    if (res_set == NULL) {
                                        printSQLerror(mySQLconn, "mysql_store_result() failed", 180, Query);
                                        return;
                                    } else if (rowsReturned > 0) {
                                        row = mysql_fetch_row (res_set);
                                        if (row != NULL && row[0] != NULL) {
                                            sprintf(value, "%llu", retRecID);
                                            if (strcmp(row[0], value) == 0) foundRecID = TRUE;
                                        }
                                    }
//                                    if (bVerboseSQL) fprintf(stderr, "Rain found RecID %s\n", foundRecID ? "True" : "False");
                                }
                                if (! foundRecID) {
                                    mysql_free_result(res_set);
                                    if (Query != NULL) {
                                        free(Query);
                                        Query = NULL;
                                    }
                                    Query = varAddString(Query, "INSERT INTO ");
                                    Query = varAddString(Query, _RAIN_TABLE_NAME);
                                    Query = varAddString(Query, " (");
                                    i = 0;
                                    while (Rain[i] != '\0') {
                                          if (values[cols+i] != NULL) {
                                              Query = varAddString(Query, Rain[i]);
                                              Query = varAddString(Query, ", ");
                                          }
                                          i++;
                                    }
                                    cpos = strlen(Query) - 1;
                                    while(cpos >= 0 && (Query[cpos] == ' ' || Query[cpos] == ','))
                                        Query[cpos--] = '\0';
                                    sprintf(value, ") VALUES (\"%llu\", ",retRecID);
                                    Query = varAddString(Query, value);
                                    i = 1;
                                    cols++;
                                    while (Rain[i] != '\0') {
                                        if (values[cols] != NULL) {
                                            Query = varAddString(Query, "\"");
                                            Query = varAddString(Query, values[cols]);
                                            Query = varAddString(Query, "\", ");
                                        }
                                        i++;
                                        cols++;
                                    }
                                    cpos = strlen(Query) - 1;
                                    while(cpos >= 0 && (Query[cpos] == ' ' || Query[cpos] == ','))
                                        Query[cpos--] = '\0';
                                    Query = varAddString(Query, ")");
                                    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                    if (mysql_query(mySQLconn, Query) != 0)
                                        printSQLerror(mySQLconn, "mysql_query() failed", 181, Query);
                                    else if (bVerboseSQL)
                                        fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                                } else {
                                    if (Updates != NULL) {
                                        free(Updates);
                                        Updates = NULL;
                                    }
                                    i = 1;
                                    cols++;
                                    while (Rain[i] != '\0') {
                                        if ((values[cols] == NULL && row[i] != NULL) || (values[cols] != NULL && row[i] == NULL) || (strcmp(values[cols], row[i]) != 0)) {
                                            Updates = varAddString(Updates,  Rain[i]);
                                            Updates = varAddString(Updates,  " = ");
                                            if (values[cols] != NULL) {
                                                Updates = varAddString(Updates,  "\"");
                                                Updates = varAddString(Updates,  values[cols]);
                                                Updates = varAddString(Updates,  "\"");
                                            } else
                                                Updates = varAddString(Updates, "NULL");
                                            Updates = varAddString(Updates,  ", ");
                                        }
                                        i++;
                                        cols++;
                                    }
                                    mysql_free_result(res_set);
                                    if (Updates != NULL) {
                                        cpos = strlen(Updates) - 1;
                                        while(cpos >= 0 && (Updates[cpos] == ' ' || Updates[cpos] == ','))
                                            Updates[cpos--] = '\0';
                                        if (Query != NULL) {
                                            free(Query);
                                            Query = NULL;
                                        }
                                        Query = varAddString(Query, "UPDATE ");
                                        Query = varAddString(Query, _RAIN_TABLE_NAME);
                                        Query = varAddString(Query, " SET ");
                                        Query = varAddString(Query, Updates);
                                        sprintf(value, " WHERE RecID = \"%llu\"", retRecID);
                                        Query = varAddString(Query, value);
                                        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                        if (mysql_query(mySQLconn, Query) != 0)
                                            printSQLerror(mySQLconn, "mysql_query() failed", 182, Query);
                                        else if (bVerboseSQL)
                                            fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                                    }
                                }
                            }

                            /* Barometer */
                            if (values[cols] == NULL) {
                                i = 0;
                                while (Barometer[i++] != '\0') cols++;
                            } else {
                                if (Query != NULL) {
                                    free(Query);
                                    Query = NULL;
                                }
                                Query = varAddString(Query, "SELECT ");
                                i = 0;
                                while (Barometer[i] != '\0') {
                                      if (i > 0) Query = varAddString(Query, ", ");
                                      Query = varAddString(Query, Barometer[i]);
                                      i++;
                                }
                                Query = varAddString(Query, " FROM ");
                                Query = varAddString(Query, _BARO_TABLE_NAME);
                                sprintf(value, " USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", retRecID);
                                Query = varAddString(Query, value);
                                if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                foundRecID = FALSE;
                                if (mysql_query(mySQLconn, Query) != 0) {
                                    printSQLerror(mySQLconn, "mysql_query() failed", 183, Query);
                                    return;
                                } else {
                                    res_set = mysql_store_result(mySQLconn);
                                    rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                                    if (res_set == NULL) {
                                        printSQLerror(mySQLconn, "mysql_store_result() failed", 184, Query);
                                        return;
                                    } else if (rowsReturned > 0) {
                                        row = mysql_fetch_row (res_set);
                                        if (row != NULL && row[0] != NULL) {
                                            sprintf(value, "%llu", retRecID);
                                            if (strcmp(row[0], value) == 0) foundRecID = TRUE;
                                        }
                                    }
//                                    if (bVerboseSQL) fprintf(stderr, "Barometer found RecID %s\n", foundRecID ? "True" : "False");
                                }
                                if (! foundRecID) {
                                    mysql_free_result(res_set);
                                    if (Query != NULL) {
                                        free(Query);
                                        Query = NULL;
                                    }
                                    Query = varAddString(Query, "INSERT INTO ");
                                    Query = varAddString(Query, _BARO_TABLE_NAME);
                                    Query = varAddString(Query, " (");
                                    i = 0;
                                    while (Barometer[i] != '\0') {
                                          if (values[cols+i] != NULL) {
                                              Query = varAddString(Query, Barometer[i]);
                                              Query = varAddString(Query, ", ");
                                          }
                                          i++;
                                    }
                                    cpos = strlen(Query) - 1;
                                    while(cpos >= 0 && (Query[cpos] == ' ' || Query[cpos] == ','))
                                        Query[cpos--] = '\0';
                                    sprintf(value, ") VALUES (\"%llu\", ",retRecID);
                                    Query = varAddString(Query, value);
                                    i = 1;
                                    cols++;
                                    while (Barometer[i] != '\0') {
                                        if (values[cols] != NULL) {
                                            Query = varAddString(Query, "\"");
                                            Query = varAddString(Query, values[cols]);
                                            Query = varAddString(Query, "\", ");
                                        }
                                        i++;
                                        cols++;
                                    }
                                    cpos = strlen(Query) - 1;
                                    while(cpos >= 0 && (Query[cpos] == ' ' || Query[cpos] == ','))
                                        Query[cpos--] = '\0';
                                    Query = varAddString(Query, ")");
                                    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                    if (mysql_query(mySQLconn, Query) != 0)
                                        printSQLerror(mySQLconn, "mysql_query() failed", 185, Query);
                                    else if (bVerboseSQL)
                                        fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                                } else {
                                    if (Updates != NULL) {
                                        free(Updates);
                                        Updates = NULL;
                                    }
                                    i = 1;
                                    cols++;
                                    while (Barometer[i] != '\0') {
                                        if ((values[cols] == NULL && row[i] != NULL) || (values[cols] != NULL && row[i] == NULL) || (strcmp(values[cols], row[i]) != 0)) {
                                            Updates = varAddString(Updates,  Barometer[i]);
                                            Updates = varAddString(Updates,  " = ");
                                            if (values[cols] != NULL) {
                                                Updates = varAddString(Updates,  "\"");
                                                Updates = varAddString(Updates,  values[cols]);
                                                Updates = varAddString(Updates,  "\"");
                                            } else
                                                Updates = varAddString(Updates, "NULL");
                                            Updates = varAddString(Updates,  ", ");
                                        }
                                        i++;
                                        cols++;
                                    }
                                    mysql_free_result(res_set);
                                    if (Updates != NULL) {
                                        cpos = strlen(Updates) - 1;
                                        while(cpos >= 0 && (Updates[cpos] == ' ' || Updates[cpos] == ','))
                                            Updates[cpos--] = '\0';
                                        if (Query != NULL) {
                                            free(Query);
                                            Query = NULL;
                                        }
                                        Query = varAddString(Query, "UPDATE ");
                                        Query = varAddString(Query, _BARO_TABLE_NAME);
                                        Query = varAddString(Query, " SET ");
                                        Query = varAddString(Query, Updates);
                                        sprintf(value, " WHERE RecID = \"%llu\"", retRecID);
                                        Query = varAddString(Query, value);
                                        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                        if (mysql_query(mySQLconn, Query) != 0)
                                            printSQLerror(mySQLconn, "mysql_query() failed", 186, Query);
                                        else if (bVerboseSQL)
                                            fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                                    }
                                }
                            }

                            /* Solar */
                            if (values[cols] == NULL) {
                                i = 0;
                                while (Solar[i++] != '\0') cols++;
                            } else {
                                if (Query != NULL) {
                                    free(Query);
                                    Query = NULL;
                                }
                                Query = varAddString(Query, "SELECT ");
                                i = 0;
                                while (Solar[i] != '\0') {
                                      if (i > 0) Query = varAddString(Query, ", ");
                                      Query = varAddString(Query, Solar[i]);
                                      i++;
                                }
                                Query = varAddString(Query, " FROM ");
                                Query = varAddString(Query, _SOLAR_TABLE_NAME);
                                sprintf(value, " USE INDEX (PRIMARY) WHERE RecID = \"%llu\"", retRecID);
                                Query = varAddString(Query, value);
                                if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                foundRecID = FALSE;
                                if (mysql_query(mySQLconn, Query) != 0) {
                                    printSQLerror(mySQLconn, "mysql_query() failed", 187, Query);
                                    return;
                                } else {
                                    res_set = mysql_store_result(mySQLconn);
                                    rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                                    if (res_set == NULL) {
                                        printSQLerror(mySQLconn, "mysql_store_result() failed", 188, Query);
                                        return;
                                    } else if (rowsReturned > 0) {
                                        row = mysql_fetch_row (res_set);
                                        if (row != NULL && row[0] != NULL) {
                                            sprintf(value, "%llu", retRecID);
                                            if (strcmp(row[0], value) == 0) foundRecID = TRUE;
                                        }
                                    }
//                                    if (bVerboseSQL) fprintf(stderr, "Solar found RecID %s\n", foundRecID ? "True" : "False");
                                }
                                if (! foundRecID) {
                                    mysql_free_result(res_set);
                                    if (Query != NULL) {
                                        free(Query);
                                        Query = NULL;
                                    }
                                    Query = varAddString(Query, "INSERT INTO ");
                                    Query = varAddString(Query, _SOLAR_TABLE_NAME);
                                    Query = varAddString(Query, " (");
                                    i = 0;
                                    while (Solar[i] != '\0') {
                                          if (values[cols+i] != NULL) {
                                              Query = varAddString(Query, Solar[i]);
                                              Query = varAddString(Query, ", ");
                                          }
                                          i++;
                                    }
                                    cpos = strlen(Query) - 1;
                                    while(cpos >= 0 && (Query[cpos] == ' ' || Query[cpos] == ','))
                                        Query[cpos--] = '\0';
                                    sprintf(value, ") VALUES (\"%llu\", ",retRecID);
                                    Query = varAddString(Query, value);
                                    i = 1;
                                    cols++;
                                    while (Solar[i] != '\0') {
                                        if (values[cols] != NULL) {
                                            Query = varAddString(Query, "\"");
                                            Query = varAddString(Query, values[cols]);
                                            Query = varAddString(Query, "\", ");
                                        }
                                        i++;
                                        cols++;
                                    }
                                    cpos = strlen(Query) - 1;
                                    while(cpos >= 0 && (Query[cpos] == ' ' || Query[cpos] == ','))
                                        Query[cpos--] = '\0';
                                    Query = varAddString(Query, ")");
                                    if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                    if (mysql_query(mySQLconn, Query) != 0)
                                        printSQLerror(mySQLconn, "mysql_query() failed", 189, Query);
                                    else if (bVerboseSQL)
                                        fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                                } else {
                                    if (Updates != NULL) {
                                        free(Updates);
                                        Updates = NULL;
                                    }
                                    i = 1;
                                    cols++;
                                    while (Solar[i] != '\0') {
                                        if ((values[cols] == NULL && row[i] != NULL) || (values[cols] != NULL && row[i] == NULL) || (strcmp(values[cols], row[i]) != 0)) {
                                            Updates = varAddString(Updates,  Solar[i]);
                                            Updates = varAddString(Updates,  " = ");
                                            if (values[cols] != NULL) {
                                                Updates = varAddString(Updates,  "\"");
                                                Updates = varAddString(Updates,  values[cols]);
                                                Updates = varAddString(Updates,  "\"");
                                            } else
                                                Updates = varAddString(Updates, "NULL");
                                            Updates = varAddString(Updates,  ", ");
                                        }
                                        i++;
                                        cols++;
                                    }
                                    mysql_free_result(res_set);
                                    if (Updates != NULL) {
                                        cpos = strlen(Updates) - 1;
                                        while(cpos >= 0 && (Updates[cpos] == ' ' || Updates[cpos] == ','))
                                            Updates[cpos--] = '\0';
                                        if (Query != NULL) {
                                            free(Query);
                                            Query = NULL;
                                        }
                                        Query = varAddString(Query, "UPDATE ");
                                        Query = varAddString(Query, _SOLAR_TABLE_NAME);
                                        Query = varAddString(Query, " SET ");
                                        Query = varAddString(Query, Updates);
                                        sprintf(value, " WHERE RecID = \"%llu\"", retRecID);
                                        Query = varAddString(Query, value);
                                        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                        if (mysql_query(mySQLconn, Query) != 0)
                                            printSQLerror(mySQLconn, "mysql_query() failed", 190, Query);
                                        else if (bVerboseSQL)
                                            fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                                    }
                                }
                            }
                        }

                        if (bVerboseSQL) fprintf(stderr, "\n");
                        for (i = 0; i < cols; i++) {
                            if (values[i] != NULL) {
                                free(values[i]);
                                values[i] = NULL;
                            }
                        }
                        break;
                    case 'H':
                        if (! iok) break;
                        if (bVerboseSQL) fprintf(stderr, "%c ", rt);
                        i = 0;
                        if (values[i] != NULL) {
                            free(values[i]);
                            values[i] = NULL;
                        }
                        ch = fgetc(fp);
                        ch = fgetc(fp);
                        while(ch != '\0') {
                            while(ch != '\3' && ch != '\0') {
                                values[i] = varAddChar(values[i], ch);
                                ch = fgetc(fp);
                            }
                            if (ch == '\3') {
                                i++;
                                if (values[i] != NULL) {
                                    free(values[i]);
                                    values[i] = NULL;
                                }
                            }
                            if (ch != '\0') ch = fgetc(fp);
                        }
                        cols = i;
                        if (!bVerboseSQL) fprintf(stderr, "%c%c", star[(iCount-1)%5], 0x0d);
                        if (bVerboseSQL) {
                            for (i = 0; i <= cols; i++) fprintf(stderr, "\"%s\" ", values[i]);
                            fprintf(stderr, "\n");
                        }

                        if (Query != NULL) {
                            free(Query);
                            Query = NULL;
                        }
                        retRecID = 0;
                        Query = varAddString(Query, "SELECT RecID FROM IndexDT USE INDEX (PRIMARY) WHERE RecID = \"");
                        Query = varAddString(Query, values[1]);
                        Query = varAddString(Query, "\" && vpID = \"");
                        Query = varAddString(Query, values[0]);
                        Query = varAddString(Query, "\"");
                        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                        if (mysql_query(mySQLconn, Query) != 0) {
                            printSQLerror(mySQLconn, "mysql_query() failed", 191, Query);
                            return;
                        } else {
                            res_set = mysql_store_result(mySQLconn);
                            rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                            if (rowsReturned > 1) {
                                mysql_free_result(res_set);
                                ErrMsg = varAddString(ErrMsg, "too many rows returned looking for RecID (");
                                sprintf(value, "%lu", rowsReturned);
                                ErrMsg = varAddString(ErrMsg, value);
                                ErrMsg = varAddString(ErrMsg, ")");
                                printSQLerror(mySQLconn, ErrMsg, 192, Query);
                                free(ErrMsg);
                                ErrMsg = NULL;
                            }
                            if (res_set == NULL) {
                                printSQLerror(mySQLconn, "mysql_store_result() failed", 193, Query);
                                return;
                            } else {
                                row = mysql_fetch_row (res_set);
                                if (row != NULL) {
                                    if (row[0] != NULL) sscanf(row[0], "%llu", &retRecID);
                                    if (bVerboseSQL) fprintf(stderr, "retRecID \"%llu\"\n", retRecID);
                                }
                                mysql_free_result(res_set);
                            }
                        }
                        if (! warned && retRecID != 0) {
                            (ImportWarning(atoi(values[1]))) && (warned = TRUE);
                            if (! bImportOR) {
                                LOOP = FALSE;
                                break;
                            }
                        }
                        iCount++;
                        if (retRecID == 0) {
                            if (Query != NULL) {
                                free(Query);
                                Query = NULL;
                            }
                            Query = varAddString(Query, "SELECT RecID FROM IndexDT USE INDEX (vpIDorgRecID) WHERE vpIDorgRecID = \"");
                            Query = varAddString(Query, values[1]);
                            Query = varAddString(Query, "\" && vpID = \"");
                            Query = varAddString(Query, values[0]);
                            Query = varAddString(Query, "\"");
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            if (mysql_query(mySQLconn, Query) != 0) {
                                printSQLerror(mySQLconn, "mysql_query() failed", 194, Query);
                                return;
                            } else {
                                res_set = mysql_store_result(mySQLconn);
                                rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                                if (rowsReturned > 1) {
                                    mysql_free_result(res_set);
                                    ErrMsg = varAddString(ErrMsg, "too many rows returned looking for RecID (");
                                    sprintf(value, "%lu", rowsReturned);
                                    ErrMsg = varAddString(ErrMsg, value);
                                    ErrMsg = varAddString(ErrMsg, ")");
                                    printSQLerror(mySQLconn, ErrMsg, 195, Query);
                                    free(ErrMsg);
                                    ErrMsg = NULL;
                                }
                                if (res_set == NULL) {
                                    printSQLerror(mySQLconn, "mysql_store_result() failed", 196, Query);
                                    return;
                                } else {
                                    row = mysql_fetch_row (res_set);
                                    if (row != NULL) {
                                        if (row[0] != NULL) sscanf(row[0], "%llu", &retRecID);
                                        if (bVerboseSQL) fprintf(stderr, "retRecID \"%llu\"\n", retRecID);
                                    }
                                    mysql_free_result(res_set);
                                }
                            }
                        }
                        if (retRecID == 0) {
                            if (Query != NULL) {
                                free(Query);
                                Query = NULL;
                            }
                            Query = varAddString(Query, "SELECT RecID FROM DT USE INDEX (vpIDorgRecID) WHERE vpIDorgRecID = \"");
                            Query = varAddString(Query, values[1]);
                            Query = varAddString(Query, "\" && vpID = \"");
                            Query = varAddString(Query, values[0]);
                            Query = varAddString(Query, "\"");
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            retRecID = 0;
                            if (mysql_query(mySQLconn, Query) != 0) {
                                printSQLerror(mySQLconn, "mysql_query() failed", 197, Query);
                                return;
                            } else {
                                res_set = mysql_store_result(mySQLconn);
                                rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                                if (rowsReturned > 1) {
                                    mysql_free_result(res_set);
                                    ErrMsg = varAddString(ErrMsg, "too many rows returned looking for RecID (");
                                    sprintf(value, "%lu", rowsReturned);
                                    ErrMsg = varAddString(ErrMsg, value);
                                    ErrMsg = varAddString(ErrMsg, ")");
                                    printSQLerror(mySQLconn, ErrMsg, 198, Query);
                                    free(ErrMsg);
                                    ErrMsg = NULL;
                                }
                                if (res_set == NULL) {
                                    printSQLerror(mySQLconn, "mysql_store_result() failed", 199, Query);
                                    return;
                                } else {
                                    row = mysql_fetch_row (res_set);
                                    if (row != NULL) {
                                        if (row[0] != NULL) sscanf(row[0], "%llu", &retRecID);
                                        if (bVerboseSQL) fprintf(stderr, "retRecID \"%llu\"\n", retRecID);
                                    }
                                    mysql_free_result(res_set);
                                }
                            }
                        }
                        foundRecID = FALSE;
                        if (retRecID != 0) {		/* this needs to be != for real */
                            if (Query != NULL) {
                                free(Query);
                                Query = NULL;
                            }
                            Query = varAddString(Query, "SELECT ");
                            i = 0;
                            while (Hourly[i] != '\0') {
                                  if (i > 0) Query = varAddString(Query,  ", ");
                                  Query = varAddString(Query, "t1.");
                                  Query = varAddString(Query, Hourly[i]);
                                  i++;
                            }
                            Query = varAddString(Query, " FROM ");
                            Query = varAddString(Query, _HOURLY_TABLE_NAME);
                            sprintf(value, " AS t1 INNER JOIN DT AS t2 USE INDEX (RecIDvpID) USING (RecID) WHERE t2.RecID = \"%llu\" && t2.vpID = \"%s\"", retRecID, values[0]);
                            Query = varAddString(Query, value);
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            if (mysql_query(mySQLconn, Query) != 0) {
                                printSQLerror(mySQLconn, "mysql_query() failed", 200, Query);
                                return;
                            } else {
                                res_set = mysql_store_result(mySQLconn);
                                rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                                if (res_set == NULL) {
                                    printSQLerror(mySQLconn, "mysql_store_result() failed", 201, Query);
                                    return;
                                } else if (rowsReturned > 0) {
                                    row = mysql_fetch_row (res_set);
                                   if (row != NULL && row[0] != NULL) {
                                        sprintf(value, "%llu", retRecID);
                                        if (strcmp(row[0], value) == 0) foundRecID = TRUE;
                                    }
                                }
//                                if (bVerboseSQL) fprintf(stderr, "Hourly found RecID %s\n", foundRecID ? "True" : "False");
                            }
                        }
                        if (! foundRecID) {
                            mysql_free_result(res_set);
                            if (Query != NULL) {
                                free(Query);
                                Query = NULL;
                            }
                            Query = varAddString(Query, "INSERT INTO ");
                            Query = varAddString(Query, _HOURLY_TABLE_NAME);
                            Query = varAddString(Query, " (");
                            i = 0;
                            while (Hourly[i] != '\0') {
                                if (values[i+1] != NULL) {	/* offset by 1 because vpID included but not in string array */
                                    Query = varAddString(Query, Hourly[i]);
                                    Query = varAddString(Query, ", ");
                                }
                                i++;
                            }
                            cpos = strlen(Query) - 1;
                            while(cpos >= 0 && (Query[cpos] == ' ' || Query[cpos] == ','))
                                Query[cpos--] = '\0';
                            sprintf(value, ") VALUES (\"%llu\", ",retRecID);
                            Query = varAddString(Query, value);
                            i = 1;
                            while (Hourly[i] != '\0') {
                                if (values[i+1] != NULL) {	/* skip vpID & RecID vpID not in string array */
                                    Query = varAddString(Query, "\"");
                                    Query = varAddString(Query, values[i+1]);
                                    Query = varAddString(Query, "\", ");
                                }
                                i++;
                            }
                            cpos = strlen(Query) - 1;
                            while(cpos >= 0 && (Query[cpos] == ' ' || Query[cpos] == ','))
                                Query[cpos--] = '\0';
                            Query = varAddString(Query, ")");
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            if (mysql_query(mySQLconn, Query) != 0)
                                printSQLerror(mySQLconn, "mysql_query() failed", 202, Query);
                            else if (bVerboseSQL)
                                fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                        } else {
                            if (Updates != NULL) {
                                free(Updates);
                                Updates = NULL;
                            }
                            i = 1;
                            cols = 2;
                            while (Hourly[i] != '\0') {
                                if ((values[cols] == NULL && row[i] != NULL) || (values[cols] != NULL && row[i] == NULL) || (strcmp(values[cols], row[i]) != 0)) {
                                    Updates = varAddString(Updates,  Hourly[i]);
                                    Updates = varAddString(Updates,  " = ");
                                    if (values[cols] != NULL) {
                                        Updates = varAddString(Updates,  "\"");
                                        Updates = varAddString(Updates,  values[cols]);
                                        Updates = varAddString(Updates,  "\"");
                                    } else
                                        Updates = varAddString(Updates, "NULL");
                                    Updates = varAddString(Updates,  ", ");
                                }
                                i++;
                                cols++;
                            }
                            mysql_free_result(res_set);
                            if (Updates != NULL) {
                                cpos = strlen(Updates) - 1;
                                while(cpos >= 0 && (Updates[cpos] == ' ' || Updates[cpos] == ','))
                                    Updates[cpos--] = '\0';
                                if (Query != NULL) {
                                    free(Query);
                                    Query = NULL;
                                }
                                Query = varAddString(Query, "UPDATE ");
                                Query = varAddString(Query, _HOURLY_TABLE_NAME);
                                Query = varAddString(Query, " SET ");
                                Query = varAddString(Query, Updates);
                                sprintf(value, " WHERE RecID = \"%llu\"", retRecID);
                                Query = varAddString(Query, value);
                                if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                if (mysql_query(mySQLconn, Query) != 0)
                                    printSQLerror(mySQLconn, "mysql_query() failed", 203, Query);
                                else if (bVerboseSQL)
                                    fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                            }
                        }
                        if (bVerboseSQL) fprintf(stderr, "\n");
                        for (i = 0; i < cols; i++) {
                            if (values[i] != NULL) {
                                free(values[i]);
                                values[i] = NULL;
                            }
                        }
                        break;
                    case 'Y':
                        if (! iok) break;
                        if (bVerboseSQL) fprintf(stderr, "%c ", rt);
                        i = 0;
                        if (values[i] != NULL) {
                            free(values[i]);
                            values[i] = NULL;
                        }
                        ch = fgetc(fp);
                        ch = fgetc(fp);
                        while(ch != '\0') {
                            while(ch != '\3' && ch != '\0') {
                                values[i] = varAddChar(values[i], ch);
                                ch = fgetc(fp);
                            }
                            if (ch == '\3') {
                                i++;
                                if (values[i] != NULL) {
                                    free(values[i]);
                                    values[i] = NULL;
                                }
                            }
                            if (ch != '\0') ch = fgetc(fp);
                        }
                        cols = i;
                        if (!bVerboseSQL) fprintf(stderr, "%c%c", star[(iCount-1)%5], 0x0d);
                        iCount++;
                        if (bVerboseSQL) {
                            for (i = 0; i <= cols; i++) fprintf(stderr, "\"%s\" ", values[i]);
                            fprintf(stderr, "\n");
                        }
                        if (Query != NULL) {
                            free(Query);
                            Query = NULL;
                        }
                        Query = varAddString(Query, "SELECT ");
                        i = 0;
                        while (YMD[i] != '\0') {
                            Query = varAddString(Query, YMD[i++]);
                            if (YMD[i] != '\0') Query = varAddString(Query, ", ");
                        }
                        Query = varAddString(Query, " FROM YMD USE INDEX (PRIMARY) WHERE vpID = \"");
                        Query = varAddString(Query, values[0]);
                        Query = varAddString(Query, "\" && YearMonthDay = \"");
                        Query = varAddString(Query, values[1]);
                        Query = varAddString(Query, "\"");
                        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                        foundYMD = FALSE;
                        if (mysql_query(mySQLconn, Query) != 0) {
                            printSQLerror(mySQLconn, "mysql_query() failed", 204, Query);
                            return;
                        } else {
                            res_set = mysql_store_result(mySQLconn);
                            rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                            if (rowsReturned > 1) {
                                mysql_free_result(res_set);
                                ErrMsg = varAddString(ErrMsg, "too many rows returned looking for RecID (");
                                sprintf(value, "%lu", rowsReturned);
                                ErrMsg = varAddString(ErrMsg, value);
                                ErrMsg = varAddString(ErrMsg, ")");
                                printSQLerror(mySQLconn, ErrMsg, 205, Query);
                                free(ErrMsg);
                                ErrMsg = NULL;
                            }
                            if (res_set == NULL) {
                                printSQLerror(mySQLconn, "mysql_store_result() failed", 206, Query);
                                return;
                            } else {
                                row = mysql_fetch_row (res_set);
                                if (row != NULL) {
                                    if (strcmp(values[0], row[0]) == 0 && strcmp(values[1], row[1]) == 0) foundYMD = TRUE;
                                }
                            }
                        }
                        if (! foundYMD) {
                            mysql_free_result(res_set);
                            if (Query != NULL) {
                                free(Query);
                                Query = NULL;
                            }
                            Query = varAddString(Query, "INSERT INTO YMD (");
                            i = 0;
                            while (YMD[i] != '\0') {
                                Query = varAddString(Query, YMD[i]);
                                i++;
                                if (YMD[i] != '\0') Query = varAddString(Query, ", ");
                            }
                            Query = varAddString(Query, ") VALUES (");
                            i = 0;
                            while (YMD[i] != '\0') {
                                if (values[i] != NULL) {
                                    Query = varAddString(Query, "\"");
                                    Query = varAddString(Query, values[i]);
                                    Query = varAddString(Query, "\"");
                                } else
                                    Query = varAddString(Query, "NULL");
                                i++;
                                if (YMD[i] != '\0') Query = varAddString(Query, ", ");
                            }
                            Query = varAddString(Query, ")");
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            if (mysql_query(mySQLconn, Query) != 0)
                                printSQLerror(mySQLconn, "mysql_query() failed", 207, Query);
                            else if (bVerboseSQL)
                                fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                        } else {
                            if (Updates != NULL) {
                                free(Updates);
                                Updates = NULL;
                            }
                            i = cols = 2;
                            while (YMD[i] != '\0') {
                                if ((values[cols] == NULL && row[i] != NULL) || (values[cols] != NULL && row[i] == NULL) || (strcmp(values[cols], row[i]) != 0)) {
                                    Updates = varAddString(Updates,  YMD[i]);
                                    Updates = varAddString(Updates,  " = ");
                                    if (values[cols] != NULL) {
                                        Updates = varAddString(Updates,  "\"");
                                        Updates = varAddString(Updates,  values[cols]);
                                        Updates = varAddString(Updates,  "\"");
                                    } else
                                        Updates = varAddString(Updates, "NULL");
                                    Updates = varAddString(Updates,  ", ");
                                }
                                i++;
                                cols++;
                            }
                            mysql_free_result(res_set);
                            if (Updates != NULL) {
                                cpos = strlen(Updates) - 1;
                                while(cpos >= 0 && (Updates[cpos] == ' ' || Updates[cpos] == ','))
                                    Updates[cpos--] = '\0';
                                if (Query != NULL) {
                                    free(Query);
                                    Query = NULL;
                                }
                                Query = varAddString(Query, "UPDATE YMD SET ");
                                Query = varAddString(Query, Updates);
                                Query = varAddString(Query, " WHERE vpID = \"");
                                Query = varAddString(Query, values[0]);
                                Query = varAddString(Query, "\" && YearMonthDay = \"");
                                Query = varAddString(Query, values[1]);
                                Query = varAddString(Query, "\"");
                                if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                if (mysql_query(mySQLconn, Query) != 0)
                                    printSQLerror(mySQLconn, "mysql_query() failed", 208, Query);
                                else if (bVerboseSQL)
                                    fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                            }
                        }
                        if (bVerboseSQL) fprintf(stderr, "\n");
                        for (i = 0; i < cols; i++) {
                            if (values[i] != NULL) {
                                free(values[i]);
                                values[i] = NULL;
                            }
                        }
                        break;
                    case 'S':
                        if (! iok) break;
                        if (bVerboseSQL) fprintf(stderr, "%c ", rt);
                        i = 0;
                        if (values[i] != NULL) {
                            free(values[i]);
                            values[i] = NULL;
                        }
                        ch = fgetc(fp);
                        ch = fgetc(fp);
                        while(ch != '\0') {
                            while(ch != '\3' && ch != '\0') {
                                values[i] = varAddChar(values[i], ch);
                                ch = fgetc(fp);
                            }
                            if (ch == '\3') {
                                i++;
                                if (values[i] != NULL) {
                                    free(values[i]);
                                    values[i] = NULL;
                                }
                            }
                            if (ch != '\0') ch = fgetc(fp);
                        }
                        cols = i;
                        if (!bVerboseSQL) fprintf(stderr, "%c%c", star[(iCount-1)%5], 0x0d);
                        iCount++;
                        if (bVerboseSQL) {
                            for (i = 0; i <= cols; i++) fprintf(stderr, "\"%s\" ", values[i]);
                            fprintf(stderr, "\n");
                        }
                        if (Query != NULL) {
                            free(Query);
                            Query = NULL;
                        }   
                        Query = varAddString(Query, "SELECT ");
                        i = 0;
                        while (Storms[i] != '\0') {
                            Query = varAddString(Query, Storms[i++]);
                            if (Storms[i] != '\0') Query = varAddString(Query, ", ");
                        }
                        Query = varAddString(Query, " FROM ");
                        Query = varAddString(Query, _STORMS_TABLE_NAME);
                        Query = varAddString(Query, " USE INDEX (PRIMARY) WHERE vpID = \"");
                        Query = varAddString(Query, values[0]);
                        Query = varAddString(Query, "\" && StormStartedDate = \"");
                        Query = varAddString(Query, values[1]);
                        Query = varAddString(Query, "\"");
                        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                        foundStorm = FALSE;
                        if (mysql_query(mySQLconn, Query) != 0) {
                            printSQLerror(mySQLconn, "mysql_query() failed", 209, Query);
                            return;
                        } else {
                            res_set = mysql_store_result(mySQLconn);
                            rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                            if (rowsReturned > 1) {
                                mysql_free_result(res_set);
                                ErrMsg = varAddString(ErrMsg, "too many rows returned looking for RecID (");
                                sprintf(value, "%lu", rowsReturned);
                                ErrMsg = varAddString(ErrMsg, value);
                                ErrMsg = varAddString(ErrMsg, ")");
                                printSQLerror(mySQLconn, ErrMsg, 210, Query);
                                free(ErrMsg);
                                ErrMsg = NULL;
                            }
                            if (res_set == NULL) {
                                printSQLerror(mySQLconn, "mysql_store_result() failed", 211, Query);
                                return;
                            } else {
                                row = mysql_fetch_row (res_set);
                                if (row != NULL) {
                                    if (strcmp(values[0], row[0]) == 0 && strcmp(values[1], row[1]) == 0) foundStorm = TRUE;
                                }
                            }
                        }
                        if (! foundStorm) {
                            mysql_free_result(res_set);
                            if (Query != NULL) {
                                free(Query);
                                Query = NULL;
                            }
                            Query = varAddString(Query, "INSERT INTO ");
                            Query = varAddString(Query, _STORMS_TABLE_NAME);
                            Query = varAddString(Query, " (");
                            i = 0;
                            while (Storms[i] != '\0') {
                                Query = varAddString(Query, Storms[i]);
                                i++;
                                if (Storms[i] != '\0') Query = varAddString(Query, ", ");
                            }
                            Query = varAddString(Query, ") VALUES (");
                            i = 0;
                            while (Storms[i] != '\0') {
                                if (values[i] != NULL) {
                                    Query = varAddString(Query, "\"");
                                    Query = varAddString(Query, values[i]);
                                    Query = varAddString(Query, "\"");
                                } else
                                    Query = varAddString(Query, "NULL");
                                i++;
                                if (Storms[i] != '\0') Query = varAddString(Query, ", ");
                            }
                            Query = varAddString(Query, ")");
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            if (mysql_query(mySQLconn, Query) != 0)
                                printSQLerror(mySQLconn, "mysql_query() failed", 212, Query);
                            else if (bVerboseSQL)
                                fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                        } else {
                            if (Updates != NULL) {
                                free(Updates);
                                Updates = NULL;
                            }
                            i = cols = 2;
                            while (Storms[i] != '\0') {
                                if ((values[cols] == NULL && row[i] != NULL) || (values[cols] != NULL && row[i] == NULL) || (strcmp(values[cols], row[i]) != 0)) {
                                    Updates = varAddString(Updates,  Storms[i]);
                                    Updates = varAddString(Updates,  " = ");
                                    if (values[cols] != NULL) {
                                        Updates = varAddString(Updates,  "\"");
                                        Updates = varAddString(Updates,  values[cols]);
                                        Updates = varAddString(Updates,  "\"");
                                    } else
                                        Updates = varAddString(Updates, "NULL");
                                    Updates = varAddString(Updates,  ", ");
                                }
                                i++;
                                cols++;
                            }
                            mysql_free_result(res_set);
                            if (Updates != NULL) {
                                cpos = strlen(Updates) - 1;
                                while(cpos >= 0 && (Updates[cpos] == ' ' || Updates[cpos] == ','))
                                    Updates[cpos--] = '\0';
                                if (Query != NULL) {
                                    free(Query);
                                    Query = NULL;
                                }
                                Query = varAddString(Query, "UPDATE ");
                                Query = varAddString(Query, _STORMS_TABLE_NAME);
                                Query = varAddString(Query, " SET ");
                                Query = varAddString(Query, Updates);
                                Query = varAddString(Query, " WHERE vpID = \"");
                                Query = varAddString(Query, values[0]);
                                Query = varAddString(Query, "\" && StormStartedDate = \"");
                                Query = varAddString(Query, values[1]);
                                Query = varAddString(Query, "\"");
                                if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                if (mysql_query(mySQLconn, Query) != 0)
                                    printSQLerror(mySQLconn, "mysql_query() failed", 213, Query);
                                else if (bVerboseSQL)
                                    fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                            }
                        }
                        if (bVerboseSQL) fprintf(stderr, "\n");
                        for (i = 0; i < cols; i++) {
                            if (values[i] != NULL) {
                                free(values[i]);
                                values[i] = NULL;
                            }
                        }
                        break;
                    case 'D':
                        if (! iok) break;
                        if (bVerboseSQL) fprintf(stderr, "%c ", rt);
                        i = 0;
                        if (values[i] != NULL) {
                            free(values[i]);
                            values[i] = NULL;
                        }
                        ch = fgetc(fp);
                        ch = fgetc(fp);
                        while(ch != '\0') {
                            while(ch != '\3' && ch != '\0') {
                                values[i] = varAddChar(values[i], ch);
                                ch = fgetc(fp);
                            }
                            if (ch == '\3') {
                                i++;
                                if (values[i] != NULL) {
                                    free(values[i]);
                                    values[i] = NULL;
                                }
                            }
                            if (ch != '\0') ch = fgetc(fp);
                        }
                        cols = i;
                        if (!bVerboseSQL) fprintf(stderr, "%c%c", star[(iCount-1)%5], 0x0d);
                        iCount++;
                        if (bVerboseSQL) {
                            for (i = 0; i <= cols; i++) fprintf(stderr, "\"%s\" ", values[i]);
                            fprintf(stderr, "\n");
                        }
                        if (Query != NULL) {
                            free(Query);
                            Query = NULL;
                        }   
                        Query = varAddString(Query, "SELECT ");
                        i = 0;
                        while (DHL[i] != '\0') {
                            Query = varAddString(Query, DHL[i++]);
                            if (DHL[i] != '\0') Query = varAddString(Query, ", ");
                        }
                        Query = varAddString(Query, " FROM ");
                        Query = varAddString(Query, _DHL_TABLE_NAME);
                        Query = varAddString(Query, " USE INDEX (PRIMARY) WHERE vpID = \"");
                        Query = varAddString(Query, values[0]);
                        Query = varAddString(Query, "\" && YearMonthDay = \"");
                        Query = varAddString(Query, values[1]);
                        Query = varAddString(Query, "\"");
                        if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                        foundDHL = FALSE;
                        if (mysql_query(mySQLconn, Query) != 0) {
                            printSQLerror(mySQLconn, "mysql_query() failed", 214, Query);
                            return;
                        } else {
                            res_set = mysql_store_result(mySQLconn);
                            rowsReturned = mysql_num_rows(res_set);     /* 0 = not found */
                            if (rowsReturned > 1) {
                                mysql_free_result(res_set);
                                ErrMsg = varAddString(ErrMsg, "too many rows returned looking for RecID (");
                                sprintf(value, "%lu", rowsReturned);
                                ErrMsg = varAddString(ErrMsg, value);
                                ErrMsg = varAddString(ErrMsg, ")");
                                printSQLerror(mySQLconn, ErrMsg, 215, Query);
                                free(ErrMsg);
                                ErrMsg = NULL;
                            }
                            if (res_set == NULL) {
                                printSQLerror(mySQLconn, "mysql_store_result() failed", 216, Query);
                                return;
                            } else {
                                row = mysql_fetch_row (res_set);
                                if (row != NULL) {
                                    if (strcmp(values[0], row[0]) == 0 && strcmp(values[1], row[1]) == 0) foundDHL = TRUE;
                                }
                            }
                        }
                        if (! foundDHL) {
                            mysql_free_result(res_set);
                            if (Query != NULL) {
                                free(Query);
                                Query = NULL;
                            }
                            Query = varAddString(Query, "INSERT INTO ");
                            Query = varAddString(Query, _DHL_TABLE_NAME);
                            Query = varAddString(Query, " (");
                            i = 0;
                            while (DHL[i] != '\0') {
                                Query = varAddString(Query, DHL[i]);
                                i++;
                                if (DHL[i] != '\0') Query = varAddString(Query, ", ");
                            }
                            Query = varAddString(Query, ") VALUES (");
                            i = 0;
                            while (DHL[i] != '\0') {
                                if (values[i] != NULL) {
                                    Query = varAddString(Query, "\"");
                                    Query = varAddString(Query, values[i]);
                                    Query = varAddString(Query, "\"");
                                } else
                                    Query = varAddString(Query, "NULL");
                                i++;
                                if (DHL[i] != '\0') Query = varAddString(Query, ", ");
                            }
                            Query = varAddString(Query, ")");
                            if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                            if (mysql_query(mySQLconn, Query) != 0)
                                printSQLerror(mySQLconn, "mysql_query() failed", 217, Query);
                            else if (bVerboseSQL)
                                fprintf(stderr, "       INSERT statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                        } else {
                            if (Updates != NULL) {
                                free(Updates);
                                Updates = NULL;
                            }
                            i = cols = 2;
                            while (DHL[i] != '\0') {
                                if ((values[cols] == NULL && row[i] != NULL) || (values[cols] != NULL && row[i] == NULL) || (strcmp(values[cols], row[i]) != 0)) {
                                    Updates = varAddString(Updates,  DHL[i]);
                                    Updates = varAddString(Updates,  " = ");
                                    if (values[cols] != NULL) {
                                        Updates = varAddString(Updates,  "\"");
                                        Updates = varAddString(Updates,  values[cols]);
                                        Updates = varAddString(Updates,  "\"");
                                    } else
                                        Updates = varAddString(Updates, "NULL");
                                    Updates = varAddString(Updates,  ", ");
                                }
                                i++;
                                cols++;
                            }
                            mysql_free_result(res_set);
                            if (Updates != NULL) {
                                cpos = strlen(Updates) - 1;
                                while(cpos >= 0 && (Updates[cpos] == ' ' || Updates[cpos] == ','))
                                    Updates[cpos--] = '\0';
                                if (Query != NULL) {
                                    free(Query);
                                    Query = NULL;
                                }
                                Query = varAddString(Query, "UPDATE ");
                                Query = varAddString(Query, _DHL_TABLE_NAME);
                                Query = varAddString(Query, " SET ");
                                Query = varAddString(Query, Updates);
                                Query = varAddString(Query, " WHERE vpID = \"");
                                Query = varAddString(Query, values[0]);
                                Query = varAddString(Query, "\" && YearMonthDay = \"");
                                Query = varAddString(Query, values[1]);
                                Query = varAddString(Query, "\"");
                                if (bVerboseSQL) fprintf(stderr, "Query: %s\n", Query);
                                if (mysql_query(mySQLconn, Query) != 0)
                                    printSQLerror(mySQLconn, "mysql_query() failed", 218, Query);
                                else if (bVerboseSQL)
                                    fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));
                            }
                        }
                        if (bVerboseSQL) fprintf(stderr, "\n");
                        for (i = 0; i < cols; i++) {
                            if (values[i] != NULL) {
                                free(values[i]);
                                values[i] = NULL;
                            }
                        }
                        break;

                    case '?': /* user entered unknown option */
                        if (bVerboseSQL) fprintf(stderr, "U rt 0x%02x ch 0x%02x\n", rt, ch);
                        return;
                    default:
                        if (bVerboseSQL) fprintf(stderr, "F rt 0x%02x ch 0x%02x\n", rt, ch);
                        break;
                }
                if (! iok && LOOP) {
                    fprintf(stderr, "\nERROR! Could not validate Export version aborting...\n");
                    LOOP = FALSE;
                }
            }
            fclose(fp);
            if (MySQLconnect) mysql_close (mySQLconn);
            if (bVerboseSQL) 
                fprintf(stderr, " \n%lu rows imported from %s\n\n", iCount, sImportDataFN);
            else
                fprintf(stdout, " \n%lu rows imported from %s\n\n", iCount, sImportDataFN);
        }
    }
    if (bProfiling) logProfileSQL(" doImport");
}


/*--------------------------------------------------------------------------
        doExport
----------------------------------------------------------------------------*/
void doExport(int cid)
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    BOOL MySQLconnect = FALSE;
    FILE *fp;
    char Query[1025];
    char value[128];
    int i;
    int cols;
    int Y;
    int M;
    int D;
    char EBD[11];
    char EED[11];
    unsigned long int eCount = 0;
    unsigned long int eCountLast = 0;
    extern char VersionM[];
    extern char *sExportBegDate;
    extern char *sExportEndDate;
    extern BOOL bConsoleIDset;

    if (! bConsoleIDset) {
        fprintf(stderr, "\nERROR: Console ID (-j) must be used to export\n");
        if (! bVerboseSQL) fprintf(stderr, "\n");
        return;
    }
    if (strlen(sExportDataFN) > 0) {
        fp = fopen(sExportDataFN, "w");
        if(fp == NULL) {
            fprintf(stderr, "can't open %s\n", sExportDataFN);
            return;
        } else {
            if (mySQLconn == NULL) {
                if (bVerboseSQL) fprintf(stderr, "doExport: setting up mySQLconn to %s\n",opt_db_name);
                if (! mySQLsetup()) {
                    if (bVerboseSQL) fprintf(stderr, "doExport: setting up mySQLconn failed\n");
                    return;
                }
                MySQLconnect = TRUE;
            }
            /* Version */
            fprintf(fp, "V%c%s%c", '\3', VersionM, '\0');
            /* IndexDT Table */
            sprintf(Query, "SELECT ");
            i = 0;
            while (IndexDT[i] != '\0') {
                if (sExportBegDate || sExportEndDate) strcat(Query, "t1.");
                strcat(Query, IndexDT[i++]);
                if (IndexDT[i] != '\0') strcat(Query, ", ");
            }
            cols = i;
            strcat(Query, " FROM IndexDT");
            if (sExportBegDate || sExportEndDate) strcat(Query, " AS t1 INNER JOIN DT AS t2 USE INDEX (YearMonthDay) USING (RecID,vpID)");
            strcat(Query, " WHERE vpID = ");
            sprintf(value, "\"%i\"", cid);
            strcat(Query, value);
            if (sExportBegDate) {
                strcat(Query, " && YearMonthDay >= ");
                strcat(Query, sExportBegDate);
            }
            if (sExportEndDate) {
               strcat(Query, " && YearMonthDay < ");
               strcat(Query, sExportEndDate);
            }
            if (bVerboseSQL) fprintf(stderr, "\nQuery: %s\n", Query);
            if (mysql_query(mySQLconn, Query) != 0) {
                printSQLerror(mySQLconn, "mysql_query() failed", 219, Query);
                fclose(fp);
                return;
            } else {
                res_set = mysql_store_result(mySQLconn);
                if (res_set == NULL) {
                    printSQLerror(mySQLconn, "mysql_store_result() failed", 220, Query);
                    fclose(fp);
                    return;
                } else {
                    while ((row = mysql_fetch_row (res_set)) != NULL) {
                        fprintf(fp, "I");
                        for (i = 0; i < cols; i++) {
                            fprintf(fp, "\3");
                            if (row[i] != NULL) fprintf(fp, "%s", row[i] ? row[i] : "");
                        }
                        fprintf(fp, "%c", '\0');
                        if (!bVerboseSQL) fprintf(stderr, "%c%c", star[(eCount-1)%5], 0x0d);
                        eCount++;
                    }
                    mysql_free_result(res_set);
                }
            }
            if (bVerboseSQL) fprintf(stderr, "%-9s eCount %lu\n", "IndexDT", eCount-eCountLast);
            eCountLast = eCount;
            /* Location table */
            sprintf(Query, "SELECT ");
            i = 0;
            while (Location[i] != '\0') {
                strcat(Query, Location[i++]);
                if (Location[i] != '\0') strcat(Query, ", ");
            }
            cols = i;
            strcat(Query, " FROM Location");
            if (bVerboseSQL) fprintf(stderr, "\nQuery: %s\n", Query);
            if (mysql_query(mySQLconn, Query) != 0) {
                printSQLerror(mySQLconn, "mysql_query() failed", 221, Query);
                fclose(fp);
                return;
            } else {
                res_set = mysql_store_result(mySQLconn);
                if (res_set == NULL) {
                    printSQLerror(mySQLconn, "mysql_store_result() failed", 222, Query);
                    fclose(fp);
                    return;
                } else {
                    while ((row = mysql_fetch_row (res_set)) != NULL) {
                        fprintf(fp, "L");
                        for (i = 0; i < cols; i++) {
                            fprintf(fp, "\3");
                            if (row[i] != NULL) fprintf(fp, "%s", row[i] ? row[i] : "");
                        }
                        fprintf(fp, "%c", '\0');
                        if (!bVerboseSQL) fprintf(stderr, "%c%c", star[(eCount-1)%5], 0x0d);
                        eCount++;
                    }
                    mysql_free_result(res_set);
                }
            }
            if (bVerboseSQL) fprintf(stderr, "%-9s eCount %lu\n", "Location", eCount-eCountLast);
            eCountLast = eCount;
            /* DT, Temperature, Humidity, Wind, Rain, Barometer, Solar tables */
            sprintf(Query, "SELECT ");
            i = cols = 0;
            while (DT[i] != '\0') {
                cols++;
                strcat(Query, "t1.");
                strcat(Query, DT[i++]);
                strcat(Query, ", ");
            }
            i = 0;
            while (Temperature[i] != '\0') {
                cols++;
                strcat(Query, "t2.");
                strcat(Query, Temperature[i++]);
                strcat(Query, ", ");
            }
            i = 0;
            while (Humidity[i] != '\0') {
                cols++;
                strcat(Query, "t3.");
                strcat(Query, Humidity[i++]);
                strcat(Query, ", ");
            }
            i = 0;
            while (Wind[i] != '\0') {
                cols++;
                strcat(Query, "t4.");
                strcat(Query, Wind[i++]);
                strcat(Query, ", ");
            }
            i = 0;
            while (Rain[i] != '\0') {
                cols++;
                strcat(Query, "t5.");
                strcat(Query, Rain[i++]);
                strcat(Query, ", ");
            }
            i = 0;
            while (Barometer[i] != '\0') {
                cols++;
                strcat(Query, "t6.");
                strcat(Query, Barometer[i++]);
                strcat(Query, ", ");
            }
            i = 0;
            while (Solar[i] != '\0') {
                cols++;
                strcat(Query, "t7.");
                strcat(Query, Solar[i++]);
                if (Solar[i] != '\0') strcat(Query, ", ");
            }
            strcat(Query, " FROM DT AS t1 LEFT JOIN Temperature AS t2 USING (RecID) LEFT JOIN Humidity AS t3 USING (RecID) LEFT JOIN Wind AS t4 USING (RecID) LEFT JOIN Rain AS t5 USING (RecID) LEFT JOIN Barometer AS t6 USING (RecID) LEFT JOIN Solar AS t7 USING (RecID) WHERE t1.vpID = ");
            sprintf(value, "\"%i\"", cid);
            strcat(Query, value);
            if (sExportBegDate) {
                strcat(Query, " && t1.YearMonthDay >= ");
                strcat(Query, sExportBegDate);
            }
            if (sExportEndDate) {
                strcat(Query, " && t1.YearMonthDay < ");
                strcat(Query, sExportEndDate);
            }
            if (bVerboseSQL) fprintf(stderr, "\nQuery: %s\n", Query);
            if (mysql_query(mySQLconn, Query) != 0) {
                printSQLerror(mySQLconn, "mysql_query() failed", 223, Query);
                fclose(fp);
                return;
            } else {
                res_set = mysql_store_result(mySQLconn);
                if (res_set == NULL) {
                    printSQLerror(mySQLconn, "mysql_store_result() failed", 224, Query);
                    fclose(fp);
                    return;
                } else {
                    while ((row = mysql_fetch_row (res_set)) != NULL) {
                        fprintf(fp, "T");
                        for (i = 0; i < cols; i++) {
                            fprintf(fp, "\3");
                            if (row[i] != NULL) fprintf(fp, "%s", row[i] ? row[i] : "");
                        }
                        fprintf(fp, "%c", '\0');
                        if (!bVerboseSQL) fprintf(stderr, "%c%c", star[(eCount-1)%5], 0x0d);
                        eCount++;
                    }
                    mysql_free_result(res_set);
                }
            }
            if (bVerboseSQL) fprintf(stderr, "%-9s eCount %lu\n", "DT et all", eCount-eCountLast);
            eCountLast = eCount;
            /* Hourly table */
            sprintf(Query, "SELECT t2.vpID, ");
            i = 0;
            while (Hourly[i] != '\0') {
                strcat(Query, "t1.");
                strcat(Query, Hourly[i++]);
                if (Hourly[i] != '\0') strcat(Query, ", ");
            }
            cols = i + 1;
            strcat(Query, " FROM Hourly AS t1 INNER JOIN DT AS t2");
            if (sExportBegDate || sExportEndDate) strcat(Query, " USE INDEX (YearMonthDay)");
            strcat(Query, " USING (RecID) WHERE vpID = ");
            sprintf(value, "\"%i\"", cid);
            strcat(Query, value);
            if (sExportBegDate) {
                strcat(Query, " && t2.YearMonthDay >= ");
                strcat(Query, sExportBegDate);
            }
            if (sExportEndDate) {
                strcat(Query, " && t2.YearMonthDay < ");
                strcat(Query, sExportEndDate);
            }
            if (bVerboseSQL) fprintf(stderr, "\nQuery: %s\n", Query);
            if (mysql_query(mySQLconn, Query) != 0) {
                printSQLerror(mySQLconn, "mysql_query() failed", 225, Query);
                fclose(fp);
                return;
            } else {
                res_set = mysql_store_result(mySQLconn);
                if (res_set == NULL) {
                    printSQLerror(mySQLconn, "mysql_store_result() failed", 226, Query);
                    fclose(fp);
                    return;
                } else {
                    while ((row = mysql_fetch_row (res_set)) != NULL) {
                        fprintf(fp, "H");
                        for (i = 0; i < cols; i++) {
                            fprintf(fp, "\3");
                            if (row[i] != NULL) fprintf(fp, "%s", row[i] ? row[i] : "");
                        }
                        fprintf(fp, "%c", '\0');
                        if (!bVerboseSQL) fprintf(stderr, "%c%c", star[(eCount-1)%5], 0x0d);
                        eCount++;
                    }
                    mysql_free_result(res_set);
                }
            }
            if (bVerboseSQL) fprintf(stderr, "%-9s eCount %lu\n", "Hourly", eCount-eCountLast);
            eCountLast = eCount;
            /* YMD table */
            sprintf(Query, "SELECT ");
            i = 0;
            while (YMD[i] != '\0') {
                strcat(Query, YMD[i++]);
                if (YMD[i] != '\0') strcat(Query, ", ");
            }
            cols = i;
            strcat(Query, " FROM YMD");
            if (sExportBegDate || sExportEndDate) strcat(Query, " USE INDEX (PRIMARY)");
            strcat(Query, " WHERE vpID = ");
            sprintf(value, "\"%i\"", cid);
            strcat(Query, value);
            if (sExportBegDate) {
                strcat(Query, " && YearMonthDay >= ");
                strcat(Query, sExportBegDate);
            }
            if (sExportEndDate) {
                strcat(Query, " && YearMonthDay < ");
                strcat(Query, sExportEndDate);
            }
            if (bVerboseSQL) fprintf(stderr, "\nQuery: %s\n", Query);
            if (mysql_query(mySQLconn, Query) != 0) {
                printSQLerror(mySQLconn, "mysql_query() failed", 227, Query);
                fclose(fp);
                return;
            } else {
                res_set = mysql_store_result(mySQLconn);
                if (res_set == NULL) {
                    printSQLerror(mySQLconn, "mysql_store_result() failed", 228, Query);
                    fclose(fp);
                    return;
                } else {
                    while ((row = mysql_fetch_row (res_set)) != NULL) {
                        fprintf(fp, "Y");
                        for (i = 0; i < cols; i++) {
                            fprintf(fp, "\3");
                            if (row[i] != NULL) fprintf(fp, "%s", row[i] ? row[i] : "");
                        }
                        fprintf(fp, "%c", '\0');
                        if (!bVerboseSQL) fprintf(stderr, "%c%c", star[(eCount-1)%5], 0x0d);
                        eCount++;
                    }
                    mysql_free_result(res_set);
                }
            }
            if (bVerboseSQL) fprintf(stderr, "%-9s eCount %lu\n", "YMD", eCount-eCountLast);
            eCountLast = eCount;
            /* Storms table */
            sprintf(Query, "SELECT "); 
            i = 0;
            while (Storms[i] != '\0') {
                strcat(Query, Storms[i++]);
                if (Storms[i] != '\0') strcat(Query, ", ");
            }
            cols = i;
            strcat(Query, " FROM Storms");
            if (sExportBegDate || sExportEndDate) strcat(Query, " USE INDEX (PRIMARY)");
            strcat(Query, " WHERE vpID = ");
            sprintf(value, "\"%i\"", cid);
            strcat(Query, value);
            if (sExportBegDate) {
                Y = M = D = 0;
                EBD[0] = '\0';
                sscanf(sExportBegDate, "%4d%2d%2d", &Y, &M, &D);
                sprintf(EBD, "%4d-%02d-%02d", Y, M, D);
                strcat(Query, " && StormStartedDate >= \"");
                strcat(Query, EBD);
                strcat(Query, "\"");
            }
            if (sExportEndDate) {
                Y = M = D = 0;
                EED[0] = '\0';
                sscanf(sExportEndDate, "%4d%2d%2d", &Y, &M, &D);
                sprintf(EED, "%4d-%02d-%02d", Y, M, D);
                strcat(Query, " && StormStartedDate < \"");
                strcat(Query, EED);
                strcat(Query, "\"");
            }
            if (bVerboseSQL) fprintf(stderr, "\nQuery: %s\n", Query);
            if (mysql_query(mySQLconn, Query) != 0) {
                printSQLerror(mySQLconn, "mysql_query() failed", 229, Query);
                fclose(fp);
                return;
            } else {
                res_set = mysql_store_result(mySQLconn);
                if (res_set == NULL) {
                    printSQLerror(mySQLconn, "mysql_store_result() failed", 230, Query);
                    fclose(fp);
                    return;
                } else {
                    while ((row = mysql_fetch_row (res_set)) != NULL) {
                        fprintf(fp, "S");
                        for (i = 0; i < cols; i++) {
                            fprintf(fp, "\3");
                            if (row[i] != NULL) fprintf(fp, "%s", row[i] ? row[i] : "");
                        }
                        fprintf(fp, "%c", '\0');
                        if (!bVerboseSQL) fprintf(stderr, "%c%c", star[(eCount-1)%5], 0x0d);
                        eCount++;
                    }
                    mysql_free_result(res_set);
                }
            }
            if (bVerboseSQL) fprintf(stderr, "%-9s eCount %lu\n", "Storms", eCount-eCountLast);
            eCountLast = eCount;
            /* THSW, UV, DewPoint from DHL table */
            sprintf(Query, "SELECT ");
            i = 0;
            while (DHL[i] != '\0') {
                strcat(Query, DHL[i++]);
                if (DHL[i] != '\0') strcat(Query, ", ");
            }
            cols = i;
            strcat(Query, " FROM DHL");
            if (sExportBegDate || sExportEndDate) strcat(Query, " USE INDEX (PRIMARY)");
            strcat(Query, " WHERE vpID = ");
            sprintf(value, "\"%i\"", cid);
            strcat(Query, value);
            if (sExportBegDate) {
                strcat(Query, " && YearMonthDay >= ");
                strcat(Query, sExportBegDate);
            }
            if (sExportEndDate) {
                strcat(Query, " && YearMonthDay < ");
                strcat(Query, sExportEndDate);
            }
            if (bVerboseSQL) fprintf(stderr, "\nQuery: %s\n", Query);
            if (mysql_query(mySQLconn, Query) != 0) {
                printSQLerror(mySQLconn, "mysql_query() failed", 231, Query);
                fclose(fp);
                return;
            } else {
                res_set = mysql_store_result(mySQLconn);
                if (res_set == NULL) {
                    printSQLerror(mySQLconn, "mysql_store_result() failed", 232, Query);
                    fclose(fp);
                    return;
                } else {
                    while ((row = mysql_fetch_row (res_set)) != NULL) {
                        fprintf(fp, "D");
                        for (i = 0; i < cols; i++) {
                            fprintf(fp, "\3");
                            if (row[i] != NULL) fprintf(fp, "%s", row[i] ? row[i] : "");
                        }
                        fprintf(fp, "%c", '\0');
                        if (!bVerboseSQL) fprintf(stderr, "%c%c", star[(eCount-1)%5], 0x0d);
                        eCount++;
                    }
                    mysql_free_result(res_set);
                }
            }
            if (bVerboseSQL) fprintf(stderr, "%-9s eCount %lu\n", "DHL", eCount-eCountLast);
            eCountLast = eCount;

            fclose(fp);
            if (MySQLconnect) mysql_close (mySQLconn);
            if (bVerboseSQL) 
                fprintf(stderr, " \n%lu rows exported to %s\n\n", eCount, sExportDataFN);
            else
                fprintf(stdout, " \n%lu rows exported to %s\n\n", eCount, sExportDataFN);
        }
    }
    if (bProfiling) logProfileSQL(" doExport");
}


/*--------------------------------------------------------------------------
        logProfileSQL
----------------------------------------------------------------------------*/
void logProfileSQL(char whereFrom[])
{
    MYSQL_RES *res_set;
    MYSQL_ROW row;
    FILE *fp;
    char Query[1024];
    int Query_ID = -1;
    int LastQuery_ID = -1;
    double Duration = 0.0;

    if (strlen(sProfileDataFN) > 0) {
        fp = fopen(sProfileDataFN, "a");
        if(fp == NULL)
            fprintf(stderr, "can't open %s\n", sProfileDataFN);
        else {
            fprintf(fp, "\nProfile for %s %s\n\n", whereFrom, getCurTime());
            strcpy(Query, "SHOW PROFILES");
            if (mysql_query(mySQLconn, Query) != 0)
                printSQLerror(mySQLconn, "mysql_query() failed", 233, Query);
            else {
                res_set = mysql_store_result(mySQLconn);
                if (res_set == NULL)
                    printSQLerror(mySQLconn, "mysql_store_result() failed", 234, Query);
                else {
                    while ((row = mysql_fetch_row (res_set)) != NULL) {
                        if (row[0] != NULL) sscanf(row[0], "%i", &Query_ID);
                        if (row[1] != NULL) sscanf(row[1], "%lf", &Duration);
                        if (row[2] != NULL) sprintf(Query, "%s", row[2]); /* need to use sprintf if there's spaces in the string */
                        if (Query_ID > LastProfileID) 
                            fprintf(fp, "%i\t%12.8lf\t%s\n", Query_ID, Duration, Query);
                        LastQuery_ID = Query_ID;
                    }
                    mysql_free_result(res_set);
                }
            }
            fclose(fp);
        }
    }
    LastProfileID = LastQuery_ID;
}


/*--------------------------------------------------------------------------
        mySQLprocess
----------------------------------------------------------------------------*/
BOOL mySQLprocess(char * pData, int cid)
{
    time_t eTT;
    int STATUS = TRUE;
    unsigned long long int InsertPtrRT = 0;
    unsigned long long int InsertPtrGR = 0;
    char Query[1024];

    if (bVerboseSQL) fprintf(stderr, "mySQLprocess %s\n", getCurTime());
    if (mySQLconn == NULL) {
        if (bVerboseSQL) fprintf(stderr, "mySQLprocess: setting up mySQLconn to %s\n",opt_db_name);
        if (! mySQLsetup()) {
            if (bVerboseSQL) fprintf(stderr, "mySQLprocess: setting up mySQLconn failed\n");
            return FALSE;
        }
    }

    LoadCnt = 0;

    chkLocation(cid);

    if(bGetRTD && bRTDSuccess) {
        if(bVerboseSQL) fprintf(stderr, "\nLoading real time data set into DB...\n");
        InsertPtrRT = setupRecID(cid, ttrt);			/* mySQLprocess rt */
        if (InsertPtrRT != 0) {
#ifdef _TEMP_TABLE_NAME
            STATUS &= TemperatureUpdate(InsertPtrRT);
#endif
#ifdef _HUM_TABLE_NAME
            STATUS &= HumidityUpdate(InsertPtrRT);
#endif
#ifdef _BARO_TABLE_NAME
            STATUS &= BarometerUpdate(InsertPtrRT);
#endif
#ifdef _WIND_TABLE_NAME
            STATUS &= WindUpdate(InsertPtrRT);
#endif
#ifdef _RAIN_TABLE_NAME
            STATUS &= RainUpdate(InsertPtrRT, cid);
#endif
#ifdef _SOLAR_TABLE_NAME
            STATUS &= SolarUpdate(InsertPtrRT, cid);
#endif
#ifdef _FORECAST_TABLE_NAME
            STATUS &= ForecastUpdate(InsertPtrRT, cid);
#endif
        }
    }

    if(bGetHLD && bHLDSuccess) {
        if(bVerboseSQL) fprintf(stderr, "\nComparing reported highs/lows data set to DB...\n");
        STATUS &= HighLowUpdate(cid);

    }

    if(bGetGD && bGDSuccess) {
        if(bVerboseSQL) fprintf(stderr, "\nLoading graph data set into DB...\n");
        InsertPtrGR = setupRecID(cid, ttgr);			/* mySQLprocess gr */
#ifdef _HOURLY_TABLE_NAME
        STATUS &= HourlyUpdate(InsertPtrGR, pData, cid);
#endif
#ifdef _STORMS_TABLE_NAME
        STATUS &= StormsUpdate(InsertPtrGR, pData, cid);
#endif
#ifdef _MINUTE_PTR_DATA_TABLE_NAME
        STATUS &= MinPtrUpdate(InsertPtrGR, pData, cid);
#endif
    }

    eTT = time(NULL);
    sprintf(Query, "UPDATE IndexDT SET eTT = \"%lu\", rt = \"%i\" WHERE vpID = \"%i\" && sTT = \"%lu\"", eTT, (int)(eTT-ttRun), cid, ttRun);
    if (bVerboseSQL) fprintf(stderr, "\nQuery: %s\n", Query);
    if (mysql_query(mySQLconn, Query) != 0)
        printSQLerror(mySQLconn, "mysql_query() failed", 235, Query);
    else if (bVerboseSQL)
        fprintf(stderr, "       UPDATE statement succeeded: %lu rows affected\n", (unsigned long)mysql_affected_rows(mySQLconn));

    if (bProfiling) disableProfileSQL("mySQLprocess");

    /* disconnect from server */
    mysql_close (mySQLconn);

    if (bVerboseSQL || ! bQuiet) fprintf(stderr, "\nLoaded %i data items into DB %s\n", LoadCnt, getCurTime());

    return TRUE;
}

/* END */

